@@ -33,6 +33,7 @@
 */
 
 #include "svgexport.h"
+#include "SvgUtil.h"
 
 #include <KarbonDocument.h>
 #include <KarbonPart.h>
@@ -48,8 +49,10 @@
 #include <plugins/artistictextshape/ArtisticTextShape.h>
 #include <pathshapes/rectangle/KoRectangleShape.h>
 #include <pathshapes/ellipse/KoEllipseShape.h>
-#include <plugins/pictureshape/PictureShape.h>
 #include <KoImageData.h>
+#include "KoFilterEffect.h"
+#include "KoFilterEffectStack.h"
+#include "KoXmlWriter.h"
 
 #include <KGenericFactory>
 #include <KMimeType>
@@ -65,8 +68,6 @@
 #include <QtGui/QLinearGradient>
 #include <QtGui/QRadialGradient>
 
-const double ScaleToUserSpace = 90.0 / 72.0;
-
 static void printIndentation( QTextStream *stream, unsigned int indent )
 {
     static const QString INDENT("  ");
@@ -80,7 +81,8 @@ K_EXPORT_COMPONENT_FACTORY( libkarbonsvgexport, SvgExportFactory( "kofficefilter
 SvgExport::SvgExport( QObject*parent, const QStringList& )
     : KoFilter(parent), m_indent( 0 ), m_indent2( 0 )
 {
-    m_userSpaceMatrix.scale( ScaleToUserSpace, ScaleToUserSpace );
+    double scaleToUserSpace = SvgUtil::toUserSpace( 1.0 );
+    m_userSpaceMatrix.scale( scaleToUserSpace, scaleToUserSpace );
 }
 
 KoFilter::ConversionStatus SvgExport::convert( const QByteArray& from, const QByteArray& to )
@@ -160,7 +162,7 @@ void SvgExport::saveLayer( KoShapeLayer * layer )
     printIndentation( m_body, m_indent++ );
     *m_body << "<g" << getID( layer ) << ">" << endl;
 
-    QList<KoShape*> sortedShapes = layer->iterator();
+    QList<KoShape*> sortedShapes = layer->childShapes();
     qSort(sortedShapes.begin(), sortedShapes.end(), KoShape::compareShapeZIndex);
 
     foreach( KoShape * shape, sortedShapes )
@@ -185,7 +187,7 @@ void SvgExport::saveGroup( KoShapeContainer * group )
         *m_body << " display=\"none\"";
     *m_body << ">" << endl;
 
-    QList<KoShape*> sortedShapes = group->iterator();
+    QList<KoShape*> sortedShapes = group->childShapes();
     qSort(sortedShapes.begin(), sortedShapes.end(), KoShape::compareShapeZIndex);
 
     foreach( KoShape * shape, sortedShapes )
@@ -227,9 +229,9 @@ void SvgExport::saveShape( KoShape * shape )
         {
             saveText( static_cast<ArtisticTextShape*>( shape ) );
         }
-        else if( shape->shapeId() == PICTURESHAPEID )
+        else if( shape->shapeId() == "PictureShape" )
         {
-            saveImage( static_cast<PictureShape*>( shape ) );
+            saveImage(shape);
         }
     }
 }
@@ -334,15 +336,16 @@ QString SvgExport::getTransform( const QMatrix &matrix, const QString &attribute
     if( isTranslation( matrix ) )
     {
         transform += QString("translate(%1, %2)")
-                    .arg( toUserSpace(matrix.dx()) )
-                    .arg( toUserSpace(matrix.dy()) );
+                    .arg( SvgUtil::toUserSpace(matrix.dx()) )
+                    .arg( SvgUtil::toUserSpace(matrix.dy()) );
     }
     else
     {
         transform += QString( "matrix(%1 %2 %3 %4 %5 %6)" )
                     .arg( matrix.m11() ).arg( matrix.m12() )
                     .arg( matrix.m21() ).arg( matrix.m22() )
-                    .arg( toUserSpace(matrix.dx()) ) .arg( toUserSpace(matrix.dy()) );
+                    .arg( SvgUtil::toUserSpace(matrix.dx()) )
+                    .arg( SvgUtil::toUserSpace(matrix.dy()) );
     }
 
     return transform + "\"";
@@ -359,8 +362,7 @@ void SvgExport::getColorStops( const QGradientStops & colorStops )
     foreach( const QGradientStop &stop, colorStops )
     {
         printIndentation( m_defs, m_indent2 );
-        *m_defs << "<stop stop-color=\"";
-        getHexColor( m_defs, stop.second );
+        *m_defs << "<stop stop-color=\"" << stop.second.name();
         *m_defs << "\" offset=\"" << QString().setNum( stop.first );
         *m_defs << "\" stop-opacity=\"" << stop.second.alphaF() << "\"" << " />" << endl;
     }
@@ -387,10 +389,10 @@ void SvgExport::getGradient( const QGradient * gradient, const QMatrix &gradient
         *m_defs << "<linearGradient id=\"" << uid << "\" ";
         *m_defs << "gradientUnits=\"userSpaceOnUse\" ";
         *m_defs << getTransform( gradientTransform, "gradientTransform" ) << " ";
-        *m_defs << "x1=\"" << toUserSpace( g->start().x() ) << "\" ";
-        *m_defs << "y1=\"" << toUserSpace( g->start().y() ) << "\" ";
-        *m_defs << "x2=\"" << toUserSpace( g->finalStop().x() ) << "\" ";
-        *m_defs << "y2=\"" << toUserSpace( g->finalStop().y() ) << "\" ";
+        *m_defs << "x1=\"" << SvgUtil::toUserSpace( g->start().x() ) << "\" ";
+        *m_defs << "y1=\"" << SvgUtil::toUserSpace( g->start().y() ) << "\" ";
+        *m_defs << "x2=\"" << SvgUtil::toUserSpace( g->finalStop().x() ) << "\" ";
+        *m_defs << "y2=\"" << SvgUtil::toUserSpace( g->finalStop().y() ) << "\" ";
         *m_defs << spreadMethod[g->spread()];
         *m_defs << ">" << endl;
 
@@ -409,11 +411,11 @@ void SvgExport::getGradient( const QGradient * gradient, const QMatrix &gradient
         *m_defs << "<radialGradient id=\"" << uid << "\" ";
         *m_defs << "gradientUnits=\"userSpaceOnUse\" ";
         *m_defs << getTransform( gradientTransform, "gradientTransform" ) << " ";
-        *m_defs << "cx=\"" << toUserSpace( g->center().x() ) << "\" ";
-        *m_defs << "cy=\"" << toUserSpace( g->center().y() ) << "\" ";
-        *m_defs << "fx=\"" << toUserSpace( g->focalPoint().x() ) << "\" ";
-        *m_defs << "fy=\"" << toUserSpace( g->focalPoint().y() ) << "\" ";
-        *m_defs << "r=\"" << QString().setNum( toUserSpace( g->radius() ) ) << "\" ";
+        *m_defs << "cx=\"" << SvgUtil::toUserSpace( g->center().x() ) << "\" ";
+        *m_defs << "cy=\"" << SvgUtil::toUserSpace( g->center().y() ) << "\" ";
+        *m_defs << "fx=\"" << SvgUtil::toUserSpace( g->focalPoint().x() ) << "\" ";
+        *m_defs << "fy=\"" << SvgUtil::toUserSpace( g->focalPoint().y() ) << "\" ";
+        *m_defs << "r=\"" << QString().setNum( SvgUtil::toUserSpace( g->radius() ) ) << "\" ";
         *m_defs << spreadMethod[g->spread()];
         *m_defs << ">" << endl;
 
@@ -501,8 +503,8 @@ void SvgExport::getPattern( KoPatternBackground * pattern, KoShape * shape )
 
     printIndentation( m_defs, m_indent2 );
     *m_defs << "<pattern id=\"" << uid << "\"";
-    *m_defs << " x=\"" << toUserSpace(offset.x()) << "\"";
-    *m_defs << " y=\"" << toUserSpace(offset.y()) << "\"";
+    *m_defs << " x=\"" << SvgUtil::toUserSpace(offset.x()) << "\"";
+    *m_defs << " y=\"" << SvgUtil::toUserSpace(offset.y()) << "\"";
 
     if( pattern->repeat() == KoPatternBackground::Stretched )
     {
@@ -512,8 +514,8 @@ void SvgExport::getPattern( KoPatternBackground * pattern, KoShape * shape )
     }
     else
     {
-        *m_defs << " width=\"" << toUserSpace(patternSize.width()) << "\"";
-        *m_defs << " height=\"" << toUserSpace(patternSize.height()) << "\"";
+        *m_defs << " width=\"" << SvgUtil::toUserSpace(patternSize.width()) << "\"";
+        *m_defs << " height=\"" << SvgUtil::toUserSpace(patternSize.height()) << "\"";
         *m_defs << " patternUnits=\"userSpaceOnUse\"";
     }
 
@@ -551,6 +553,7 @@ void SvgExport::getStyle( KoShape * shape, QTextStream * stream )
 {
     getFill( shape, stream );
     getStroke( shape, stream );
+    getEffects( shape, stream );
     if( ! shape->isVisible() )
         *stream << " display=\"none\"";
 }
@@ -567,9 +570,7 @@ void SvgExport::getFill( KoShape * shape, QTextStream *stream )
     KoColorBackground * cbg = dynamic_cast<KoColorBackground*>( shape->background() );
     if( cbg )
     {
-        *stream << " fill=\"";
-        getHexColor( stream, cbg->color() );
-        *stream << "\"";
+        *stream << " fill=\"" << cbg->color().name() << "\"";
         *stream << " fill-opacity=\"" << cbg->color().alphaF() << "\"";
     }
     KoGradientBackground * gbg = dynamic_cast<KoGradientBackground*>( shape->background() );
@@ -609,11 +610,11 @@ void SvgExport::getStroke( KoShape *shape, QTextStream *stream )
     else if( line->lineBrush().gradient() )
         getGradient( line->lineBrush().gradient(), line->lineBrush().matrix() );
     else
-        getHexColor( stream, line->color() );
+        *stream << line->color().name();
     *stream << "\"";
 
     *stream << " stroke-opacity=\"" << line->color().alphaF() << "\"";
-    *stream << " stroke-width=\"" << toUserSpace(line->lineWidth()) << "\"";
+    *stream << " stroke-width=\"" << SvgUtil::toUserSpace(line->lineWidth()) << "\"";
 
     if( line->capStyle() == Qt::FlatCap )
         *stream << " stroke-linecap=\"butt\"";
@@ -653,14 +654,31 @@ void SvgExport::getStroke( KoShape *shape, QTextStream *stream )
     }
 }
 
-void SvgExport::getHexColor( QTextStream *stream, const QColor & color )
+void SvgExport::getEffects( KoShape *shape, QTextStream *stream )
 {
-    // Convert the various color-spaces to hex
-    QString Output;
+    KoFilterEffectStack * filterStack = shape->filterEffectStack();
+    if (!filterStack)
+        return;
 
-    Output.sprintf( "#%02x%02x%02x", color.red(), color.green(), color.blue() );
+    QList<KoFilterEffect*> filterEffects = filterStack->filterEffects();
+    if (!filterEffects.count())
+        return;
 
-    *stream << Output;
+    QString uid = createUID();
+
+    printIndentation( m_defs, m_indent2 );
+
+    QByteArray ba;
+    QBuffer buffer(&ba);
+    buffer.open(QIODevice::WriteOnly);
+    KoXmlWriter writer(&buffer,m_indent2*2);
+
+    filterStack->save(writer, uid);
+    
+    *m_defs << ba;
+    *m_defs << endl;
+
+    *stream << " filter=\"url(#" << uid << ")\"";
 }
 
 void SvgExport::saveText( ArtisticTextShape * text )
@@ -668,8 +686,7 @@ void SvgExport::saveText( ArtisticTextShape * text )
     printIndentation( m_body, m_indent++ );
     *m_body << "<text" << getID( text );
 
-    getFill( text, m_body );
-    getStroke( text, m_body );
+    getStyle( text, m_body );
 
     QFont font = text->font();
 
@@ -742,9 +759,9 @@ void SvgExport::saveText( ArtisticTextShape * text )
     *m_body << "</text>" << endl;
 }
 
-void SvgExport::saveImage( PictureShape * picture )
+void SvgExport::saveImage(KoShape *picture)
 {
-    KoImageData * imageData = dynamic_cast<KoImageData*>( picture->userData() );
+    KoImageData *imageData = qobject_cast<KoImageData*>(picture->userData());
     if( ! imageData )
     {
         qWarning() << "Picture has no image data. Omitting.";
@@ -776,8 +793,7 @@ void SvgExport::saveImage( PictureShape * picture )
         QByteArray ba;
         QBuffer buffer(&ba);
         buffer.open(QIODevice::WriteOnly);
-        if( imageData->saveToFile( buffer ) )
-        {
+        if (imageData->saveData(buffer)) {
             const QString mimeType( KMimeType::findByContent( ba )->name() );
             *m_body << " xlink:href=\"data:" << mimeType << ";base64," << ba.toBase64() <<  "\"";
         }
@@ -786,7 +802,7 @@ void SvgExport::saveImage( PictureShape * picture )
     {
         // write to a temp file first
         KTemporaryFile imgFile;
-        if( imageData->saveToFile( imgFile ) )
+        if( imageData->saveData( imgFile ) )
         {
             // tz: TODO the new version of KoImageData has the extension save inside maybe that can be used
             // get the mime type from the temp file content
@@ -819,11 +835,5 @@ void SvgExport::saveImage( PictureShape * picture )
     *m_body << " />" << endl;
 }
 
-double SvgExport::toUserSpace( double value )
-{
-    return value * ScaleToUserSpace;
-}
-
-
 #include "svgexport.moc"
 
