@@ -27,14 +27,12 @@
 #include <kmessagebox.h>
 #include <kgenericfactory.h>
 #include <KoFilterChain.h>
+#include <KoZoomController.h>
 #include <QApplication>
-#include <q3ptrlist.h>
-#include <q3sortedlist.h>
+#include <QList>
 #include <QFile>
 #include <QTextStream>
 #include <QPen>
-//Added by qt3to4:
-#include <Q3ValueList>
 #include <QByteArray>
 
 // KOffice
@@ -1165,7 +1163,7 @@ KoFilter::ConversionStatus GNUMERICExport::convert( const QByteArray& from, cons
         // KSpread 200% gives zoom() = 2.5, this in GNumeric = 250%
         tmp = gnumeric_doc.createElement("gmr:Zoom");
         if (view)
-            tmp.appendChild(gnumeric_doc.createTextNode(QString::number(canvas->view()->zoom())));
+            tmp.appendChild(gnumeric_doc.createTextNode(QString::number(view->zoomController()->zoomAction()->effectiveZoom())));
         else
             tmp.appendChild(gnumeric_doc.createTextNode("1.0"));
         sheet.appendChild(tmp);
