@@ -67,7 +67,6 @@ public:
         m_timepoint(QDateTime::currentDateTime(Qt::UTC)), m_fatalerror(false)
     {
         createDocument();
-        structureStack.setAutoDelete(true);
         StackItem *stackItem=new(StackItem);
         stackItem->elementType=ElementTypeBottom;
         stackItem->m_frameset=mainFramesetElement; // The default frame set.
@@ -76,7 +75,9 @@ public:
     }
     virtual ~StructureParser()
     {
-        structureStack.clear();
+        while(!structureStack.isEmpty()) {
+            delete structureStack.pop();
+        }
     }
 public:
     virtual bool startDocument(void);
@@ -173,7 +174,7 @@ bool StructureParser::StartElementC(StackItem* stackItem, StackItem* stackCurren
     else
     {//we are not nested correctly, so consider it a parse error!
         kError(30506) << "parse error <c> tag nested neither in <p> nor in <c> nor in <a> but in "
-            << stackCurrent->itemName << endl;
+            << stackCurrent->itemName;
         return false;
     }
     return true;
@@ -204,7 +205,7 @@ bool charactersElementC (StackItem* stackItem, QDomDocument& mainDocument, const
     }
     else
     {
-        kError(30506) << "Internal error (in charactersElementC)" << endl;
+        kError(30506) << "Internal error (in charactersElementC)";
     }
 
 	return true;
@@ -223,7 +224,7 @@ bool EndElementC (StackItem* stackItem, StackItem* stackCurrent)
     }
     else
     {
-        kError(30506) << "Wrong element type!! Aborting! (</c> in StructureParser::endElement)" << endl;
+        kError(30506) << "Wrong element type!! Aborting! (</c> in StructureParser::endElement)";
         return false;
     }
     return true;
@@ -252,7 +253,7 @@ bool StructureParser::StartElementA(StackItem* stackItem, StackItem* stackCurren
         if (stackItem->strTemp1[0]=='#')
         {
             kWarning(30506) << "Anchor <a> to bookmark: " << stackItem->strTemp1 << endl
-                << " Processing <a> like <c>" << endl;
+                << " Processing <a> like <c>";
             // We have a reference to a bookmark. Therefore treat it as a normal content <c>
             return StartElementC(stackItem, stackCurrent, attributes);
         }
@@ -260,7 +261,7 @@ bool StructureParser::StartElementA(StackItem* stackItem, StackItem* stackCurren
     else
     {//we are not nested correctly, so consider it a parse error!
         kError(30506) << "parse error <a> tag not a child of <p> but of "
-            << stackCurrent->itemName << endl;
+            << stackCurrent->itemName;
         return false;
     }
     return true;
@@ -277,7 +278,7 @@ static bool EndElementA (StackItem* stackItem, StackItem* stackCurrent, QDomDocu
 {
     if (stackItem->elementType!=ElementTypeAnchor)
     {
-        kError(30506) << "Wrong element type!! Aborting! (</a> in StructureParser::endElement)" << endl;
+        kError(30506) << "Wrong element type!! Aborting! (</a> in StructureParser::endElement)";
         return false;
     }
 
@@ -379,7 +380,7 @@ bool EndElementP (StackItem* stackItem)
 {
     if (stackItem->elementType!=ElementTypeParagraph)
     {
-        kError(30506) << "Wrong element type!! Aborting! (in endElementP)" << endl;
+        kError(30506) << "Wrong element type!! Aborting! (in endElementP)";
         return false;
     }
     stackItem->stackElementText.normalize();
@@ -446,7 +447,7 @@ static bool StartElementField(StackItem* stackItem, StackItem* stackCurrent,
     else
     {//we are not nested correctly, so consider it a parse error!
         kError(30506) << "parse error <field> tag not nested in <p> but in "
-            << stackCurrent->itemName << endl;
+            << stackCurrent->itemName;
         return false;
     }
     return true;
@@ -479,7 +480,7 @@ static bool StartElementS(StackItem* stackItem, StackItem* /*stackCurrent*/,
         kDebug(30506) <<" Style name:" << strStyleName << endl
             << " Based on: " << strBasedOn  << endl
             << " Level: " << level << endl
-            << " Props: " << attributes.value("props") << endl;
+            << " Props: " << attributes.value("props");
     }
 
     return true;
@@ -493,7 +494,7 @@ bool StructureParser::StartElementImage(StackItem* stackItem, StackItem* stackCu
     if ((stackCurrent->elementType!=ElementTypeParagraph) && (stackCurrent->elementType!=ElementTypeContent))
     {//we are not nested correctly, so consider it a parse error!
         kError(30506) << "parse error <image> tag nested neither in <p> nor in <c> but in "
-            << stackCurrent->itemName << endl;
+            << stackCurrent->itemName;
         return false;
     }
     stackItem->elementType=ElementTypeEmpty;
@@ -625,12 +626,12 @@ bool StructureParser::EndElementD (StackItem* stackItem)
 {
     if (stackItem->elementType!=ElementTypeRealData)
     {
-        kError(30506) << "Wrong element type!! Aborting! (in endElementD)" << endl;
+        kError(30506) << "Wrong element type!! Aborting! (in endElementD)";
         return false;
     }
     if (!m_chain)
     {
-        kError(30506) << "No filter chain! Aborting! (in endElementD)" << endl;
+        kError(30506) << "No filter chain! Aborting! (in endElementD)";
         return false;
     }
 
@@ -655,7 +656,7 @@ bool StructureParser::EndElementD (StackItem* stackItem)
     else
     {
         kWarning(30506) << "Unknown or unsupported mime type: "
-            << stackItem->strTemp1 << endl;
+            << stackItem->strTemp1;
         return true;
     }
 
@@ -680,7 +681,7 @@ bool StructureParser::EndElementD (StackItem* stackItem)
     KoStoreDevice* out=m_chain->storageFile(strStoreName, KoStore::Write);
     if(!out)
     {
-        kError(30506) << "Unable to open output file for: " << stackItem->fontName << " Storage: " << strStoreName << endl;
+        kError(30506) << "Unable to open output file for: " << stackItem->fontName << " Storage: " << strStoreName;
         return false;
     }
 
@@ -738,16 +739,16 @@ static bool CharactersElementM (StackItem* stackItem, const QString & ch)
 
 bool StructureParser::EndElementM (StackItem* stackItem)
 {
-    if (stackItem->elementType!=ElementTypeRealData)
+    if (stackItem->elementType!=ElementTypeRealMetaData)
     {
-        kError(30506) << "Wrong element type!! Aborting! (in endElementM)" << endl;
+        kError(30506) << "Wrong element type!! Aborting! (in endElementM)";
         return false;
     }
 
     if (stackItem->strTemp1.isEmpty())
     {
         // Probably an internal error!
-        kError(30506) << "Key name was erased! Aborting! (in endElementM)" << endl;
+        kError(30506) << "Key name was erased! Aborting! (in endElementM)";
         return false;
     }
 
@@ -787,7 +788,7 @@ static bool StartElementBR(StackItem* stackItem, StackItem* stackCurrent,
     else
     {//we are not nested correctly, so consider it a parse error!
         kError(30506) << "parse error <br> tag not nested in <p> or <c> but in "
-            << stackCurrent->itemName << endl;
+            << stackCurrent->itemName;
         return false;
     }
     return true;
@@ -815,7 +816,7 @@ static bool StartElementPBR(StackItem* /*stackItem*/, StackItem* stackCurrent,
 
     if (!nodeList.count())
     {
-        kError(30506) << "Unable to find <LAYOUT> element! Aborting! (in StartElementPBR)" <<endl;
+        kError(30506) << "Unable to find <LAYOUT> element! Aborting! (in StartElementPBR)";
         return false;
     }
 
@@ -823,7 +824,7 @@ static bool StartElementPBR(StackItem* /*stackItem*/, StackItem* stackCurrent,
     QDomNode newNode=nodeList.item(0).cloneNode(true); // We make a deep cloning of the first element/node
     if (newNode.isNull())
     {
-        kError(30506) << "Unable to clone <LAYOUT> element! Aborting! (in StartElementPBR)" <<endl;
+        kError(30506) << "Unable to clone <LAYOUT> element! Aborting! (in StartElementPBR)";
         return false;
     }
     paragraphElementOut.appendChild(newNode);
@@ -832,7 +833,7 @@ static bool StartElementPBR(StackItem* /*stackItem*/, StackItem* stackCurrent,
     QDomElement oldLayoutElement=nodeList.item(0).toElement();
     if (oldLayoutElement.isNull())
     {
-        kError(30506) << "Cannot find old <LAYOUT> element! Aborting! (in StartElementPBR)" <<endl;
+        kError(30506) << "Cannot find old <LAYOUT> element! Aborting! (in StartElementPBR)";
         return false;
     }
     // We have now to add a element <PAGEBREAKING>
@@ -900,8 +901,7 @@ static bool StartElementPageSize(QDomElement& paperElement, const QXmlAttributes
         QString strUnits = attributes.value("units").trimmed();
 
         kDebug(30506) <<"Explicit page size:"
-         << height << " " << strUnits << " x " << width << " " << strUnits
-         << endl;
+         << height << " " << strUnits << " x " << width << " " << strUnits;
 
         if (strUnits=="cm")
         {
@@ -936,7 +936,7 @@ static bool StartElementPageSize(QDomElement& paperElement, const QXmlAttributes
         // At least one of the two values is ridiculous
     {
         kWarning(30506) << "Page width or height is too small: "
-         << kwordHeight << "x" << kwordWidth << endl;
+         << kwordHeight << "x" << kwordWidth;
         // As we have no correct page size, we assume we have A4
         kwordFormat = KoPageFormat::IsoA4Size;
         kwordHeight = CentimetresToPoints(29.7);
@@ -947,7 +947,7 @@ static bool StartElementPageSize(QDomElement& paperElement, const QXmlAttributes
 
     if (paperElement.isNull())
     {
-        kError(30506) << "<PAPER> element cannot be accessed! Aborting!" << endl;
+        kError(30506) << "<PAPER> element cannot be accessed! Aborting!";
         return false;
     }
 
@@ -968,17 +968,17 @@ bool StructureParser::complexForcedPageBreak(StackItem* stackItem)
 
     if (!clearStackUntilParagraph(auxilaryStack))
     {
-        kError(30506) << "Could not clear stack until a paragraph!" << endl;
+        kError(30506) << "Could not clear stack until a paragraph!";
         return false;
     }
 
     // Now we are a child of a <p> element!
 
-    bool success=StartElementPBR(stackItem,structureStack.current(),mainDocument);
+    bool success=StartElementPBR(stackItem,structureStack.top(),mainDocument);
 
     // Now restore the stack
 
-    StackItem* stackCurrent=structureStack.current();
+    StackItem* stackCurrent=structureStack.top();
     StackItem* item;
     while (auxilaryStack.count()>0)
     {
@@ -1045,7 +1045,7 @@ static bool EndElementIW(StackItem* stackItem, StackItem* /*stackCurrent*/,
 {
     if (stackItem->elementType!=ElementTypeIgnoreWord)
     {
-        kError(30506) << "Wrong element type!! Aborting! (in endElementIW)" << endl;
+        kError(30506) << "Wrong element type!! Aborting! (in endElementIW)";
         return false;
     }
     QDomElement wordElement=mainDocument.createElement("SPELLCHECKIGNOREWORD");
@@ -1114,7 +1114,7 @@ bool StructureParser::StartElementTable(StackItem* stackItem, StackItem* stackCu
     for ( i=0, it=widthList.constBegin(); i<columns; ++i, ++it )
     {
         kDebug(30506) <<"Column width:" << (*it) <<" cooked" << ValueWithLengthUnit(*it);
-        stackItem->m_doubleArray.at(i+1) = ValueWithLengthUnit(*it) + stackItem->m_doubleArray.at(i);
+        stackItem->m_doubleArray[i+1] = ValueWithLengthUnit(*it) + stackItem->m_doubleArray[i];
     }
     // ### TODO: in case of automatic column widths, we have not any width given by AbiWord
 
@@ -1171,7 +1171,7 @@ bool StructureParser::StartElementCell(StackItem* stackItem, StackItem* stackCur
 #if 1
     if (stackCurrent->elementType!=ElementTypeTable)
     {
-        kError(30506) << "Wrong element type!! Aborting! (in StructureParser::endElementCell)" << endl;
+        kError(30506) << "Wrong element type!! Aborting! (in StructureParser::endElementCell)";
         return false;
     }
 
@@ -1182,7 +1182,7 @@ bool StructureParser::StartElementCell(StackItem* stackItem, StackItem* stackCur
 
     if (tableName.isEmpty())
     {
-        kError(30506) << "Table name is empty! Aborting!" << endl;
+        kError(30506) << "Table name is empty! Aborting!";
         return false;
     }
 
@@ -1198,8 +1198,8 @@ bool StructureParser::StartElementCell(StackItem* stackItem, StackItem* stackCur
         // We do not know the right position of this column, so improvise. (### TODO)
         // We play on the fact that QByteArray uses shallow copies by default.
         //  (We do want that the change is known at <table> level)
-        stackItem->m_doubleArray.resize( stackItem->m_doubleArray.size() + 1, Q3GArray::SpeedOptim );
-        stackItem->m_doubleArray.at(col+1) = stackItem->m_doubleArray.at(col) + 72; // Try 1 inch
+        stackItem->m_doubleArray.resize( stackItem->m_doubleArray.size() + 1 );
+        stackItem->m_doubleArray[col+1] = stackItem->m_doubleArray[col] + 72; // Try 1 inch
     }
 
     const QString frameName(i18nc("Frameset name","Table %3, row %1, column %2",row,col,stackCurrent->strTemp2)); // As the stack could be wrong, be careful and use the string as last!
@@ -1218,8 +1218,8 @@ bool StructureParser::StartElementCell(StackItem* stackItem, StackItem* stackCur
     framesetsPluralElement.appendChild(framesetElement);
 
     QDomElement frameElementOut(mainDocument.createElement("FRAME"));
-    frameElementOut.setAttribute( "left", stackItem->m_doubleArray.at(col) );
-    frameElementOut.setAttribute( "right", stackItem->m_doubleArray.at(col+1) );
+    frameElementOut.setAttribute( "left", stackItem->m_doubleArray[col] );
+    frameElementOut.setAttribute( "right", stackItem->m_doubleArray[col+1] );
     frameElementOut.setAttribute("top",0);
     frameElementOut.setAttribute("bottom",0);
     frameElementOut.setAttribute("runaround",1);
@@ -1249,16 +1249,16 @@ bool StructureParser :: startElement( const QString&, const QString&, const QStr
 
     if (structureStack.isEmpty())
     {
-        kError(30506) << "Stack is empty!! Aborting! (in StructureParser::startElement)" << endl;
+        kError(30506) << "Stack is empty!! Aborting! (in StructureParser::startElement)";
         return false;
     }
 
     // Create a new stack element copying the top of the stack.
-    StackItem *stackItem=new StackItem(*structureStack.current());
+    StackItem *stackItem=new StackItem(*structureStack.top());
 
     if (!stackItem)
     {
-        kError(30506) << "Could not create Stack Item! Aborting! (in StructureParser::startElement)" << endl;
+        kError(30506) << "Could not create Stack Item! Aborting! (in StructureParser::startElement)";
         return false;
     }
 
@@ -1268,32 +1268,32 @@ bool StructureParser :: startElement( const QString&, const QString&, const QStr
 
     if ((name=="c")||(name=="C"))
     {
-        success=StartElementC(stackItem,structureStack.current(),attributes);
+        success=StartElementC(stackItem,structureStack.top(),attributes);
     }
     else if ((name=="p")||(name=="P"))
     {
-        success=StartElementP(stackItem,structureStack.current(),mainDocument,
+        success=StartElementP(stackItem,structureStack.top(),mainDocument,
             styleDataMap,attributes);
     }
     else if ((name=="section")||(name=="SECTION"))
     {
-        success=StartElementSection(stackItem,structureStack.current(),attributes);
+        success=StartElementSection(stackItem,structureStack.top(),attributes);
     }
     else if (name=="a")
     {
-        success=StartElementA(stackItem,structureStack.current(),attributes);
+        success=StartElementA(stackItem,structureStack.top(),attributes);
     }
     else if (name=="br") // NOTE: Not sure if it only exists in lower case!
     {
         // We have a forced line break
-        StackItem* stackCurrent=structureStack.current();
+        StackItem* stackCurrent=structureStack.top();
         success=StartElementBR(stackItem,stackCurrent,mainDocument);
     }
     else if (name=="cbr") // NOTE: Not sure if it only exists in lower case!
     {
         // We have a forced column break (not supported by KWord)
         stackItem->elementType=ElementTypeEmpty;
-        StackItem* stackCurrent=structureStack.current();
+        StackItem* stackCurrent=structureStack.top();
         if (stackCurrent->elementType==ElementTypeContent)
         {
             kWarning(30506) << "Forced column break found! Transforming to forced page break";
@@ -1307,7 +1307,7 @@ bool StructureParser :: startElement( const QString&, const QString&, const QStr
         else
         {
             kError(30506) << "Forced column break found out of turn! Aborting! Parent: "
-                << stackCurrent->itemName <<endl;
+                << stackCurrent->itemName;
             success=false;
         }
     }
@@ -1315,7 +1315,7 @@ bool StructureParser :: startElement( const QString&, const QString&, const QStr
     {
         // We have a forced page break
         stackItem->elementType=ElementTypeEmpty;
-        StackItem* stackCurrent=structureStack.current();
+        StackItem* stackCurrent=structureStack.top();
         if (stackCurrent->elementType==ElementTypeContent)
         {
             success=complexForcedPageBreak(stackItem);
@@ -1327,7 +1327,7 @@ bool StructureParser :: startElement( const QString&, const QString&, const QStr
         else
         {
             kError(30506) << "Forced page break found out of turn! Aborting! Parent: "
-                << stackCurrent->itemName <<endl;
+                << stackCurrent->itemName;
             success=false;
         }
     }
@@ -1335,26 +1335,26 @@ bool StructureParser :: startElement( const QString&, const QString&, const QStr
         // Does only exist as lower case tag!
     {
         stackItem->elementType=ElementTypeEmpty;
-        stackItem->stackElementText=structureStack.current()->stackElementText; // TODO: reason?
+        stackItem->stackElementText=structureStack.top()->stackElementText; // TODO: reason?
         success=StartElementPageSize(m_paperElement,attributes);
     }
     else if ((name=="field") //TODO: upper-case?
         || (name=="f")) // old deprecated name for <field>
     {
-        success=StartElementField(stackItem,structureStack.current(),mainDocument,attributes);
+        success=StartElementField(stackItem,structureStack.top(),mainDocument,attributes);
     }
     else if (name=="s") // Seems only to exist as lower case
     {
-        success=StartElementS(stackItem,structureStack.current(),attributes,styleDataMap);
+        success=StartElementS(stackItem,structureStack.top(),attributes,styleDataMap);
     }
     else if ((name=="image") //TODO: upper-case?
         || (name=="i")) // old deprecated name for <image>
     {
-        success=StartElementImage(stackItem,structureStack.current(),attributes);
+        success=StartElementImage(stackItem,structureStack.top(),attributes);
     }
     else if (name=="d") // TODO: upper-case?
     {
-        success=StartElementD(stackItem,structureStack.current(),attributes);
+        success=StartElementD(stackItem,structureStack.top(),attributes);
     }
     else if (name=="iw") // No upper-case
     {
@@ -1363,24 +1363,24 @@ bool StructureParser :: startElement( const QString&, const QString&, const QStr
     }
     else if (name=="m") // No upper-case
     {
-        success=StartElementM(stackItem,structureStack.current(),attributes);
+        success=StartElementM(stackItem,structureStack.top(),attributes);
     }
     else if (name=="foot") // No upper-case
     {
-        success=StartElementFoot(stackItem,structureStack.current(),attributes);
+        success=StartElementFoot(stackItem,structureStack.top(),attributes);
     }
     else if (name=="table") // No upper-case
     {
-        success=StartElementTable(stackItem,structureStack.current(), attributes);
+        success=StartElementTable(stackItem,structureStack.top(), attributes);
     }
     else if (name=="cell") // No upper-case
     {
-        success=StartElementCell(stackItem,structureStack.current(),attributes);
+        success=StartElementCell(stackItem,structureStack.top(),attributes);
     }
     else
     {
         stackItem->elementType=ElementTypeUnknown;
-        stackItem->stackElementText=structureStack.current()->stackElementText; // TODO: reason?
+        stackItem->stackElementText=structureStack.top()->stackElementText; // TODO: reason?
         success=true;
     }
     if (success)
@@ -1401,7 +1401,7 @@ bool StructureParser :: endElement( const QString&, const QString& , const QStri
 
     if (structureStack.isEmpty())
     {
-        kError(30506) << "Stack is empty!! Aborting! (in StructureParser::endElement)" << endl;
+        kError(30506) << "Stack is empty!! Aborting! (in StructureParser::endElement)";
         return false;
     }
 
@@ -1410,7 +1410,7 @@ bool StructureParser :: endElement( const QString&, const QString& , const QStri
     StackItem *stackItem=structureStack.pop();
     if ((name=="c")||(name=="C"))
     {
-        success=EndElementC(stackItem,structureStack.current());
+        success=EndElementC(stackItem,structureStack.top());
     }
     else if ((name=="p")||(name=="P"))
     {
@@ -1421,12 +1421,12 @@ bool StructureParser :: endElement( const QString&, const QString& , const QStri
         if (stackItem->elementType==ElementTypeContent)
         {
             // Anchor to a bookmark (not supported by KWord))
-            success=EndElementC(stackItem,structureStack.current());
+            success=EndElementC(stackItem,structureStack.top());
         }
         else
         {
             // Normal anchor
-            success=EndElementA(stackItem,structureStack.current(), mainDocument);
+            success=EndElementA(stackItem,structureStack.top(), mainDocument);
         }
     }
     else if (name=="d")
@@ -1435,7 +1435,7 @@ bool StructureParser :: endElement( const QString&, const QString& , const QStri
     }
     else if (name=="iw") // No upper-case
     {
-        success=EndElementIW(stackItem,structureStack.current(), mainDocument, m_ignoreWordsElement);
+        success=EndElementIW(stackItem,structureStack.top(), mainDocument, m_ignoreWordsElement);
     }
     else if (name=="m") // No upper-case
     {
@@ -1443,13 +1443,13 @@ bool StructureParser :: endElement( const QString&, const QString& , const QStri
     }
     else
     {
-        success=true; // No problem, so authorisation to continue parsing
+        success=true; // No problem, so authorization to continue parsing
     }
     if (!success)
     {
         // If we have no success, then it was surely a tag mismatch. Help debugging!
         kError(30506) << "Found tag name: " << name
-            << " expected: " << stackItem->itemName << endl;
+            << " expected: " << stackItem->itemName;
     }
     delete stackItem;
     return success;
@@ -1473,13 +1473,13 @@ bool StructureParser :: characters ( const QString & ch )
     // DEBUG end
     if (structureStack.isEmpty())
     {
-        kError(30506) << "Stack is empty!! Aborting! (in StructureParser::characters)" << endl;
+        kError(30506) << "Stack is empty!! Aborting! (in StructureParser::characters)";
         return false;
     }
 
     bool success=false;
 
-    StackItem *stackItem=structureStack.current();
+    StackItem *stackItem=structureStack.top();
 
     if ((stackItem->elementType==ElementTypeContent)
         || (stackItem->elementType==ElementTypeAnchorContent))
@@ -1501,7 +1501,7 @@ bool StructureParser :: characters ( const QString & ch )
         {
             // We have a parsing error, so abort!
             kError(30506) << "Empty element "<< stackItem->itemName
-                <<" is not empty! Aborting! (in StructureParser::characters)" << endl;
+                <<" is not empty! Aborting! (in StructureParser::characters)";
         }
     }
     else if (stackItem->elementType==ElementTypeRealData)
@@ -1611,7 +1611,7 @@ bool StructureParser::endDocument(void)
 bool StructureParser::warning(const QXmlParseException& exception)
 {
     kWarning(30506) << "XML parsing warning: line " << exception.lineNumber()
-        << " col " << exception.columnNumber() << " message: " << exception.message() << endl;
+        << " col " << exception.columnNumber() << " message: " << exception.message();
     return true;
 }
 
@@ -1619,14 +1619,14 @@ bool StructureParser::error(const QXmlParseException& exception)
 {
     // A XML error is recoverable, so it is only a KDE warning
     kWarning(30506) << "XML parsing error: line " << exception.lineNumber()
-        << " col " << exception.columnNumber() << " message: " << exception.message() << endl;
+        << " col " << exception.columnNumber() << " message: " << exception.message();
     return true;
 }
 
 bool StructureParser::fatalError (const QXmlParseException& exception)
 {
     kError(30506) << "XML parsing fatal error: line " << exception.lineNumber()
-        << " col " << exception.columnNumber() << " message: " << exception.message() << endl;
+        << " col " << exception.columnNumber() << " message: " << exception.message();
     m_fatalerror=true;
     KMessageBox::error(NULL, i18n("An error has occurred while parsing the AbiWord file.\nAt line: %1, column %2\nError message: %3",exception.lineNumber(),exception.columnNumber(), i18n( "QXml", exception.message() ) ),
         i18n("AbiWord Import Filter"),0);
@@ -1735,7 +1735,7 @@ bool StructureParser::clearStackUntilParagraph(StackItemStack& auxilaryStack)
             {
                 // Something has gone wrong!
                 kError(30506) << "Cannot clear this element: "
-                    << item->itemName << endl;
+                    << item->itemName;
                 return false;
             }
         }
@@ -1792,13 +1792,13 @@ KoFilter::ConversionStatus ABIWORDImport::convert( const QByteArray& from, const
 
     if ( !in )
     {
-        kError(30506) << "Cannot create device for uncompressing! Aborting!" << endl;
+        kError(30506) << "Cannot create device for uncompressing! Aborting!";
         return KoFilter::FileNotFound; // ### TODO: better error?
     }
 
     if (!in->open(QIODevice::ReadOnly))
     {
-        kError(30506) << "Cannot open file for uncompressing! Aborting!" << endl;
+        kError(30506) << "Cannot open file for uncompressing! Aborting!";
         delete in;
         return KoFilter::FileNotFound;
     }
@@ -1809,7 +1809,7 @@ KoFilter::ConversionStatus ABIWORDImport::convert( const QByteArray& from, const
 
     if (!reader.parse( source ))
     {
-        kError(30506) << "Import: Parsing unsuccessful. Aborting!" << endl;
+        kError(30506) << "Import: Parsing unsuccessful. Aborting!";
         delete in;
         if (!handler.wasFatalError())
         {
@@ -1828,7 +1828,7 @@ KoFilter::ConversionStatus ABIWORDImport::convert( const QByteArray& from, const
     out=m_chain->storageFile( "documentinfo.xml", KoStore::Write );
     if(!out)
     {
-        kError(30506) << "AbiWord Import unable to open output file! (Documentinfo)" << endl;
+        kError(30506) << "AbiWord Import unable to open output file! (Documentinfo)";
         KMessageBox::error(NULL, i18n("Unable to save document information."),i18n("AbiWord Import Filter"),0);
         return KoFilter::StorageCreationError;
     }
@@ -1842,7 +1842,7 @@ KoFilter::ConversionStatus ABIWORDImport::convert( const QByteArray& from, const
     out=m_chain->storageFile( "root", KoStore::Write );
     if(!out)
     {
-        kError(30506) << "AbiWord Import unable to open output file! (Root)" << endl;
+        kError(30506) << "AbiWord Import unable to open output file! (Root)";
         KMessageBox::error(NULL, i18n("Unable to save main document."),i18n("AbiWord Import Filter"),0);
         return KoFilter::StorageCreationError;
     }
