@@ -36,7 +36,7 @@
 #include <QFile>
 #include <qdom.h>
 //Added by qt3to4:
-#include <Q3ValueList>
+#include <QList>
 #include <QByteArray>
 
 #include <kdebug.h>
@@ -68,10 +68,10 @@ KWEFKWordLeader::~KWEFKWordLeader(void)
 }
 
 static FrameAnchor *findAnchor ( const KoPictureKey& key,
-                                 Q3ValueList<ParaData>& paraList )
+                                 QList<ParaData>& paraList )
 {
     kDebug(30508) <<"findAnchor" << key.toString();
-    Q3ValueList<ParaData>::Iterator paraIt;
+    QList<ParaData>::Iterator paraIt;
 
     for ( paraIt = paraList.begin (); paraIt != paraList.end (); ++paraIt )
     {
@@ -98,7 +98,7 @@ static void ProcessHardBrkTag ( QDomNode myNode, void* tagData, KWEFKWordLeader*
 {
     // <HARDBRK>
     bool* flag = (bool*) tagData;
-    Q3ValueList<AttrProcessing> attrProcessingList;
+    QList<AttrProcessing> attrProcessingList;
     attrProcessingList << AttrProcessing ( "frame", *flag );
     ProcessAttributes (myNode, attrProcessingList);
     if (*flag)
@@ -146,8 +146,8 @@ void KWEFKWordLeader::createBookmarkFormatData( ParaData& paraData )
 {
     const int paraCount = m_paraCountMap[ m_currentFramesetName ];
 
-    Q3ValueList<Bookmark>::ConstIterator it;
-    for (it = m_bookmarkList.begin(); it != m_bookmarkList.end(); ++it )
+    QList<Bookmark>::ConstIterator it;
+    for (it = m_bookmarkList.constBegin(); it != m_bookmarkList.constEnd(); ++it )
     {
         if ( (*(it)).m_frameset != m_currentFramesetName )
         {
@@ -176,7 +176,7 @@ static void ProcessParagraphTag ( QDomNode         myNode,
     kDebug (30508) <<"ProcessParagraphTag () - Begin";
 #endif
 
-    Q3ValueList<ParaData> *paraList = (Q3ValueList<ParaData> *) tagData;
+    QList<ParaData> *paraList = (QList<ParaData> *) tagData;
 
     AllowNoAttributes (myNode);
 
@@ -189,7 +189,7 @@ static void ProcessParagraphTag ( QDomNode         myNode,
     
     ParaData paraData;
 
-    Q3ValueList<TagProcessing> tagProcessingList;
+    QList<TagProcessing> tagProcessingList;
     tagProcessingList << TagProcessing ( "TEXT",    ProcessTextTag,     &paraData.text           )
                       << TagProcessing ( "FORMATS", ProcessFormatsTag,  &paraData.formattingList )
                       << TagProcessing ( "LAYOUT",  ProcessLayoutTag,   &paraData.layout         );
@@ -241,7 +241,7 @@ static void ProcessFrameTag ( QDomNode myNode, void *tagData,
     frameAnchor->frame.tWidth=0.0;
     frameAnchor->frame.bWidth=0.0;
 
-    Q3ValueList<AttrProcessing> attrProcessingList;
+    QList<AttrProcessing> attrProcessingList;
     attrProcessingList
         << AttrProcessing ( "right",        frameAnchor->frame.right  )
         << AttrProcessing ( "left",         frameAnchor->frame.left   )
@@ -328,7 +328,7 @@ static void ProcessPictureAnchor( QDomNode myNode, KWEFKWordLeader *leader, Fram
 {
     frameAnchor->type = frameType;
  
-    Q3ValueList<TagProcessing> tagProcessingList;
+    QList<TagProcessing> tagProcessingList;
     tagProcessingList
         << TagProcessing ( "FRAME",   ProcessFrameTag, frameAnchor )
         << TagProcessing ( "PICTURE", ProcessImageTag, &frameAnchor->picture.key )
@@ -347,8 +347,8 @@ static void ProcessTableAnchor( QDomNode myNode, KWEFKWordLeader *leader, FrameA
 {
     frameAnchor->type = 6; // Table
 
-    Q3ValueList<ParaData> cellParaList;
-    Q3ValueList<TagProcessing> tagProcessingList;
+    QList<ParaData> cellParaList;
+    QList<TagProcessing> tagProcessingList;
     tagProcessingList << TagProcessing ( "FRAME",     ProcessFrameTag,     frameAnchor   )
                         << TagProcessing ( "PARAGRAPH", ProcessParagraphTag, &cellParaList );
     ProcessSubtags (myNode, tagProcessingList, leader);
@@ -364,7 +364,7 @@ static void ProcessFramesetTag ( QDomNode        myNode,
     kDebug (30508) <<"ProcessFramesetTag () - Begin";
 #endif
 
-    Q3ValueList<ParaData> *paraList = (Q3ValueList<ParaData> *) tagData;
+    QList<ParaData> *paraList = (QList<ParaData> *) tagData;
 
     int     frameType = -1;
     int     frameInfo = -1;
@@ -376,7 +376,7 @@ static void ProcessFramesetTag ( QDomNode        myNode,
 
     const QString oldName ( leader->m_currentFramesetName );
     
-    Q3ValueList<AttrProcessing> attrProcessingList;
+    QList<AttrProcessing> attrProcessingList;
     attrProcessingList << AttrProcessing ( "name",        leader->m_currentFramesetName      )
                        << AttrProcessing ( "frameType",   frameType )
                        << AttrProcessing ( "frameInfo",   frameInfo )
@@ -402,7 +402,7 @@ static void ProcessFramesetTag ( QDomNode        myNode,
                 {
                     // Normal Text
                     kDebug(30508) <<"Processing Frameset:" << leader->m_currentFramesetName;
-                    Q3ValueList<TagProcessing> tagProcessingList;
+                    QList<TagProcessing> tagProcessingList;
                     tagProcessingList.append(TagProcessing ( "FRAME" ));
                     tagProcessingList.append(TagProcessing ( "PARAGRAPH", ProcessParagraphTag,  paraList ));
                     ProcessSubtags (myNode, tagProcessingList, leader);
@@ -412,7 +412,7 @@ static void ProcessFramesetTag ( QDomNode        myNode,
                     // header for first page
                     HeaderData header;
                     header.page = HeaderData::PAGE_FIRST;
-                    Q3ValueList<TagProcessing> tagProcessingList;
+                    QList<TagProcessing> tagProcessingList;
                     tagProcessingList.append(TagProcessing ( "FRAME" ));
                     tagProcessingList.append(TagProcessing ( "PARAGRAPH", ProcessParagraphTag,  &header.para ));
 
@@ -424,7 +424,7 @@ static void ProcessFramesetTag ( QDomNode        myNode,
                     // header for even page
                     HeaderData header;
                     header.page = HeaderData::PAGE_EVEN;
-                    Q3ValueList<TagProcessing> tagProcessingList;
+                    QList<TagProcessing> tagProcessingList;
                     tagProcessingList.append(TagProcessing ( "FRAME" ));
                     tagProcessingList.append(TagProcessing ( "PARAGRAPH", ProcessParagraphTag,  &header.para ));
                     ProcessSubtags (myNode, tagProcessingList, leader);
@@ -435,7 +435,7 @@ static void ProcessFramesetTag ( QDomNode        myNode,
                     // header for odd page (or all page, if hType=0)
                     HeaderData header;
                     header.page = (leader->headerType() != 0 ) ? HeaderData::PAGE_ODD : HeaderData::PAGE_ALL;
-                    Q3ValueList<TagProcessing> tagProcessingList;
+                    QList<TagProcessing> tagProcessingList;
                     tagProcessingList.append(TagProcessing ( "FRAME" ));
                     tagProcessingList.append(TagProcessing ( "PARAGRAPH", ProcessParagraphTag,  &header.para ));
                     ProcessSubtags (myNode, tagProcessingList, leader);
@@ -446,7 +446,7 @@ static void ProcessFramesetTag ( QDomNode        myNode,
                     // footer for first page
                     FooterData footer;
                     footer.page = FooterData::PAGE_FIRST;
-                    Q3ValueList<TagProcessing> tagProcessingList;
+                    QList<TagProcessing> tagProcessingList;
                     tagProcessingList.append(TagProcessing ( "FRAME" ));
                     tagProcessingList.append(TagProcessing ( "PARAGRAPH", ProcessParagraphTag,  &footer.para ));
                     ProcessSubtags (myNode, tagProcessingList, leader);
@@ -457,7 +457,7 @@ static void ProcessFramesetTag ( QDomNode        myNode,
                     // footer for even page
                     FooterData footer;
                     footer.page = FooterData::PAGE_EVEN;
-                    Q3ValueList<TagProcessing> tagProcessingList;
+                    QList<TagProcessing> tagProcessingList;
                     tagProcessingList.append(TagProcessing ( "FRAME" ));
                     tagProcessingList.append(TagProcessing ( "PARAGRAPH", ProcessParagraphTag,  &footer.para ));
                     ProcessSubtags (myNode, tagProcessingList, leader);
@@ -468,7 +468,7 @@ static void ProcessFramesetTag ( QDomNode        myNode,
                     // footer for odd page (or all page, if fType=0)
                     FooterData footer;
                     footer.page = (leader->footerType() != 0) ? FooterData::PAGE_ODD : FooterData::PAGE_ALL;
-                    Q3ValueList<TagProcessing> tagProcessingList;
+                    QList<TagProcessing> tagProcessingList;
                     tagProcessingList.append(TagProcessing ( "FRAME" ));
                     tagProcessingList.append(TagProcessing ( "PARAGRAPH", ProcessParagraphTag,  &footer.para ));
                     ProcessSubtags (myNode, tagProcessingList, leader);
@@ -494,7 +494,7 @@ static void ProcessFramesetTag ( QDomNode        myNode,
                         {
                             bool found = false;
                             KoPictureKey key( grpMgr );
-                            Q3ValueList<FrameAnchor>::Iterator it;
+                            QList<FrameAnchor>::Iterator it;
                             for ( it = leader->m_nonInlinedTableAnchors.begin(); it !=  leader->m_nonInlinedTableAnchors.end(); ++it )
                             {
                                 if ( (*it).key == key )
@@ -586,7 +586,7 @@ static void ProcessFramesetsTag ( QDomNode        myNode,
                                   KWEFKWordLeader *leader )
 {
     AllowNoAttributes (myNode);
-    Q3ValueList<TagProcessing> tagProcessingList;
+    QList<TagProcessing> tagProcessingList;
     tagProcessingList << TagProcessing ( "FRAMESET", ProcessFramesetTag, tagData );
     ProcessSubtags (myNode, tagProcessingList, leader);
 }
@@ -608,7 +608,7 @@ static void ProcessStylesPluralTag (QDomNode myNode, void *, KWEFKWordLeader *le
 
     leader->doOpenStyles ();
 
-    Q3ValueList<TagProcessing> tagProcessingList;
+    QList<TagProcessing> tagProcessingList;
     tagProcessingList << TagProcessing ( "STYLE", ProcessStyleTag, leader );
     ProcessSubtags (myNode, tagProcessingList, leader);
 
@@ -624,7 +624,7 @@ static void ProcessPaperBordersTag (QDomNode myNode, void*, KWEFKWordLeader* lea
     double top    = 0.0;
     double bottom = 0.0;
 
-    Q3ValueList<AttrProcessing> attrProcessingList;
+    QList<AttrProcessing> attrProcessingList;
     if ( leader->m_oldSyntax )
     {
         attrProcessingList
@@ -671,7 +671,7 @@ static void ProcessPaperTag (QDomNode myNode, void *, KWEFKWordLeader *leader)
     double columnspacing = 36.0; // Half-inch
     int numPages = -1;
 
-    Q3ValueList<AttrProcessing> attrProcessingList;
+    QList<AttrProcessing> attrProcessingList;
     attrProcessingList << AttrProcessing ( "format",              format      )
                        << AttrProcessing ( "width",               width       )
                        << AttrProcessing ( "height",              height      )
@@ -714,7 +714,7 @@ static void ProcessPaperTag (QDomNode myNode, void *, KWEFKWordLeader *leader)
     leader->doFullPaperFormat (format, width, height, orientation);
     leader->doFullPaperFormatOther( columns, columnspacing, numPages );
 
-    Q3ValueList<TagProcessing> tagProcessingList;
+    QList<TagProcessing> tagProcessingList;
     tagProcessingList
         << TagProcessing ( "PAPERBORDERS", ProcessPaperBordersTag, NULL )
         ;
@@ -733,7 +733,7 @@ static void ProcessVariableSettingsTag (QDomNode myNode, void *, KWEFKWordLeader
     int modificationMonth = -1;
     int modificationDay = -1;
 
-    Q3ValueList<AttrProcessing> attrProcessingList;
+    QList<AttrProcessing> attrProcessingList;
     attrProcessingList << AttrProcessing ( "startingPageNumber", vs.startingPageNumber )
                        << AttrProcessing ( "displaylink", vs.displaylink )
                        << AttrProcessing ( "underlinelink", vs.underlinelink )
@@ -795,7 +795,7 @@ static void ProcessSpellCheckIgnoreWordTag (QDomNode myNode, void *, KWEFKWordLe
 {
     QString ignoreword;
 
-    Q3ValueList<AttrProcessing> attrProcessingList;
+    QList<AttrProcessing> attrProcessingList;
     attrProcessingList
         << AttrProcessing ( "word", ignoreword )
         ;
@@ -813,7 +813,7 @@ static void ProcessSpellCheckIgnoreListTag (QDomNode myNode, void *, KWEFKWordLe
 
     leader->doOpenSpellCheckIgnoreList ();
 
-    Q3ValueList<TagProcessing> tagProcessingList;
+    QList<TagProcessing> tagProcessingList;
     tagProcessingList << TagProcessing ( "SPELLCHECKIGNOREWORD", ProcessSpellCheckIgnoreWordTag, leader );
     ProcessSubtags (myNode, tagProcessingList, leader);
 
@@ -825,7 +825,7 @@ static void ProcessPixmapsKeyTag ( QDomNode         myNode,
                                    void            *tagData,
                                    KWEFKWordLeader *leader )
 {
-    Q3ValueList<ParaData> *paraList = (Q3ValueList<ParaData> *) tagData;
+    QList<ParaData> *paraList = (QList<ParaData> *) tagData;
 
     KoPictureKey key;
 
@@ -839,7 +839,7 @@ static void ProcessPixmapsKeyTag ( QDomNode         myNode,
     // NOTE: we must always search in both inlined and non-inlined pictures. A picture can be used in both ways and a few times in each!
     
     // Process inlined pictures
-    Q3ValueList<ParaData>::Iterator paraIt;
+    QList<ParaData>::Iterator paraIt;
 
     for ( paraIt = paraList->begin(); paraIt != paraList->end(); ++paraIt )
     {
@@ -858,7 +858,7 @@ static void ProcessPixmapsKeyTag ( QDomNode         myNode,
         }
     }
     // Process non-inline pictures
-    Q3ValueList<FrameAnchor>::Iterator it;
+    QList<FrameAnchor>::Iterator it;
     for ( it = leader->m_nonInlinedPictureAnchors.begin(); it !=  leader->m_nonInlinedPictureAnchors.end(); ++it )
     {
         if ( (*it).key == key )
@@ -884,15 +884,15 @@ static void ProcessPixmapsTag ( QDomNode         myNode,
 {
     AllowNoAttributes (myNode);
 
-    Q3ValueList<TagProcessing> tagProcessingList;
+    QList<TagProcessing> tagProcessingList;
     tagProcessingList << TagProcessing ( "KEY", ProcessPixmapsKeyTag, tagData );
     ProcessSubtags (myNode, tagProcessingList, leader);
 }
 
 
-static void FreeCellParaLists ( Q3ValueList<ParaData> &paraList )
+static void FreeCellParaLists ( QList<ParaData> &paraList )
 {
-    Q3ValueList<ParaData>::Iterator paraIt;
+    QList<ParaData>::Iterator paraIt;
 
     for ( paraIt = paraList.begin (); paraIt != paraList.end (); ++paraIt )
     {
@@ -904,7 +904,7 @@ static void FreeCellParaLists ( Q3ValueList<ParaData> &paraList )
         {
             if ( (*formattingIt).id == 6 && (*formattingIt).frameAnchor.type == 6 )
             {
-                Q3ValueList<TableCell>::Iterator cellIt;
+                QList<TableCell>::Iterator cellIt;
 
                 for ( cellIt = (*formattingIt).frameAnchor.table.cellList.begin ();
                       cellIt != (*formattingIt).frameAnchor.table.cellList.end ();
@@ -925,7 +925,7 @@ static void ProcessFootnoteFramesetTag ( QDomNode myNode, void *tagData, KWEFKWo
     int frameType = -1, frameInfo = -1;
     bool visible = false;
 
-    Q3ValueList<AttrProcessing> attrProcessingList;
+    QList<AttrProcessing> attrProcessingList;
     attrProcessingList
         << AttrProcessing ( "name",        frameName      )
         << AttrProcessing ( "frameType",   frameType )
@@ -946,7 +946,7 @@ static void ProcessFootnoteFramesetTag ( QDomNode myNode, void *tagData, KWEFKWo
     {
         FootnoteData footnote;
         footnote.frameName = frameName;
-        Q3ValueList<TagProcessing> tagProcessingList;
+        QList<TagProcessing> tagProcessingList;
         tagProcessingList.append(TagProcessing ( "FRAME" ));
         tagProcessingList.append(TagProcessing ( "PARAGRAPH", ProcessParagraphTag,  &footnote.para ));
         ProcessSubtags (myNode, tagProcessingList, leader);
@@ -959,18 +959,18 @@ static void ProcessFootnoteFramesetsTag ( QDomNode myNode, void *tagData, KWEFKW
 {
     AllowNoAttributes (myNode);
 
-    Q3ValueList<TagProcessing> tagProcessingList;
+    QList<TagProcessing> tagProcessingList;
     tagProcessingList << TagProcessing ( "FRAMESET", ProcessFootnoteFramesetTag, tagData );
     ProcessSubtags (myNode, tagProcessingList, leader);
 }
 
 static void ProcessBookmarkItemTag ( QDomNode myNode, void* tag, KWEFKWordLeader *leader )
 {
-    Q3ValueList<Bookmark> * bookmarkList = static_cast< Q3ValueList<Bookmark> * > ( tag );
+    QList<Bookmark> * bookmarkList = static_cast< QList<Bookmark> * > ( tag );
 
     Bookmark bookmark;
 
-    Q3ValueList<AttrProcessing> attrProcessingList;
+    QList<AttrProcessing> attrProcessingList;
     attrProcessingList
         << AttrProcessing ( "name", bookmark.m_name )
         << AttrProcessing ( "cursorIndexStart", bookmark.m_cursorIndexStart )
@@ -995,7 +995,7 @@ static void ProcessBookmarksTag ( QDomNode myNode, void* tag, KWEFKWordLeader *l
 {
     AllowNoAttributes (myNode);
 
-    Q3ValueList<TagProcessing> tagProcessingList;
+    QList<TagProcessing> tagProcessingList;
     tagProcessingList << TagProcessing ( "BOOKMARKITEM", ProcessBookmarkItemTag, tag );
     ProcessSubtags (myNode, tagProcessingList, leader);
 }
@@ -1007,7 +1007,7 @@ void ProcessDocTag ( QDomNode         myNode,
 
     QString editor, author;
     
-    Q3ValueList<AttrProcessing> attrProcessingList;
+    QList<AttrProcessing> attrProcessingList;
 
     attrProcessingList
         << AttrProcessing ( "xmlns" )
@@ -1073,14 +1073,14 @@ void ProcessDocTag ( QDomNode         myNode,
         ProcessStylesPluralTag (nodeStyles, NULL, leader);
 
     // Process framesets, but only to find and extract footnotes (also endnotes)
-    Q3ValueList<FootnoteData> footnotes;
+    QList<FootnoteData> footnotes;
     QDomNode nodeFramesets=myNode.namedItem("FRAMESETS");
     if ( !nodeFramesets.isNull() )
         ProcessFootnoteFramesetsTag(nodeFramesets, &footnotes, leader );
 
     // Process all framesets and pictures
-    Q3ValueList<TagProcessing> tagProcessingList;
-    Q3ValueList<ParaData> paraList;
+    QList<TagProcessing> tagProcessingList;
+    QList<ParaData> paraList;
 
     tagProcessingList
         << TagProcessing ( "PAPER" ) // Already done
@@ -1190,7 +1190,7 @@ bool KWEFKWordLeader::doVariableSettings (const VariableSettingsData &varSetting
 }
 
 
-bool KWEFKWordLeader::doFullDocument (const Q3ValueList<ParaData> &paraList)
+bool KWEFKWordLeader::doFullDocument (const QList<ParaData> &paraList)
 {
     if ( m_worker )
         return m_worker->doFullDocument (paraList);
@@ -1262,7 +1262,7 @@ bool KWEFKWordLeader::doFooter ( const FooterData& footer )
     return false;
 }
 
-bool KWEFKWordLeader::doDeclareNonInlinedFramesets( Q3ValueList<FrameAnchor>& pictureAnchors, Q3ValueList<FrameAnchor>& tableAnchors )
+bool KWEFKWordLeader::doDeclareNonInlinedFramesets( QList<FrameAnchor>& pictureAnchors, QList<FrameAnchor>& tableAnchors )
 {
     if ( m_worker )
         return m_worker->doDeclareNonInlinedFramesets( pictureAnchors, tableAnchors );
