@@ -29,13 +29,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <QBuffer>
-#include <q3cstring.h>
 #include <QFile>
 #include <QFont>
 #include <QImage>
 #include <QTextCodec>
-#include <q3valuelist.h>
-#include <q3valuestack.h>
+#include <QList>
 
 #include <kdebug.h>
 #include <kgenericfactory.h>
@@ -160,7 +158,7 @@ public:
 			kDebug (30509) <<"Want to seek to" << absloc
 					  				<< " but EOF is at " << m_outfp_eof
 									<< "; so writing " << absloc - m_outfp_eof
-									<< " zeros" << endl;
+									<< " zeros";
 
 			if (fseek (m_outfp, m_outfp_eof, SEEK_SET))
 			{
@@ -242,8 +240,8 @@ private:
 	QTextCodec *m_codec;
 	QTextEncoder *m_encoder;
 
-	Q3ValueList <HeaderData> m_headerData;
-	Q3ValueList <FooterData> m_footerData;
+	QList <HeaderData> m_headerData;
+	QList <FooterData> m_footerData;
 
 	int m_headerType, m_footerType;
 	bool m_hasHeader, m_isHeaderOnFirstPage;
@@ -277,7 +275,7 @@ public:
 		m_device = new WRIDevice;
 		if (!m_device)
 		{
-			kError (30509) << "Could not allocate memory for Device" << endl;
+			kError (30509) << "Could not allocate memory for Device";
 			return;
 		}
 
@@ -374,7 +372,7 @@ public:
 								<< format << ", "
 								<< width << ", "
 								<< height << ", "
-								<< orientation << ")" << endl;
+								<< orientation << ")";
 
 		// TODO: does "format" or "orientation" matter?
 
@@ -391,7 +389,7 @@ public:
 								<< top << ", "
 								<< left << ", "
 								<< bottom << ", "
-								<< right << ")" << endl;
+								<< right << ")";
 
 		m_topMargin = MSWrite::Word (Point2Twip (top));
 		m_leftMargin = MSWrite::Word (Point2Twip (left));
@@ -406,7 +404,7 @@ public:
 		m_pageNumberStart = MSWrite::Word (varSettings.startingPageNumber);
 
 		kDebug (30509) <<"doVariableSettings pageNumberStart="
-								<< m_pageNumberStart << endl;
+								<< m_pageNumberStart;
 		return true;
 	}
 
@@ -424,7 +422,7 @@ public:
 	{
 		kDebug (30509) <<"doPageInfo (headerType=" << headerType
 								<< ", footerType=" << footerType
-								<< ")" << endl;
+								<< ")";
 
 		m_headerType = headerType;
 		switch (headerType)
@@ -463,7 +461,7 @@ public:
 		return true;
 	}
 
-	bool isParaListEmpty (const Q3ValueList <ParaData> &para)
+	bool isParaListEmpty (const QList <ParaData> &para)
 	{
 		if (para.count () == 1)
 		{
@@ -567,7 +565,7 @@ public:
 		bool wroteFooter = false;
 		m_inWhat = Footer;
 
-		for (Q3ValueList <FooterData>::Iterator it = m_footerData.begin ();
+		for (QList <FooterData>::Iterator it = m_footerData.begin ();
 				it != m_footerData.end ();
 				it++)
 		{
@@ -589,7 +587,7 @@ public:
 		bool wroteHeader = false;
 		m_inWhat = Header;
 
-		for (Q3ValueList <HeaderData>::Iterator it = m_headerData.begin ();
+		for (QList <HeaderData>::Iterator it = m_headerData.begin ();
 			it != m_headerData.end ();
 			it++)
 		{
@@ -621,7 +619,7 @@ public:
 
 #if 0
 		// dump remaining header paragraphs at the start of the body
-		for (Q3ValueList <HeaderData>::Iterator it = m_headerData.begin ();
+		for (QList <HeaderData>::Iterator it = m_headerData.begin ();
 				it != m_headerData.end ();
 				it++)
 		{
@@ -631,7 +629,7 @@ public:
 		}
 
 		// dump remaining footer paragraphs too
-		for (Q3ValueList <FooterData>::Iterator it = m_footerData.begin ();
+		for (QList <FooterData>::Iterator it = m_footerData.begin ();
 				it != m_footerData.end ();
 				it++)
 		{
@@ -896,11 +894,9 @@ public:
 		// Note: not from LibMSWrite's wmf.cpp
 		kDebug (30509) <<"\t\tBIH: width(pt)=" << width
 								<< " height(pt)=" << height
-								<< " BPP=" << bih.getBitsPerPixel ()
-								<< endl;
+								<< " BPP=" << bih.getBitsPerPixel();
 		kDebug (30509) <<"\t\tBIH: xPixelsPerMeter=" << bih.getXPixelsPerMeter ()
-								<< " yPixelsPerMeter=" << bih.getYPixelsPerMeter ()
-								<< endl;
+								<< " yPixelsPerMeter=" << bih.getYPixelsPerMeter ();
 
 
 		// write WMF Header
@@ -981,7 +977,7 @@ public:
 	static double getDimen72DPI (const int measurement, const int dotsPerMeter)
 	{
 		kDebug (30509) <<"\t\tgetDimen72DPI (measurement=" << measurement
-								<< ",dotsPerMeter=" << dotsPerMeter << ")" << endl;
+								<< ",dotsPerMeter=" << dotsPerMeter << ")";
 
 		// Can't get resolution?
 		// Assume that we are already 72dpi
@@ -1012,7 +1008,7 @@ public:
 								const bool ignoreIndent)
 	{
 		kDebug (30509) <<"--------------------------" << endl
-								<< "processImage()" << endl;
+								<< "processImage()";
 
 
 		// Write supports images in 3 formats:
@@ -1058,7 +1054,7 @@ public:
 					KoWmfPaint wmf;
 					if (!wmf.load (imageData))
 					{
-						kError (30509) << "Could not open WMF - Invalid Format!" << endl;
+						kError (30509) << "Could not open WMF - Invalid Format!";
 						return true;
 					}
 
@@ -1091,7 +1087,7 @@ public:
 					}
 					else if (wmf.isEnhanced ())
 					{
-						kError (30509) << "Enhanced WMF unsupported by QWmf, internal error!" << endl;
+						kError (30509) << "Enhanced WMF unsupported by QWmf, internal error!";
 
 						return true;
 					}
@@ -1108,8 +1104,7 @@ public:
 
 				kDebug (30509) <<"\tNow WMF: width=" << imageActualWidth
 										<< " height=" << imageActualHeight
-										<< " size=" << imageSize
-										<< endl;
+										<< " size=" << imageSize;
 
 				// we're done!
 				break;
@@ -1120,7 +1115,7 @@ public:
 				QImage image (imageData);
 				if (image.isNull ())
 				{
-					kError (30509) << "QImage IsNull: Line=" << __LINE__ << endl;
+					kError (30509) << "QImage IsNull: Line=" << __LINE__;
 					return true;
 				}
 
@@ -1132,8 +1127,7 @@ public:
 
 				kDebug (30509) <<"\tNow BMP: width=" << imageActualWidth
 										<< " height=" << imageActualHeight
-										<< " size=" << imageSize
-										<< endl;
+										<< " size=" << imageSize;
 
 				QByteArray imageWMF;
 					// input device
@@ -1149,7 +1143,7 @@ public:
 					// BMP --> WMF
 					if (!BMP2WMF (inDevice, outDevice))
 					{
-						kError (30509) << "BMP to WMF conversion error" << endl;
+						kError (30509) << "BMP to WMF conversion error";
 						return true;
 					}
 
@@ -1166,7 +1160,7 @@ public:
 					QImage image (imageData);
 					if (image.isNull())
 					{
-						kError (30509) << "QImage isNull: Line=" << __LINE__ << endl;
+						kError (30509) << "QImage isNull: Line=" << __LINE__;
 						return true;
 					}
 
@@ -1176,14 +1170,13 @@ public:
 
 				kDebug (30509) <<"\tForeign format: width=" << imageActualWidth
 										<< " height=" << imageActualHeight
-										<< " size=" << imageSize
-										<< endl;
+										<< " size=" << imageSize;
 
                 // read foreign image
                 QImage inImage;
                 if ( !inImage.loadFromData(imageData) )
                 {
-                    kError (30509) << "Could not read foreign format" << endl;
+                    kError (30509) << "Could not read foreign format";
                     return true;
                 }
 
@@ -1195,7 +1188,7 @@ public:
                 // write BMP
                 if ( !inImage.save( &outBuffer, "BMP" ) )
                 {
-                    kError (30509) << "Could not convert to BMP" << endl;
+                    kError (30509) << "Could not convert to BMP";
                     return true;
                 }
 
@@ -1207,13 +1200,12 @@ public:
 
 
 		kDebug (30509) <<"\tActual dimensions: width=" << imageActualWidth
-								<< " height=" << imageActualHeight << endl;
+								<< " height=" << imageActualHeight;
 
 		kDebug (30509) <<"\tKOffice position: left=" << frameAnchor.frame.left
 								<< " right=" << frameAnchor.frame.right
 								<< " top=" << frameAnchor.frame.top
-								<< " bottom=" << frameAnchor.frame.bottom
-								<< endl;
+								<< " bottom=" << frameAnchor.frame.bottom;
 
 		kDebug (30509) <<"\tIndent=" << MSWrite::Word (Point2Twip (frameAnchor.frame.left)) - m_leftMargin;
 		if (ignoreIndent)
@@ -1223,8 +1215,7 @@ public:
 		double displayedHeight = Point2Twip (frameAnchor.frame.bottom - frameAnchor.frame.top + 1);
 
 		kDebug (30509) <<"\tdisplayedWidth=" << displayedWidth
-								<< " displayedHeight=" << displayedHeight
-								<< endl;
+								<< " displayedHeight=" << displayedHeight;
 
 
 		//
@@ -1279,16 +1270,14 @@ public:
 
 
 		kDebug (30509) <<"processImage() successful!" << endl
-								<< "==========================" << endl
-																			<< endl
-																			<< endl;
+								<< "==========================";
 		return true;
 	}
 
 	bool processTable (const Table &table)
 	{
 		// just dump the table out for now (no layout)
-		for (Q3ValueList <TableCell>::ConstIterator it = table.cellList.begin ();
+		for (QList <TableCell>::ConstIterator it = table.cellList.begin ();
 				it != table.cellList.end ();
 				it++)
 		{
@@ -1403,9 +1392,9 @@ public:
 	#endif
 	}
 
-	bool doFullParagraphList (const Q3ValueList <ParaData> &paraList)
+	bool doFullParagraphList (const QList <ParaData> &paraList)
 	{
-		for (Q3ValueList <ParaData>::ConstIterator it = paraList.begin ();
+		for (QList <ParaData>::ConstIterator it = paraList.begin ();
 				it != paraList.end ();
 				it ++)
 		{
@@ -1469,7 +1458,7 @@ public:
 	#if 0
 		kDebug (30509) <<"Indent:" << Point2Twip (layout.indentFirst) <<""
 												<< Point2Twip (layout.indentLeft) << " "
-												<< Point2Twip (layout.indentRight) << endl;
+												<< Point2Twip (layout.indentRight);
 	#endif
 
 		// Line Spacing
@@ -1812,9 +1801,9 @@ public:
 
 		#ifdef DEBUG_PROCESS_TEXT
 			kDebug (30509) <<"Parent string:  upto=" << upto
-									<< ",length=" << stringUnicode.length () << endl;
+									<< ",length=" << stringUnicode.length ();
 			kDebug (30509) <<"Child string:   length=" << length
-									<< " (specialLoc=" << specialLocation << ")" << endl;
+									<< " (specialLoc=" << specialLocation << ")";
 		#endif
 
 			//
@@ -1906,26 +1895,26 @@ MSWriteExport::~MSWriteExport ()
 
 KoFilter::ConversionStatus MSWriteExport::convert (const QByteArray &from, const QByteArray &to)
 {
-	kDebug (30509) <<"MSWriteExport $Date: 2007-09-29 16:32:49 +0200 (Sat, 29 Sep 2007) $ using LibMSWrite"
-			  				<< MSWrite::Version << endl;
+	kDebug (30509) <<"MSWriteExport $Date: 2009-08-03 21:45:31 +0200 (Mon, 03 Aug 2009) $ using LibMSWrite"
+			  				<< MSWrite::Version;
 
 	if (to != "application/x-mswrite" || from != "application/x-kword")
 	{
-		kError (30509) << "Internal error!  Filter not implemented?" << endl;
+		kError (30509) << "Internal error!  Filter not implemented?";
 		return KoFilter::NotImplemented;
 	}
 
 	KWordMSWriteWorker *worker = new KWordMSWriteWorker;
 	if (!worker)
 	{
-		kError (30509) << "Could not allocate memory for worker" << endl;
+		kError (30509) << "Could not allocate memory for worker";
 		return KoFilter::OutOfMemory;
 	}
 
 	KWEFKWordLeader *leader = new KWEFKWordLeader (worker);
 	if (!leader)
 	{
-		kError (30509) << "Could not allocate memory for leader" << endl;
+		kError (30509) << "Could not allocate memory for leader";
 		delete worker;
 		return KoFilter::OutOfMemory;
 	}
