@@ -26,14 +26,12 @@
 #include <QFileInfo>
 #include <qfontinfo.h>
 #include <QFontDatabase>
-#include <q3picture.h>
 #include <QImage>
 #include <QRegExp>
 #include <QColor>
 #include <QDateTime>
-//Added by qt3to4:
 #include <QTextStream>
-#include <Q3ValueList>
+#include <QList>
 
 #include <klocale.h>
 #include <kglobal.h>
@@ -136,7 +134,7 @@ QString RTFWorker::makeTable(const FrameAnchor& anchor)
     const bool oldInTable = m_inTable;
     m_inTable = true;
 
-    Q3ValueList<TableCell>::ConstIterator itCell;
+    QList<TableCell>::ConstIterator itCell;
     for (itCell=anchor.table.cellList.begin();
         itCell!=anchor.table.cellList.end(); itCell++)
     {
@@ -163,7 +161,7 @@ QString RTFWorker::makeTable(const FrameAnchor& anchor)
         }
 
         kDebug(30515) <<"Cell:" << debugRowCurrent <<"," << debugCellCurrent
-            << " left: " << frame.left << " right: " << frame.right << " top: " << frame.top << " bottom " << frame.bottom << endl;
+            << " left: " << frame.left << " right: " << frame.right << " top: " << frame.top << " bottom " << frame.bottom;
         textCellHeader += writeBorder('t',qRound(PT_TO_TWIP(frame.tWidth)),frame.tColor);
         textCellHeader += writeBorder('l',qRound(PT_TO_TWIP(frame.lWidth)),frame.lColor);
         textCellHeader += writeBorder('b',qRound(PT_TO_TWIP(frame.bWidth)),frame.bColor);
@@ -171,10 +169,10 @@ QString RTFWorker::makeTable(const FrameAnchor& anchor)
         textCellHeader += WritePositiveKeyword("\\cellx",qRound(PT_TO_TWIP(frame.right) - m_paperMarginRight)); //right border of cell
 
         QString endOfParagraph;
-        Q3ValueList<ParaData> *paraList = (*itCell).paraList;
-        Q3ValueList<ParaData>::ConstIterator it;
-		Q3ValueList<ParaData>::ConstIterator end(paraList->end());
-        for (it=paraList->begin();it!=end;++it)
+        QList<ParaData> *paraList = (*itCell).paraList;
+        QList<ParaData>::ConstIterator it;
+                QList<ParaData>::ConstIterator end(paraList->constEnd());
+        for (it=paraList->constBegin();it!=end;++it)
         {
             rowText += endOfParagraph;
             rowText += ProcessParagraphData( (*it).text,(*it).layout,(*it).formattingList);
@@ -453,11 +451,11 @@ QString RTFWorker::ProcessParagraphData ( const QString &paraText,
         else
         {
             if (layout.counter.numbering!=0)
-		{
-        	    markup += "\\pnlvl";
-        	    markup += QString::number(layout.counter.depth + 1);
-        	    markup += "\\pnprev1";
-		}
+                {
+                    markup += "\\pnlvl";
+                    markup += QString::number(layout.counter.depth + 1);
+                    markup += "\\pnprev1";
+                }
         else if (layout.counter.style==1)
         {
         markup += "\\pnlvlbody";
@@ -572,7 +570,7 @@ QString RTFWorker::ProcessParagraphData ( const QString &paraText,
                 //Retrieve text
                 partialText=paraText.mid ( (*paraFormatDataIt).pos, (*paraFormatDataIt).len );
                 content +=formatTextParagraph(partialText, formatRef, *paraFormatDataIt);
-	    }
+            }
             else if (4==(*paraFormatDataIt).id)
             {
                 // ### TODO: put date/time fields into own method.
@@ -726,15 +724,15 @@ QString RTFWorker::ProcessParagraphData ( const QString &paraText,
                     // Footnote
                     QString value = (*paraFormatDataIt).variable.getFootnoteValue();
                     bool automatic = (*paraFormatDataIt).variable.getFootnoteAuto();
-                    Q3ValueList<ParaData> *paraList = (*paraFormatDataIt).variable.getFootnotePara();
+                    QList<ParaData> *paraList = (*paraFormatDataIt).variable.getFootnotePara();
                     if( paraList )
                     {
                         QString fstr;
-                        Q3ValueList<ParaData>::ConstIterator it;
-                        Q3ValueList<ParaData>::ConstIterator end(paraList->end());
+                        QList<ParaData>::ConstIterator it;
+                        QList<ParaData>::ConstIterator end(paraList->constEnd());
                         const QString prefixSaved = m_prefix;
                         m_prefix.clear();
-                        for (it=paraList->begin();it!=end;++it)
+                        for (it=paraList->constBegin();it!=end;++it)
                             fstr += ProcessParagraphData( (*it).text, (*it).layout,(*it).formattingList);
                         m_prefix = prefixSaved;
 
@@ -758,32 +756,32 @@ QString RTFWorker::ProcessParagraphData ( const QString &paraText,
                 kDebug(30515) <<"Found an anchor of type:" << (*paraFormatDataIt).frameAnchor.type;
                 // We have an image, a clipart or a table
 
-		if (6==(*paraFormatDataIt).frameAnchor.type)
+                if (6==(*paraFormatDataIt).frameAnchor.type)
                 {
 
 
-		    if (!content.isEmpty())
-		    {
-			str += m_prefix;
-			str += markup;
-			str += " {";
-			str += content;
-			str += '}';
-			str += m_eol;
-			content.clear();
-			if (!m_inTable)
-			{
-			    m_prefix = "\\par";
-			}
-		    }
-		    str += makeTable((*paraFormatDataIt).frameAnchor);
-		}
+                    if (!content.isEmpty())
+                    {
+                        str += m_prefix;
+                        str += markup;
+                        str += " {";
+                        str += content;
+                        str += '}';
+                        str += m_eol;
+                        content.clear();
+                        if (!m_inTable)
+                        {
+                            m_prefix = "\\par";
+                        }
+                    }
+                    str += makeTable((*paraFormatDataIt).frameAnchor);
+                }
                 else if ((2==(*paraFormatDataIt).frameAnchor.type) || (5==(*paraFormatDataIt).frameAnchor.type))
                 {
                     content += makeImage((*paraFormatDataIt).frameAnchor);
 
                 }
-	    }
+            }
         }
     }
 
@@ -793,14 +791,14 @@ QString RTFWorker::ProcessParagraphData ( const QString &paraText,
     if (!content.isEmpty())
     {
         str += m_prefix;
-	str += markup;
-	str += " {";
-	str += content;
-	str += '}';
-	str += m_eol;
-	if (m_inTable==false)
-	{
-	   m_prefix = "\\par";
+        str += markup;
+        str += " {";
+        str += content;
+        str += '}';
+        str += m_eol;
+        if (m_inTable==false)
+        {
+           m_prefix = "\\par";
         }
     }
     if (str.isEmpty())
@@ -820,7 +818,7 @@ bool RTFWorker::doFullParagraph(const QString& paraText,
     kDebug(30515) <<"Entering RTFWorker::doFullParagraph" << endl << paraText;
     QString par = ProcessParagraphData( paraText, layout, paraFormatDataList);
     m_textBody += par;
-    kDebug(30515) <<"Quiting RTFWorker::doFullParagraph";
+    kDebug(30515) <<"Quitting RTFWorker::doFullParagraph";
     return true;
 }
 
@@ -841,8 +839,8 @@ bool RTFWorker::doHeader(const HeaderData& header)
 
     str += " {";
 
-    Q3ValueList<ParaData>::ConstIterator it;
-	Q3ValueList<ParaData>::ConstIterator end(header.para.end());
+    QList<ParaData>::ConstIterator it;
+        QList<ParaData>::ConstIterator end(header.para.end());
     for (it=header.para.begin();it!=end;++it)
         content += ProcessParagraphData( (*it).text,(*it).layout,(*it).formattingList);
 
@@ -876,8 +874,8 @@ bool RTFWorker::doFooter(const FooterData& footer)
 
     str += " {";
 
-    Q3ValueList<ParaData>::ConstIterator it;
-	Q3ValueList<ParaData>::ConstIterator end(footer.para.end());
+    QList<ParaData>::ConstIterator it;
+        QList<ParaData>::ConstIterator end(footer.para.end());
     for (it=footer.para.begin();it!=end;++it)
         content += ProcessParagraphData( (*it).text,(*it).layout,(*it).formattingList);
 
@@ -900,13 +898,13 @@ bool RTFWorker::doOpenFile(const QString& filenameOut, const QString& /*to*/)
 
     if (!m_ioDevice)
     {
-        kError(30515) << "No output file! Aborting!" << endl;
+        kError(30515) << "No output file! Aborting!";
         return false;
     }
 
     if ( !m_ioDevice->open (QIODevice::WriteOnly) )
     {
-        kError(30515) << "Unable to open output file!" << endl;
+        kError(30515) << "Unable to open output file!";
         return false;
     }
 
@@ -1016,9 +1014,9 @@ void RTFWorker::writeColorData(void)
 {
     *m_streamOut << "{\\colortbl;";
     uint count;
-    Q3ValueList<QColor>::ConstIterator it;
-    for (count=0, it=m_colorList.begin();
-        it!=m_colorList.end();
+    QList<QColor>::ConstIterator it;
+    for (count=0, it=m_colorList.constBegin();
+        it!=m_colorList.constEnd();
         count++, it++)
     {
         *m_streamOut << "\\red" << (*it).red();
@@ -1034,9 +1032,9 @@ void RTFWorker::writeStyleData(void)
     *m_streamOut << "{\\stylesheet" << m_eol;
 
     uint count;
-    Q3ValueList<LayoutData>::ConstIterator it;
-    for (count=0, it=m_styleList.begin();
-        it!=m_styleList.end();
+    QList<LayoutData>::ConstIterator it;
+    for (count=0, it=m_styleList.constBegin();
+        it!=m_styleList.constEnd();
         count++, it++)
     {
         *m_streamOut << "{";
@@ -1048,8 +1046,8 @@ void RTFWorker::writeStyleData(void)
         // \snext must be the last keyword before the style name
         // Find the number of the following style
         uint counter=0;  // counts position in style table starting at 0
-        Q3ValueList < LayoutData > ::ConstIterator it2;
-        for( it2 =  m_styleList.begin(); it2 != m_styleList.end(); counter++, ++it2 )
+        QList < LayoutData > ::ConstIterator it2;
+        for( it2 =  m_styleList.constBegin(); it2 != m_styleList.constEnd(); counter++, ++it2 )
         {
             if ( (*it2).styleName == (*it).styleFollowing )
             {
@@ -1137,7 +1135,7 @@ bool RTFWorker::doFullDocumentInfo(const KWEFDocumentInfo& docInfo)
     }
 
     // Now add who we are in a \comment
-    QString revision("$Revision: 911800 $");
+    QString revision("$Revision: 1016271 $");
     m_textDocInfo += "{\\comment ";
     m_textDocInfo += "Generated by KWord's RTF Export Filter";
     m_textDocInfo += revision.mid(10).remove('$'); // has a leading and a trailing space.
@@ -1674,10 +1672,10 @@ QString RTFWorker::lookupColor(const QString& markup, const QColor& color)
     uint counter=1;  // counts position in color table starting at 1
     QString strColor(markup);  // Holds RTF markup for the color
 
-    Q3ValueList < QColor > ::ConstIterator it;
+    QList < QColor > ::ConstIterator it;
 
     // search color table for this color
-    for( it =  m_colorList.begin(); it != m_colorList.end(); counter++, ++it )
+    for( it =  m_colorList.constBegin(); it != m_colorList.constEnd(); counter++, ++it )
     {
         if ( (*it) == color )
         {
@@ -1701,11 +1699,11 @@ QString RTFWorker::lookupStyle(const QString& styleName, LayoutData& returnLayou
     uint counter=0;  // counts position in style table starting at 0
     QString strMarkup("\\s");  // Holds RTF markup for the style
 
-    Q3ValueList < LayoutData > ::ConstIterator it;
-	Q3ValueList < LayoutData > ::ConstIterator end(m_styleList.end());
+    QList < LayoutData > ::ConstIterator it;
+        QList < LayoutData > ::ConstIterator end(m_styleList.constEnd());
 
     // search color table for this color
-    for( it =  m_styleList.begin(); it != end; counter++, ++it )
+    for( it =  m_styleList.constBegin(); it != end; counter++, ++it )
     {
         if ( (*it).styleName == styleName )
         {
