@@ -13,7 +13,7 @@
    Copyright (C) 2005-2006 Tim Beaulen <tbscope@gmail.com>
    Copyright (C) 2005 Yann Bodson <yann.bodson@online.fr>
    Copyright (C) 2005-2006 Boudewijn Rempt <boud@valdyas.org>
-   Copyright (C) 2005-2008 Jan Hambrecht <jaham@gmx.net>
+   Copyright (C) 2005-2009 Jan Hambrecht <jaham@gmx.net>
    Copyright (C) 2005-2006 Peter Simonsson <psn@linux.se>
    Copyright (C) 2005-2006 Sven Langkamp <sven.langkamp@gmail.com>
    Copyright (C) 2005-2006 Inge Wallin <inge@lysator.liu.se>
@@ -59,6 +59,7 @@
 #include "KarbonPrintJob.h"
 #include "KarbonZoomController.h"
 #include "KarbonSmallStylePreview.h"
+#include "KarbonDocumentMergeCommand.h"
 //#include "karbon_drag.h"
 
 #include <KoMainWindow.h>
@@ -71,12 +72,10 @@
 #include <KoRuler.h>
 #include <KoToolManager.h>
 #include <KoToolProxy.h>
-#include <KoToolDocker.h>
 #include <KoShapeManager.h>
 #include <KoShapeController.h>
 #include <KoShapeContainer.h>
 #include <KoShapeGroup.h>
-#include <KoShapeUngroupCommand.h>
 #include <KoShapeCreateCommand.h>
 #include <KoShapeDeleteCommand.h>
 #include <KoShapeReorderCommand.h>
@@ -87,9 +86,11 @@
 #include <KoZoomAction.h>
 #include <KoZoomHandler.h>
 #include <KoPathShape.h>
+#include <KoPathPointData.h>
 #include <KoPathCombineCommand.h>
 #include <KoPathSeparateCommand.h>
 #include <KoPathReverseCommand.h>
+#include <KoPathPointMoveCommand.h>
 #include <KoToolBoxFactory.h>
 #include <KoParameterShape.h>
 #include <KoRulerController.h>
@@ -100,6 +101,8 @@
 #include <KoCutController.h>
 #include <KoCopyController.h>
 #include <KoPasteController.h>
+#include <KoSnapGuide.h>
+#include <KoSnapStrategy.h>
 
 // kde header
 #include <kaction.h>
@@ -107,7 +110,7 @@
 #include <klocale.h>
 #include <kiconloader.h>
 #include <kmessagebox.h>
-#include <kdeversion.h>
+
 #include <kcomponentdata.h>
 #include <kactioncollection.h>
 #include <kxmlguifactory.h>
@@ -143,7 +146,7 @@
 #define KARBON_DEBUG_VIEW
 
 #ifdef KARBON_DEBUG_VIEW
-#define debugView(text) kDebug(38000) <<"KARBON_DEBUG_VIEW:" << text
+#define debugView(text) kDebug(38000) << "(" << this << ") " << text
 #else
 #define debugView(text)
 #endif
@@ -152,7 +155,13 @@ class KarbonView::Private
 {
 public:
     Private( KarbonPart * p)
-        : part(p), canvas( 0 )
+        : part(p), canvas( 0 ), canvasController(0), horizRuler(0), vertRuler(0)
+        , closePath(0), combinePath(0)
+        , separatePath(0), reversePath(0), intersectPath(0), subtractPath(0)
+        , unitePath(0), excludePath(0), pathSnapToGrid(0), configureAction(0)
+        , deleteSelectionAction(0), viewAction(0), showRulerAction(0)
+        , snapGridAction(0), showPageMargins(0), showGuidesAction(0)
+        , status(0), cursorCoords(0), smallPreview(0), zoomActionWidget(0)
     {}
 
     KarbonPart * part;
@@ -162,8 +171,6 @@ public:
     KoRuler * vertRuler;
 
     // actions:
-    KAction * groupObjects;
-    KAction * ungroupObjects;
     KAction * closePath;
     KAction * combinePath;
     KAction * separatePath;
@@ -172,6 +179,7 @@ public:
     KAction * subtractPath;
     KAction * unitePath;
     KAction * excludePath;
+    KAction * pathSnapToGrid;
     KAction * configureAction;
     KAction * deleteSelectionAction;
 
@@ -184,17 +192,17 @@ public:
     //Status Bar
     QLabel * status;       ///< ordinary status
     QLabel * cursorCoords; ///< cursor coordinates
+    KarbonSmallStylePreview * smallPreview; ///< small style preview
+    QWidget * zoomActionWidget; ///< zoom action widget
 };
 
 KarbonView::KarbonView( KarbonPart* p, QWidget* parent )
-    : KoView( p, parent ), KXMLGUIBuilder( shell() ), d( new Private( p ) )
+    : KoView( p, parent ), d( new Private( p ) )
 {
-    debugView("KarbonView::KarbonView");
+    debugView("");
 
     setComponentData( KarbonFactory::componentData(), true );
 
-    setClientBuilder( this );
-
     if( !p->isReadWrite() )
         setXMLFile( QString::fromLatin1( "karbon_readonly.rc" ) );
     else
@@ -204,7 +212,7 @@ KarbonView::KarbonView( KarbonPart* p, QWidget* parent )
     d->canvas = new KarbonCanvas( p );
     d->canvas->setParent( this );
     d->canvas->setDocumentViewMargin( viewMargin );
-    connect( d->canvas->shapeManager()->selection(), SIGNAL( selectionChanged() ), 
+    connect( d->canvas->shapeManager()->selection(), SIGNAL( selectionChanged() ),
              this, SLOT( selectionChanged() ) );
 
     d->canvasController = new KoCanvasController(this);
@@ -218,13 +226,13 @@ KarbonView::KarbonView( KarbonPart* p, QWidget* parent )
     d->canvasController->show();
 
     // set up status bar message
-    d->status = new QLabel( QString(), statusBar() );
+    d->status = new QLabel( QString(), this );
     d->status->setAlignment( Qt::AlignLeft | Qt::AlignVCenter );
     d->status->setMinimumWidth( 300 );
     addStatusBarItem( d->status, 1 );
     connect( KoToolManager::instance(), SIGNAL(changedStatusText(const QString &)),
              d->status, SLOT(setText(const QString &)) );
-    d->cursorCoords = new QLabel( QString(), statusBar() );
+    d->cursorCoords = new QLabel( QString(), this );
     d->cursorCoords->setAlignment( Qt::AlignLeft | Qt::AlignVCenter );
     d->cursorCoords->setMinimumWidth( 50 );
     addStatusBarItem( d->cursorCoords, 0 );
@@ -233,21 +241,28 @@ KarbonView::KarbonView( KarbonPart* p, QWidget* parent )
     // set up the zoom controller
     KarbonZoomController * zoomController = new KarbonZoomController( d->canvasController, actionCollection() );
     zoomController->setPageSize( d->part->document().pageSize() );
-    addStatusBarItem( zoomController->zoomAction()->createWidget( statusBar() ), 0 );
+    d->zoomActionWidget = zoomController->zoomAction()->createWidget( statusBar() );
+    addStatusBarItem( d->zoomActionWidget, 0 );
     zoomController->setZoomMode( KoZoomMode::ZOOM_PAGE );
     connect( zoomController, SIGNAL(zoomedToSelection()), this, SLOT(zoomSelection()));
     connect( zoomController, SIGNAL(zoomedToAll()), this, SLOT(zoomDrawing()));
-    
-    KarbonSmallStylePreview * smallPreview = new KarbonSmallStylePreview( statusBar() );
-    connect( smallPreview, SIGNAL(fillApplied()), this, SLOT(applyFillToSelection()) );
-    connect( smallPreview, SIGNAL(strokeApplied()), this, SLOT(applyStrokeToSelection()) );
-    addStatusBarItem( smallPreview, 0 );
+
+    d->smallPreview = new KarbonSmallStylePreview( this );
+    connect( d->smallPreview, SIGNAL(fillApplied()), this, SLOT(applyFillToSelection()) );
+    connect( d->smallPreview, SIGNAL(strokeApplied()), this, SLOT(applyStrokeToSelection()) );
+    addStatusBarItem( d->smallPreview, 0 );
 
     // layout:
     QGridLayout *layout = new QGridLayout();
     layout->setMargin(0);
     layout->addWidget(d->canvasController, 1, 1);
 
+    if( shell() )
+    {
+        KoToolManager::instance()->addController( d->canvasController );
+        KoToolManager::instance()->registerTools( actionCollection(), d->canvasController );
+    }
+
     initActions();
 
     unsigned int max = part()->maxRecentFiles();
@@ -275,17 +290,15 @@ KarbonView::KarbonView( KarbonPart* p, QWidget* parent )
     connect( d->canvasController, SIGNAL(canvasOffsetYChanged(int)), this, SLOT(pageOffsetChanged()));
     connect( d->canvasController, SIGNAL(canvasMousePositionChanged(const QPoint &)),
             this, SLOT(mousePositionChanged(const QPoint&)));
-    connect( d->vertRuler, SIGNAL(guideLineCreated(Qt::Orientation,int)), 
+    connect( d->vertRuler, SIGNAL(guideLineCreated(Qt::Orientation,int)),
              d->canvasController, SLOT( addGuideLine(Qt::Orientation,int) ) );
-    connect( d->horizRuler, SIGNAL(guideLineCreated(Qt::Orientation,int)), 
+    connect( d->horizRuler, SIGNAL(guideLineCreated(Qt::Orientation,int)),
              d->canvasController, SLOT( addGuideLine(Qt::Orientation,int) ) );
 
     updateRuler();
 
     if( shell() )
     {
-        KoToolManager::instance()->addController( d->canvasController );
-        KoToolManager::instance()->registerTools( actionCollection(), d->canvasController );
         // set the first layer active
         d->canvasController->canvas()->shapeManager()->selection()->setActiveLayer( part()->document().layers().first() );
 
@@ -301,8 +314,8 @@ KarbonView::KarbonView( KarbonPart* p, QWidget* parent )
             setDockerManager(dockerMng);
         }
 
-        connect( d->canvasController, SIGNAL( toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, KoView *) ),
-             dockerMng, SLOT( newOptionWidgets(const  QMap<QString, QWidget *> &, KoView *) ) );
+        connect( d->canvasController, SIGNAL( toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*) ),
+             dockerMng, SLOT( newOptionWidgets(const  QMap<QString, QWidget *> &, QWidget*) ) );
 
         KoToolManager::instance()->requestToolActivation( d->canvasController );
 
@@ -320,10 +333,15 @@ KarbonView::KarbonView( KarbonPart* p, QWidget* parent )
 
 KarbonView::~KarbonView()
 {
-    debugView("KarbonView::~KarbonView()");
+    debugView("");
 
     KoToolManager::instance()->removeCanvasController( d->canvasController );
 
+    removeStatusBarItem(d->status);
+    removeStatusBarItem(d->cursorCoords);
+    removeStatusBarItem(d->smallPreview);
+    removeStatusBarItem(d->zoomActionWidget);
+    
     delete d;
 }
 
@@ -337,9 +355,14 @@ KarbonCanvas * KarbonView::canvasWidget() const
     return d->canvas;
 }
 
+KoZoomController * KarbonView::zoomController() const
+{
+    return 0;
+}
+
 void KarbonView::resizeEvent( QResizeEvent* /*event*/ )
 {
-    debugView("KarbonView::resizeEvent()");
+    debugView("");
 
     if(!d->showRulerAction)
         return;
@@ -352,7 +375,7 @@ void KarbonView::resizeEvent( QResizeEvent* /*event*/ )
 
 void KarbonView::dropEvent( QDropEvent *e )
 {
-    debugView("KarbonView::dropEvent()");
+    debugView("");
 
     //Accepts QColor - from Color Manager's KColorPatch
     QColor color = KColorMimeData::fromMimeData( e->mimeData() );
@@ -396,39 +419,84 @@ void KarbonView::dropEvent( QDropEvent *e )
 
 void KarbonView::fileImportGraphic()
 {
-    debugView("KarbonView::fileImportGraphic()");
+    debugView("");
 
     QStringList filter;
-    filter << "application/x-karbon" << "image/svg+xml" << "image/x-wmf" << "image/x-eps" << "application/postscript";
-    KFileDialog *dialog = new KFileDialog(KUrl("foo"), "", 0);
+    filter << part()->nativeFormatMimeType();
+    filter << "application/x-karbon";
+    filter << "image/svg+xml";
+    filter << "application/x-wpg";
+    filter << "image/x-wmf";
+    filter << "image/x-eps";
+    filter << "application/postscript";
+    
+    QPointer<KFileDialog> dialog = new KFileDialog(KUrl(), "", 0);
     dialog->setCaption(i18n("Choose Graphic to Add"));
     dialog->setModal(true);
-    dialog->setMimeFilter( filter, "application/x-karbon" );
+    dialog->setMimeFilter( filter );
     if(dialog->exec()!=QDialog::Accepted) {
         delete dialog;
         return;
     }
-    QString fname = dialog->selectedFile();
-    //kDebug(38000) <<"in :" << fname.latin1();
-    //kDebug(38000) <<"part()->document()->nativeFormatMimeType().latin1() :" << part()->nativeFormatMimeType();
-    //kDebug(38000) <<"dialog->currentMimeFilter().latin1() :" << dialog->currentMimeFilter().latin1();
-    if( part()->nativeFormatMimeType() == dialog->currentMimeFilter().toLatin1() )
-        part()->mergeNativeFormat( fname );
-    else
-    {
-        KoFilterManager man( part() );
-        KoFilter::ConversionStatus status;
-        QString importedFile = man.importDocument( fname, status );
-        part()->mergeNativeFormat( importedFile );
-        if( !importedFile.isEmpty() )
-            unlink( QFile::encodeName( importedFile ) );
-    }
+    QString fname = dialog ? dialog->selectedFile() : QString();
+    QString currentMimeFilter = dialog ? dialog->currentMimeFilter() : QString();
     delete dialog;
+    
+    
+    KarbonPart importPart;
+    // use data centers of this document for importing
+    importPart.document().useExternalDataCenterMap( part()->document().dataCenterMap() );
+
+    bool success = true;
+
+    // check if we have an empty mime type (probably because the "All supported files"
+    // filter was active)
+    if (currentMimeFilter.isEmpty()) {
+        // get mime type from file
+        KMimeType::Ptr mimeType = KMimeType::findByFileContent(fname);
+        if (mimeType && mimeType->is(importPart.nativeFormatMimeType()))
+            currentMimeFilter = importPart.nativeFormatMimeType();
+    }
+    if (importPart.nativeFormatMimeType() == currentMimeFilter) {
+        // directly load the native format
+        success = importPart.loadNativeFormat( fname );
+        if ( !success ) {
+            importPart.showLoadingErrorDialog();
+        }
+    } else {
+        // use import filters to load the file
+        KoFilterManager man( &importPart );
+        KoFilter::ConversionStatus status = KoFilter::OK;
+        QString importedFile = man.importDocument( fname, status );
+        if (status != KoFilter::OK) {
+            importPart.showLoadingErrorDialog();
+            success = false;
+        } 
+        else if( !importedFile.isEmpty() ) {
+            success = importPart.loadNativeFormat( importedFile );
+            if (!success) {
+                importPart.showLoadingErrorDialog();
+            }
+            // remove the temporary file created during format conversion
+            unlink( QFile::encodeName( importedFile ) );
+        }
+    }
+        
+    if (success) {
+        QList<KoShape*> importedShapes = importPart.document().shapes();
+            
+        KarbonDocumentMergeCommand * cmd = new KarbonDocumentMergeCommand(part(), &importPart);
+        d->canvas->addCommand( cmd );
+
+        foreach( KoShape * shape, importedShapes ) {
+            d->canvas->shapeManager()->selection()->select( shape, false );
+        }
+    }
 }
 
 void KarbonView::selectionDuplicate()
 {
-    debugView("KarbonView::selectionDuplicate()");
+    debugView("");
     d->canvas->toolProxy()->copy();
     // TODO bring back copy offset
     d->canvas->toolProxy()->paste();
@@ -436,7 +504,7 @@ void KarbonView::selectionDuplicate()
 
 void KarbonView::editSelectAll()
 {
-    debugView("KarbonView::editSelectAll()");
+    debugView("");
 
     KoSelection* selection = d->canvas->shapeManager()->selection();
     if( ! selection )
@@ -456,7 +524,7 @@ void KarbonView::editSelectAll()
 
 void KarbonView::editDeselectAll()
 {
-    debugView("KarbonView::editDeselectAll()");
+    debugView("");
 
     KoSelection* selection = d->canvas->shapeManager()->selection();
     if( selection )
@@ -467,63 +535,63 @@ void KarbonView::editDeselectAll()
 
 void KarbonView::editDeleteSelection()
 {
-    debugView("KarbonView::editDeleteSelection()");
+    debugView("");
 
     d->canvas->toolProxy()->deleteSelection();
 }
 
 void KarbonView::selectionDistributeHorizontalCenter()
 {
-    debugView("KarbonView::selectionDistributeHorizontalCenter()");
+    debugView("");
 
     selectionDistribute(KoShapeDistributeCommand::HorizontalCenterDistribution);
 }
 
 void KarbonView::selectionDistributeHorizontalGap()
 {
-    debugView("KarbonView::selectionDistributeHorizontalGap()");
+    debugView("");
 
     selectionDistribute(KoShapeDistributeCommand::HorizontalGapsDistribution);
 }
 
 void KarbonView::selectionDistributeHorizontalLeft()
 {
-    debugView("KarbonView::selectionDistributeHorizontalLeft()");
+    debugView("");
 
     selectionDistribute(KoShapeDistributeCommand::HorizontalLeftDistribution);
 }
 
 void KarbonView::selectionDistributeHorizontalRight()
 {
-    debugView("KarbonView::selectionDistributeHorizontalRight()");
+    debugView("");
 
     selectionDistribute(KoShapeDistributeCommand::HorizontalRightDistribution);
 }
 
 void KarbonView::selectionDistributeVerticalCenter()
 {
-    debugView("KarbonView::selectionDistributeVerticalCenter()");
+    debugView("");
 
     selectionDistribute(KoShapeDistributeCommand::VerticalCenterDistribution);
 }
 
 void KarbonView::selectionDistributeVerticalGap()
 {
-    debugView("KarbonView::selectionDistributeVerticalGap()");
+    debugView("");
 
     selectionDistribute(KoShapeDistributeCommand::VerticalGapsDistribution);
 }
 
 void KarbonView::selectionDistributeVerticalBottom()
 {
-    debugView("KarbonView::selectionDistributeVerticalBottom()");
+    debugView("");
 
     selectionDistribute(KoShapeDistributeCommand::VerticalBottomDistribution);
 }
 
 void KarbonView::selectionDistributeVerticalTop()
 {
-    debugView("KarbonView::selectionDistributeVerticalTop()");
+    debugView("");
 
     selectionDistribute(KoShapeDistributeCommand::VerticalTopDistribution);
 }
@@ -542,109 +610,16 @@ void KarbonView::selectionDistribute(KoShapeDistributeCommand::Distribute distri
     d->canvas->addCommand( cmd );
 }
 
-void KarbonView::selectionBringToFront()
-{
-    debugView("KarbonView::selectionBringToFront()");
-
-    QList<KoShape*> selectedShapes = d->canvas->shapeManager()->selection()->selectedShapes( KoFlake::TopLevelSelection );
-    d->canvas->addCommand( KoShapeReorderCommand::createCommand( selectedShapes, d->canvas->shapeManager(), KoShapeReorderCommand::BringToFront ) );
-}
-
-void KarbonView::selectionMoveUp()
-{
-    debugView("KarbonView::selectionMoveUp()");
-
-    QList<KoShape*> selectedShapes = d->canvas->shapeManager()->selection()->selectedShapes( KoFlake::TopLevelSelection );
-    d->canvas->addCommand( KoShapeReorderCommand::createCommand( selectedShapes, d->canvas->shapeManager(), KoShapeReorderCommand::RaiseShape ) );
-}
-
-void KarbonView::selectionMoveDown()
-{
-    debugView("KarbonView::selectionMoveDown()");
-
-    QList<KoShape*> selectedShapes = d->canvas->shapeManager()->selection()->selectedShapes( KoFlake::TopLevelSelection );
-    d->canvas->addCommand( KoShapeReorderCommand::createCommand( selectedShapes, d->canvas->shapeManager(), KoShapeReorderCommand::LowerShape ) );
-}
-
-void KarbonView::selectionSendToBack()
-{
-    debugView("KarbonView::selectionSendToBack()");
-
-    QList<KoShape*> selectedShapes = d->canvas->shapeManager()->selection()->selectedShapes( KoFlake::TopLevelSelection );
-    d->canvas->addCommand( KoShapeReorderCommand::createCommand( selectedShapes, d->canvas->shapeManager(), KoShapeReorderCommand::SendToBack ) );
-}
-
-void KarbonView::groupSelection()
-{
-    debugView("KarbonView::groupSelection()");
-
-    KoSelection* selection = d->canvas->shapeManager()->selection();
-    if( ! selection )
-        return;
-
-    QList<KoShape*> selectedShapes = selection->selectedShapes( KoFlake::TopLevelSelection );
-    QList<KoShape*> groupedShapes;
-
-    // only group shapes with an unselected parent
-    foreach( KoShape* shape, selectedShapes )
-    {
-        if( selectedShapes.contains( shape->parent() ) )
-            continue;
-        groupedShapes << shape;
-    }
-    KoShapeGroup *group = new KoShapeGroup();
-    if( selection->activeLayer() )
-        selection->activeLayer()->addChild( group );
-    QUndoCommand *cmd = new QUndoCommand( i18n("Group shapes") );
-    new KoShapeCreateCommand( d->part, group, cmd );
-    new KoShapeGroupCommand( group, groupedShapes, cmd );
-    d->canvas->addCommand( cmd );
-}
-
-void KarbonView::ungroupSelection()
-{
-    debugView("KarbonView::ungroupSelection()");
-
-    KoSelection* selection = d->canvas->shapeManager()->selection();
-    if( ! selection )
-        return;
-
-    QList<KoShape*> selectedShapes = selection->selectedShapes( KoFlake::TopLevelSelection );
-    QList<KoShape*> containerSet;
-
-    // only ungroup shape containers with an unselected parent
-    foreach( KoShape* shape, selectedShapes )
-    {
-        if( selectedShapes.contains( shape->parent() ) )
-            continue;
-        containerSet << shape;
-    }
-
-    QUndoCommand *cmd = new QUndoCommand( i18n("Ungroup shapes") );
-
-    // add a ungroup command for each found shape container to the macro command
-    foreach( KoShape* shape, containerSet )
-    {
-        KoShapeContainer *container = dynamic_cast<KoShapeContainer*>( shape );
-        if( container )
-        {
-            new KoShapeUngroupCommand( container, container->iterator(), cmd );
-            new KoShapeDeleteCommand( d->part, container, cmd );
-        }
-    }
-    d->canvas->addCommand( cmd );
-}
-
 void KarbonView::closePath()
 {
-    debugView("KarbonView::closePath()");
+    debugView("");
 
     // TODO add the new close path command here
 }
 
 void KarbonView::combinePath()
 {
-    debugView("KarbonView::combinePath()");
+    debugView("");
 
     KoSelection* selection = d->canvas->shapeManager()->selection();
     if( ! selection )
@@ -658,6 +633,9 @@ void KarbonView::combinePath()
         KoPathShape *path = dynamic_cast<KoPathShape*>( shape );
         if( path )
         {
+            KoParameterShape * paramShape = dynamic_cast<KoParameterShape*>(path);
+            if( paramShape && paramShape->isParametricShape() )
+                continue;
             paths << path;
             selection->deselect( shape );
         }
@@ -669,7 +647,7 @@ void KarbonView::combinePath()
 
 void KarbonView::separatePath()
 {
-    debugView("KarbonView::separatePath()");
+    debugView("");
     KoSelection* selection = d->canvas->shapeManager()->selection();
     if( ! selection )
         return;
@@ -693,7 +671,7 @@ void KarbonView::separatePath()
 
 void KarbonView::reversePath()
 {
-    debugView("KarbonView::reversePath()");
+    debugView("");
 
     QList<KoPathShape*> paths = selectedPathShapes();
     if( paths.size() )
@@ -755,9 +733,63 @@ void KarbonView::booleanOperation( KarbonBooleanCommand::BooleanOperation operat
     }
 }
 
+void KarbonView::pathSnapToGrid()
+{
+    KoSelection* selection = d->canvas->shapeManager()->selection();
+    if( ! selection )
+        return;
+    
+    QList<KoShape*> selectedShapes = selection->selectedShapes();
+    QList<KoPathPointData> points;
+    QList<QPointF> offsets;
+    
+    // store current grid snap state
+    bool oldSnapToGrid = part()->gridData().snapToGrid();
+    // enable grid snapping
+    part()->gridData().setSnapToGrid(true);
+    
+    KoSnapGuide snapGuide(d->canvas);
+    snapGuide.enableSnapStrategies(KoSnapStrategy::Grid);
+    snapGuide.setSnapDistance(INT_MAX);
+    
+    foreach( KoShape* shape, selectedShapes )
+    {
+        KoParameterShape * paramShape = dynamic_cast<KoParameterShape*>(shape);
+        if( paramShape && paramShape->isParametricShape() )
+            continue;
+        
+        KoPathShape *path = dynamic_cast<KoPathShape*>( shape );
+        if( ! path )
+            continue;
+
+        uint subpathCount = path->subpathCount();
+        for( uint i = 0; i < subpathCount; ++i )
+        {
+            uint pointCount = path->pointCountSubpath(i);
+            for( uint j = 0; j < pointCount; ++j )
+            {
+                KoPathPointIndex index(i,j);
+                KoPathPoint * p = path->pointByIndex(index);
+                if( !p )
+                    continue;
+                
+                QPointF docPoint = path->shapeToDocument(p->point());
+                QPointF offset = snapGuide.snap(docPoint, 0) - docPoint;
+                points.append( KoPathPointData(path,index) );
+                offsets.append( offset );
+            }
+        }
+    }
+    
+    // reset grid snapping state to old value
+    part()->gridData().setSnapToGrid(oldSnapToGrid);
+    
+    d->canvas->addCommand( new KoPathPointMoveCommand( points, offsets ) );
+}
+
 void KarbonView::viewModeChanged(bool outlineMode)
 {
-    debugView("KarbonView::viewModeChanged()");
+    debugView("");
 
     d->canvas->enableOutlineMode( outlineMode );
     d->canvas->updateCanvas(d->canvas->canvasWidget()->rect());
@@ -802,7 +834,7 @@ void KarbonView::zoomDrawing()
 
 void KarbonView::initActions()
 {
-    debugView("KarbonView::initActions()");
+    debugView("");
 
     // view ----->
     d->viewAction  = new KToggleAction(i18n("Outline &Mode"), this);
@@ -819,7 +851,7 @@ void KarbonView::initActions()
         return;
 
     // edit ----->
-    KAction *action = actionCollection()->addAction(KStandardAction::Cut,  "edit_cut", 0, 0);
+    QAction *action = actionCollection()->addAction(KStandardAction::Cut,  "edit_cut", 0, 0);
     new KoCutController(d->canvas, action);
     action = actionCollection()->addAction(KStandardAction::Copy,  "edit_copy", 0, 0);
     new KoCopyController(d->canvas, action);
@@ -845,26 +877,6 @@ void KarbonView::initActions()
     actionDuplicate->setShortcut(QKeySequence("Ctrl+D"));
     connect(actionDuplicate, SIGNAL(triggered()), this, SLOT(selectionDuplicate()));
 
-    KAction *actionBringToFront  = new KAction(KIcon("object-order-front"), i18n("Bring to &Front"), this);
-    actionCollection()->addAction("object_move_totop", actionBringToFront );
-    actionBringToFront->setShortcut(QKeySequence("Ctrl+Shift+]"));
-    connect(actionBringToFront, SIGNAL(triggered()), this, SLOT(selectionBringToFront()));
-
-    KAction *actionRaise  = new KAction(KIcon("object-order-raise"), i18n("&Raise"), this);
-    actionCollection()->addAction("object_move_up", actionRaise );
-    actionRaise->setShortcut(QKeySequence("Ctrl+]"));
-    connect(actionRaise, SIGNAL(triggered()), this, SLOT(selectionMoveUp()));
-
-    KAction *actionLower  = new KAction(KIcon("object-order-lower"), i18nc("Lower selected shapes", "&Lower"), this);
-    actionCollection()->addAction("object_move_down", actionLower );
-    actionLower->setShortcut(QKeySequence("Ctrl+["));
-    connect(actionLower, SIGNAL(triggered()), this, SLOT(selectionMoveDown()));
-
-    KAction *actionSendToBack  = new KAction(KIcon("object-order-back"), i18n("Send to &Back"), this);
-    actionCollection()->addAction("object_move_tobottom", actionSendToBack );
-    actionSendToBack->setShortcut(QKeySequence("Ctrl+Shift+["));
-    connect(actionSendToBack, SIGNAL(triggered()), this, SLOT(selectionSendToBack()));
-
     KAction *actionDistributeHorizontalCenter  = new KAction(i18n("Distribute Center (Horizontal)"), this);
     actionCollection()->addAction("object_distribute_horizontal_center", actionDistributeHorizontalCenter );
     connect(actionDistributeHorizontalCenter, SIGNAL(triggered()), this, SLOT(selectionDistributeHorizontalCenter()));
@@ -905,10 +917,6 @@ void KarbonView::initActions()
     connect( d->showRulerAction, SIGNAL(triggered()), this, SLOT(showRuler()));
 
     KToggleAction *gridAction = d->part->gridData().gridToggleAction(d->canvas);
-    // XXX remove the translated strings when the string freeze is lifted, the KoGridData should have those
-    gridAction->setText(i18n("Show Grid"));
-    gridAction->setCheckedState(KGuiItem(i18n("Hide Grid")));
-    gridAction->setToolTip(i18n("Shows or hides grid"));
     actionCollection()->addAction("view_grid", gridAction);
 
     d->showGuidesAction  = new KToggleAction(KIcon("guides"), i18n("Show Guides"), this);
@@ -923,15 +931,15 @@ void KarbonView::initActions()
     d->snapGridAction->setToolTip(i18n( "Snaps to grid"));
     connect(d->snapGridAction, SIGNAL(triggered()), this, SLOT(snapToGrid()));
 
-    d->groupObjects  = new KAction(KIcon("object-group"), i18n("&Group Objects"), this);
-    actionCollection()->addAction("selection_group", d->groupObjects );
-    d->groupObjects->setShortcut(QKeySequence("Ctrl+G"));
-    connect(d->groupObjects, SIGNAL(triggered()), this, SLOT(groupSelection()));
+    action = actionCollection()->action("object_group");
+    if ( action ) {
+        action->setShortcut(QKeySequence("Ctrl+G"));
+    }
+    action = actionCollection()->action("object_ungroup");
+    if ( action ) {
+        action->setShortcut(QKeySequence("Ctrl+Shift+G"));
+    }
 
-    d->ungroupObjects  = new KAction(KIcon("object-ungroup"), i18n("&Ungroup Objects"), this);
-    actionCollection()->addAction("selection_ungroup", d->ungroupObjects );
-    d->ungroupObjects->setShortcut(QKeySequence("Ctrl+Shift+G"));
-    connect(d->ungroupObjects, SIGNAL(triggered()), this, SLOT(ungroupSelection()));
     // object <-----
 
     // path ------->
@@ -982,6 +990,12 @@ void KarbonView::initActions()
     //d->excludePath->setShortcut(QKeySequence("Shift+Ctrl+K"));
     d->excludePath->setEnabled( false );
     connect(d->excludePath, SIGNAL(triggered()), this, SLOT(excludePaths()));
+    
+    d->pathSnapToGrid = new KAction(i18n("Snap Path to Grid"), this);
+    actionCollection()->addAction("path_snap_to_grid", d->pathSnapToGrid );
+    d->pathSnapToGrid->setEnabled( false );
+    connect(d->pathSnapToGrid, SIGNAL(triggered()), this, SLOT(pathSnapToGrid()));
+    
     // path <-----
 
     d->configureAction  = new KAction(KIcon("configure"), i18n("Configure Karbon..."), this);
@@ -1022,7 +1036,7 @@ void KarbonView::mousePositionChanged( const QPoint &position )
 
 void KarbonView::reorganizeGUI()
 {
-    debugView("KarbonView::reorganizeGUI()");
+    debugView("");
 
     if( d->snapGridAction )
         d->snapGridAction->setChecked( part()->gridData().snapToGrid() );
@@ -1032,7 +1046,7 @@ void KarbonView::reorganizeGUI()
 
 void KarbonView::setNumberOfRecentFiles( unsigned int number )
 {
-    debugView(QString("KarbonView::setNumberOfRecentFiles(%1)").arg(number));
+    debugView(QString("to %1").arg(number));
 
     if( shell() )    // 0L when embedded into konq !
         shell()->setMaxRecentItems( number );
@@ -1040,7 +1054,7 @@ void KarbonView::setNumberOfRecentFiles( unsigned int number )
 
 void KarbonView::showRuler()
 {
-    debugView("KarbonView::showRuler()");
+    debugView("");
 
     if( shell() && d->showRulerAction->isChecked() )
     {
@@ -1057,7 +1071,7 @@ void KarbonView::showRuler()
 
 void KarbonView::togglePageMargins(bool b)
 {
-    debugView(QString("KarbonView::togglePageMargins(%1)").arg(b));
+    debugView(QString("parameter %1").arg(b));
 
     ((KToggleAction*)actionCollection()->action("view_show_margins"))->setChecked(b);
     d->canvas->setShowPageMargins( b );
@@ -1078,7 +1092,7 @@ void KarbonView::updateRuler()
 
 void KarbonView::showGuides()
 {
-    debugView("KarbonView::showGuides()");
+    debugView("");
 
     d->part->guidesData().setShowGuideLines( d->showGuidesAction->isChecked() );
     d->canvas->update();
@@ -1086,7 +1100,7 @@ void KarbonView::showGuides()
 
 void KarbonView::snapToGrid()
 {
-    debugView("KarbonView::snapToGrid()");
+    debugView("");
 
     d->part->gridData().setSnapToGrid( d->snapGridAction->isChecked() );
     d->canvas->update();
@@ -1094,50 +1108,54 @@ void KarbonView::snapToGrid()
 
 void KarbonView::configure()
 {
-    debugView("KarbonView::configure()");
+    debugView("");
 
-    KarbonConfigureDialog dialog( this );
-    dialog.exec();
+    QPointer<KarbonConfigureDialog> dialog = new KarbonConfigureDialog( this );
+    dialog->exec();
+    delete dialog;
+    d->part->reorganizeGUI();
+    d->canvas->update();
 }
 
 void KarbonView::pageLayout()
 {
-    debugView("KarbonView::pageLayout()");
+    debugView("");
 
-    KoPageLayoutDialog dlg( this, part()->pageLayout() );
-    dlg.showPageSpread( false );
-    dlg.showTextDirection( false );
-    dlg.setPageSpread( false );
+    QPointer<KoPageLayoutDialog> dlg = new KoPageLayoutDialog( this, part()->pageLayout() );
+    dlg->showPageSpread( false );
+    dlg->showTextDirection( false );
+    dlg->setPageSpread( false );
 
-    if( dlg.exec() == QDialog::Accepted )
+    if( dlg->exec() == QDialog::Accepted )
     {
-        part()->setPageLayout( dlg.pageLayout() );
-        emit pageLayoutChanged();
+        if( dlg ) {
+            part()->setPageLayout( dlg->pageLayout() );
+            emit pageLayoutChanged();
+        }
     }
+    delete dlg;
 }
 
 void KarbonView::selectionChanged()
 {
-    debugView("KarbonView::selectionChanged()");
+    debugView("");
 
     KoSelection *selection = d->canvas->shapeManager()->selection();
     int count = selection->selectedShapes( KoFlake::FullSelection ).count();
 
-    d->groupObjects->setEnabled( count > 1 );
-    d->ungroupObjects->setEnabled( false );
     d->closePath->setEnabled( false );
     d->combinePath->setEnabled( false );
     d->excludePath->setEnabled( false );
     d->intersectPath->setEnabled( false );
     d->subtractPath->setEnabled( false );
     d->unitePath->setEnabled( false );
+    d->pathSnapToGrid->setEnabled(false);
 
     kDebug(38000) << count <<" shapes selected";
 
     if( count > 0 )
     {
         uint selectedPaths = 0;
-        uint selectedGroups = 0;
         uint selectedParametrics = 0;
         // check for different shape types for enabling specific actions
         foreach( KoShape* shape, selection->selectedShapes( KoFlake::FullSelection ) )
@@ -1151,15 +1169,8 @@ void KarbonView::selectionChanged()
                     selectedPaths++;
             }
         }
-        foreach( KoShape* shape, selection->selectedShapes( KoFlake::TopLevelSelection ) )
-        {
-            if( dynamic_cast<KoShapeGroup*>( shape ) )
-                selectedGroups++;
-        }
-        kDebug(38000) << selectedGroups <<" group shapes selected";
         kDebug(38000) << selectedPaths <<" path shapes selected";
         kDebug(38000) << selectedParametrics <<" parameter shapes selected";
-        d->ungroupObjects->setEnabled( selectedGroups > 0 );
         //TODO enable action when the ClosePath command is ported
         //d->closePath->setEnabled( selectedPaths > 0 );
         d->combinePath->setEnabled( selectedPaths > 1 );
@@ -1169,7 +1180,7 @@ void KarbonView::selectionChanged()
         d->intersectPath->setEnabled( selectedPaths + selectedParametrics == 2 );
         d->subtractPath->setEnabled( selectedPaths + selectedParametrics == 2 );
         d->unitePath->setEnabled( selectedPaths + selectedParametrics == 2 );
-        
+        d->pathSnapToGrid->setEnabled( selectedPaths > 0 );
         // if only one shape selected, set its parent layer as the active layer
         if( count == 1 )
         {
@@ -1190,14 +1201,14 @@ void KarbonView::selectionChanged()
 
 void KarbonView::setCursor( const QCursor &c )
 {
-    debugView("KarbonView::setCursor(QCursor)");
+    debugView("");
 
     d->canvas->setCursor( c );
 }
 
 void KarbonView::createLayersTabDock()
 {
-    debugView("KarbonView::createLayersTabDock()");
+    debugView("");
 
     KarbonLayerDockerFactory layerFactory;
     KarbonLayerDocker * layerDocker = qobject_cast<KarbonLayerDocker*>(createDockWidget(&layerFactory));
@@ -1212,7 +1223,7 @@ void KarbonView::createLayersTabDock()
 
 void KarbonView::updateReadWrite( bool readwrite )
 {
-    debugView("KarbonView::updateReadWrite( bool )");
+    debugView("");
     kDebug(38000) <<"writable state =" << readwrite;
 }
 
@@ -1255,7 +1266,7 @@ void KarbonView::applyFillToSelection()
     KoSelection *selection = d->canvas->shapeManager()->selection();
     if( ! selection->count() )
         return;
-    
+
     KoShape * shape = selection->firstSelectedShape();
     d->canvas->addCommand( new KoShapeBackgroundCommand( selection->selectedShapes(), shape->background() ) );
 }
@@ -1265,7 +1276,7 @@ void KarbonView::applyStrokeToSelection()
     KoSelection *selection = d->canvas->shapeManager()->selection();
     if( ! selection->count() )
         return;
-    
+
     KoShape * shape = selection->firstSelectedShape();
     d->canvas->addCommand( new KoShapeBorderCommand( selection->selectedShapes(), shape->border() ) );
 }
