@@ -205,7 +205,7 @@ QVariant KarbonLayerModel::data( const QModelIndex &index, int role ) const
             if( container )
             {
                 bbox = QRectF();
-                foreach( KoShape* shape, container->iterator() )
+                foreach( KoShape* shape, container->childShapes() )
                     bbox = bbox.united( shape->outline().boundingRect() );
             }
             return qreal(bbox.width()) / bbox.height();
@@ -272,7 +272,7 @@ KoDocumentSectionModel::PropertyList KarbonLayerModel::properties( KoShape* shap
 {
     PropertyList l;
     l << Property(i18nc("Visibility state of the shape","Visible"), SmallIcon("14_layer_visible"), SmallIcon("14_layer_novisible"), shape->isVisible());
-    l << Property(i18nc("Lock state of the shape","Locked"), SmallIcon("object-locked"), SmallIcon("object-unlocked"), shape->isLocked());
+    l << Property(i18nc("Lock state of the shape","Locked"), SmallIcon("object-locked"), SmallIcon("object-unlocked"), shape->isGeometryProtected());
     l << Property( i18nc("The z-index of the shape", "zIndex"), QString("%1").arg( shape->zIndex() ) );
     return l;
 }
@@ -280,12 +280,15 @@ KoDocumentSectionModel::PropertyList KarbonLayerModel::properties( KoShape* shap
 void KarbonLayerModel::setProperties( KoShape* shape, const PropertyList &properties )
 {
     bool oldVisibleState = shape->isVisible();
-    bool oldLockedState = shape->isLocked();
+    bool oldLockedState = shape->isGeometryProtected();
+    bool newVisibleState = properties.at( 0 ).state.toBool();
+    bool newLockedState = properties.at( 1 ).state.toBool();
+    
+    shape->setVisible( newVisibleState );
+    shape->setGeometryProtected( newLockedState );
+    shape->setSelectable( !newLockedState );
 
-    shape->setVisible( properties.at( 0 ).state.toBool() );
-    shape->setLocked( properties.at( 1 ).state.toBool() );
-
-    if( ( oldVisibleState != shape->isVisible() ) || ( oldLockedState != shape->isLocked() ) )
+    if( ( oldVisibleState != shape->isVisible() ) || ( oldLockedState != shape->isGeometryProtected() ) )
         shape->update();
 }
 
@@ -297,7 +300,7 @@ QImage KarbonLayerModel::createThumbnail( KoShape* shape, const QSize &thumbSize
 
     KoShapeContainer * container = dynamic_cast<KoShapeContainer*>( shape );
     if( container )
-        shapes = container->iterator();
+        shapes = container->childShapes();
     else
         shapes.append( shape );
 
@@ -319,12 +322,12 @@ QImage KarbonLayerModel::createThumbnail( KoShape* shape, const QSize &thumbSize
 
 KoShape * KarbonLayerModel::childFromIndex( KoShapeContainer *parent, int row ) const
 {
-    return parent->iterator().at( row );
+    return parent->childShapes().at( row );
 }
 
 int KarbonLayerModel::indexFromChild( KoShapeContainer *parent, KoShape *child ) const
 {
-    return parent->iterator().indexOf( child );
+    return parent->childShapes().indexOf( child );
 }
 
 Qt::DropActions KarbonLayerModel::supportedDropActions () const
@@ -447,7 +450,7 @@ bool KarbonLayerModel::dropMimeData( const QMimeData * data, Qt::DropAction acti
             cmd->setText( i18n("Reparent shapes") );
 
             foreach( KoShape * shape, toplevelShapes )
-                new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, cmd );
+                new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, QList<KoShape*>(), cmd );
 
             new KoShapeGroupCommand( group, toplevelShapes, cmd );
             KoCanvasController * canvasController = KoToolManager::instance()->activeCanvasController();
@@ -479,7 +482,7 @@ bool KarbonLayerModel::dropMimeData( const QMimeData * data, Qt::DropAction acti
                     }
 
                     clipped.append( shape->parent()->childClipped( shape ) );
-                    new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, cmd );
+                    new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, QList<KoShape*>(), cmd );
                 }
 
                 // shapes are dropped on a container, so add them to the container
