@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
 
    Copyright 2007-2008 Johannes Simon <johannes.simon@gmail.com>
+   Copyright 2009      Inge Wallin    <inge@lysator.liu.se>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -18,6 +19,7 @@
    Boston, MA 02110-1301, USA.
 */
 
+// Own
 #include "ChartConfigWidget.h"
 
 // Qt
@@ -34,6 +36,9 @@
 #include <KDebug>
 #include <KMessageBox>
 
+// KOffice
+#include <interfaces/KoChartModel.h>
+
 // KDChart
 #include <KDChartChart>
 #include <KDChartPosition>
@@ -44,7 +49,7 @@
 #include <KDChartDataValueAttributes>
 
 // KChart
-#include "ProxyModel.h"
+#include "ChartProxyModel.h"
 #include "PlotArea.h"
 #include "Legend.h"
 #include "DataSet.h"
@@ -57,8 +62,7 @@
 #include "TableEditorDialog.h"
 #include "commands/ChartTypeCommand.h"
 #include "CellRegionStringValidator.h"
-#include <interfaces/KoChartModel.h>
-#include "TableModel.h"
+#include "ChartTableModel.h"
 
 using namespace KChart;
 
@@ -69,16 +73,22 @@ public:
     Private( QWidget *parent );
     ~Private();
 
-    // Basic properties of the chart.
+    // The owner of this struct.
     ChartShape            *shape;
-    ChartType           type;
-    ChartSubtype        subtype;
+
+    // Basic properties of the chart.
+    // FIXME: why are they needed here?
+    ChartType              type;
+    ChartSubtype           subtype;
     bool                   threeDMode;
+
+    // Layouts and widgets.
     QVBoxLayout           *leftLayout;
     QVBoxLayout           *rightLayout;
     Ui::ChartConfigWidget  ui;
-    bool sourceIsSpreadSheet;
+    bool                   sourceIsSpreadSheet; // FIXME: Rename to isExternalDataSource?
     
+    // Menus
     QMenu *dataSetBarChartMenu;
     QMenu *dataSetLineChartMenu;
     QMenu *dataSetAreaChartMenu;
@@ -97,12 +107,17 @@ public:
     QAction  *percentAreaChartAction;
 
     QAction  *circleChartAction;
-    
+    QAction  *ringChartAction;
     QAction  *radarChartAction;
     
     QAction  *scatterChartAction;
-    
-    // chart type selection actions
+    QAction  *bubbleChartAction;
+
+    QAction  *stockChartAction;
+    QAction  *surfaceChartAction;
+    QAction  *ganttChartAction;
+
+    // chart type selection actions for datasets
     QAction  *dataSetNormalBarChartAction;
     QAction  *dataSetStackedBarChartAction;
     QAction  *dataSetPercentBarChartAction;
@@ -115,11 +130,16 @@ public:
     QAction  *dataSetStackedAreaChartAction;
     QAction  *dataSetPercentAreaChartAction;
     
-    QAction *dataSetCircleChartAction;
-    QAction *dataSetScatterChartAction;
-    QAction *dataSetRadarChartAction;
+    QAction  *dataSetCircleChartAction;
+    QAction  *dataSetRingChartAction;
+    QAction  *dataSetRadarChartAction;
+    QAction  *dataSetScatterChartAction;
+    QAction  *dataSetBubbleChartAction;
+    QAction  *dataSetStockChartAction;
+    QAction  *dataSetSurfaceChartAction;
+    QAction  *dataSetGanttChartAction;
 
-    // Table Editor
+    // Table Editor (a.k.a. the data editor)
     TableEditorDialog    *tableEditorDialog;
 
     // Legend
@@ -131,9 +151,10 @@ public:
     
     int                   selectedDataSet;
     int                   selectedDataSet_CellRegionDialog;
-    
-    QList<Axis*> dataSetAxes;
-    QList<Axis*> axes;
+
+    // Axes
+    QList<Axis*>    dataSetAxes;
+    QList<Axis*>    axes;
     QList<DataSet*> dataSets;
 
     // Dialogs
@@ -144,10 +165,13 @@ public:
     CellRegionStringValidator *cellRegionStringValidator;
 };
 
+
 ChartConfigWidget::Private::Private( QWidget *parent )
-    : newAxisDialog( parent ),
-      axisScalingDialog( parent ),
-      cellRegionDialog( parent )
+    : newAxisDialog( parent )
+    , axisScalingDialog( parent )
+    , cellRegionDialog( parent )
+    , tableEditorDialog( 0 )
+      
 {
     lastHorizontalAlignment = 1; // Qt::AlignCenter
     lastVerticalAlignment   = 1; // Qt::AlignCenter
@@ -173,8 +197,13 @@ ChartConfigWidget::Private::Private( QWidget *parent )
     dataSetStackedAreaChartAction = 0;
     dataSetPercentAreaChartAction = 0;
     dataSetCircleChartAction = 0;
+    dataSetRingChartAction = 0;
     dataSetScatterChartAction = 0;
     dataSetRadarChartAction = 0;
+    dataSetStockChartAction = 0;
+    dataSetBubbleChartAction = 0;
+    dataSetSurfaceChartAction = 0;
+    dataSetGanttChartAction = 0;
 }
 
 ChartConfigWidget::Private::~Private()
@@ -182,40 +211,65 @@ ChartConfigWidget::Private::~Private()
 }
 
 
+// ================================================================
+//                     class ChartConfigWidget
+
+
 ChartConfigWidget::ChartConfigWidget()
     : d( new Private( this ) )
 {
     setObjectName("Chart Type");
     d->ui.setupUi( this );
     
-    // Chart type button
+    // Chart type button with its associated menu
     QMenu *chartTypeMenu = new QMenu( this );
     chartTypeMenu->setIcon( KIcon( "chart_bar_beside" ) );
     
+    // Bar charts
     QMenu *barChartMenu = chartTypeMenu->addMenu( KIcon( "chart_bar_beside" ), i18n( "Bar Chart" ) );
     d->normalBarChartAction  = barChartMenu->addAction( KIcon( "chart_bar_beside" ), i18n("Normal") );
     d->stackedBarChartAction = barChartMenu->addAction( KIcon( "chart_bar_layer" ), i18n("Stacked") );
     d->percentBarChartAction = barChartMenu->addAction( KIcon( "chart_bar_percent" ), i18n("Percent") );
     
+    // Line charts
     QMenu *lineChartMenu = chartTypeMenu->addMenu( KIcon( "chart_line_normal" ), i18n( "Line Chart" ) );
     d->normalLineChartAction  = lineChartMenu->addAction( KIcon( "chart_line_normal" ), i18n("Normal") );
     d->stackedLineChartAction = lineChartMenu->addAction( KIcon( "chart_line_stacked" ), i18n("Stacked") );
     d->percentLineChartAction = lineChartMenu->addAction( KIcon( "chart_line_percent" ), i18n("Percent") );
-    
+
+    // Area charts
     QMenu *areaChartMenu = chartTypeMenu->addMenu( KIcon( "chart_area_normal" ), i18n( "Area Chart" ) );
     d->normalAreaChartAction  = areaChartMenu->addAction( KIcon( "chart_area_normal" ), i18n("Normal") );
     d->stackedAreaChartAction = areaChartMenu->addAction( KIcon( "chart_area_stacked" ), i18n("Stacked") );
     d->percentAreaChartAction = areaChartMenu->addAction( KIcon( "chart_area_percent" ), i18n("Percent") );
     
     chartTypeMenu->addSeparator();
-    
-    d->radarChartAction = chartTypeMenu->addAction( KIcon( "chart_polar_normal" ), i18n("Polar Chart") );
+
+    // Circular charts: pie and ring
     d->circleChartAction = chartTypeMenu->addAction( KIcon( "chart_pie_normal" ), i18n("Pie Chart") );
-    
+    d->ringChartAction = chartTypeMenu->addAction( KIcon( "chart_ring_normal" ), i18n("Ring Chart") );
+
     chartTypeMenu->addSeparator();
-    
+
+    // Polar charts: radar
+    d->radarChartAction = chartTypeMenu->addAction( KIcon( "chart_polar_normal" ), i18n("Polar Chart") );
+
+    chartTypeMenu->addSeparator();
+
+    // X/Y charts: scatter and bubble
     d->scatterChartAction = chartTypeMenu->addAction( KIcon( "chart_scatter_normal" ), i18n("Scatter Chart") );
-    
+    d->bubbleChartAction = chartTypeMenu->addAction( KIcon( "chart_bubble_normal" ), i18n("Bubble Chart") );
+    d->bubbleChartAction->setEnabled( false );
+
+    chartTypeMenu->addSeparator();
+
+    d->stockChartAction = chartTypeMenu->addAction( KIcon( "chart_stock_normal" ), i18n("Stock Chart") );
+    d->stockChartAction->setEnabled( false );
+    d->surfaceChartAction = chartTypeMenu->addAction( KIcon( "chart_surface_normal" ), i18n("Surface Chart") );
+    d->surfaceChartAction->setEnabled( false );
+    d->ganttChartAction = chartTypeMenu->addAction( KIcon( "chart_gantt_normal" ), i18n("Gantt Chart") );
+    d->ganttChartAction->setEnabled( false );
+
     d->ui.chartTypeMenu->setMenu( chartTypeMenu );
     
     connect( chartTypeMenu, SIGNAL( triggered( QAction* ) ), 
@@ -223,6 +277,7 @@ ChartConfigWidget::ChartConfigWidget()
     
     // Data set chart type button
     QMenu *dataSetChartTypeMenu = new QMenu( this );
+
     // Default chart type is a bar chart
     dataSetChartTypeMenu->setIcon( KIcon( "chart_bar_beside" ) );       
 
@@ -242,8 +297,10 @@ ChartConfigWidget::ChartConfigWidget()
     d->dataSetStackedAreaChartAction = d->dataSetAreaChartMenu->addAction( KIcon( "chart_area_stacked" ), i18n("Stacked") );
     d->dataSetPercentAreaChartAction = d->dataSetAreaChartMenu->addAction( KIcon( "chart_area_percent" ), i18n("Percent") );
     
-    d->dataSetRadarChartAction = dataSetChartTypeMenu->addAction( KIcon( "chart_polar_normal" ), i18n("Polar Chart") );
     d->dataSetCircleChartAction = dataSetChartTypeMenu->addAction( KIcon( "chart_pie_normal" ), i18n("Pie Chart") );
+    d->dataSetRingChartAction = dataSetChartTypeMenu->addAction( KIcon( "chart_ring_normal" ), i18n("Ring Chart") );
+    d->dataSetRadarChartAction = dataSetChartTypeMenu->addAction( KIcon( "chart_polar_normal" ), i18n("Polar Chart") );
+
     d->dataSetScatterChartAction = dataSetChartTypeMenu->addAction( KIcon( "chart_scatter_normal" ), i18n("Scatter Chart") );
     
     d->ui.dataSetChartTypeMenu->setMenu( dataSetChartTypeMenu );
@@ -254,8 +311,6 @@ ChartConfigWidget::ChartConfigWidget()
     connect( d->ui.dataSetHasChartType, SIGNAL( toggled( bool ) ),
              this,                      SLOT( ui_dataSetHasChartTypeChanged( bool ) ) );
              
-    initTableEditorDialog();
-    
     // "Plot Area" tab
     connect( d->ui.showTitle,    SIGNAL( toggled( bool ) ),
              this,               SIGNAL( showTitleChanged( bool ) ) );
@@ -322,11 +377,12 @@ ChartConfigWidget::~ChartConfigWidget()
     delete d;
 }
 
-void ChartConfigWidget::initTableEditorDialog()
+void ChartConfigWidget::deleteSubDialogs()
 {
-    // Quick-access settings
-    d->tableEditorDialog = new TableEditorDialog;
-    d->tableEditorDialog->hide();
+    if ( d->tableEditorDialog ) {
+        delete d->tableEditorDialog;
+        d->tableEditorDialog = 0;
+    }
 }
 
 void ChartConfigWidget::open( KoShape* shape )
@@ -334,7 +390,10 @@ void ChartConfigWidget::open( KoShape* shape )
     if ( ! shape ) {
         return;
     }
-
+ 
+    // There are 3 shapes that we can select: the full chart shape,
+    // the plotarea and the legend.
+    //
     // Find the selected shape and adapt the tool option window to
     // which of the subshapes of the chart widget that was actually
     // selected.  Then select the tab depending on which one it was.
@@ -359,7 +418,7 @@ void ChartConfigWidget::open( KoShape* shape )
     }
 
     KoChart::ChartModel *spreadSheetModel = qobject_cast<KoChart::ChartModel*>( d->shape->model() );
-    TableModel *tableModel = qobject_cast<TableModel*>( d->shape->model() );
+    ChartTableModel *tableModel = qobject_cast<ChartTableModel*>( d->shape->model() );
     d->sourceIsSpreadSheet = ( spreadSheetModel != 0 && tableModel == 0 );
     
     // Update the axis titles
@@ -398,7 +457,6 @@ void ChartConfigWidget::open( KoShape* shape )
     else {
 	// This part is run when the data source is not external,
 	// i.e. the data is handled by the chart shape itself.
-        d->tableEditorDialog->setProxyModel( d->shape->proxyModel() );
         connect( d->ui.editData, SIGNAL( clicked( bool ) ),
                  this,           SLOT( slotShowTableEditor( bool ) ) );
     }
@@ -458,23 +516,48 @@ void ChartConfigWidget::chartTypeSelected( QAction *action )
 
     // also known as polar chart.
     else if ( action == d->radarChartAction ) {
-        type = RadarChartType;
+        type    = RadarChartType;
         subtype = NoChartSubtype;
     }
 
     // also known as pie chart
     else if ( action == d->circleChartAction ) {
-        type = CircleChartType;
+        type    = CircleChartType;
+        subtype = NoChartSubtype;
+    }
+    else if ( action == d->ringChartAction ) {
+        type    = RingChartType;
         subtype = NoChartSubtype;
     }
     
     else if ( action == d->scatterChartAction ) {
-        type = ScatterChartType;
+        type    = ScatterChartType;
         subtype = NoChartSubtype;
     }
+
+    else if ( action == d->stockChartAction ) {
+        type    = StockChartType;
+        subtype = NoChartSubtype;
+    }
+
+    else if ( action == d->bubbleChartAction ) {
+        type    = BubbleChartType;
+        subtype = NoChartSubtype;
+    }
+
+    else if ( action == d->surfaceChartAction ) {
+        type    = SurfaceChartType;
+        subtype = NoChartSubtype;
+    }
+
+    else if ( action == d->ganttChartAction ) {
+        type    = GanttChartType;
+        subtype = NoChartSubtype;
+    }
+
     
     // o Make sure polar and cartesian plots can't conflict and
-    // don't allow the user to mix these two types
+    //   don't allow the user to mix these two types
     // o Hide axis configuration options for polar plots
     if ( isPolar( type ) ) {
         setPolarChartTypesEnabled( true );
@@ -498,6 +581,7 @@ void ChartConfigWidget::chartTypeSelected( QAction *action )
 void ChartConfigWidget::setPolarChartTypesEnabled( bool enabled )
 {
     d->dataSetCircleChartAction->setEnabled( enabled );
+    d->dataSetRingChartAction->setEnabled( enabled );
     d->dataSetRadarChartAction->setEnabled( enabled );
 }
 
@@ -510,6 +594,13 @@ void ChartConfigWidget::setCartesianChartTypesEnabled( bool enabled )
     d->dataSetLineChartMenu->setEnabled( enabled );
     d->dataSetAreaChartMenu->setEnabled( enabled );
     d->dataSetScatterChartAction->setEnabled( enabled );
+    // FIXME: Enable for:
+    // pie, ring?
+    //NYI: 
+    //stock
+    //bubble
+    //surface
+    //gantt
 }
 
 void ChartConfigWidget::ui_dataSetPieExplodeFactorChanged( int percent )
@@ -547,7 +638,7 @@ void ChartConfigWidget::dataSetChartTypeSelected( QAction *action )
     if ( d->selectedDataSet < 0 )
         return;
     
-    ChartType     type = LastChartType;
+    ChartType     type    = LastChartType;
     ChartSubtype  subtype = NoChartSubtype;
     
     if ( action == d->dataSetNormalBarChartAction ) {
@@ -587,9 +678,17 @@ void ChartConfigWidget::dataSetChartTypeSelected( QAction *action )
         type = RadarChartType;
     else if ( action == d->dataSetCircleChartAction )
         type = CircleChartType;
+    else if ( action == d->dataSetRingChartAction )
+        type = RingChartType;
     else if ( action == d->dataSetScatterChartAction )
         type = ScatterChartType;
     
+    // FIXME: Not supported by KChart yet:
+    //stock
+    //bubble
+    //surface
+    //gantt
+
     DataSet *dataSet = d->dataSets[ d->selectedDataSet ];
     Q_ASSERT( dataSet );
     if ( !dataSet )
@@ -772,6 +871,7 @@ void ChartConfigWidget::update()
     if (    d->type    != d->shape->chartType()
          || d->subtype != d->shape->chartSubType() )
     {
+        // Update the chart type specific settings in the "Data Sets" tab
         bool needSeparator = false;
         if ( d->shape->chartType() == BarChartType ) {
             d->ui.barProperties->show();
@@ -781,11 +881,17 @@ void ChartConfigWidget::update()
             d->ui.barProperties->hide();
             d->ui.pieProperties->show();
             needSeparator = true;
+        } else if ( d->shape->chartType() == RingChartType ) {
+            d->ui.barProperties->hide();
+            d->ui.pieProperties->show();
+            needSeparator = true;
         } else {
             d->ui.barProperties->hide();
             d->ui.pieProperties->hide();
         }
         d->ui.propertiesSeparator->setVisible( needSeparator );
+
+        // Set the chart type icon in the chart type button.
         switch ( d->shape->chartSubType() ) {
         case NormalChartSubtype:
             switch ( d->shape->chartType() ) {
@@ -875,11 +981,13 @@ void ChartConfigWidget::update()
             setPolarChartTypesEnabled( false );
             setCartesianChartTypesEnabled( true );
         }
-        
+
+        // ...and finally save the new chart type and subtype.
         d->type    = d->shape->chartType();
         d->subtype = d->shape->chartSubType();
     }
     
+    // If the datasets have changed, set up the new ones.
     if ( d->shape->plotArea()->dataSets() != d->dataSets ) {
         d->dataSets = d->shape->plotArea()->dataSets();
         d->ui.dataSets->clear();
@@ -891,11 +999,13 @@ void ChartConfigWidget::update()
             d->ui.dataSets->addItem( title );
             d->cellRegionDialog.dataSets->addItem( title );
         }
+
         // Select the first data set
         ui_dataSetSelectionChanged( 0 );
         ui_dataSetSelectionChanged_CellRegionDialog( 0 );
     }
-    
+
+    // If the "3D" checkbox is checked, then adapt the chart to that.
     d->ui.threeDLook->setChecked( d->shape->isThreeD() );
     if ( d->shape->legend() ) {
         d->ui.legendTitle->blockSignals( true );
@@ -909,10 +1019,17 @@ void ChartConfigWidget::update()
 
 void ChartConfigWidget::slotShowTableEditor( bool show )
 {
-    if ( show )
+    if ( !d->tableEditorDialog ) {
+        d->tableEditorDialog = new TableEditorDialog;
+        d->tableEditorDialog->setProxyModel( d->shape->proxyModel() );
+    }
+
+    if ( show ) {
         d->tableEditorDialog->hide();
-    else
+    }
+    else {
         d->tableEditorDialog->show();
+    }
 }
 
 
