@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
 
    Copyright 2007-2008 Johannes Simon <johannes.simon@gmail.com>
-   Copyright      2008 Inge Wallin     <inge@lysator.liu.se>
+   Copyright 2008-2009 Inge Wallin    <inge@lysator.liu.se>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,7 +19,8 @@
  * Boston, MA 02110-1301, USA.
 */
 
-// Local
+
+// Own
 #include "DataSet.h"
 
 // Qt
@@ -36,12 +37,12 @@
 #include <KDChartTextAttributes>
 #include <KDChartAbstractDiagram>
 #include <KDChartMeasure>
+#include "KDChartModel.h"
 
 // KChart
-#include "ProxyModel.h"
+#include "ChartProxyModel.h"
 #include "Axis.h"
 #include "PlotArea.h"
-#include "KDChartModel.h"
 
 
 using namespace KChart;
@@ -81,14 +82,16 @@ public:
     QPen pen;
     QBrush brush;
     int num;
+
+    // The different CellRegions for a dataset
+    // Note: These are all 1-dimensional, i.e. vectors.
+    CellRegion labelDataRegion; // one cell that holds the label
+    CellRegion yDataRegion;     // normal y values
+    CellRegion xDataRegion;     // x values -- only for scatter & bubble charts
+    CellRegion customDataRegion;// used for bubble width in bubble charts
+    CellRegion categoryDataRegion; // x labels -- same for all datasets
     
-    CellRegion xDataRegion;
-    CellRegion yDataRegion;
-    CellRegion customDataRegion;
-    CellRegion labelDataRegion;
-    CellRegion categoryDataRegion;
-    
-    ProxyModel *model;
+    ChartProxyModel *model;
     KDChart::AbstractDiagram *kdDiagram;
     int kdDataSetNumber;
     
@@ -136,7 +139,8 @@ void DataSet::Private::updateSize()
     newSize = qMax( newSize, categoryDataRegion.cellCount() );
     
     size = newSize;
-    
+    // FIXME: The first test below can *never* fail because of the
+    //        assignment just above. (iw)
     if ( newSize != size && !blockSignals && kdChartModel ) {
         kdChartModel->dataSetSizeChanged( parent, size );
     }
@@ -158,6 +162,7 @@ ChartType DataSet::Private::effectiveChartType() const
 void DataSet::Private::updateDiagram() const
 {
     if ( kdDiagram && kdDataSetNumber >= 0 && size > 0 ) {
+
 	// FIXME: This should be done in a more proper way.
 	// The problem here is that line diagrams don't use the brush
 	// to paint their lines, but the pen. We on the other hand set
@@ -198,12 +203,16 @@ QVariant DataSet::Private::data( const CellRegion &region, int index ) const
     // source model.
     QPoint dataPoint = region.pointAtIndex( index );
 
+    // FIXME: Why not use this immediately if true?
     const bool verticalHeaderData   = dataPoint.x() == 0;
     const bool horizontalHeaderData = dataPoint.y() == 0;
 
     // Check if the data point is valid
     const bool validDataPoint = isValidDataPoint( dataPoint );
-    Q_ASSERT( validDataPoint );
+
+    // Remove, since it makes kspread crash when inserting a chart for
+    // a 1x1 cell region.
+    //Q_ASSERT( validDataPoint );
     if ( !validDataPoint )
         return QVariant();    
 
@@ -225,7 +234,7 @@ QVariant DataSet::Private::data( const CellRegion &region, int index ) const
     return data;
 }
 
-DataSet::DataSet( ProxyModel *proxyModel )
+DataSet::DataSet( ChartProxyModel *proxyModel )
     : d( new Private( this ) )
 {
     d->model = proxyModel;
@@ -235,6 +244,8 @@ DataSet::~DataSet()
 {
     if ( d->attachedAxis )
         d->attachedAxis->detachDataSet( this, true );
+
+    delete d;
 }
 
 
@@ -263,7 +274,8 @@ Axis *DataSet::attachedAxis() const
     return d->attachedAxis;
 }
 
-ProxyModel *DataSet::model() const
+// FIXME: Should this method also be called proxyModel?
+ChartProxyModel *DataSet::model() const
 {
     return d->model;
 }
@@ -380,7 +392,9 @@ void DataSet::setShowValues( bool showValues )
     
     KDChart::DataValueAttributes attributes = d->kdDiagram->dataValueAttributes( d->kdDataSetNumber );
     attributes.setVisible( showValues );
-    // FIXME: This should be a dynamic property that can be changed by the user (for 2.1)
+
+    // FIXME: This should be a dynamic property that can be changed by
+    //        the user (for 2.1)
     KDChart::TextAttributes textAttributes = attributes.textAttributes();
     textAttributes.setFontSize( KDChart::Measure( 6, KDChartEnums::MeasureCalculationModeAbsolute ) );
     attributes.setTextAttributes( textAttributes );
@@ -744,6 +758,7 @@ int DataSet::dimension() const
     const ChartType chartType = ( d->chartType != LastChartType
 				  ? d->chartType
 				  : d->globalChartType );
+    // FIXME BUG: Ring, Surface
     switch ( chartType ) {
     case BarChartType:
     case AreaChartType:
@@ -753,9 +768,9 @@ int DataSet::dimension() const
     case SurfaceChartType:
 	return 1;
 
+    case RingChartType:
     case ScatterChartType:
     case GanttChartType:
-    case RingChartType:
 	return 2;
 
     case BubbleChartType:
@@ -770,7 +785,7 @@ int DataSet::dimension() const
 	return 0;
     }
 	
-    // Avoid warnings from compiler
+    // Avoid warnings from the compiler.
     return 0;
 }
 
