@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
 
    Copyright 2007-2008 Johannes Simon <johannes.simon@gmail.com>
+   Copyright 2009      Inge Wallin <inge@lysator.liu.se>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -18,14 +19,16 @@
    Boston, MA 02110-1301, USA.
 */
 
-// Local
+// Own
 #include "PlotArea.h"
-#include "Axis.h"
-#include "DataSet.h"
-#include "Surface.h"
-#include "ThreeDScene.h"
-#include "ProxyModel.h"
-#include "Legend.h"
+
+// Qt
+#include <QPointF>
+#include <QSizeF>
+#include <QList>
+#include <QImage>
+#include <QPainter>
+#include <kdebug.h>
 
 // KOffice
 #include <KoXmlReader.h>
@@ -42,17 +45,11 @@
 #include <KoShapeBackground.h>
 #include <KoOdfGraphicStyles.h>
 
-// Qt
-#include <QPointF>
-#include <QSizeF>
-#include <QList>
-#include <QImage>
-#include <QPainter>
-
 // KDChart
 #include <KDChartChart>
 #include <KDChartCartesianAxis>
 #include <KDChartAbstractDiagram>
+
 #include <KDChartAbstractCartesianDiagram>
 #include <KDChartAbstractCoordinatePlane>
 #include <KDChartBarAttributes>
@@ -70,6 +67,15 @@
 #include <KDChartRingDiagram>
 #include <KDChartPolarDiagram>
 
+// KChart
+#include "Legend.h"
+#include "Surface.h"
+#include "Axis.h"
+#include "DataSet.h"
+#include "ThreeDScene.h"
+#include "ChartProxyModel.h"
+
+
 using namespace KChart;
 
 const int MAX_PIXMAP_SIZE = 1000;
@@ -77,35 +83,49 @@ const int MAX_PIXMAP_SIZE = 1000;
 class PlotArea::Private
 {
 public:
-    Private();
+    Private( ChartShape *parent );
     ~Private();
 
     // The parent chart shape
     ChartShape *shape;
 
-    // The list of axes
-    QList<Axis*> axes;
-    
-    QList<KDChart::AbstractDiagram*> kdDiagrams;
-    
-    KDChart::Chart *kdChart;
-    KDChart::AbstractCoordinatePlane *kdPlane;
-    
-    Surface *wall;
-    Surface *floor;
-    
-    ChartType chartType;
-    ChartSubtype chartSubtype;
-    bool threeD;
-    
-    ThreeDScene *threeDScene;
-    
-    int gapBetweenBars;
-    int gapBetweenSets;
-    
-    QList<KoShape*> automaticallyHiddenAxisTitles;
+    // ----------------------------------------------------------------
+    // Parts and properties of the chart
 
-    // We can rerender faster if we cache KDChart's output
+    ChartType     chartType;
+    ChartSubtype  chartSubtype;
+    
+    Surface       *wall;
+    Surface       *floor;       // Only used in 3D charts
+    
+    // The axes
+    QList<Axis*>     axes;
+    QList<KoShape*>  automaticallyHiddenAxisTitles;
+    
+    // 3D properties
+    bool          threeD;
+    ThreeDScene  *threeDScene;
+    
+    // ----------------------------------------------------------------
+    // Data specific to each types
+
+    // 1. Bar charts
+    bool  vertical;
+    int   gapBetweenBars;
+    int   gapBetweenSets;
+    
+    // 2. Pie charts
+    int   pieExplodeFactor;     // in percents
+
+    // ----------------------------------------------------------------
+    // The embedded KD Chart
+
+    // The KD Chart parts
+    KDChart::Chart                    *kdChart;
+    KDChart::AbstractCoordinatePlane  *kdPlane;
+    QList<KDChart::AbstractDiagram*>   kdDiagrams;
+    
+    // Caching: We can rerender faster if we cache KDChart's output
     QImage   image;
     bool     paintPixmap;
     QPointF  lastZoomLevel;
@@ -113,57 +133,71 @@ public:
     mutable bool pixmapRepaintRequested;
 };
 
-PlotArea::Private::Private()
+PlotArea::Private::Private( ChartShape *parent )
 {
-    shape = 0;
-    kdChart = new KDChart::Chart();
-    kdPlane = new KDChart::CartesianCoordinatePlane( kdChart );
-    wall = 0;
-    floor = 0;
-    chartType = BarChartType;
+    shape = parent;
+
+    // Default type: normal bar chart
+    chartType    = BarChartType;
     chartSubtype = NormalChartSubtype;
-    threeD = false;
+
+    wall  = 0;
+    floor = 0;
+
+    threeD      = false;
     threeDScene = 0;
+
+    // Data specific for bar charts
+    vertical       = false;
     gapBetweenBars = 0;
     gapBetweenSets = 100;
+
+    // KD Chart stuff
+    kdChart = new KDChart::Chart();
+    kdPlane = new KDChart::CartesianCoordinatePlane( kdChart );
+
+    // Cache
     pixmapRepaintRequested = true;
-    paintPixmap = true;
+    paintPixmap            = true;
 }
 
 PlotArea::Private::~Private()
 {
+    delete kdPlane;
+    delete kdChart;
 }
 
 
 PlotArea::PlotArea( ChartShape *parent )
-    : KoShape()
-    , d( new Private )
+    : QObject()
+    , KoShape()
+    , d( new Private( parent ) )
 {
-    Q_ASSERT( parent );
-    
     setShapeId( ChartShapeId );
-    d->shape = parent;
     
+    Q_ASSERT( d->shape );
     Q_ASSERT( d->shape->proxyModel() );
     
     connect( d->shape->proxyModel(), SIGNAL( modelReset() ),
-             this, SLOT( dataSetCountChanged() ) );
+             this,                   SLOT( dataSetCountChanged() ) );
     connect( d->shape->proxyModel(), SIGNAL( modelResetComplete() ),
-             this, SLOT( update() ) );
+             this,                   SLOT( update() ) );
+    // FIXME: The following signatures don't match.  Bug?
     connect( d->shape->proxyModel(), SIGNAL( rowsInserted( const QModelIndex, int, int ) ),
-             this, SLOT( dataSetCountChanged() ) );
+             this,                   SLOT( dataSetCountChanged() ) );
     connect( d->shape->proxyModel(), SIGNAL( rowsRemoved( const QModelIndex, int, int ) ),
-             this, SLOT( dataSetCountChanged() ) );
+             this,                   SLOT( dataSetCountChanged() ) );
     connect( d->shape->proxyModel(), SIGNAL( columnsInserted( const QModelIndex, int, int ) ),
-             this, SLOT( update() ) );
+             this,                   SLOT( update() ) );
     connect( d->shape->proxyModel(), SIGNAL( columnsRemoved( const QModelIndex, int, int ) ),
-             this, SLOT( update() ) );
+             this,                   SLOT( update() ) );
     connect( d->shape->proxyModel(), SIGNAL( dataChanged() ),
-             this, SLOT( update() ) );
+             this,                   SLOT( update() ) );
 }
 
 PlotArea::~PlotArea()
 {
+    delete d;
 }
 
 
@@ -171,17 +205,19 @@ void PlotArea::init()
 {
     d->kdChart->resize( size().toSize() );
     d->kdChart->replaceCoordinatePlane( d->kdPlane );
+
     KDChart::FrameAttributes attr = d->kdChart->frameAttributes();
     attr.setVisible( false );
     d->kdChart->setFrameAttributes( attr );
     
     d->wall = new Surface( this );
-    d->floor = new Surface( this );
+    //d->floor = new Surface( this );
     
-    // There need to be at least these two axes. Do not delete, but hide them instead.
+    // There need to be at least these two axes. Do not delete, but
+    // hide them instead.
     Axis *xAxis = new Axis( this );
-    xAxis->setPosition( BottomAxisPosition );
     Axis *yAxis = new Axis( this );
+    xAxis->setPosition( BottomAxisPosition );
     yAxis->setPosition( LeftAxisPosition );
     yAxis->setShowMajorGrid( true );
     d->axes.append( xAxis );
@@ -192,16 +228,15 @@ void PlotArea::dataSetCountChanged()
 {
     if ( !yAxis() )
         return;
-    foreach( DataSet *dataSet, proxyModel()->dataSets() )
-    {
-        if ( !dataSet->attachedAxis() )
-        {
+
+    foreach( DataSet *dataSet, proxyModel()->dataSets() ) {
+        if ( !dataSet->attachedAxis() ) {
             yAxis()->attachDataSet( dataSet );
         }
     }
 }
 
-ProxyModel *PlotArea::proxyModel() const
+ChartProxyModel *PlotArea::proxyModel() const
 {
     return d->shape->proxyModel();
 }
@@ -222,41 +257,31 @@ int PlotArea::dataSetCount() const
     return proxyModel()->dataSets().size();
 }
 
-Surface *PlotArea::wall() const
-{
-    return d->wall;
-}
-
-Surface *PlotArea::floor() const
-{
-    return d->floor;
-}
-
 Axis *PlotArea::xAxis() const
 {
-    foreach( Axis *axis, d->axes )
-    {
+    foreach( Axis *axis, d->axes ) {
         if ( axis->orientation() == Qt::Horizontal )
             return axis;
     }
+
     return 0;
 }
 
 Axis *PlotArea::yAxis() const
 {
-    foreach( Axis *axis, d->axes )
-    {
+    foreach( Axis *axis, d->axes ) {
         if ( axis->orientation() == Qt::Vertical )
             return axis;
     }
+
     return 0;
 }
 
 Axis *PlotArea::secondaryXAxis() const
 {
     bool firstXAxisFound = false;
-    foreach( Axis *axis, d->axes )
-    {
+
+    foreach( Axis *axis, d->axes ) {
         if ( axis->orientation() == Qt::Horizontal ) {
             if ( firstXAxisFound )
                 return axis;
@@ -264,14 +289,15 @@ Axis *PlotArea::secondaryXAxis() const
                 firstXAxisFound = true;
         }
     }
+
     return 0;
 }
 
 Axis *PlotArea::secondaryYAxis() const
 {
     bool firstYAxisFound = false;
-    foreach( Axis *axis, d->axes )
-    {
+
+    foreach( Axis *axis, d->axes ) {
         if ( axis->orientation() == Qt::Vertical ) {
             if ( firstYAxisFound )
                 return axis;
@@ -279,6 +305,7 @@ Axis *PlotArea::secondaryYAxis() const
                 firstYAxisFound = true;
         }
     }
+
     return 0;
 }
 
@@ -314,20 +341,18 @@ int PlotArea::gapBetweenSets() const
 
 bool PlotArea::addAxis( Axis *axis )
 {
-    if ( d->axes.contains( axis ) )
-    {
-    	qWarning() << "PlotArea::addAxis(): This axis has already been added.";
+    if ( d->axes.contains( axis ) ) {
+    	qWarning() << "PlotArea::addAxis(): Trying to add already added axis.";
     	return false;
     }
-    if ( !axis )
-    {
+
+    if ( !axis ) {
     	qWarning() << "PlotArea::addAxis(): Pointer to axis is NULL!";
     	return false;
     }
     d->axes.append( axis );
     
-    if ( axis->dimension() == XAxisDimension )
-    {
+    if ( axis->dimension() == XAxisDimension ) {
         foreach ( Axis *_axis, d->axes )
             _axis->registerKdXAxis( axis->kdAxis() );
     }
@@ -339,20 +364,18 @@ bool PlotArea::addAxis( Axis *axis )
 
 bool PlotArea::removeAxis( Axis *axis )
 {
-    if ( !d->axes.contains( axis ) )
-    {
-    	qWarning() << "PlotArea::removeAxis(): This axis has not been added previously.";
+    if ( !d->axes.contains( axis ) ) {
+    	qWarning() << "PlotArea::removeAxis(): Trying to remove non-added axis.";
     	return false;
     }
-    if ( !axis )
-    {
+
+    if ( !axis ) {
     	qWarning() << "PlotArea::removeAxis(): Pointer to axis is NULL!";
     	return false;
     }
     d->axes.removeAll( axis );
     
-    if ( axis->dimension() == XAxisDimension )
-    {
+    if ( axis->dimension() == XAxisDimension ) {
         foreach ( Axis *_axis, d->axes )
             _axis->deregisterKdXAxis( axis->kdAxis() );
     }
@@ -369,38 +392,38 @@ bool PlotArea::removeAxis( Axis *axis )
 
 void PlotArea::setChartType( ChartType type )
 {
-    if ( !isPolar( d->chartType ) && isPolar( type ) )
-    {
-        foreach ( Axis *axis, d->axes )
-        {
+    // Lots of things to do if the old and new types of coordinate
+    // systems don't match.
+    if ( !isPolar( d->chartType ) && isPolar( type ) ) {
+        foreach ( Axis *axis, d->axes ) {
             if ( !axis->title()->isVisible() )
                 continue;
+
             axis->title()->setVisible( false );
             d->automaticallyHiddenAxisTitles.append( axis->title() );
         }
     }
-    else if ( isPolar( d->chartType ) && !isPolar( type ) )
-    {
-        foreach ( KoShape *title, d->automaticallyHiddenAxisTitles )
-        {
+    else if ( isPolar( d->chartType ) && !isPolar( type ) ) {
+        foreach ( KoShape *title, d->automaticallyHiddenAxisTitles ) {
             title->setVisible( true );
         }
         d->automaticallyHiddenAxisTitles.clear();
     }
-    
-    if ( d->chartType != ScatterChartType && type == ScatterChartType )
-    {
+
+    // FIXME BUG: This seems buggy.
+    // FIXME NOW: Make this a switch (see comment below)
+    // Set the dimensionality of the data points.
+    if ( d->chartType != ScatterChartType && type == ScatterChartType ) {
         d->shape->proxyModel()->setDataDimensions( 2 );
     }
-    else if ( d->chartType == ScatterChartType && type != ScatterChartType )
-    {
+    // FIXME: This is not really true.  Could also be 3 (bubble charts)
+    else if ( d->chartType == ScatterChartType && type != ScatterChartType ) {
         d->shape->proxyModel()->setDataDimensions( 1 );
     }
     
     d->chartType = type;
     
-    foreach ( Axis *axis, d->axes )
-    {
+    foreach ( Axis *axis, d->axes ) {
         axis->plotAreaChartTypeChanged( type );
     }
     
@@ -411,8 +434,7 @@ void PlotArea::setChartSubType( ChartSubtype subType )
 {
     d->chartSubtype = subType;
     
-    foreach ( Axis *axis, d->axes )
-    {
+    foreach ( Axis *axis, d->axes ) {
         axis->plotAreaChartSubTypeChanged( subType );
     }
     
@@ -429,6 +451,11 @@ void PlotArea::setThreeD( bool threeD )
     requestRepaint();
 }
 
+
+// ----------------------------------------------------------------
+//                         loading and saving
+
+
 bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
                         KoShapeLoadingContext &context )
 {
@@ -436,27 +463,72 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
     styleStack.save();
 
     styleStack.clear();
+
+    // Find out about things that are in the plotarea style.
+    // 
+    // These things include chart subtype, special things for some
+    // chart types like line charts, stock charts, etc.
     if ( plotAreaElement.hasAttributeNS( KoXmlNS::chart, "style-name" ) ) {
         context.odfLoadingContext().fillStyleStack( plotAreaElement, KoXmlNS::chart, "style-name", "chart" );
 
         styleStack.setTypeProperties( "graphic" );
-
         styleStack.setTypeProperties( "chart" );
 
-        if ( styleStack.hasProperty( KoXmlNS::chart, "percentage" ) && styleStack.property( KoXmlNS::chart, "percentage" ) == "true" )
+        // Set subtypes stacked or percent.
+        // These are valid for Bar, Line, Area and Radar types.
+        if ( styleStack.hasProperty( KoXmlNS::chart, "percentage" )
+             && styleStack.property( KoXmlNS::chart, "percentage" ) == "true" )
+        {
             setChartSubType( PercentChartSubtype );
-        else if ( styleStack.hasProperty( KoXmlNS::chart, "stacked" ) && styleStack.property( KoXmlNS::chart, "stacked" ) == "true" )
+        }
+        else if ( styleStack.hasProperty( KoXmlNS::chart, "stacked" )
+                  && styleStack.property( KoXmlNS::chart, "stacked" ) == "true" )
+        {
             setChartSubType( StackedChartSubtype );
+        }
+
+        // Data specific to bar charts
+        if ( styleStack.hasProperty( KoXmlNS::chart, "vertical" )
+             && styleStack.property( KoXmlNS::chart, "vertical" ) == "true" )
+        {
+            d->vertical = true;
+        }
+
+        // Data direction: It's in the plotarea style.
+        if ( styleStack.hasProperty( KoXmlNS::chart, "series-source" ) ) {
+            const QString  seriesSource
+                = styleStack.property( KoXmlNS::chart, "series-source" );
+
+            kDebug(35001) << "series-source=" << seriesSource;
+            if ( seriesSource == "rows" )
+                proxyModel()->setDataDirection( Qt::Horizontal );
+            else if ( seriesSource == "columns" )
+                proxyModel()->setDataDirection( Qt::Vertical );
+            else
+                // Use the default value for wrong values (not "rows" or "columns")
+                proxyModel()->setDataDirection( Qt::Vertical );
+        }
+
+        // Special properties for various chart types
+#if 0
+        switch () {
+        case BarChartType:
+            if ( styleStack )
+                ;
+        }
+#endif
     }
+
     loadOdfAttributes( plotAreaElement, context, OdfAllAttributes );
     
-    KoOdfStylesReader &stylesReader = context.odfLoadingContext().stylesReader();
+    //KoOdfStylesReader &stylesReader = context.odfLoadingContext().stylesReader();
     
-    KoXmlElement dataHasLabelsElem = KoXml::namedItemNS( plotAreaElement, 
-                                                         KoXmlNS::chart, "data-source-has-labels" );
+    // Find out if the data table contains labels as first row and/or column.
+    // This is in the plot-area element itself.
     if ( plotAreaElement.hasAttributeNS( KoXmlNS::chart,
                                          "data-source-has-labels" ) ) {
 
+        // Yes, it does.  Now find out how.
         const QString  dataSourceHasLabels
             = plotAreaElement.attributeNS( KoXmlNS::chart,
                                            "data-source-has-labels" );
@@ -482,14 +554,13 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
     }
 
     // Remove all axes before loading new ones
-    KoXmlElement e;
     while( !d->axes.isEmpty() ) {
         Axis *axis = d->axes.takeLast();
         Q_ASSERT( axis );
         delete axis;
     }
     
-    // A data set is always attached to an axis, so load them first
+    // A data set is always attached to an axis, so load them first.
     KoXmlElement n;
     forEachElement ( n, plotAreaElement ) {
         if ( n.namespaceURI() != KoXmlNS::chart )
@@ -501,9 +572,16 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
             addAxis( axis );
         }
     }
-    // Load data sets
+
+    // Now, after the axes, load the datasets.
+    // Note that this only contains properties of the datasets, the
+    // actual data is not stored here.
+    //
+    // FIXME: Isn't the proxy model a strange place to store this data?
     d->shape->proxyModel()->loadOdf( plotAreaElement, context );
 
+    // Now load the surfaces (wall and possibly floor)
+    // FIXME: Use named tags instead of looping?
     forEachElement ( n, plotAreaElement ) {
         if ( n.namespaceURI() != KoXmlNS::chart )
             continue;
@@ -512,6 +590,9 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
             d->wall->loadOdf( n, context );
         }
         else if ( n.localName() == "floor" ) {
+            // The floor is not always present, so allocate it if needed.
+            if ( !d->floor )
+                d->floor = new Surface( this );
             d->floor->loadOdf( n, context );
         }
         else if ( n.localName() != "axis" && n.localName() != "series" ) {
@@ -529,12 +610,12 @@ bool PlotArea::loadOdf( const KoXmlElement &plotAreaElement,
 void PlotArea::saveOdf( KoShapeSavingContext &context ) const
 {
     KoXmlWriter &bodyWriter = context.xmlWriter();
-    KoGenStyles &mainStyles = context.mainStyles();
-    
+    //KoGenStyles &mainStyles = context.mainStyles();
     bodyWriter.startElement( "chart:plot-area" );
     
-    KoGenStyle plotAreaStyle;
-    plotAreaStyle = KoGenStyle( KoGenStyle::StyleGraphicAuto, "chart" );
+    // FIXME: Somehow this style gets the name gr2 instead of ch2.
+    //        Fix that as well.
+    KoGenStyle plotAreaStyle( KoGenStyle::StyleChartAuto, "chart" );
     
     // Data direction
     const Qt::Orientation direction = proxyModel()->dataDirection();
@@ -544,11 +625,12 @@ void PlotArea::saveOdf( KoShapeSavingContext &context ) const
     // Save chart subtype
     saveOdfSubType( bodyWriter, plotAreaStyle );
     
-    bodyWriter.addAttribute( "chart:style-name", saveStyle( plotAreaStyle, context ) );
+    bodyWriter.addAttribute( "chart:style-name",
+                             saveStyle( plotAreaStyle, context ) );
     
     const QSizeF s( size() );
     const QPointF p( position() );
-    bodyWriter.addAttributePt( "svg:width", s.width() );
+    bodyWriter.addAttributePt( "svg:width",  s.width() );
     bodyWriter.addAttributePt( "svg:height", s.height() );
     bodyWriter.addAttributePt( "svg:x", p.x() );
     bodyWriter.addAttributePt( "svg:y", p.y() );
@@ -568,8 +650,9 @@ void PlotArea::saveOdf( KoShapeSavingContext &context ) const
             dataSourceHasLabels = "none";
     // Note: this is saved in the plotarea attributes and not the style.
     bodyWriter.addAttribute( "chart:data-source-has-labels",
-                            dataSourceHasLabels );
+                             dataSourceHasLabels );
 
+    // Save the axes.
     if ( isCartesian( d->chartType ) ) {
         foreach( Axis *axis, d->axes ) {
             axis->saveOdf( context );
@@ -579,8 +662,10 @@ void PlotArea::saveOdf( KoShapeSavingContext &context ) const
     // Save data series
     d->shape->proxyModel()->saveOdf( context );
 
-    d->floor->saveOdf( context, "chart:floor" );
+    // Save the floor and wall of the plotarea.
     d->wall->saveOdf( context, "chart:wall" );
+    if ( d->floor )
+        d->floor->saveOdf( context, "chart:floor" );
 
     // TODO chart:series
     // TODO chart:grid
@@ -588,8 +673,11 @@ void PlotArea::saveOdf( KoShapeSavingContext &context ) const
     bodyWriter.endElement(); // chart:plot-area
 }
 
-void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter, KoGenStyle& plotAreaStyle ) const
+void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter,
+                               KoGenStyle& plotAreaStyle ) const
 {
+    Q_UNUSED( xmlWriter );
+
     switch ( d->chartType ) {
     case BarChartType:
         switch( d->chartSubtype ) {
@@ -603,12 +691,15 @@ void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter, KoGenStyle& plotAreaStyle
             plotAreaStyle.addProperty( "chart:percentage", "true" );
             break;
         }
-    if ( d->threeD ) {
-        plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
-        // FIXME: Save all 3D attributes too.
-    }
-        plotAreaStyle.addProperty( "chart:vertical", "false" ); // FIXME
-        plotAreaStyle.addProperty( "chart:lines-used", 0 ); // FIXME: for now
+        if ( d->threeD ) {
+            plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
+            // FIXME: Save all 3D attributes too.
+        }
+        // Data specific to bar charts
+        if ( d->vertical )
+            plotAreaStyle.addProperty( "chart:vertical", "true" );
+        // Don't save this if zero, because that's the default.
+        //plotAreaStyle.addProperty( "chart:lines-used", 0 ); // FIXME: for now
         break;
 
     case LineChartType:
@@ -623,10 +714,10 @@ void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter, KoGenStyle& plotAreaStyle
             plotAreaStyle.addProperty( "chart:percentage", "true" );
             break;
         }
-    if ( d->threeD ) {
-        plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
-        // FIXME: Save all 3D attributes too.
-    }
+        if ( d->threeD ) {
+            plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
+            // FIXME: Save all 3D attributes too.
+        }
         // FIXME: What does this mean?
         plotAreaStyle.addProperty( "chart:symbol-type", "automatic" );
         break;
@@ -636,27 +727,47 @@ void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter, KoGenStyle& plotAreaStyle
         case NoChartSubtype:
         case NormalChartSubtype:
         break;
-    case StackedChartSubtype:
+        case StackedChartSubtype:
             plotAreaStyle.addProperty( "chart:stacked", "true" );
             break;
         case PercentChartSubtype:
             plotAreaStyle.addProperty( "chart:percentage", "true" );
             break;
         }
-        //plotAreaStyle.addProperty( "chart:lines-used", 0 ); // #### for now
-    if ( d->threeD ) {
-        plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
-        // FIXME: Save all 3D attributes too.
-    }
+
+        if ( d->threeD ) {
+            plotAreaStyle.addProperty( "chart:three-dimensional", "true" );
+            // FIXME: Save all 3D attributes too.
+        }
+
+    break;
 
     case CircleChartType:
         // FIXME
         break;
-    break;
 
     case RingChartType:
+        // FIXME
+        break;
+
     case ScatterChartType:
+        // FIXME
+        break;
     case RadarChartType:
+        // Save subtype of the Radar chart.
+        switch( d->chartSubtype ) {
+        case NoChartSubtype:
+        case NormalChartSubtype:
+            break;
+        case StackedChartSubtype:
+            plotAreaStyle.addProperty( "chart:stacked", "true" );
+            break;
+        case PercentChartSubtype:
+            plotAreaStyle.addProperty( "chart:percentage", "true" );
+            break;
+        }
+        break;
+
     case StockChartType:
     case BubbleChartType:
     case SurfaceChartType:
@@ -675,8 +786,6 @@ void PlotArea::saveOdfSubType( KoXmlWriter& xmlWriter, KoGenStyle& plotAreaStyle
 
 void PlotArea::setGapBetweenBars( int percent )
 {
-    // FIXME: Redundant data (dangerous!), but needed for getter method
-    // This information should instead by retrieved dynamically
     d->gapBetweenBars = percent;
 
     emit gapBetweenBarsChanged( percent );
@@ -684,8 +793,6 @@ void PlotArea::setGapBetweenBars( int percent )
 
 void PlotArea::setGapBetweenSets( int percent )
 {
-    // FIXME: Redundant data (dangerous!), but needed for getter method
-    // This information should instead by retrieved dynamically
     d->gapBetweenSets = percent;
 
     emit gapBetweenSetsChanged( percent );
@@ -693,6 +800,8 @@ void PlotArea::setGapBetweenSets( int percent )
 
 void PlotArea::setPieExplodeFactor( DataSet *dataSet, int percent )
 {
+    d->pieExplodeFactor = percent;
+
     emit pieExplodeFactorChanged( dataSet, percent );
 }
 
@@ -726,6 +835,7 @@ bool PlotArea::deregisterKdDiagram( KDChart::AbstractDiagram *diagram )
 {
     if ( !d->kdDiagrams.contains( diagram ) )
         return false;
+
     d->kdDiagrams.removeAll( diagram );
     return true;
 }
@@ -736,6 +846,7 @@ void PlotArea::update() const
     requestRepaint();
     foreach( Axis* axis, d->axes )
         axis->update();
+
     KoShape::update();
 }
 
@@ -787,8 +898,7 @@ void PlotArea::paint( QPainter& painter, const KoViewConverter& converter )
     painter.setClipRect( paintRect );
     
     // Paint the background
-    if( background() )
-    {
+    if ( background() ) {
         QPainterPath p;
         p.addRect( paintRect );
         background()->paint( painter, p );
@@ -798,18 +908,24 @@ void PlotArea::paint( QPainter& painter, const KoViewConverter& converter )
     QPointF zoomLevel;
     converter.zoom( &zoomLevel.rx(), &zoomLevel.ry() );
 
-    // Only repaint the pixmap if it is scheduled, the zoom level changed or the shape was resized
+    // Only repaint the pixmap if it is scheduled, the zoom level
+    // changed or the shape was resized.
     /*if (    d->pixmapRepaintRequested
          || d->lastZoomLevel != zoomLevel
          || d->lastSize      != size()
          || !d->paintPixmap ) {
-        // TODO: What if two zoom levels are constantly being requested?
-        // At the moment, this *is* the case, due to the fact
-        // that the shape is also rendered in the page overview
-        // in KPresenter
-        // Every time the window is hidden and shown again, a repaint is
-        // requested --> laggy performance, especially when quickly
-        // switching through windows
+        // TODO (js): What if two zoom levels are constantly being
+        //            requested?  At the moment, this *is* the case,
+        //            due to the fact that the shape is also rendered
+        //            in the page overview in KPresenter Every time
+        //            the window is hidden and shown again, a repaint
+        //            is requested --> laggy performance, especially
+        //            when quickly switching through windows.
+        //
+        // ANSWER (iw): what about having a small mapping between size
+        //              in pixels and pixmaps?  The size could be 2 or
+        //              at most 3.  We could manage the replacing
+        //              using LRU.
         paintPixmap( painter, converter );
         d->pixmapRepaintRequested = false;
         d->lastZoomLevel = zoomLevel;
