@@ -22,9 +22,9 @@
 #include "TestProxyModel.h"
 
 // KChart
-#include "ProxyModel.h"
+#include "ChartProxyModel.h"
 #include "DataSet.h"
-#include "TableModel.h"
+#include "ChartTableModel.h"
 #include "CellRegion.h"
 
 // Qt
@@ -72,7 +72,7 @@ TestProxyModel::TestProxyModel()
 
 void TestProxyModel::initTestCase()
 {
-    m_sourceModel = new TableModel;
+    m_sourceModel = new ChartTableModel;
     m_sourceModel->setRowCount( 4 );
     m_sourceModel->setColumnCount( 5 );
     
@@ -108,7 +108,7 @@ void TestProxyModel::initTestCase()
     QRect selection( QPoint( 1, 1 ),
                      QSize( m_sourceModel->columnCount(), m_sourceModel->rowCount() ) );
     
-    m_proxyModel = new ProxyModel;
+    m_proxyModel = new ChartProxyModel;
     m_proxyModel->setSourceModel( m_sourceModel, QVector<QRect>( 1, selection ) );
 }
 
