@@ -417,12 +417,12 @@ bool CalendarWeekdays::load( KoXmlElement &element, XMLLoaderObject &status ) {
     bool ok;
     int dayNo = QString(element.attribute("day","-1")).toInt(&ok);
     if (dayNo < 0 || dayNo > 6) {
-        kError()<<"Illegal weekday: "<<dayNo<<endl;
+        kError()<<"Illegal weekday: "<<dayNo;
         return true; // we continue anyway
     }
     CalendarDay *day = m_weekdays.value( dayNo + 1 );
     if ( day == 0 ) {
-        kError()<<"No weekday: "<<dayNo<<endl;
+        kError()<<"No weekday: "<<dayNo;
         return false;
     }
     if (!day->load( element, status ) )
@@ -475,7 +475,7 @@ void CalendarWeekdays::setState(int weekday, int state) {
     day->setState(state);
 }
 
-const QList<TimeInterval*> &CalendarWeekdays::intervals(int weekday) const { 
+QList<TimeInterval*> CalendarWeekdays::intervals(int weekday) const { 
     CalendarDay *day = m_weekdays.value( weekday );
     Q_ASSERT(day);
     return day->workingIntervals();
@@ -677,6 +677,14 @@ void Calendar::setTimeZone( const KTimeZone &tz )
     }
 }
 
+void Calendar::setDefault( bool on )
+{
+    m_default = on;
+    if ( m_project ) {
+        m_project->changed( this );
+    }
+}
+
 // Note: only project should do this
 void Calendar::setId(const QString& id) {
     //kDebug()<<id;
@@ -711,7 +719,7 @@ bool Calendar::load( KoXmlElement &element, XMLLoaderObject &status ) {
     KTimeZone tz = KSystemTimeZones::zone( element.attribute( "timezone" ) );
     if ( tz.isValid() ) {
         setTimeZone( tz );
-    } else kWarning()<<"No timezone specified, use default (local)"<<endl;
+    } else kWarning()<<"No timezone specified, use default (local)";
     bool m_default = (bool)element.attribute("default","0").toInt();
     if ( m_default ) {
         status.project().setDefaultCalendar( this );
@@ -731,19 +739,19 @@ bool Calendar::load( KoXmlElement &element, XMLLoaderObject &status ) {
             if ( day->load( e, status ) ) {
                 if (!day->date().isValid()) {
                     delete day;
-                    kError()<<m_name<<": Failed to load calendarDay - Invalid date"<<endl;
+                    kError()<<m_name<<": Failed to load calendarDay - Invalid date";
                 } else {
                     CalendarDay *d = findDay(day->date());
                     if (d) {
                         // already exists, keep the new
                         delete takeDay(d);
-                        kWarning()<<m_name<<" Load calendarDay - Date already exists"<<endl;
+                        kWarning()<<m_name<<" Load calendarDay - Date already exists";
                     }
                     addDay(day);
                 }
             } else {
                 delete day;
-                kError()<<"Failed to load calendarDay"<<endl;
+                kError()<<"Failed to load calendarDay";
                 return true; //false; don't throw away the whole calendar
             }
         }
@@ -896,7 +904,7 @@ Duration Calendar::effort(const QDate &date, const QTime &start, int length, Sch
         } else if (day->state() == CalendarDay::NonWorking) {
             return Duration::zeroDuration;
         } else {
-            kError()<<"Invalid state: "<<day->state()<<endl;
+            kError()<<"Invalid state: "<<day->state();
             return Duration::zeroDuration;
         }
     }
@@ -920,7 +928,7 @@ Duration Calendar::effort(const DateTime &start, const DateTime &end, Schedule *
     Duration eff;
     if (!start.isValid() || !end.isValid() || end < start) {
         if ( sch && sch->resource() ) kDebug()<<sch->resource()->name()<<sch->name()<<"Available:"<<sch->resource()->availableFrom()<<sch->resource()->availableUntil();
-        kError()<<"Illegal datetime: "<<start<<", "<<end<<endl;
+        kError()<<"Illegal datetime: "<<start<<", "<<end;
         return eff;
     }
     if ( start == end ) {
@@ -1021,8 +1029,8 @@ DateTimeInterval Calendar::firstInterval(const DateTime &start, const DateTime &
     }
     //kDebug()<<"tospec:"<<s.toString()<<" -"<<e.toString();
     // Multiple days
-    for ( QDate date = start.date(); date <= e.date(); date = date.addDays(1) ) {
-        if (date > start.date()) {
+    for ( QDate date = s.date(); date <= e.date(); date = date.addDays(1) ) {
+        if (date > s.date()) {
             startTime = QTime(0, 0, 0);
         }
         if (date < e.date()) {
@@ -1045,7 +1053,7 @@ DateTimeInterval Calendar::firstInterval(const DateTime &start, const DateTime &
             return dti;
         }
     }
-    //kWarning()<<"Didn't find an interval ("<<start<<", "<<end<<")"<<endl;
+    //kWarning()<<"Didn't find an interval ("<<start<<", "<<end<<")";
     return DateTimeInterval(DateTime(), DateTime());
 }
 
@@ -1225,6 +1233,11 @@ void StandardWorktime::changed()
     }
 }
 
+QList<double> StandardWorktime::scales() const
+{
+    return QList<double>() << year() / month() << month() / week() << week() / day() << day();
+}
+
 bool StandardWorktime::load( KoXmlElement &element, XMLLoaderObject &status ) {
     //kDebug();
     m_year = Duration::fromString(element.attribute("year"), Duration::Format_Hour); 
@@ -1241,8 +1254,8 @@ bool StandardWorktime::load( KoXmlElement &element, XMLLoaderObject &status ) {
         if (e.tagName() == "calendar") {
             // pre 0.6 version stored base calendar in standard worktime
             if ( status.version() >= "0.6" ) {
-                kWarning()<<"Old format, calendar in standard worktime"<<endl;
-                kWarning()<<"Tries to load anyway"<<endl;
+                kWarning()<<"Old format, calendar in standard worktime";
+                kWarning()<<"Tries to load anyway";
             }
             // try to load anyway
             Calendar *calendar = new Calendar;
@@ -1253,7 +1266,7 @@ bool StandardWorktime::load( KoXmlElement &element, XMLLoaderObject &status ) {
                 status.setBaseCalendar( calendar );
             } else {
                 delete calendar;
-                kError()<<"Failed to load calendar"<<endl;
+                kError()<<"Failed to load calendar";
             }
         }
     }
@@ -1275,17 +1288,18 @@ void StandardWorktime::save(QDomElement &element) const {
 void CalendarDay::printDebug(const QString& _indent) {
     QString indent = _indent;
     QString s[] = {"None", "Non-working", "Working"};
-    kDebug()<<indent<<""<<m_date.toString()<<" ="<<s[m_state];
+    qDebug()<<indent<<""<<m_date.toString()<<" ="<<s[m_state];
     if (m_state == CalendarDay::Working) {
         indent += "  ";
         foreach (TimeInterval *i, m_workingIntervals) {
-            kDebug()<<indent<<" Interval:"<<i->first<<" to"<<i->second;
+            QString end = i->endsMidnight() ? "midnight" : i->endTime().toString( Qt::ISODate );
+            qDebug()<<indent<<" Interval:"<<i->first.toString( Qt::ISODate )<<"to"<<end;
         }
     }
     
 }
 void CalendarWeekdays::printDebug(const QString& indent) {
-    kDebug()<<indent<<"Weekdays ------";
+    qDebug()<<indent<<"Weekdays ------";
     int c=1;
     foreach ( CalendarDay *d, m_weekdays ) {
         d->printDebug(indent + "  Day " + c++ + ": ");
@@ -1293,21 +1307,21 @@ void CalendarWeekdays::printDebug(const QString& indent) {
 
 }
 void Calendar::printDebug(const QString& indent) {
-    kDebug()<<indent<<"Calendar"<<m_id<<": '"<<m_name;
-    kDebug()<<indent<<"  Parent:"<<(m_parent ? m_parent->name() :"No parent");
+    qDebug()<<indent<<"Calendar"<<m_id<<":"<<m_name<<(isDefault()?" (Default calendar)":"");
+    qDebug()<<indent<<"  Parent:"<<(m_parent ? m_parent->name() :"No parent");
     m_weekdays->printDebug(indent + "  ");
-    kDebug()<<indent<<"  Days --------";
+    qDebug()<<indent<<"  Days --------";
     foreach (CalendarDay *d, m_days) {
         d->printDebug(indent + "  ");
     }
 }
 
 void StandardWorktime::printDebug(const QString& indent) {
-    kDebug()<<indent<<"StandardWorktime";
-    kDebug()<<indent<<"Year:"<<m_year.toString();
-    kDebug()<<indent<<"Month:"<<m_month.toString();
-    kDebug()<<indent<<"Week:"<<m_week.toString();
-    kDebug()<<indent<<"Day:"<<m_day.toString();
+    qDebug()<<indent<<"Estimate conversion";
+    qDebug()<<indent<<"Year:"<<m_year.toString()<<"("<<year()<<"hours )";
+    qDebug()<<indent<<"Month:"<<m_month.toString()<<"("<<month()<<"hours )";
+    qDebug()<<indent<<"Week:"<<m_week.toString()<<"("<<week()<<"hours )";
+    qDebug()<<indent<<"Day:"<<m_day.toString()<<"("<<day()<<"hours )";
 }
 
 #endif
