@@ -93,6 +93,8 @@ public:
 
     virtual void execute();
     virtual void unexecute();
+
+    bool isEmpty() const { return cmds.isEmpty(); }
     
 protected:
     QList<QUndoCommand*> cmds;
@@ -646,6 +648,32 @@ private:
 
 };
 
+class KPLATOKERNEL_EXPORT ModifyResourceRequestUnitsCmd : public NamedCommand
+{
+public:
+    ModifyResourceRequestUnitsCmd( ResourceRequest *request, int oldvalue, int newvalue, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    ResourceRequest *m_request;
+    int m_oldvalue, m_newvalue;
+
+};
+
+class KPLATOKERNEL_EXPORT ModifyResourceGroupRequestUnitsCmd : public NamedCommand
+{
+public:
+    ModifyResourceGroupRequestUnitsCmd( ResourceGroupRequest *request, int oldvalue, int newvalue, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    ResourceGroupRequest *m_request;
+    int m_oldvalue, m_newvalue;
+
+};
+
 class KPLATOKERNEL_EXPORT ModifyEstimateCmd : public NamedCommand
 {
 public:
@@ -1488,6 +1516,17 @@ private:
     bool oldvalue, newvalue;
 };
 
+class KPLATOKERNEL_EXPORT ModifyScheduleManagerSchedulerCmd : public NamedCommand
+{
+public:
+    ModifyScheduleManagerSchedulerCmd( ScheduleManager &sm, int value, const QString& name = 0 );
+    void execute();
+    void unexecute();
+
+private:
+    ScheduleManager &m_sm;
+    int oldvalue, newvalue;
+};
 
 class KPLATOKERNEL_EXPORT ModifyStandardWorktimeYearCmd : public NamedCommand
 {
@@ -1622,6 +1661,127 @@ private:
     WBSDefinition m_newvalue, m_oldvalue;
 };
 
+class KPLATOKERNEL_EXPORT InsertProjectCmd : public MacroCommand
+{
+public:
+    InsertProjectCmd( Project &project, Node *parent, Node *after, const QString& name = QString() );
+
+    void execute();
+    void unexecute();
+
+protected:
+    void addCalendars( Calendar *calendar, Calendar *parent, QMap<Calendar*, Calendar*> &map );
+    void addChildNodes( Node *node );
+
+private:
+    Project *m_project;
+    Node *m_parent;
+    Node *m_after;
+
+};
+
+class KPLATOKERNEL_EXPORT WorkPackageAddCmd : public NamedCommand
+{
+public:
+    WorkPackageAddCmd( Project *project, Node *node, WorkPackage *wp, const QString& name = QString() );
+    ~WorkPackageAddCmd();
+    void execute();
+    void unexecute();
+private:
+    Project *m_project;
+    Node *m_node;
+    WorkPackage *m_wp;
+    bool m_mine;
+
+};
+
+class KPLATOKERNEL_EXPORT ModifyProjectLocaleCmd : public MacroCommand
+{
+public:
+    ModifyProjectLocaleCmd( Project &project, const QString &name );
+    void execute();
+    void unexecute();
+private:
+    Project &m_project;
+};
+
+class KPLATOKERNEL_EXPORT ModifyCurrencySymolCmd : public NamedCommand
+{
+public:
+    ModifyCurrencySymolCmd(  KLocale *locale, const QString &value, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    KLocale *m_locale;
+    QString m_newvalue;
+    QString m_oldvalue;
+};
+
+class  KPLATOKERNEL_EXPORT ModifyCurrencyFractionalDigitsCmd : public NamedCommand
+{
+public:
+    ModifyCurrencyFractionalDigitsCmd(  KLocale *locale, int value, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    KLocale *m_locale;
+    int m_newvalue;
+    int m_oldvalue;
+};
+
+class  KPLATOKERNEL_EXPORT ModifyPositivePrefixCurrencySymolCmd : public NamedCommand
+{
+public:
+    ModifyPositivePrefixCurrencySymolCmd(  KLocale *locale, bool value, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    KLocale *m_locale;
+    bool m_newvalue;
+    bool m_oldvalue;
+};
+
+class  KPLATOKERNEL_EXPORT ModifyNegativePrefixCurrencySymolCmd : public NamedCommand
+{
+public:
+    ModifyNegativePrefixCurrencySymolCmd(  KLocale *locale, bool value, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    KLocale *m_locale;
+    bool m_newvalue;
+    bool m_oldvalue;
+};
+
+class  KPLATOKERNEL_EXPORT ModifyPositiveMonetarySignPositionCmd : public NamedCommand
+{
+public:
+    ModifyPositiveMonetarySignPositionCmd( KLocale *locale, int value, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    KLocale *m_locale;
+    int m_newvalue;
+    int m_oldvalue;
+};
+
+class  KPLATOKERNEL_EXPORT ModifyNegativeMonetarySignPositionCmd : public NamedCommand
+{
+public:
+    ModifyNegativeMonetarySignPositionCmd( KLocale *locale, int value, const QString& name = QString() );
+    void execute();
+    void unexecute();
+
+private:
+    KLocale *m_locale;
+    int m_newvalue;
+    int m_oldvalue;
+};
 
 }  //KPlato namespace
 
