@@ -155,7 +155,7 @@ Node *Node::projectNode() {
     if (m_parent)
         return m_parent->projectNode();
 
-    kError()<<"Ooops, no parent and no project found"<<endl;
+    kError()<<"Ooops, no parent and no project found";
     return 0;
 }
 
@@ -166,7 +166,7 @@ const Node *Node::projectNode() const {
     if (m_parent)
         return m_parent->projectNode();
 
-    kError()<<"Ooops, no parent and no project found"<<endl;
+    kError()<<"Ooops, no parent and no project found";
     return 0;
 }
 
@@ -852,7 +852,7 @@ bool Node::isStartNode() const {
 bool Node::setId(const QString& id) {
     //kDebug()<<id;
     if (id.isEmpty()) {
-        kError()<<"id is empty"<<endl;
+        kError()<<"id is empty";
         m_id = id;
         return false;
     }
@@ -863,11 +863,11 @@ bool Node::setId(const QString& id) {
             removeId();
         } else if (n) {
             //Hmmm, shouldn't happen
-            kError()<<"My id '"<<m_id<<"' already used for different node: "<<n->name()<<endl;
+            kError()<<"My id '"<<m_id<<"' already used for different node: "<<n->name();
         }
     }
     if (findNode(id)) {
-        kError()<<"id '"<<id<<"' is already used for different node: "<<findNode(id)->name()<<endl;
+        kError()<<"id '"<<id<<"' is already used for different node: "<<findNode(id)->name();
         m_id = QString(); // hmmm
         return false;
     }
@@ -1046,7 +1046,7 @@ Schedule *Node::findSchedule(const Schedule::Type type) {
 void Node::setScheduleDeleted(long id, bool on) {
     Schedule *ns = findSchedule(id);
     if (ns == 0) {
-        kError()<<m_name<<" Could not find schedule with id="<<id<<endl;
+        kError()<<m_name<<" Could not find schedule with id="<<id;
     } else {
         ns->setDeleted(on);
     }
@@ -1368,7 +1368,12 @@ bool Estimate::load(KoXmlElement &element, XMLLoaderObject &status) {
         m_optimisticEstimate = scale(  Duration::fromString(element.attribute("optimistic")), m_unit, s );
         m_pessimisticEstimate = scale( Duration::fromString(element.attribute("pessimistic")), m_unit, s );
     } else {
-        m_unit = (Duration::Unit)(element.attribute("unit", QString().number(Duration::Unit_ms) ).toInt());
+        if ( status.version() <= "0.6.2" ) {
+            // 0 pos in unit is now Unit_Y, so add 3 to get the correct new unit
+            m_unit = (Duration::Unit)(element.attribute("unit", QString().number(Duration::Unit_ms - 3) ).toInt() + 3);
+        } else {
+            m_unit = Duration::unitFromString( element.attribute( "unit" ) );
+        }
         m_expectedEstimate = element.attribute("expected", "0.0").toDouble();
         m_optimisticEstimate = element.attribute("optimistic", "0.0").toDouble();
         m_pessimisticEstimate = element.attribute("pessimistic", "0.0").toDouble();
@@ -1389,7 +1394,7 @@ void Estimate::save(QDomElement &element) const {
         me.setAttribute("calendar-id", m_calendar->id() );
     }
     me.setAttribute("risk", risktypeToString());
-    me.setAttribute("unit", m_unit);
+    me.setAttribute("unit", Duration::unitToString( m_unit ) );
 }
 
 QString Estimate::typeToString( bool trans ) const {
@@ -1399,8 +1404,7 @@ QString Estimate::typeToString( bool trans ) const {
 QStringList Estimate::typeToStringList( bool trans ) {
     return QStringList() 
             << (trans ? i18n("Effort") : QString("Effort"))
-            << (trans ? i18n("FixedDuration") : QString("FixedDuration"))
-            << (trans ? i18n("Length") : QString("Length"));
+            << (trans ? i18n("Duration") : QString("Duration"));
 }
 
 void Estimate::setType(Type type)
@@ -1415,12 +1419,12 @@ void Estimate::setType(Type type)
 void Estimate::setType(const QString& type) {
     if (type == "Effort")
         setType(Type_Effort);
-    else if (type == "FixedDuration")
-        setType(Type_FixedDuration);
-    else if (type == "Length")
-        setType(Type_Length);
+    else if (type == "Duration" || /*old format*/ type == "FixedDuration")
+        setType(Type_Duration);
+    else if (/*old format*/type == "Length")
+        setType(Type_Duration);
     else if (type == "Type_FixedDuration") // Typo, keep old xml files working
-        setType(Type_FixedDuration);
+        setType(Type_Duration);
     else
         setType(Type_Effort); // default
 }
@@ -1445,6 +1449,15 @@ void Estimate::setRisktype(const QString& type) {
         setRisktype(Risk_None); // default
 }
 
+void Estimate::setCalendar( Calendar *calendar )
+{
+    m_calendar = calendar;
+    m_expectedCached = false;
+    m_optimisticCached = false;
+    m_pessimisticCached = false;
+    changed();
+}
+
 void Estimate::setExpectedEstimate( double value)
 {
     m_expectedEstimate = value;
@@ -1543,57 +1556,83 @@ Duration Estimate::expectedValue() const
 
 double Estimate::scale( const Duration &value, Duration::Unit unit, const QList<double> &scales )
 {
+    //kDebug()<<value.toDouble( unit )<<","<<unit<<scales;
     QList<double> lst = scales;
     switch ( lst.count() ) {
         case 0:
-            lst << 24.0; // add hours in day
+            lst << 365.0 / 30; // add months in a year
         case 1:
-            lst << 60.0; // add minutes in hour
+            lst << 30.0 / 7.0; // add weeks in a month
         case 2:
-            lst << 60.0; // add seconds in minute
+            lst << 7.0; // add days in a week
         case 3:
+            lst << 24.0; // add hours in day
+        case 4:
+            lst << 60.0; // add minutes in hour
+        case 5:
+            lst << 60.0; // add seconds in minute
+        case 6:
             lst << 1000.0; // add milliseconds in second
         default:
             break;
     }
     double v = (double)value.milliseconds();
     if (unit == Duration::Unit_ms) return v;
-    v /= lst[3];
+    v /= lst[6];
     if (unit == Duration::Unit_s) return v;
-    v /= lst[2];
+    v /= lst[5];
     if (unit == Duration::Unit_m) return v;
-    v /= lst[1];
+    v /= lst[4];
     if (unit == Duration::Unit_h) return v;
-    v /= lst[0];
+    v /= lst[3];
+    if (unit == Duration::Unit_d) return v;
+    v /= lst[2];
+    if (unit == Duration::Unit_w) return v;
+    v /= lst[1];
+    if (unit == Duration::Unit_M) return v;
+    v /= lst[0]; // Year
     //kDebug()<<value.toString()<<","<<unit<<"="<<v;
     return v;
 }
 
 Duration Estimate::scale( double value, Duration::Unit unit, const QList<double> &scales )
 {
+    //kDebug()<<value<<","<<unit<<scales;
     QList<double> lst = scales;
     switch ( lst.count() ) {
-        case 0:
+        case Duration::Unit_Y:
+            lst << 365.0 / 30.0; // add months in a year
+        case Duration::Unit_M:
+            lst << 30.0 / 7.0; // add weeks in a month
+        case Duration::Unit_w:
+            lst << 7.0; // add days in a week
+        case Duration::Unit_d:
             lst << 24.0; // add hours in day
-        case 1:
+        case Duration::Unit_h:
             lst << 60.0; // add minutes in hour
-        case 2:
+        case Duration::Unit_m:
             lst << 60.0; // add seconds in minute
-        case 3:
+        case Duration::Unit_s:
             lst << 1000.0; // add milliseconds in second
         default:
             break;
     }
     double v = value;
     switch ( unit ) {
-        case Duration::Unit_d:
+        case Duration::Unit_Y:
             v *= lst[0];
-        case Duration::Unit_h:
+        case Duration::Unit_M:
             v *= lst[1];
-        case Duration::Unit_m:
+        case Duration::Unit_w:
             v *= lst[2];
-        case Duration::Unit_s:
+        case Duration::Unit_d:
             v *= lst[3];
+        case Duration::Unit_h:
+            v *= lst[4];
+        case Duration::Unit_m:
+            v *= lst[5];
+        case Duration::Unit_s:
+            v *= lst[6];
         case Duration::Unit_ms:
             break; // nothing
     }
@@ -1604,7 +1643,7 @@ Duration Estimate::scale( double value, Duration::Unit unit, const QList<double>
 QList<double> Estimate::scales() const
 {
     QList<double> s;
-    if ( m_type == Type_FixedDuration ) {
+    if ( m_type == Type_Duration && m_calendar == 0 ) {
         return s; // Use default scaling ( 24h a day...)
     }
     if ( m_parent == 0 ) {
@@ -1614,7 +1653,7 @@ QList<double> Estimate::scales() const
     if ( p == 0 ) {
         return s;
     }
-    s << p->standardWorktime()->day();
+    s << p->standardWorktime()->scales();
     return s;
 }
 
@@ -1622,38 +1661,37 @@ QList<double> Estimate::scales() const
 #ifndef NDEBUG
 void Node::printDebug(bool children, const QByteArray& _indent) {
     QByteArray indent = _indent;
-    kDebug()<<indent<<"  Unique node identity="<<m_id;
     if ( m_estimate ) m_estimate->printDebug(indent);
     QString s = "  Constraint: " + constraintToString();
     if (m_constraint == MustStartOn || m_constraint == StartNotEarlier || m_constraint == FixedInterval)
-        kDebug()<<indent<<s<<" ("<<constraintStartTime().toString()<<")";
+        qDebug()<<indent<<s<<" ("<<constraintStartTime().toString()<<")";
     if (m_constraint == MustFinishOn || m_constraint == FinishNotLater || m_constraint == FixedInterval)
-        kDebug()<<indent<<s<<" ("<<constraintEndTime().toString()<<")";
+        qDebug()<<indent<<s<<" ("<<constraintEndTime().toString()<<")";
     Schedule *cs = m_currentSchedule; 
     if (cs) {
-        kDebug()<<indent<<"  Current schedule:"<<"id="<<cs->id()<<" '"<<cs->name()<<"' type:"<<cs->type();
+        qDebug()<<indent<<"  Current schedule:"<<"id="<<cs->id()<<" '"<<cs->name()<<"' type:"<<cs->type();
     } else {
-        kDebug()<<indent<<"  Current schedule: None";
+        qDebug()<<indent<<"  Current schedule: None";
     }
     foreach (Schedule *sch, m_schedules) {
         sch->printDebug(indent+"  ");
     }
-    kDebug()<<indent<<"  Parent:"<<(m_parent ? m_parent->name() : QString("None"));
-    kDebug()<<indent<<"  Level:"<<level();
-    kDebug()<<indent<<"  No of predecessors:"<<m_dependParentNodes.count();
+    qDebug()<<indent<<"  Parent:"<<(m_parent ? m_parent->name() : QString("None"));
+    qDebug()<<indent<<"  Level:"<<level();
+    qDebug()<<indent<<"  No of predecessors:"<<m_dependParentNodes.count();
     QListIterator<Relation*> pit(m_dependParentNodes);
-    //kDebug()<<indent<<"  Dependent parents="<<pit.count();
+    //qDebug()<<indent<<"  Dependent parents="<<pit.count();
     while (pit.hasNext()) {
         pit.next()->printDebug(indent);
     }
-    kDebug()<<indent<<"  No of successors:"<<m_dependChildNodes.count();
+    qDebug()<<indent<<"  No of successors:"<<m_dependChildNodes.count();
     QListIterator<Relation*> cit(m_dependChildNodes);
-    //kDebug()<<indent<<"  Dependent children="<<cit.count();
+    //qDebug()<<indent<<"  Dependent children="<<cit.count();
     while (cit.hasNext()) {
         cit.next()->printDebug(indent);
     }
 
-    //kDebug()<<indent;
+    //qDebug()<<indent;
     indent += "  ";
     if (children) {
         QListIterator<Node*> it(m_nodes);
@@ -1669,18 +1707,18 @@ void Node::printDebug(bool children, const QByteArray& _indent) {
 #ifndef NDEBUG
 void Estimate::printDebug(const QByteArray& _indent) {
     QByteArray indent = _indent;
-    kDebug()<<indent<<"  Estimate:";
+    qDebug()<<indent<<"  Estimate:";
     indent += "  ";
-    kDebug()<<indent<<"  Expected:"<<m_expectedEstimate<<Duration::unitToString(m_unit);
-    kDebug()<<indent<<"  Optimistic:"<<m_optimisticEstimate<<Duration::unitToString(m_unit);
-    kDebug()<<indent<<"  Pessimistic:"<<m_pessimisticEstimate<<Duration::unitToString(m_unit);
+    qDebug()<<indent<<"  Expected:"<<m_expectedEstimate<<Duration::unitToString(m_unit);
+    qDebug()<<indent<<"  Optimistic:"<<m_optimisticEstimate<<Duration::unitToString(m_unit);
+    qDebug()<<indent<<"  Pessimistic:"<<m_pessimisticEstimate<<Duration::unitToString(m_unit);
     
-    kDebug()<<indent<<"  Risk:"<<risktypeToString();
-    kDebug()<<indent<<"  Pert expected:      "<<pertExpected().toString()<<endl;
-    kDebug()<<indent<<"  Pert optimistic:    "<<pertOptimistic().toString()<<endl;
-    kDebug()<<indent<<"  Pert pessimistic:   "<<pertPessimistic().toString()<<endl;
-    kDebug()<<indent<<"  Pert variance:      "<<variance()<<endl;
-    kDebug()<<indent<<"  Pert std deviation: "<<deviation()<<endl;
+    qDebug()<<indent<<"  Risk:"<<risktypeToString();
+    qDebug()<<indent<<"  Pert expected:      "<<pertExpected().toString();
+    qDebug()<<indent<<"  Pert optimistic:    "<<pertOptimistic().toString();
+    qDebug()<<indent<<"  Pert pessimistic:   "<<pertPessimistic().toString();
+    qDebug()<<indent<<"  Pert variance:      "<<variance();
+    qDebug()<<indent<<"  Pert std deviation: "<<deviation();
 }
 #endif
 
