@@ -29,6 +29,7 @@
 #include "kptdatetime.h"
 #include "kptschedule.h"
 #include "kptdocuments.h"
+#include "kptresource.h"
 
 #include <QObject>
 #include <QHash>
@@ -245,7 +246,7 @@ public:
     virtual DateTime mustStartOn() const { return m_constraintStartTime; }
     virtual DateTime mustFinishOn() const { return m_constraintEndTime; }
 
-    virtual ResourceGroupRequest *resourceGroupRequest(ResourceGroup * /*group*/) const { return 0; }
+    virtual ResourceGroupRequest *resourceGroupRequest(const ResourceGroup * /*group*/) const { return 0; }
     virtual QStringList requestNameList() const { return QStringList(); }
     virtual bool containsRequest( const QString &/*identity*/ ) const { return false; }
     virtual ResourceRequest *resourceRequest( const QString &/*name*/ ) const { return 0; }
@@ -485,7 +486,8 @@ public:
     /// Set parent schedule recursivly
     virtual void setParentSchedule(Schedule *sch);
     
-    virtual ResourceRequestCollection *requests() const { return 0; }
+    const ResourceRequestCollection &requests() const { return m_requests; }
+    ResourceRequestCollection &requests() { return m_requests; }
     
     virtual uint state( long ) const { return State_None; }
     
@@ -634,6 +636,8 @@ protected:
     
     Documents m_documents;
     
+    ResourceRequestCollection m_requests;
+
 private:
     void init();
         
@@ -670,8 +674,7 @@ public:
     /// Type defines the types of estimates
     enum Type {
         Type_Effort,        /// Changing amount of resources changes the task duration
-        Type_FixedDuration, /// Changing amount of resources will not change the tasks duration
-        Type_Length         /// Duration in work time defined by m_calendar
+        Type_Duration       /// Changing amount of resources will not change the tasks duration
     };
     /// Return the node that owns this Estimate
     Node *parentNode() const { return m_parent; }
@@ -689,10 +692,10 @@ public:
     /// Return a stringlist of all estimate types. Translated if @p trans = true.
     static QStringList typeToStringList( bool trans=false );
     
-    /// Return the calendar used when Type is Length
+    /// Return the calendar used when Type is Duration
     Calendar *calendar() const { return m_calendar; }
-    /// Set the calendar to be used when Type is Length
-    void setCalendar( Calendar *calendar ) { m_calendar = calendar; changed(); }
+    /// Set the calendar to be used when Type is Duration
+    void setCalendar( Calendar *calendar );
     
     enum Risktype { Risk_None, Risk_Low, Risk_High };
     Risktype risktype() const { return m_risktype; }
@@ -830,7 +833,7 @@ private:
     Type m_type;
     Risktype m_risktype;
     
-    /// Calendar used when Type is Type_Length
+    /// Calendar may be used when Type is Type_Duration
     Calendar *m_calendar;
     
 #ifndef NDEBUG
