@@ -28,6 +28,7 @@
 #include "kptschedule.h"
 #include "kptwbsdefinition.h"
 #include "kptxmlloaderobject.h"
+#include "kptschedulerplugin.h"
 
 #include <KoXmlReader.h>
 
@@ -46,21 +47,34 @@
 namespace KPlato
 {
 
-
 Project::Project( Node *parent )
         : Node( parent ),
         m_accounts( *this ),
         m_defaultCalendar( 0 ),
-        m_taskDefaults( new Task() )
+        emptyConfig( new ConfigBase() ),
+        m_config( emptyConfig ),
+        m_schedulerPlugins()
+{
+    //kDebug()<<"("<<this<<")";
+    init();
+}
+
+Project::Project( ConfigBase &config, Node *parent )
+        : Node( parent ),
+        m_accounts( *this ),
+        m_defaultCalendar( 0 ),
+        emptyConfig( 0 ),
+        m_config( &config ),
+        m_schedulerPlugins()
 {
     //kDebug()<<"("<<this<<")";
-    m_constraint = Node::MustStartOn;
-    m_standardWorktime = new StandardWorktime();
     init();
 }
 
 void Project::init()
 {
+    m_constraint = Node::MustStartOn;
+    m_standardWorktime = new StandardWorktime();
     m_spec = KDateTime::Spec::LocalZone();
     if ( !m_spec.timeZone().isValid() ) {
         m_spec.setType( KTimeZone() );
@@ -85,7 +99,8 @@ Project::~Project()
     while ( !m_managers.isEmpty() )
         delete m_managers.takeFirst();
     
-    delete m_taskDefaults;
+    m_config = 0; //not mine, don't delete
+    delete emptyConfig;
 }
 
 int Project::type() const { return Node::Type_Project; }
@@ -124,7 +139,7 @@ void Project::generateUniqueIds()
 void Project::calculate( Schedule *schedule, const DateTime &dt )
 {
     if ( schedule == 0 ) {
-        kError() << "Schedule == 0, cannot calculate" << endl;
+        kError() << "Schedule == 0, cannot calculate";
         return ;
     }
     m_currentSchedule = schedule;
@@ -134,7 +149,7 @@ void Project::calculate( Schedule *schedule, const DateTime &dt )
 void Project::calculate( const DateTime &dt )
 {
     if ( m_currentSchedule == 0 ) {
-        kError() << "No current schedule to calculate" << endl;
+        kError() << "No current schedule to calculate";
         return ;
     }
     KLocale *locale = KGlobal::locale();
@@ -176,9 +191,9 @@ void Project::calculate( const DateTime &dt )
         emit scheduleChanged( cs );
         emit changed();
     } else if ( type() == Type_Subproject ) {
-        kWarning() << "Subprojects not implemented" << endl;
+        kWarning() << "Subprojects not implemented";
     } else {
-        kError() << "Illegal project type: " << type() << endl;
+        kError() << "Illegal project type: " << type();
     }
 }
 
@@ -198,13 +213,12 @@ void Project::calculate( ScheduleManager &sm )
         incProgress();
         sm.setCalculateAll( false );
         sm.createSchedules();
-        sm.setRecalculateFrom( KDateTime::currentLocalDateTime() );
         if ( sm.parentManager() ) {
             sm.expected()->startTime = sm.parentManager()->expected()->startTime;
             sm.expected()->earlyStart = sm.parentManager()->expected()->earlyStart;
         }
         incProgress();
-        calculate( sm.expected(), KDateTime::currentLocalDateTime() );
+        calculate( sm.expected(), sm.recalculateFrom() );
     } else {
         if ( sm.optimistic() ) {
             maxprogress += nodes * 3;
@@ -225,15 +239,15 @@ void Project::calculate( ScheduleManager &sm )
         setCurrentSchedule( sm.expected()->id() );
     }
     emit sigProgress( maxprogress );
-    emit projectCalculated( &sm );
     emit scheduleManagerChanged( &sm );
+    emit projectCalculated( &sm );
     emit changed();
 }
 
 void Project::calculate( Schedule *schedule )
 {
     if ( schedule == 0 ) {
-        kError() << "Schedule == 0, cannot calculate" << endl;
+        kError() << "Schedule == 0, cannot calculate";
         return ;
     }
     m_currentSchedule = schedule;
@@ -333,9 +347,9 @@ void Project::calculate()
         emit scheduleChanged( cs );
         emit changed();
     } else if ( type() == Type_Subproject ) {
-        kWarning() << "Subprojects not implemented" << endl;
+        kWarning() << "Subprojects not implemented";
     } else {
-        kError() << "Illegal project type: " << type() << endl;
+        kError() << "Illegal project type: " << type();
     }
 }
 
@@ -369,7 +383,7 @@ bool Project::calcCriticalPath( bool fromEnd )
 
 void Project::calcCriticalPathList( MainSchedule *cs )
 {
-    //kDebug()<<m_name<<", "<<cs->name()<<endl;
+    //kDebug()<<m_name<<", "<<cs->name();
     cs->clearCriticalPathList();
     foreach ( Node *n, allNodes() ) {
         if ( n->numDependParentNodes() == 0 && n->inCriticalPath( cs->id() ) ) {
@@ -379,19 +393,19 @@ void Project::calcCriticalPathList( MainSchedule *cs )
         }
     }
     cs->criticalPathListCached = true;
-    //kDebug()<<*(criticalPathList( cs->id() ))<<endl;
+    //kDebug()<<*(criticalPathList( cs->id() ));
 }
 
 void Project::calcCriticalPathList( MainSchedule *cs, Node *node )
 {
-    //kDebug()<<node->name()<<", "<<cs->id()<<endl;
+    //kDebug()<<node->name()<<", "<<cs->id();
     bool newPath = false;
     QList<Node*> lst = *( cs->currentCriticalPath() );
     foreach ( Relation *r, node->dependChildNodes() ) {
         if ( r->child()->inCriticalPath( cs->id() ) ) {
             if ( newPath ) {
                 cs->addCriticalPath( &lst );
-                //kDebug()<<node->name()<<" new path"<<endl;
+                //kDebug()<<node->name()<<" new path";
             }
             cs->addCriticalPathNode( r->child() );
             calcCriticalPathList( cs, r->child() );
@@ -404,7 +418,7 @@ const QList< QList<Node*> > *Project::criticalPathList( long id )
 {
     Schedule *s = schedule( id );
     if ( s == 0 ) {
-        //kDebug()<<"No schedule with id="<<id<<endl;
+        //kDebug()<<"No schedule with id="<<id;
         return 0;
     }
     MainSchedule *ms = static_cast<MainSchedule*>( s );
@@ -419,7 +433,7 @@ QList<Node*> Project::criticalPath( long id, int index )
 {
     Schedule *s = schedule( id );
     if ( s == 0 ) {
-        //kDebug()<<"No schedule with id="<<id<<endl;
+        //kDebug()<<"No schedule with id="<<id;
         return QList<Node*>();
     }
     MainSchedule *ms = static_cast<MainSchedule*>( s );
@@ -456,7 +470,7 @@ Duration *Project::getRandomDuration()
 DateTime Project::checkStartConstraints( const DateTime &dt ) const
 {
     DateTime t = dt;
-    foreach ( Node *n, nodeIdDict.values() ) {
+    foreach ( Node *n, nodeIdDict ) {
         if ( n->type() == Node::Type_Task || n->type() == Node::Type_Milestone ) {
             switch ( n->constraint() ) {
                 case Node::FixedInterval:
@@ -474,7 +488,7 @@ DateTime Project::checkStartConstraints( const DateTime &dt ) const
 DateTime Project::checkEndConstraints( const DateTime &dt ) const
 {
     DateTime t = dt;
-    foreach ( Node *n, nodeIdDict.values() ) {
+    foreach ( Node *n, nodeIdDict ) {
         if ( n->type() == Node::Type_Task || n->type() == Node::Type_Milestone ) {
             switch ( n->constraint() ) {
                 case Node::FixedInterval:
@@ -629,12 +643,41 @@ void Project::initiateCalculationLists( MainSchedule &sch )
 bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
 {
     //kDebug()<<"--->";
+    // load locale first
+    KoXmlNode n = element.firstChild();
+    for ( ; ! n.isNull(); n = n.nextSibling() ) {
+        if ( ! n.isElement() ) {
+            continue;
+        }
+        KoXmlElement e = n.toElement();
+        if ( e.tagName() == "locale" ) {
+            KLocale *l = locale();
+            l->setCurrencySymbol( e.attribute( "currency-symbol", l->currencySymbol() ) );
+
+//            l->setMonetaryDecimalSymbol( e.attribute( "monetary-decimal-symbol", l->monetaryDecimalSymbol() ) );
+
+//            l->setMonetaryThousandsSeparator( e.attribute( "monetary-thousands-separator", l->monetaryThousandsSeparator() ) );
+            if ( e.hasAttribute( "positive-monetary-sign-position" ) ) {
+                l->setPositiveMonetarySignPosition( (KLocale::SignPosition)( e.attribute( "positive-monetary-sign-position" ).toInt() ) );
+            }
+            if ( e.hasAttribute( "positive-prefix-currency-symbol" ) ) {
+                l->setPositivePrefixCurrencySymbol( e.attribute( "positive-prefix-currency-symbol" ).toInt() );
+            }
+            if ( e.hasAttribute( "negative-monetary-sign-position" ) ) {
+                l->setNegativeMonetarySignPosition( (KLocale::SignPosition)( e.attribute( "negative-monetary-sign-position" ).toInt() ) );
+            }
+            if ( e.hasAttribute( "negative-prefix-currency-symbol" ) ) {
+                l->setNegativePrefixCurrencySymbol( e.attribute( "negative-prefix-currency-symbol" ).toInt() );
+            }
+            //qDebug()<<"project load:"<<l->currencySymbol();
+        }
+    }
     QList<Calendar*> cals;
     QString s;
     bool ok = false;
     QString id = element.attribute( "id" );
     if ( !setId( id ) ) {
-        kWarning() << "Id must be unique: " << id << endl;
+        kWarning() << "Id must be unique: " << id;
     }
     m_name = element.attribute( "name" );
     m_leader = element.attribute( "leader" );
@@ -642,7 +685,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
     KTimeZone tz = KSystemTimeZones::zone( element.attribute( "timezone" ) );
     if ( tz.isValid() ) {
         m_spec = KDateTime::Spec( tz );
-    } else kWarning()<<"No timezone specified, using default (local)"<<endl;
+    } else kWarning()<<"No timezone specified, using default (local)";
     status.setProjectSpec( m_spec );
     
     // Allow for both numeric and text
@@ -652,7 +695,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
         setConstraint( s );
     if ( m_constraint != Node::MustStartOn &&
             m_constraint != Node::MustFinishOn ) {
-        kError() << "Illegal constraint: " << constraintToString() << endl;
+        kError() << "Illegal constraint: " << constraintToString();
         setConstraint( Node::MustStartOn );
     }
     s = element.attribute( "start-time" );
@@ -665,7 +708,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
     // Load the project children
     // Do calendars first, they only refrence other calendars
     //kDebug()<<"Calendars--->";
-    KoXmlNode n = element.firstChild();
+    n = element.firstChild();
     for ( ; ! n.isNull(); n = n.nextSibling() ) {
         if ( ! n.isElement() ) {
             continue;
@@ -680,7 +723,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
                 cals.append( child ); // temporary, reorder later
             } else {
                 // TODO: Complain about this
-                kError() << "Failed to load calendar" << endl;
+                kError() << "Failed to load calendar";
                 delete child;
             }
         } else if ( e.tagName() == "standard-worktime" ) {
@@ -689,7 +732,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
             if ( child->load( e, status ) ) {
                 setStandardWorktime( child );
             } else {
-                kError() << "Failed to load standard worktime" << endl;
+                kError() << "Failed to load standard worktime";
                 delete child;
             }
         }
@@ -720,7 +763,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
         cals = lst;
     } while ( added );
     if ( ! cals.isEmpty() ) {
-        kError()<<"All calendars not saved!"<<endl;
+        kError()<<"All calendars not saved!";
     }
     //kDebug()<<"Calendars<---";
     // Resource groups and resources, can reference calendars
@@ -788,7 +831,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
             // Load accounts
             // References tasks
             if ( !m_accounts.load( e, *this ) ) {
-                kError() << "Failed to load accounts" << endl;
+                kError() << "Failed to load accounts";
             }
         } else if ( e.tagName() == "relation" ) {
             //kDebug()<<"Relation--->";
@@ -797,7 +840,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
             Relation * child = new Relation();
             if ( !child->load( e, *this ) ) {
                 // TODO: Complain about this
-                kError() << "Failed to load relation" << endl;
+                kError() << "Failed to load relation";
                 delete child;
             }
             //kDebug()<<"Relation<---";
@@ -830,7 +873,7 @@ bool Project::load( KoXmlElement &element, XMLLoaderObject &status )
                         if ( add )
                             addScheduleManager( sm );
                     } else {
-                        kError() << "Failed to load schedule manager" << endl;
+                        kError() << "Failed to load schedule manager";
                         delete sm;
                     }
                 } else {
@@ -865,6 +908,17 @@ void Project::save( QDomElement &element ) const
 
     m_wbsDefinition.saveXML( me );
     
+    QDomElement loc = me.ownerDocument().createElement( "locale" );
+    me.appendChild( loc );
+    const KLocale *l = locale();
+    loc.setAttribute( "currency-symbol", l->currencySymbol() );
+//    loc.setAttribute( "monetary-decimal-symbol", l->monetaryDecimalSymbol() );
+//    loc.setAttribute( "monetary-thousands-separator", l->monetaryThousandsSeparator() );
+    loc.setAttribute( "positive-monetary-sign-position", l->positiveMonetarySignPosition() );
+    loc.setAttribute( "positive-prefix-currency-symbol", l->positivePrefixCurrencySymbol() );
+    loc.setAttribute( "negative-monetary-sign-position", l->negativeMonetarySignPosition() );
+    loc.setAttribute( "negative-prefix-currency-symbol", l->negativePrefixCurrencySymbol() );
+    
     m_accounts.save( me );
 
     // save calendars
@@ -921,17 +975,6 @@ void Project::saveWorkPackageXML( QDomElement &element, const Node *node, long i
     me.setAttribute( "start-time", m_constraintStartTime.toString( KDateTime::ISODate ) );
     me.setAttribute( "end-time", m_constraintEndTime.toString( KDateTime::ISODate ) );
 
-//    m_accounts.save( me );
-
-    // save calendars
-/*    foreach ( Calendar *c, calendarIdDict.values() ) {
-        c->save( me );
-    }*/
-    // save standard worktime
-//     if ( m_standardWorktime )
-//         m_standardWorktime->save( me );
-
-    // save project resources, must be after calendars
     QListIterator<ResourceGroup*> git( m_resourceGroups );
     while ( git.hasNext() ) {
         git.next() ->saveWorkPackageXML( me, node->assignedResources( id ) );
@@ -1077,7 +1120,7 @@ bool Project::addSubTask( Node* task, int index, Node* parent, bool emitSignal )
         p = this;
     }
     if ( !registerNodeId( task ) ) {
-        kError() << "Failed to register node id, can not add subtask: " << task->name() << endl;
+        kError() << "Failed to register node id, can not add subtask: " << task->name();
         return false;
     }
     int i = index == -1 ? p->numChildren() : index;
@@ -1168,7 +1211,7 @@ bool Project::canIndentTask( Node* node )
         return false;
     }
     if ( parentNode->findChildNode( node ) == -1 ) {
-        kError() << "Tasknot found???" << endl;
+        kError() << "Tasknot found???";
         return false;
     }
     Node *sib = node->siblingBefore();
@@ -1219,7 +1262,7 @@ bool Project::canUnindentTask( Node* node )
     }
     int index = parentNode->findChildNode( node );
     if ( -1 == index ) {
-        kError() << "Tasknot found???" << endl;
+        kError() << "Tasknot found???";
         return false;
     }
     return true;
@@ -1252,7 +1295,7 @@ bool Project::canMoveTaskUp( Node* node )
         return false;
     }
     if ( parentNode->findChildNode( node ) == -1 ) {
-        kError() << "Tasknot found???" << endl;
+        kError() << "Tasknot found???";
         return false;
     }
     if ( node->siblingBefore() ) {
@@ -1280,7 +1323,7 @@ bool Project::canMoveTaskDown( Node* node )
         return false;
     }
     if ( parentNode->findChildNode( node ) == -1 ) {
-        kError() << "Tasknot found???" << endl;
+        kError() << "Tasknot found???";
         return false;
     }
     if ( node->siblingAfter() ) {
@@ -1322,6 +1365,15 @@ QString Project::uniqueNodeId( int seed )
     return ident;
 }
 
+QString Project::uniqueNodeId( const QList<QString> &existingIds, int seed )
+{
+    QString id = uniqueNodeId( seed );
+    while ( existingIds.contains( id ) ) {
+        id = uniqueNodeId( seed );
+    }
+    return id;
+}
+
 bool Project::removeId( const QString &id )
 {
     //kDebug() <<"id=" << id;
@@ -1339,7 +1391,7 @@ void Project::insertId( const QString &id, Node *node )
 bool Project::registerNodeId( Node *node )
 {
     if ( node->id().isEmpty() ) {
-        kError() << "Id is empty." << endl;
+        kError() << "Id is empty.";
         return false;
     }
     Node *rn = findNode( node->id() );
@@ -1348,7 +1400,7 @@ bool Project::registerNodeId( Node *node )
         return true;
     }
     if ( rn != node ) {
-        kError() << "Id already exists for different task: " << node->id() << endl;
+        kError() << "Id already exists for different task: " << node->id();
         return false;
     }
     return true;
@@ -1402,7 +1454,7 @@ ResourceGroup *Project::group( const QString& id )
 
 ResourceGroup *Project::groupByName( const QString& name ) const
 {
-    foreach ( QString k, resourceGroupIdDict.keys() ) {
+    foreach ( const QString &k, resourceGroupIdDict.keys() ) {
         ResourceGroup *g = resourceGroupIdDict[ k ];
         if ( g->name() == name ) {
             Q_ASSERT( k == g->id() );
@@ -1460,7 +1512,7 @@ Resource *Project::resource( const QString& id )
 
 Resource *Project::resourceByName( const QString& name ) const
 {
-    foreach ( QString k, resourceIdDict.keys() ) {
+    foreach ( const QString &k, resourceIdDict.keys() ) {
         Resource *r = resourceIdDict[ k ];
         if ( r->name() == name ) {
             Q_ASSERT( k == r->id() );
@@ -2025,8 +2077,8 @@ ScheduleManager *Project::findScheduleManager( const QString &name ) const
 QList<ScheduleManager*> Project::allScheduleManagers() const
 {
     QList<ScheduleManager*> lst;
-    lst << m_managers;
     foreach ( ScheduleManager *sm, m_managers ) {
+        lst << sm;
         lst << sm->allChildren();
     }
     return lst;
@@ -2294,17 +2346,22 @@ QList<Node*> Project::flatNodeList( Node *parent )
     return lst;
 }
 
-void Project::setTaskDefaults( const Task &task )
+void Project::setSchedulerPlugins( const QMap<QString, SchedulerPlugin*> &plugins )
 {
-    delete m_taskDefaults;
-    m_taskDefaults = new Task( task );
+    m_schedulerPlugins = plugins;
+    kDebug()<<m_schedulerPlugins;
+}
+
+void Project::emitLocaleChanged()
+{
+    emit localeChanged();
 }
 
 #ifndef NDEBUG
 void Project::printDebug( bool children, const QByteArray& _indent )
 {
     QByteArray indent = _indent;
-    kDebug() << indent <<"+ Project node:" << Node::name(); //FIXME: QT3 support
+    qDebug() << indent <<"+ Project node:" << Node::name()<<" id="<<id();
     indent += '!';
     QListIterator<ResourceGroup*> it( resourceGroups() );
     while ( it.hasNext() )
@@ -2315,10 +2372,10 @@ void Project::printDebug( bool children, const QByteArray& _indent )
 void Project::printCalendarDebug( const QByteArray& _indent )
 {
     QByteArray indent = _indent;
-    kDebug() << indent <<"-------- Calendars debug printout --------";
+    qDebug() << indent <<"-------- Calendars debug printout --------";
     foreach ( Calendar *c, calendarIdDict ) {
         c->printDebug( indent + "--" );
-        kDebug();
+        qDebug();
     }
     if ( m_standardWorktime )
         m_standardWorktime->printDebug();
