@@ -34,6 +34,7 @@
 #include "kptduration.h"
 #include "kptresource.h"
 #include "kptwbsdefinition.h"
+#include "kptconfigbase.h"
 
 #include <QMap>
 #include <QList>
@@ -51,6 +52,7 @@ class StandardWorktime;
 class ScheduleManager;
 class XMLLoaderObject;
 class Task;
+class SchedulerPlugin;
 
 /**
  * Project is the main node in a project, it contains child nodes and
@@ -74,6 +76,7 @@ class KPLATOKERNEL_EXPORT Project : public Node
     Q_OBJECT
 public:
     explicit Project( Node *parent = 0 );
+    explicit Project( ConfigBase &config, Node *parent = 0 );
     ~Project();
 
     /// Returns the node type. Can be Type_Project or Type_Subproject.
@@ -323,6 +326,9 @@ public:
     /// Create a unique id.
     QString uniqueNodeId( int seed = 1 );
 
+    /// Create a unique id.
+    QString uniqueNodeId( const QList<QString> &existingIds, int seed = 1 );
+
     ResourceGroup *findResourceGroup( const QString &id ) const
     {
         if ( resourceGroupIdDict.contains( id ) )
@@ -376,7 +382,6 @@ public:
     bool setCalendarId( Calendar *calendar );
     /// returns a unique calendar id
     QString uniqueCalendarId() const;
-
     /// Return reference to WBS Definition
     WBSDefinition &wbsDefinition();
     /// Set WBS Definition to @p def
@@ -474,11 +479,27 @@ public:
     void generateUniqueNodeIds();
     void generateUniqueIds();
     
-    void setTaskDefaults( const Task &task );
-    const Task &taskDefaults() const { return *m_taskDefaults; }
+    const ConfigBase &config() const { return *m_config; }
+    /// Set configuration data
+    void setConfig( ConfigBase *config ) { m_config = config; }
+
+    const Task &taskDefaults() const { return m_config->taskDefaults(); }
+
+    /// Return locale. (Used for currency, everything else is from KGlobal::locale)
+    KLocale *locale() { return m_config->locale(); }
+    /// Return locale. (Used for currency, everything else is from KGlobal::locale)
+    const KLocale *locale() const { return m_config->locale(); }
+    /// Signal that locale data has changed
+    void emitLocaleChanged();
     
     void incProgress();
 
+    void setSchedulerPlugins( const QMap<QString, SchedulerPlugin*> &plugins );
+    const QMap<QString, SchedulerPlugin*> &schedulerPlugins() const { return m_schedulerPlugins; }
+
+    void initiateCalculation( MainSchedule &sch );
+    void initiateCalculationLists( MainSchedule &sch );
+    
 signals:
     /// Emitted when anything in the project is changed (use with care)
     void changed();
@@ -558,7 +579,10 @@ signals:
     void relationRemoved( Relation *rel );
     /// Emitted when the relation @p rel has been modified.
     void relationModified( Relation *rel );
-    
+
+    /// Emitted when locale data has changed
+    void localeChanged();
+
 protected:
     /// Calculate the schedule.
     void calculate( Schedule *scedule );
@@ -592,9 +616,6 @@ protected:
     DateTime checkStartConstraints( const DateTime &dt ) const;
     DateTime checkEndConstraints( const DateTime &dt ) const;
 
-    void initiateCalculation( MainSchedule &sch );
-    void initiateCalculationLists( MainSchedule &sch );
-
     bool legalParents( const Node *par, const Node *child ) const;
     bool legalChildren( const Node *par, const Node *child ) const;
 
@@ -614,9 +635,13 @@ private:
     //use in pert to store the project slack
     int m_projectSlack;
 
-    Task *m_taskDefaults;
+    ConfigBase *emptyConfig;
+    ConfigBase *m_config; // this one is not owned by me, don't delete
+
     int m_progress;
 
+    QMap<QString, SchedulerPlugin*> m_schedulerPlugins;
+
 #ifndef NDEBUG
 public:
     void printDebug( bool children, const QByteArray& indent );
