@@ -33,19 +33,47 @@
 namespace KPlato
 {
 
-void ProjectTester::printDebug( Project *p, Task *t ) const {
-    qDebug()<<"Debug info -------------------------------------";
-    qDebug()<<"project target start:"<<p->constraintStartTime().toString();
-    qDebug()<<"project target end:"<<p->constraintEndTime().toString();
-    qDebug()<<"project start time:"<<p->startTime().toString();
-    qDebug()<<"project end time:"<<p->endTime().toString();
+void ProjectTester::printDebug( Calendar *c, const QString &str, bool full ) const {
+    qDebug()<<"Debug info: Calendar"<<c->name()<<str;
+    for ( int wd = 1; wd <= 7; ++wd ) {
+        CalendarDay *d = c->weekday( wd );
+        qDebug()<<"   "<<wd<<":"<<d->stateToString( d->state() );
+        foreach ( TimeInterval *t,  d->workingIntervals() ) {
+            qDebug()<<"      interval:"<<t->first<<t->second;
+        }
+    }
+    foreach ( const CalendarDay *d, c->days() ) {
+        qDebug()<<"   "<<d->date()<<":";
+        foreach ( TimeInterval *t,  d->workingIntervals() ) {
+            qDebug()<<"      interval:"<<t->first<<t->second;
+        }
+    }
+}
 
-    qDebug()<<"earlyStart:"<<t->earlyStart().toString();
-    qDebug()<<"lateStart:"<<t->lateStart().toString();
-    qDebug()<<"earlyFinish:"<<t->earlyFinish().toString();
-    qDebug()<<"lateFinish:"<<t->lateFinish().toString();
-    qDebug()<<"startTime:"<<t->startTime().toString();
-    qDebug()<<"endTime:"<<t->endTime().toString();
+void ProjectTester::printDebug( Resource *r, const QString &str, bool full ) const {
+    qDebug()<<"Debug info: Resource"<<r->name()<<str;
+    qDebug()<<"Group:"<<r->parentGroup()->name()<<"Type:"<<r->parentGroup()->typeToString();
+    qDebug()<<"Available:"<<r->availableFrom().toString()<<r->availableUntil().toString()<<r->units()<<"%";
+    qDebug()<<"Type:"<<r->typeToString();
+    qDebug()<<"External appointments:"<<r->numExternalAppointments();
+    foreach ( Appointment *a, r->externalAppointmentList() ) {
+        qDebug()<<"   appointment:"<<a->startTime().toString()<<a->endTime().toString();
+    }
+}
+
+void ProjectTester::printDebug( Project *p, Task *t, const QString &str, bool full ) const {
+    qDebug()<<"Debug info: Project"<<p->name()<<str;
+    qDebug()<<"project target start:"<<p->constraintStartTime().toString();
+    qDebug()<<"  project target end:"<<p->constraintEndTime().toString();
+    qDebug()<<"  project start time:"<<p->startTime().toString();
+    qDebug()<<"    project end time:"<<p->endTime().toString();
+    qDebug()<<"Task"<<t->name()<<t->typeToString()<<t->constraintToString();
+    qDebug()<<"     earlyStart:"<<t->earlyStart().toString();
+    qDebug()<<"      lateStart:"<<t->lateStart().toString();
+    qDebug()<<"    earlyFinish:"<<t->earlyFinish().toString();
+    qDebug()<<"     lateFinish:"<<t->lateFinish().toString();
+    qDebug()<<"      startTime:"<<t->startTime().toString();
+    qDebug()<<"        endTime:"<<t->endTime().toString();
     switch ( t->constraint() ) {
         case Node::MustStartOn:
         case Node::StartNotEarlier:
@@ -55,15 +83,35 @@ void ProjectTester::printDebug( Project *p, Task *t ) const {
             qDebug()<<"startConstraint:"<<t->constraintStartTime().toString();
         case Node::MustFinishOn:
         case Node::FinishNotLater:
-            qDebug()<<"endConstraint:"<<t->constraintEndTime().toString();
+            qDebug()<<"  endConstraint:"<<t->constraintEndTime().toString();
             break;
         default: break;
     }
+    foreach ( ResourceGroupRequest *gr, t->requests().requests() ) {
+        qDebug()<<"Group request:"<<gr->group()->name()<<gr->units();
+        foreach ( ResourceRequest *rr, gr->resourceRequests() ) {
+            qDebug()<<"  Resource request:"<<rr->resource()->name()<<rr->units()<<"%"<<rr->resource()->availableFrom().toString()<<rr->resource()->availableUntil().toString();
+        }
+    }
+    Schedule *s = t->currentSchedule();
+    if ( s == 0 ) {
+        return;
+    }
+    qDebug();
+    qDebug()<<"Estimate   :"<<t->estimate()->expectedEstimate()<<Duration::unitToString(t->estimate()->unit());
+    foreach ( Appointment *a, s->appointments() ) {
+        qDebug()<<"Resource:"<<a->resource()->resource()->name()<<a->startTime().toString()<<a->endTime().toString();
+        if ( ! full ) { continue; }
+        foreach( const AppointmentInterval &i, a->intervals() ) {
+            qDebug()<<"  "<<i.startTime().toString()<<i.endTime().toString()<<i.load();
+        }
+    }
 }
 
-void ProjectTester::printSchedulingLog( const ScheduleManager &sm ) const
+void ProjectTester::printSchedulingLog( const ScheduleManager &sm, const QString &s ) const
 {
-    qDebug()<<"Scheduling log ---------------------------------";
+    qDebug()<<"Scheduling log"<<s;
+    qDebug()<<"Scheduling:"<<sm.name()<<(sm.recalculate()?QString("recalculate from %1").arg(sm.recalculateFrom().toString()):"");
     foreach ( const QString &s, sm.expected()->logMessages() ) {
         qDebug()<<s;
     }
@@ -72,6 +120,21 @@ void ProjectTester::printSchedulingLog( const ScheduleManager &sm ) const
 void ProjectTester::initTestCase()
 {
     m_project = new Project();
+    // standard worktime defines 8 hour day as default
+    QVERIFY( m_project->standardWorktime() );
+    QCOMPARE( m_project->standardWorktime()->day(), 8.0 );
+    m_calendar = new Calendar();
+    m_calendar->setDefault( true );
+    QTime t1( 9, 0, 0 );
+    QTime t2 ( 17, 0, 0 );
+    int length = t1.msecsTo( t2 );
+    for ( int i=1; i <= 7; ++i ) {
+        CalendarDay *d = m_calendar->weekday( i );
+        d->setState( CalendarDay::Working );
+        d->addInterval( t1, length );
+    }
+    m_project->addCalendar( m_calendar );
+
     m_task = 0;
 }
 
@@ -149,8 +212,8 @@ void ProjectTester::schedule()
     m_project->addTask( t, m_project );
     t->estimate()->setUnit( Duration::Unit_d );
     t->estimate()->setExpectedEstimate( 1.0 );
-    t->estimate()->setType( Estimate::Type_FixedDuration );
-    
+    t->estimate()->setType( Estimate::Type_Duration );
+
     ScheduleManager *sm = m_project->createScheduleManager( "Test Plan" );
     m_project->addScheduleManager( sm );
     m_project->calculate( *sm );
@@ -158,21 +221,17 @@ void ProjectTester::schedule()
     QCOMPARE( t->startTime(), m_project->startTime() );
     QCOMPARE( t->endTime(), DateTime(t->startTime().addDays( 1 )) );
     
-    // standard worktime defines 8 hour day as default
-    Calendar *c = new Calendar();
-    c->setDefault( true );
-    for ( int i=1; i <= 7; ++i ) {
-        CalendarDay *d = c->weekday( i );
-        d->setState( CalendarDay::Working );
-        d->addInterval( t1, length );
-    }
-    m_project->addCalendar( c );
-    
+    t->estimate()->setCalendar( m_calendar );
+    m_project->calculate( *sm );
+
+    QCOMPARE( t->startTime(), m_calendar->firstAvailableAfter( m_project->startTime(), m_project->endTime() ) );
+    QCOMPARE( t->endTime(), DateTime( t->startTime().addMSecs( length ) ) );
+
     ResourceGroup *g = new ResourceGroup();
     m_project->addResourceGroup( g );
     Resource *r = new Resource();
     r->setAvailableFrom( QDateTime( yesterday, QTime() ) );
-    r->setCalendar( c );
+    r->setCalendar( m_calendar );
     m_project->addResource( g, r );
 
     ResourceGroupRequest *gr = new ResourceGroupRequest( g );
@@ -181,7 +240,7 @@ void ProjectTester::schedule()
     gr->addResourceRequest( rr );
     t->estimate()->setType( Estimate::Type_Effort );
     
-    //kDebug()<<"Calculate forward, Task: ASAP -----------------------------------";
+    QString s = "Calculate forward, Task: ASAP -----------------------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     sm = m_project->createScheduleManager( "Test Plan" );
     m_project->addScheduleManager( sm );
@@ -196,7 +255,7 @@ void ProjectTester::schedule()
     QCOMPARE( t->endTime(), t->startTime() + Duration( 0, 8, 0 ) );
     QVERIFY( t->schedulingError() == false );
 
-    //kDebug()<<"Calculate forward, Task: ASAP, Resource 50% load ------------------";
+    s = "Calculate forward, Task: ASAP, Resource 50% available -----------------";
     r->setUnits( 50 );
     sm = m_project->createScheduleManager( "Test Plan" );
     m_project->addScheduleManager( sm );
@@ -211,13 +270,35 @@ void ProjectTester::schedule()
     QCOMPARE( t->endTime(), t->startTime() + Duration( 1, 8, 0 ) );
     QVERIFY( t->schedulingError() == false );
     
-    //kDebug()<<"Calculate forward, Task: ASAP, Resource available tomorrow --------";
+    s = "Calculate forward, Task: ASAP, Resource 50% available, Request 50% load ---------";
+    r->setUnits( 50 );
+    rr->setUnits( 50 );
+    sm = m_project->createScheduleManager( "Test Plan" );
+    m_project->addScheduleManager( sm );
+    m_project->calculate( *sm );
+    
+//    printDebug( m_project, t, s );
+    
+    QCOMPARE( t->earlyStart(), m_project->startTime() );
+    QVERIFY( t->lateStart() >=  t->earlyStart() );
+    QVERIFY( t->earlyFinish() <= t->endTime() );
+    QVERIFY( t->lateFinish() >= t->endTime() );
+
+    QCOMPARE( t->startTime(), DateTime( today, t1 ) );
+    QCOMPARE( t->endTime(), t->startTime() + Duration( 3, 8, 0 ) );
+    QVERIFY( t->schedulingError() == false );
+    
+    s = "Calculate forward, Task: ASAP, Resource available tomorrow --------";
     r->setAvailableFrom( QDateTime( tomorrow, QTime() ) );
     r->setUnits( 100 );
+    rr->setUnits( 100 );
     sm = m_project->createScheduleManager( "Test Plan" );
     m_project->addScheduleManager( sm );
     m_project->calculate( *sm );
 
+//    printDebug( m_project, t, s);
+//    printSchedulingLog( *sm, s );
+    
     QCOMPARE( t->earlyStart(), m_project->startTime() );
     QVERIFY( t->lateStart() >=  t->earlyStart() );
     QVERIFY( t->earlyFinish() <= t->endTime() );
@@ -227,7 +308,7 @@ void ProjectTester::schedule()
     QCOMPARE( t->endTime(), t->startTime() + Duration( 0, 8, 0 ) );
     QVERIFY( t->schedulingError() == false );
     
-    //kDebug()<<"Calculate forward, Task: ALAP -----------------------------------";
+    s = "Calculate forward, Task: ALAP -----------------------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime(0,0,0) ) );
     t->setConstraint( Node::ALAP );
     r->setAvailableFrom( QDateTime( yesterday, QTime() ) );
@@ -244,7 +325,7 @@ void ProjectTester::schedule()
     QCOMPARE( t->endTime(), t->startTime() + Duration( 0, 8, 0 ) );
     QVERIFY( t->schedulingError() == false );
 
-    //kDebug()<<"Calculate forward, Task: MustStartOn -----------------------------------";
+    s = "Calculate forward, Task: MustStartOn -----------------------------------";
     r->setAvailableFrom( QDateTime( yesterday, QTime() ) );
     t->setConstraint( Node::MustStartOn );
     t->setConstraintStartTime( DateTime( nextweek, t1 ) );
@@ -262,7 +343,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
     
     // Calculate backwards
-    //kDebug()<<"Calculate backward, Task: MustStartOn -----------------------------------";
+    s = "Calculate backward, Task: MustStartOn -----------------------------------";
     m_project->setConstraintEndTime( DateTime( nextweek.addDays( 1 ), QTime() ) );
     sm = m_project->createScheduleManager( "Test Plan" );
     sm->setSchedulingDirection( true );
@@ -279,7 +360,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate backword
-    //kDebug()<<"Calculate backwards, Task: MustFinishOn -----------------------------------";
+    s = "Calculate backwards, Task: MustFinishOn -----------------------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime(0,0,0) ) );
     m_project->setConstraintEndTime( DateTime( nextweek.addDays( 1 ), QTime() ) );
     t->setConstraint( Node::MustFinishOn );
@@ -299,7 +380,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, Task: MustFinishOn -----------------------------------";
+    s = "Calculate forwards, Task: MustFinishOn -----------------------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::MustFinishOn );
     t->setConstraintEndTime( DateTime( tomorrow, t2 ) );
@@ -318,7 +399,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, Task: StartNotEarlier -----------------------------------";
+    s = "Calculate forwards, Task: StartNotEarlier -----------------------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::StartNotEarlier );
     t->setConstraintStartTime( DateTime( today, t2 ) );
@@ -337,7 +418,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate backward
-    //kDebug()<<"Calculate backwards, Task: StartNotEarlier -----------------------------------";
+    s = "Calculate backwards, Task: StartNotEarlier -----------------------------------";
     m_project->setConstraintEndTime( DateTime( nextweek, QTime() ) );
     t->setConstraint( Node::StartNotEarlier );
     t->setConstraintStartTime( DateTime( today, t2 ) );
@@ -346,7 +427,7 @@ void ProjectTester::schedule()
     m_project->addScheduleManager( sm );
     m_project->calculate( *sm );
 
-    //printDebug( m_project, t );
+    //printDebug( m_project, t, s );
 
     QVERIFY( t->earlyStart() <= t->constraintStartTime() );
     QVERIFY( t->lateStart() >=  t->constraintStartTime() );
@@ -358,7 +439,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, Task: FinishNotLater -----------------------------------";
+    s = "Calculate forwards, Task: FinishNotLater -----------------------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::FinishNotLater );
     t->setConstraintEndTime( DateTime( tomorrow.addDays( 1 ), t2 ) );
@@ -380,7 +461,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate backward
-    //kDebug()<<"Calculate backwards, Task: FinishNotLater -----------------------------------";
+    s = "Calculate backwards, Task: FinishNotLater -----------------------------------";
     m_project->setConstraintStartTime( DateTime( nextweek, QTime() ) );
     t->setConstraint( Node::FinishNotLater );
     t->setConstraintEndTime( DateTime( tomorrow, t2 ) );
@@ -390,7 +471,7 @@ void ProjectTester::schedule()
     m_project->addScheduleManager( sm );
     m_project->calculate( *sm );
 
-    //printDebug( m_project, t );
+    //printDebug( m_project, t, s );
 
     QCOMPARE( t->earlyStart(), m_project->startTime() );
     QCOMPARE( t->lateStart(),  t->startTime() );
@@ -402,7 +483,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, Task: FixedInterval -----------------------------------";
+    s = "Calculate forward, Task: FixedInterval -----------------------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::FixedInterval );
     t->setConstraintStartTime( DateTime( tomorrow, t1 ) );
@@ -413,7 +494,7 @@ void ProjectTester::schedule()
     m_project->addScheduleManager( sm );
     m_project->calculate( *sm );
 
-    //printDebug( m_project, t );
+    //printDebug( m_project, t, s );
 
     QCOMPARE( t->earlyStart(), m_project->constraintStartTime() );
     QCOMPARE( t->lateStart(), t->constraintStartTime() );
@@ -425,7 +506,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, Task: FixedInterval -----------------------------------";
+    s = "Calculate forwards, Task: FixedInterval -----------------------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::FixedInterval );
     t->setConstraintStartTime( DateTime( tomorrow, QTime() ) ); // outside working hours
@@ -446,7 +527,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, Task: Milestone, ASAP-------------------------";
+    s = "Calculate forwards, Task: Milestone, ASAP-------------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::ASAP );
     t->estimate()->clear();
@@ -456,10 +537,7 @@ void ProjectTester::schedule()
     m_project->addScheduleManager( sm );
     m_project->calculate( *sm );
 
-    //kDebug()<<t->earlyStart()<<m_project->constraintStartTime();
-    //kDebug()<<t->lateStart()<<t->earlyStart();
-    //kDebug()<<t->earlyFinish()<<t->earlyStart();
-    //kDebug()<<t->lateFinish()<<t->earlyFinish();
+    //printDebug( m_project, t, s );
 
     QCOMPARE( t->earlyStart(), m_project->constraintStartTime() );
     QCOMPARE( t->lateStart(), t->earlyStart() );
@@ -471,7 +549,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate backward
-    //kDebug()<<"Calculate backwards, Task: Milestone, ASAP-------------------------";
+    s = "Calculate backwards, Task: Milestone, ASAP-------------------------";
     m_project->setConstraintEndTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::ASAP );
     t->estimate()->clear();
@@ -491,7 +569,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, Task: Milestone, ALAP-------------------------";
+    s = "Calculate forwards, Task: Milestone, ALAP-------------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::ALAP );
     t->estimate()->clear();
@@ -511,7 +589,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate backward
-    //kDebug()<<"Calculate backwards, Task: Milestone, ALAP-------------------------";
+    s = "Calculate backwards, Task: Milestone, ALAP-------------------------";
     m_project->setConstraintEndTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::ALAP );
     t->estimate()->clear();
@@ -531,7 +609,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, Task: Milestone, MustStartOn ------------------";
+    s = "Calculate forwards, Task: Milestone, MustStartOn ------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::MustStartOn );
     t->setConstraintStartTime( DateTime( tomorrow, t1 ) );
@@ -551,7 +629,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate backward
-    //kDebug()<<"Calculate backwards, Task: Milestone, MustStartOn ------------------";
+    s = "Calculate backwards, Task: Milestone, MustStartOn ------------------";
     m_project->setConstraintEndTime( DateTime( tomorrow, QTime() ) );
     t->setConstraint( Node::MustStartOn );
     t->setConstraintStartTime( DateTime( today, t1 ) );
@@ -571,7 +649,7 @@ void ProjectTester::schedule()
     QVERIFY( t->schedulingError() == false );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, Task: Milestone, MustFinishOn ------------------";
+    s = "Calculate forwards, Task: Milestone, MustFinishOn ------------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::MustFinishOn );
     t->setConstraintEndTime( DateTime( tomorrow, t1 ) );
@@ -593,7 +671,7 @@ void ProjectTester::schedule()
     QCOMPARE( m_project->endTime(), t->endTime() );
 
     // Calculate backward
-    //kDebug()<<"Calculate backwards, Task: Milestone, MustFinishOn ------------------";
+    s = "Calculate backwards, Task: Milestone, MustFinishOn ------------------";
     m_project->setConstraintEndTime( DateTime( tomorrow, QTime() ) );
     t->setConstraint( Node::MustFinishOn );
     t->setConstraintEndTime( DateTime( today, t1 ) );
@@ -615,7 +693,7 @@ void ProjectTester::schedule()
     QCOMPARE( m_project->startTime(), t->startTime() );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, Task: Milestone, StartNotEarlier ---------------";
+    s = "Calculate forwards, Task: Milestone, StartNotEarlier ---------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::StartNotEarlier );
     t->setConstraintEndTime( DateTime( tomorrow, t1 ) );
@@ -637,7 +715,7 @@ void ProjectTester::schedule()
     QCOMPARE( m_project->endTime(), t->endTime() );
 
     // Calculate backward
-    //kDebug()<<"Calculate backwards, Task: Milestone, StartNotEarlier ---------------";
+    s = "Calculate backwards, Task: Milestone, StartNotEarlier ---------------";
     m_project->setConstraintEndTime( DateTime( tomorrow, QTime() ) );
     t->setConstraint( Node::StartNotEarlier );
     t->setConstraintStartTime( DateTime( today, t1 ) );
@@ -659,7 +737,7 @@ void ProjectTester::schedule()
     QCOMPARE( m_project->startTime(), t->startTime() );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, Task: Milestone, FinishNotLater ---------------";
+    s = "Calculate forwards, Task: Milestone, FinishNotLater ---------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::FinishNotLater );
     t->setConstraintEndTime( DateTime( tomorrow, t1 ) );
@@ -681,7 +759,7 @@ void ProjectTester::schedule()
     QCOMPARE( m_project->endTime(), t->endTime() );
 
     // Calculate backward
-    //kDebug()<<"Calculate backwards, Task: Milestone, FinishNotLater ---------------";
+    s = "Calculate backwards, Task: Milestone, FinishNotLater ---------------";
     m_project->setConstraintEndTime( DateTime( tomorrow, QTime() ) );
     t->setConstraint( Node::FinishNotLater );
     t->setConstraintEndTime( DateTime( today, t1 ) );
@@ -703,8 +781,9 @@ void ProjectTester::schedule()
     QCOMPARE( m_project->startTime(), t->startTime() );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, 2 Task, no overbooking ----------";
+    s = "Calculate forward, 2 Tasks, no overbooking ----------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
+    m_project->setConstraintEndTime( DateTime( today, QTime() ).addDays( 4 ) );
     t->setConstraint( Node::ASAP );
     t->estimate()->setUnit( Duration::Unit_d );
     t->estimate()->setExpectedEstimate( 2.0 );
@@ -724,22 +803,10 @@ void ProjectTester::schedule()
     m_project->addScheduleManager( sm );
     m_project->calculate( *sm );
 
-    //kDebug()<<"estimate   :"<<t->estimate()->expectedEstimate()<<Duration::unitToString(t->estimate()->unit());
-    //kDebug()<<"earlyStart :"<<t->earlyStart();
-    //kDebug()<<"lateStart  :"<<t->lateStart();
-    //kDebug()<<"earlyFinish:"<<t->earlyFinish();
-    //kDebug()<<"lateFinish :"<<t->lateFinish();
-    //kDebug()<<"startTime  :"<<t->startTime();
-    //kDebug()<<"endTime    :"<<t->endTime();
+//    printDebug( m_project, t, s );
+//    printDebug( m_project, tsk2, s );
+//    printSchedulingLog( *sm, s );
     
-    //kDebug()<<"estimate   :"<<tsk2->estimate()->expectedEstimate()<<Duration::unitToString(tsk2->estimate()->unit());
-    //kDebug()<<"earlyStart :"<<tsk2->earlyStart();
-    //kDebug()<<"lateStart  :"<<tsk2->lateStart();
-    //kDebug()<<"earlyFinish:"<<tsk2->earlyFinish();
-    //kDebug()<<"lateFinish :"<<tsk2->lateFinish();
-    //kDebug()<<"startTime  :"<<tsk2->startTime();
-    //kDebug()<<"endTime    :"<<tsk2->endTime();
-
     QCOMPARE( t->earlyStart(), m_project->constraintStartTime() );
     QCOMPARE( t->lateStart(), tsk2->startTime() );
     QCOMPARE( t->earlyFinish(), DateTime( tomorrow, t2 ) );
@@ -766,7 +833,7 @@ void ProjectTester::schedule()
     QCOMPARE( m_project->endTime(), tsk2->endTime() );
 
     // Calculate forward
-    //kDebug()<<"Calculate forwards, 2 Task, relation ---------------";
+    s = "Calculate forward, 2 Tasks, relation ---------------";
     m_project->setConstraintStartTime( DateTime( today, QTime() ) );
     t->setConstraint( Node::ASAP );
     t->estimate()->setUnit( Duration::Unit_d );
@@ -782,6 +849,10 @@ void ProjectTester::schedule()
     m_project->addScheduleManager( sm );
     m_project->calculate( *sm );
 
+//    printDebug( m_project, t, s );
+//    printDebug( m_project, tsk2, s );
+//    printSchedulingLog( *sm, s );
+
     QCOMPARE( t->earlyStart(), m_project->constraintStartTime() );
     QCOMPARE( t->lateStart(), DateTime( today, t1 ) );
     QCOMPARE( t->earlyFinish(), DateTime( tomorrow, t2 ) );
@@ -806,21 +877,25 @@ void ProjectTester::schedule()
 
 void ProjectTester::scheduleFullday()
 {
-    Calendar c("Test");
+    QString s = "Full day, 1 resource works 24 hours a day -------------";
+    Calendar *c = new Calendar("Test");
     QTime t1(0,0,0);
     int length = 24*60*60*1000;
 
     for ( int i = 1; i <= 7; ++i ) {
-        CalendarDay *wd1 = c.weekday(i);
+        CalendarDay *wd1 = c->weekday(i);
         wd1->setState(CalendarDay::Working);
         wd1->addInterval(TimeInterval(t1, length));
-    } 
+    }
+    m_project->addCalendar( c );
+
     ResourceGroup *g = new ResourceGroup();
     g->setName( "G1" );
     m_project->addResourceGroup( g );
     Resource *r = new Resource();
     r->setName( "R1" );
-    r->setCalendar( &c );
+    r->setCalendar( c );
+    r->setAvailableUntil( r->availableFrom().addDays( 21 ) );
     m_project->addResource( g, r );
 
     Task *t = m_project->createTask( m_project );
@@ -831,17 +906,65 @@ void ProjectTester::scheduleFullday()
     t->estimate()->setType( Estimate::Type_Effort );
     
     ResourceGroupRequest *gr = new ResourceGroupRequest( g );
-    gr->addResourceRequest( new ResourceRequest( r ) );
+    gr->addResourceRequest( new ResourceRequest( r, 100 ) );
     t->addRequest( gr );
 
     ScheduleManager *sm = m_project->createScheduleManager( "Test Plan" );
     m_project->addScheduleManager( sm );
     m_project->calculate( *sm );
     
+//     printDebug( c, s );
+//     printDebug( m_project, t, s );
+//     printDebug( r, s, true );
+//     printSchedulingLog( *sm, s );
+
     QCOMPARE( t->startTime(), m_project->startTime() );
     QCOMPARE( t->endTime(), DateTime(t->startTime().addDays( 14 )) );
     
+    s = "Full day, 8 hour shifts, 3 resources ---------------";
+    int hour = 60*60*1000;
+    Calendar *c1 = new Calendar("Test 1");
+    for ( int i = 1; i <= 7; ++i ) {
+        CalendarDay *wd1 = c1->weekday(i);
+        wd1->setState(CalendarDay::Working);
+        wd1->addInterval(TimeInterval(QTime( 6, 0, 0 ), 8*hour));
+    }
+    m_project->addCalendar( c1 );
+    Calendar *c2 = new Calendar("Test 2");
+    for ( int i = 1; i <= 7; ++i ) {
+        CalendarDay *wd1 = c2->weekday(i);
+        wd1->setState(CalendarDay::Working);
+        wd1->addInterval(TimeInterval(QTime( 14, 0, 0 ), 8*hour));
+    }
+    m_project->addCalendar( c2 );
+    Calendar *c3 = new Calendar("Test 3");
+    for ( int i = 1; i <= 7; ++i ) {
+        CalendarDay *wd1 = c3->weekday(i);
+        wd1->setState(CalendarDay::Working);
+        wd1->addInterval(TimeInterval(QTime( 0, 0, 0 ), 6*hour));
+        wd1->addInterval(TimeInterval(QTime( 22, 0, 0 ), 2*hour));
+    }
+    m_project->addCalendar( c3 );
+    
+    r->setCalendar( c1 );
 
+    r = new Resource();
+    r->setName( "R2" );
+    r->setCalendar( c2 );
+    gr->addResourceRequest( new ResourceRequest( r, 100 ) );
+
+    r = new Resource();
+    r->setName( "R3" );
+    r->setCalendar( c3 );
+    gr->addResourceRequest( new ResourceRequest( r, 100 ) );
+
+    m_project->calculate( *sm );
+
+//    printDebug( m_project, t, s );
+//    printSchedulingLog( *sm, s );
+
+    QCOMPARE( t->startTime(), m_project->startTime() );
+    QCOMPARE( t->endTime(), DateTime(t->startTime().addDays( 14 )) );
 }
 
 void ProjectTester::scheduleWithExternalAppointments()
@@ -873,7 +996,7 @@ void ProjectTester::scheduleWithExternalAppointments()
     r->addExternalAppointment( "Ext-1", "External project 1", targetstart, targetstart.addDays( 1 ), 100 );
     r->addExternalAppointment( "Ext-1", "External project 1", targetend.addDays( -1 ), targetend, 100 );
 
-    Task *t = project.createTask( m_project );
+    Task *t = project.createTask( &project );
     t->setName( "T1" );
     project.addTask( t, &project );
     t->estimate()->setUnit( Duration::Unit_h );
@@ -881,14 +1004,17 @@ void ProjectTester::scheduleWithExternalAppointments()
     t->estimate()->setType( Estimate::Type_Effort );
     
     ResourceGroupRequest *gr = new ResourceGroupRequest( g );
-    gr->addResourceRequest( new ResourceRequest( r ) );
+    gr->addResourceRequest( new ResourceRequest( r, 100 ) );
     t->addRequest( gr );
 
     ScheduleManager *sm = project.createScheduleManager( "Test Plan" );
     project.addScheduleManager( sm );
     project.calculate( *sm );
     
-    //printSchedulingLog( *sm );
+    QString s = "Schedule with external appointments ----------";
+//     printDebug( r, s );
+//     printDebug( &project, t, s );
+//     printSchedulingLog( *sm );
 
     QCOMPARE( t->startTime(), targetstart + Duration( 1, 0, 0 ) );
     QCOMPARE( t->endTime(), t->startTime() + Duration( 0, 8, 0 ) );
@@ -929,6 +1055,195 @@ void ProjectTester::scheduleWithExternalAppointments()
 
 }
 
+void ProjectTester::reschedule()
+{
+    Project project;
+    project.setName( "P1" );
+    DateTime targetstart = DateTime( QDate::currentDate(), QTime(0,0,0) );
+    DateTime targetend = DateTime( targetstart.addDays( 7 ) );
+    project.setConstraintStartTime( targetstart );
+    project.setConstraintEndTime( targetend);
+
+    Calendar *c = new Calendar("Test");
+    QTime t1(8,0,0);
+    int length = 8*60*60*1000; // 8 hours
+
+    for ( int i = 1; i <= 7; ++i ) {
+        CalendarDay *wd1 = c->weekday(i);
+        wd1->setState(CalendarDay::Working);
+        wd1->addInterval(TimeInterval(t1, length));
+    }
+    project.addCalendar( c );
+    ResourceGroup *g = new ResourceGroup();
+    g->setName( "G1" );
+    project.addResourceGroup( g );
+    Resource *r = new Resource();
+    r->setName( "R1" );
+    r->setCalendar( c );
+    project.addResource( g, r );
+
+    QString s = "Re-schedule; schedule tasks T1, T2, T3 ---------------";
+
+    Task *task1 = project.createTask( &project );
+    task1->setName( "T1" );
+    project.addTask( task1, &project );
+    task1->estimate()->setUnit( Duration::Unit_h );
+    task1->estimate()->setExpectedEstimate( 8.0 );
+    task1->estimate()->setType( Estimate::Type_Effort );
+    
+    ResourceGroupRequest *gr = new ResourceGroupRequest( g );
+    gr->addResourceRequest( new ResourceRequest( r, 100 ) );
+    task1->addRequest( gr );
+
+    Task *task2 = project.createTask( &project );
+    task2->setName( "T2" );
+    project.addTask( task2, &project );
+    task2->estimate()->setUnit( Duration::Unit_h );
+    task2->estimate()->setExpectedEstimate( 8.0 );
+    task2->estimate()->setType( Estimate::Type_Effort );
+    
+    gr = new ResourceGroupRequest( g );
+    gr->addResourceRequest( new ResourceRequest( r, 100 ) );
+    task2->addRequest( gr );
+
+    Task *task3 = project.createTask( &project );
+    task3->setName( "T3" );
+    project.addTask( task3, &project );
+    task3->estimate()->setUnit( Duration::Unit_h );
+    task3->estimate()->setExpectedEstimate( 8.0 );
+    task3->estimate()->setType( Estimate::Type_Effort );
+    
+    gr = new ResourceGroupRequest( g );
+    gr->addResourceRequest( new ResourceRequest( r, 100 ) );
+    task3->addRequest( gr );
+
+    Relation *rel = new Relation( task1, task2 );
+    project.addRelation( rel );
+    rel = new Relation( task1, task3 );
+    project.addRelation( rel );
+
+    ScheduleManager *sm = project.createScheduleManager( "Plan" );
+    project.addScheduleManager( sm );
+    project.calculate( *sm );
+    
+//    printDebug( &project, task1, s, true );
+//    printDebug( &project, task2, s, true );
+//    printDebug( &project, task3, s, true );
+//    printSchedulingLog( *sm );
+
+    QCOMPARE( task1->startTime(), c->firstAvailableAfter( targetstart, targetend ) );
+    QCOMPARE( task1->endTime(), task1->startTime() + Duration( 0, 8, 0 ) );
+    
+    QCOMPARE( task2->startTime(), c->firstAvailableAfter( task1->endTime(), targetend ) );
+    QCOMPARE( task2->endTime(), task2->startTime() + Duration( 0, 8, 0 ) );
+
+    QCOMPARE( task3->startTime(), c->firstAvailableAfter( task2->endTime(), targetend ) );
+    QCOMPARE( task3->endTime(), task3->startTime() + Duration( 0, 8, 0 ) );
+
+
+    DateTime restart = task1->endTime();
+    s = QString( "Re-schedule; re-schedule from %1 - tasks T1 (finished), T2, T3 ------" ).arg( restart.toString() );
+
+    task1->completion().setStarted( true );
+    task1->completion().setPercentFinished( task1->endTime().date(), 100 );
+    task1->completion().setFinished( true );
+
+    ScheduleManager *child = project.createScheduleManager( "Plan.1" );
+    project.addScheduleManager( child, sm );
+    child->setRecalculate( true );
+    child->setRecalculateFrom( restart );
+    project.calculate( *child );
+
+//     printDebug( &project, task1, s, true );
+//     printDebug( &project, task2, s, true );
+//     printDebug( &project, task3, s, true );
+//     printSchedulingLog( *child, s );
+
+    QCOMPARE( task1->startTime(), c->firstAvailableAfter( targetstart, targetend ) );
+    QCOMPARE( task1->endTime(), task1->startTime() + Duration( 0, 8, 0 ) );
+    
+    QCOMPARE( task2->startTime(), c->firstAvailableAfter( qMax(task1->endTime(), restart ), targetend ) );
+    QCOMPARE( task2->endTime(), task2->startTime() + Duration( 0, 8, 0 ) );
+
+    QCOMPARE( task3->startTime(), c->firstAvailableAfter( task2->endTime(), targetend ) );
+    QCOMPARE( task3->endTime(), task3->startTime() + Duration( 0, 8, 0 ) );
+}
+
+void ProjectTester::materialResource()
+{
+    Project project;
+    project.setName( "P1" );
+    DateTime targetstart = DateTime( QDate::currentDate(), QTime(0,0,0) );
+    DateTime targetend = DateTime( targetstart.addDays( 7 ) );
+    project.setConstraintStartTime( targetstart );
+    project.setConstraintEndTime( targetend);
+
+    Calendar *c = new Calendar("Test");
+    QTime t1(8,0,0);
+    int length = 8*60*60*1000; // 8 hours
+
+    for ( int i = 1; i <= 7; ++i ) {
+        CalendarDay *wd1 = c->weekday(i);
+        wd1->setState(CalendarDay::Working);
+        wd1->addInterval(TimeInterval(t1, length));
+    }
+    project.addCalendar( c );
+    
+    Task *task1 = project.createTask( &project );
+    task1->setName( "T1" );
+    project.addTask( task1, &project );
+    task1->estimate()->setUnit( Duration::Unit_h );
+    task1->estimate()->setExpectedEstimate( 8.0 );
+    task1->estimate()->setType( Estimate::Type_Effort );
+
+    QString s = "Calculate forward, Task: ASAP, Working + material resource --------";
+    qDebug()<<s;
+    ResourceGroup *g = new ResourceGroup();
+    project.addResourceGroup( g );
+    Resource *r = new Resource();
+    r->setName( "Work" );
+    r->setAvailableFrom( targetstart );
+    r->setCalendar( c );
+    project.addResource( g, r );
+
+    ResourceGroup *mg = new ResourceGroup();
+    mg->setType( ResourceGroup::Type_Material );
+    project.addResourceGroup( mg );
+    Resource *mr = new Resource();
+    mr->setType( Resource::Type_Material );
+    mr->setName( "Material" );
+    mr->setAvailableFrom( targetstart );
+    mr->setCalendar( c );
+    project.addResource( mg, mr );
+
+    ResourceGroupRequest *gr = new ResourceGroupRequest( g );
+    task1->addRequest( gr );
+    ResourceRequest *rr = new ResourceRequest( r, 100 );
+    gr->addResourceRequest( rr );
+
+    ResourceGroupRequest *mgr = new ResourceGroupRequest( mg );
+    task1->addRequest( mgr );
+    ResourceRequest *mrr = new ResourceRequest( mr, 100 );
+    mgr->addResourceRequest( mrr );
+
+    ScheduleManager *sm = project.createScheduleManager( "Test Plan" );
+    project.addScheduleManager( sm );
+    project.calculate( *sm );
+
+    printDebug( r, s);
+    printDebug( mr, s);
+    printDebug( &project, task1, s);
+     printSchedulingLog( *sm, s );
+
+    QCOMPARE( task1->earlyStart(), targetstart );
+    QVERIFY( task1->lateStart() >=  task1->earlyStart() );
+    QVERIFY( task1->earlyFinish() <= task1->endTime() );
+    QVERIFY( task1->lateFinish() >= task1->endTime() );
+    
+    QCOMPARE( task1->startTime(), DateTime( r->availableFrom().date(), t1 ) );
+    QCOMPARE( task1->endTime(), task1->startTime() + Duration( 0, 8, 0 ) );
+    QVERIFY( task1->schedulingError() == false );
+}
 
 } //namespace KPlato
 
