@@ -162,6 +162,9 @@ Qt::ItemFlags CalendarItemModel::flags( const QModelIndex &index ) const
     flags |= Qt::ItemIsDragEnabled;
     if ( calendar ( index ) ) {
         switch ( index.column() ) {
+            case 0:/*Name*/
+                flags |= ( Qt::ItemIsEditable | Qt::ItemIsUserCheckable );
+                break;
             case 1: 
                 if ( parent( index ).isValid() ) {
                     flags &= ~Qt::ItemIsEditable;
@@ -262,12 +265,17 @@ QVariant CalendarItemModel::name( const Calendar *a, int role ) const
     switch ( role ) {
         case Qt::DisplayRole:
         case Qt::EditRole:
-        case Qt::ToolTipRole:
             return a->name();
-            break;
+        case Qt::ToolTipRole:
+            if ( a->isDefault() ) {
+                return i18nc( "1=calendar name", "%1 (Default calendar)", a->name() );
+            }
+            return a->name();
         case Qt::StatusTipRole:
         case Qt::WhatsThisRole:
             return QVariant();
+        case Qt::CheckStateRole:
+            return a->isDefault() ? Qt::Checked : Qt::Unchecked;
     }
     return QVariant();
 }
@@ -278,8 +286,27 @@ bool CalendarItemModel::setName( Calendar *a, const QVariant &value, int role )
         case Qt::EditRole:
             if ( value.toString() != a->name() ) {
                 emit executeCommand( new CalendarModifyNameCmd( a, value.toString(), i18n( "Modify Calendar Name" ) ) );
+                return true;
             }
-            return true;
+            break;
+        case Qt::CheckStateRole: {
+            switch ( value.toInt() ) {
+                case Qt::Unchecked:
+                    if ( a->isDefault() ) {
+                        emit executeCommand( new ProjectModifyDefaultCalendarCmd( m_project, 0, i18n( "De-select as default calendar" ) ) );
+                        return true;
+                    }
+                    break;
+                case Qt::Checked:
+                    if ( ! a->isDefault() ) {
+                        emit executeCommand( new ProjectModifyDefaultCalendarCmd( m_project, a, i18n( "Select as default calendar" ) ) );
+                        return true;
+                    }
+                    break;
+                default: break;
+            }
+        }
+        default: break;
     }
     return false;
 }
@@ -294,7 +321,7 @@ QVariant CalendarItemModel::timeZone( const Calendar *a, int role ) const
             return i18n( a->timeZone().name().toUtf8() );
         case Role::EnumList: {
             QStringList lst;
-            foreach ( QString s, KSystemTimeZones::timeZones()->zones().keys() ) {
+            foreach ( const QString &s, KSystemTimeZones::timeZones()->zones().keys() ) {
                 lst << i18n( s.toUtf8() );
             }
             lst.sort();
@@ -321,7 +348,7 @@ bool CalendarItemModel::setTimeZone( Calendar *a, const QVariant &value, int rol
             QStringList lst = timeZone( a, Role::EnumList ).toStringList();
             QString name = lst.value( value.toInt() );
             KTimeZone tz;
-            foreach ( QString s, KSystemTimeZones::timeZones()->zones().keys() ) {
+            foreach ( const QString &s, KSystemTimeZones::timeZones()->zones().keys() ) {
                 if ( name == i18n( s.toUtf8() ) ) {
                     tz = KSystemTimeZones::zone( s );
                     break;
@@ -368,7 +395,8 @@ bool CalendarItemModel::setData( const QModelIndex &index, const QVariant &value
     if ( ! index.isValid() ) {
         return ItemModelBase::setData( index, value, role );
     }
-    if ( ( flags( index ) &Qt::ItemIsEditable ) == 0 || role != Qt::EditRole ) {
+    if ( ( flags( index ) &( Qt::ItemIsEditable | Qt::CheckStateRole ) ) == 0 ) {
+        Q_ASSERT( true );
         return false;
     }
     Calendar *a = calendar( index );
@@ -441,7 +469,7 @@ QMimeData *CalendarItemModel::mimeData( const QModelIndexList & indexes ) const
     QByteArray encodedData;
     QDataStream stream(&encodedData, QIODevice::WriteOnly);
     QList<int> rows;
-    foreach (QModelIndex index, indexes) {
+    foreach (const QModelIndex &index, indexes) {
         if ( index.isValid() && !rows.contains( index.row() ) ) {
             kDebug()<<index.row();
             Calendar *c = calendar( index );
@@ -852,7 +880,7 @@ CalendarDay *CalendarDayItemModel::day( const QModelIndex &index ) const
     return static_cast<CalendarDay*>( index.internalPointer() );
 }
 
-QItemDelegate *CalendarDayItemModel::createDelegate( int column, QWidget *parent ) const
+QAbstractItemDelegate *CalendarDayItemModel::createDelegate( int column, QWidget *parent ) const
 {
     switch ( column ) {
         default: return 0;
@@ -963,8 +991,8 @@ QVariant DateTableDataModel::weekNumberData( int week, int role ) const
 }
 
 //-------------
-DateTableDateDelegate::DateTableDateDelegate()
-    : KDateTableDateDelegate()
+DateTableDateDelegate::DateTableDateDelegate( QObject *parent )
+    : KDateTableDateDelegate( parent )
 {
 }
 
