@@ -162,7 +162,17 @@ int NodeChartModel::dataSetCount( const ChartAxisIndex &index ) const
         return 0;
     }
     if ( index.isValid() && index.userData == AxisSet ) {
-        return 3;
+        int count = 0;
+        if ( m_dataShown.showCost && index.number() == 0 ) {
+            if ( m_dataShown.showBCWSCost ) ++count;
+            if ( m_dataShown.showBCWPCost ) ++count;
+            if ( m_dataShown.showACWPCost ) ++count;
+        } else { // always effort
+            if ( m_dataShown.showBCWSEffort ) ++count;
+            if ( m_dataShown.showBCWPEffort ) ++count;
+            if ( m_dataShown.showACWPEffort ) ++count;
+        }
+        return count;
     }
     return 0;
 }
@@ -207,7 +217,7 @@ QVariant NodeChartModel::data( const ChartDataIndex &idx, const ChartAxisIndex &
                 if ( axisType == X_Axis ) {
                     //kDebug()<<"X-axis"<<idx<<axis;
                     QDate s = startDate();
-                    foreach ( QDate d, m_bcwp.days().keys() ) {
+                    foreach ( const QDate &d, m_bcwp.days().keys() ) {
                         lst << s.daysTo( d );
                     }
                 } else if ( axisType == Y_Axis ) {
@@ -236,7 +246,7 @@ QVariant NodeChartModel::data( const ChartDataIndex &idx, const ChartAxisIndex &
                 if ( axisType == X_Axis ) {
                     //kDebug()<<"X-axis"<<idx<<axis;
                     QDate s = startDate();
-                    foreach ( QDate d, m_bcwp.days().keys() ) {
+                    foreach ( const QDate &d, m_bcwp.days().keys() ) {
                         lst << s.daysTo( d );
                     }
                 } else if ( axisType == Y_Axis ) {
@@ -262,7 +272,7 @@ QVariant NodeChartModel::data( const ChartDataIndex &idx, const ChartAxisIndex &
                 if ( axisType == X_Axis ) {
                     //kDebug()<<"X-axis"<<idx;
                     QDate s = startDate();
-                    foreach ( QDate d, m_acwp.days().keys() ) {
+                    foreach ( const QDate &d, m_acwp.days().keys() ) {
                         lst << s.daysTo( d );
                     }
                 } else if ( axisType == Y_Axis ) {
@@ -351,27 +361,36 @@ ChartDataIndex NodeChartModel::index( int number, const ChartAxisIndex &idx ) co
     if ( m_dataShown.showCost && idx.number() == 0 ) {
         switch ( number ) {
             case 0:
-                if ( type & 1 ) return createDataIndex( number, idx, BCWS );
-                //fall trough
-            case 1:
-                if ( type & 2 ) return createDataIndex( number, idx, BCWP );
-                //fall trough
-            case 2:
+                if ( type & 1 ) return createDataIndex( 0, idx, BCWS );
+                if ( type & 2 ) return createDataIndex( 1, idx, BCWP );
+                if ( type & 4 ) return createDataIndex( 2, idx, ACWP );
+                break;
+           case 1: {
+                if ( (type & 1) && (type & 2) ) return createDataIndex( 1, idx, BCWP );
+                if ( type & 4 ) return createDataIndex( 2, idx, ACWP );
+                break; }
+           case 2:
                 if ( type & 4 ) return createDataIndex( number, idx, ACWP );
+                if ( type & 4 ) return createDataIndex( 2, idx, ACWP );
+                break;
         }
     }
-    if ( m_dataShown.showEffort && idx.number() == m_dataShown.showCost ? 1 : 0 ) {
-        switch ( number ) {
-            case 0:
-                if ( type & 8 ) return createDataIndex( number, idx, BCWS );
-                //fall trough
-            case 1:
-                if ( type & 16 ) return createDataIndex( number, idx, BCWP );
-                //fall trough
-            case 2:
-                if ( type & 32 ) return createDataIndex( number, idx, ACWP );
-        }
-    }
+     if ( m_dataShown.showEffort && idx.number() == (m_dataShown.showCost ? 1 : 0) ) {
+         switch ( number ) {
+             case 0:
+                if ( type & 8 ) return createDataIndex( 0, idx, BCWS );
+                if ( type & 16 ) return createDataIndex( 1, idx, BCWP );
+                if ( type & 32 ) return createDataIndex( 2, idx, ACWP );
+                break;
+             case 1:
+                if ( (type & 8) && (type & 16) ) return createDataIndex( 1, idx, BCWP );
+                if ( type & 32 ) return createDataIndex( 2, idx, ACWP );
+                break;
+             case 2:
+                if ( type & 32 ) return createDataIndex( 2, idx, ACWP );
+                break;
+         }
+     }
     return ChartDataIndex();
 }
     
@@ -427,7 +446,7 @@ QVariant NodeChartModel::axisData( const ChartAxisIndex &index, int role ) const
         }
         if ( index.userData == X_Axis ) {
             if ( parent( index ).number() == 0 ) {
-                return "Time"; // left y-axis
+                return i18n( "Time" ); // left y-axis
             }
             if ( parent( index ).number() == 1 ) {
                 // Top axis label
@@ -438,11 +457,11 @@ QVariant NodeChartModel::axisData( const ChartAxisIndex &index, int role ) const
             //kDebug()<<"Y axis:"<<index;
             if ( m_dataShown.showCost && parent( index ).number() == 0 ) {
                 //kDebug()<<"Y-axis Cost";
-                return "Cost"; // left y-axis
+                return i18n( "Cost" ); // left y-axis
             }
             if ( m_dataShown.showEffort && parent( index ).number() == ( m_dataShown.showCost ? 1 : 0 ) ) {
                 //kDebug()<<"Y-axis Hours";
-                return "Hours"; // right y-axis
+                return i18n( "Hours" ); // right y-axis
             }
         }
         //kDebug()<<index<<"No data";
