@@ -21,6 +21,7 @@
 
 #include "kptglobal.h"
 #include "kptitemmodelbase.h"
+#include "kpttaskcompletedelegate.h"
 #include "kptcommand.h"
 #include "kptnode.h"
 #include "kptproject.h"
@@ -29,7 +30,6 @@
 
 #include <QAbstractItemModel>
 #include <QMimeData>
-#include <QItemDelegate>
 #include <QModelIndex>
 
 #include <kglobal.h>
@@ -110,6 +110,7 @@ void TaskStatusItemModel::setProject( Project *project )
 {
     clear();
     if ( m_project ) {
+        disconnect( m_project, SIGNAL( localeChanged() ), this, SLOT( slotLayoutChanged() ) );
         disconnect( m_project, SIGNAL( wbsDefinitionChanged() ), this, SLOT( slotWbsDefinitionChanged() ) );
         disconnect( m_project, SIGNAL( nodeChanged( Node* ) ), this, SLOT( slotNodeChanged( Node* ) ) );
         disconnect( m_project, SIGNAL( nodeToBeAdded( Node* ) ), this, SLOT( slotNodeToBeInserted(  Node*, int ) ) );
@@ -123,6 +124,7 @@ void TaskStatusItemModel::setProject( Project *project )
     m_project = project;
     m_nodemodel.setProject( project );
     if ( project ) {
+        connect( m_project, SIGNAL( localeChanged() ), this, SLOT( slotLayoutChanged() ) );
         connect( m_project, SIGNAL( wbsDefinitionChanged() ), this, SLOT( slotWbsDefinitionChanged() ) );
         connect( m_project, SIGNAL( nodeChanged( Node* ) ), this, SLOT( slotNodeChanged( Node* ) ) );
         connect( m_project, SIGNAL( nodeToBeAdded( Node*, int ) ), this, SLOT( slotNodeToBeInserted(  Node*, int ) ) );
@@ -200,7 +202,7 @@ void TaskStatusItemModel::refresh()
     QDate end = m_nodemodel.now().addDays( m_period );
     
     foreach( Node* n, m_project->allNodes() ) {
-        if ( n->type() != Node::Type_Task ) {
+        if ( n->type() != Node::Type_Task && n->type() != Node::Type_Milestone ) {
             continue;
         }
         Task *t = static_cast<Task*>( n );
@@ -235,7 +237,10 @@ Qt::ItemFlags TaskStatusItemModel::flags( const QModelIndex &index ) const
     Qt::ItemFlags flags = QAbstractItemModel::flags( index );
     flags &= ~( Qt::ItemIsEditable | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled );
     Node *n = node( index );
-    if ( n == 0 || n->type() != Node::Type_Task || m_id == -1 || ! n->isScheduled( m_id ) ) {
+    if ( ! m_readWrite || n == 0 || m_id == -1 || ! n->isScheduled( m_id ) ) {
+        return flags;
+    }
+    if ( n->type() != Node::Type_Task && n->type() != Node::Type_Milestone ) {
         return flags;
     }
     Task *t = static_cast<Task*>( n );
@@ -244,6 +249,11 @@ Qt::ItemFlags TaskStatusItemModel::flags( const QModelIndex &index ) const
             case NodeModel::NodeActualStart:
                 flags |= Qt::ItemIsEditable;
                 break;
+            case NodeModel::NodeCompleted:
+                if ( t->state() & Node::State_ReadyToStart ) {
+                    flags |= Qt::ItemIsEditable;
+                }
+                break;
             default: break;
         }
     } else if ( ! t->completion().isFinished() ) {
@@ -362,12 +372,24 @@ QVariant TaskStatusItemModel::name( int row, int role ) const
 
 bool TaskStatusItemModel::setCompletion( Node *node, const QVariant &value, int role )
 {
-    if ( role == Qt::EditRole && node->type() == Node::Type_Task ) {
+    if ( role != Qt::EditRole ) {
+        return false;
+    }
+    if ( node->type() == Node::Type_Task ) {
         Completion &c = static_cast<Task*>( node )->completion();
-        QDate date = QDate::currentDate();
+        QDateTime dt = QDateTime::currentDateTime();
+        QDate date = dt.date();
         // xgettext: no-c-format
         MacroCommand *m = new MacroCommand( i18n( "Modify % Completed" ) );
+        if ( ! c.isStarted() ) {
+            m->addCommand( new ModifyCompletionStartedCmd( c, true ) );
+            m->addCommand( new ModifyCompletionStartTimeCmd( c, dt ) );
+        }
         m->addCommand( new ModifyCompletionPercentFinishedCmd( c, date, value.toInt() ) );
+        if ( value.toInt() == 100 ) {
+            m->addCommand( new ModifyCompletionFinishedCmd( c, true ) );
+            m->addCommand( new ModifyCompletionFinishTimeCmd( c, dt ) );
+        }
         emit executeCommand( m ); // also adds a new entry if necessary
         if ( c.entrymode() == Completion::EnterCompleted ) {
             Duration planned = static_cast<Task*>( node )->plannedEffort( m_nodemodel.id() );
@@ -382,6 +404,22 @@ bool TaskStatusItemModel::setCompletion( Node *node, const QVariant &value, int
         }
         return true;
     }
+    if ( node->type() == Node::Type_Milestone ) {
+        Completion &c = static_cast<Task*>( node )->completion();
+        if ( value.toInt() > 0 ) {
+            QDateTime dt = QDateTime::currentDateTime();
+            QDate date = dt.date();
+            MacroCommand *m = new MacroCommand( i18n( "Set finished" ) );
+            m->addCommand( new ModifyCompletionStartedCmd( c, true ) );
+            m->addCommand( new ModifyCompletionStartTimeCmd( c, dt ) );
+            m->addCommand( new ModifyCompletionFinishedCmd( c, true ) );
+            m->addCommand( new ModifyCompletionFinishTimeCmd( c, dt ) );
+            m->addCommand( new ModifyCompletionPercentFinishedCmd( c, date, 100 ) );
+            emit executeCommand( m ); // also adds a new entry if necessary
+            return true;
+        }
+        return false;
+    }
     return false;
 }
 
@@ -392,7 +430,7 @@ bool TaskStatusItemModel::setRemainingEffort( Node *node, const QVariant &value,
         double d( value.toList()[0].toDouble() );
         Duration::Unit unit = static_cast<Duration::Unit>( value.toList()[1].toInt() );
         Duration dur( d, unit );
-        emit executeCommand( new ModifyCompletionRemainingEffortCmd( t->completion(), QDate::currentDate(), dur, i18n( "Modify Remainig Effort" ) ) );
+        emit executeCommand( new ModifyCompletionRemainingEffortCmd( t->completion(), QDate::currentDate(), dur, i18n( "Modify Remaining Effort" ) ) );
         return true;
     }
     return false;
@@ -419,7 +457,7 @@ bool TaskStatusItemModel::setStartedTime( Node *node, const QVariant &value, int
             if ( t == 0 ) {
                 return false;
             }
-            MacroCommand *m = new MacroCommand( headerData( NodeModel::NodeActualStart, Qt::Horizontal, Qt::DisplayRole ).toString() ); //FIXME: proper description when string freeze is lifted
+            MacroCommand *m = new MacroCommand( i18n( "Modify actual start time" ) );
             if ( ! t->completion().isStarted() ) {
                 m->addCommand( new ModifyCompletionStartedCmd( t->completion(), true ) );
             }
@@ -447,7 +485,7 @@ bool TaskStatusItemModel::setFinishedTime( Node *node, const QVariant &value, in
             if ( t == 0 ) {
                 return false;
             }
-            MacroCommand *m = new MacroCommand( headerData( NodeModel::NodeActualFinish, Qt::Horizontal, Qt::DisplayRole ).toString() ); //FIXME: proper description when string freeze is lifted
+            MacroCommand *m = new MacroCommand( i18n( "Modify actual finish time" ) );
             if ( ! t->completion().isFinished() ) {
                 m->addCommand( new ModifyCompletionFinishedCmd( t->completion(), true ) );
                 if ( t->completion().percentFinished() < 100 ) {
@@ -560,9 +598,10 @@ QVariant TaskStatusItemModel::alignment( int column ) const
     return QVariant();
 }
 
-QItemDelegate *TaskStatusItemModel::createDelegate( int column, QWidget *parent ) const
+QAbstractItemDelegate *TaskStatusItemModel::createDelegate( int column, QWidget *parent ) const
 {
     switch ( column ) {
+        case NodeModel::NodeCompleted: return new TaskCompleteDelegate( parent );
         case NodeModel::NodeRemainingEffort: return new DurationSpinBoxDelegate( parent );
         case NodeModel::NodeActualEffort: return new DurationSpinBoxDelegate( parent );
         default: return 0;
@@ -607,7 +646,7 @@ QMimeData *TaskStatusItemModel::mimeData( const QModelIndexList & indexes ) cons
     QByteArray encodedData;
     QDataStream stream(&encodedData, QIODevice::WriteOnly);
     QList<int> rows;
-    foreach (QModelIndex index, indexes) {
+    foreach (const QModelIndex &index, indexes) {
         if ( index.isValid() && !rows.contains( index.row() ) ) {
             //kDebug()<<index.row();
             Node *n = node( index );
@@ -675,6 +714,12 @@ void TaskStatusItemModel::slotWbsDefinitionChanged()
     }
 }
 
+void TaskStatusItemModel::slotLayoutChanged()
+{
+    //kDebug()<<node->name();
+    emit layoutAboutToBeChanged();
+    emit layoutChanged();
+}
 
 } // namespace KPlato
 
