@@ -74,14 +74,6 @@ CalendarTreeView::CalendarTreeView( QWidget *parent )
     setItemDelegateForColumn( 1, new EnumDelegate( this ) ); // timezone
     
     connect( header(), SIGNAL( customContextMenuRequested ( const QPoint& ) ), this, SLOT( headerContextMenuRequested( const QPoint& ) ) );
-    connect( this, SIGNAL( activated ( const QModelIndex ) ), this, SLOT( slotActivated( const QModelIndex ) ) );
-
-
-}
-
-void CalendarTreeView::slotActivated( const QModelIndex index )
-{
-    kDebug()<<index.column();
 }
 
 void CalendarTreeView::headerContextMenuRequested( const QPoint &pos )
@@ -112,7 +104,7 @@ void CalendarTreeView::focusOutEvent ( QFocusEvent * event )
 void CalendarTreeView::selectionChanged( const QItemSelection &sel, const QItemSelection &desel )
 {
     //kDebug()<<sel.indexes().count();
-    //foreach( QModelIndex i, selectionModel()->selectedIndexes() ) { kDebug()<<i.row()<<","<<i.column(); }
+    //foreach( const QModelIndex &i, selectionModel()->selectedIndexes() ) { kDebug()<<i.row()<<","<<i.column(); }
     TreeViewBase::selectionChanged( sel, desel );
     emit selectionChanged( selectionModel()->selectedIndexes() );
 }
@@ -142,7 +134,7 @@ Calendar *CalendarTreeView::selectedCalendar() const
 QList<Calendar*> CalendarTreeView::selectedCalendars() const
 {
     QList<Calendar *> lst;
-    foreach ( QModelIndex i, selectionModel()->selectedRows() ) {
+    foreach ( const QModelIndex &i, selectionModel()->selectedRows() ) {
         Calendar *a = model()->calendar( i );
         if ( a ) {
             lst << a;
@@ -196,7 +188,8 @@ void CalendarTreeView::dragMoveEvent(QDragMoveEvent *event)
 
 //--------------------
 CalendarDayView::CalendarDayView( QWidget *parent )
-    : QTableView( parent )
+    : QTableView( parent ),
+    m_readwrite( false )
 {
     setTabKeyNavigation( false );
     setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Fixed );
@@ -322,19 +315,17 @@ void CalendarDayView::setCurrentCalendar( Calendar *calendar )
     model()->setCalendar( calendar );
 }
 
-void CalendarDayView::slotActivated( const QModelIndex index )
-{
-    kDebug()<<index.column();
-}
-
 void CalendarDayView::headerContextMenuRequested( const QPoint &pos )
 {
-//    kDebug()<<k_funcinfo<<header()->logicalIndexAt(pos)<<" at"<<pos;
+//    kDebug()<<header()->logicalIndexAt(pos)<<" at"<<pos;
 }
 
 void CalendarDayView::contextMenuEvent ( QContextMenuEvent *event )
 {
     //kDebug();
+    if ( ! isReadWrite() ) {
+        return;
+    }
     KMenu menu;
     menu.addAction( actionSetWork );
     menu.addAction( actionSetVacation );
@@ -412,7 +403,7 @@ CalendarEditor::CalendarEditor( KoDocument *part, QWidget *parent )
     m_datePicker->dateTable()->setWeekNumbersEnabled( true );
     m_datePicker->dateTable()->setGridEnabled( true );
     m_datePicker->dateTable()->setSelectionMode( KDateTable::ExtendedSelection );
-    m_datePicker->dateTable()->setDateDelegate( new DateTableDateDelegate() );
+    m_datePicker->dateTable()->setDateDelegate( new DateTableDateDelegate( m_datePicker->dateTable() ) );
     m_datePicker->dateTable()->setModel( m_model );
     m_datePicker->dateTable()->setPopupMenuEnabled( true );
 
@@ -486,6 +477,9 @@ void CalendarEditor::setGuiActive( bool activate )
 void CalendarEditor::slotContextMenuDate( KMenu *menu, const QList<QDate> &dates )
 {
     kDebug()<<menu<<dates;
+    if ( ! isReadWrite() ) {
+        return;
+    }
     if ( dates.isEmpty() ) {
         m_currentMenuDateList << m_datePicker->date();
     } else {
@@ -499,7 +493,7 @@ void CalendarEditor::slotContextMenuDate( KMenu *menu, const QList<QDate> &dates
 void CalendarEditor::slotContextMenuDate( KMenu *menu, const QDate &date )
 {
     kDebug()<<menu<<date;
-    if ( ! date.isValid() ) {
+    if ( ! isReadWrite() || ! date.isValid() ) {
         return;
     }
     m_currentMenuDateList << date;
@@ -510,6 +504,9 @@ void CalendarEditor::slotContextMenuDate( KMenu *menu, const QDate &date )
 
 void CalendarEditor::slotContextMenuCalendar( QModelIndex index, const QPoint& pos )
 {
+    if ( ! isReadWrite() ) {
+        return;
+    }
     //kDebug()<<index.row()<<","<<index.column()<<":"<<pos;
     QString name;
 /*    if ( index.isValid() ) {
@@ -527,6 +524,9 @@ void CalendarEditor::slotContextMenuCalendar( QModelIndex index, const QPoint& p
 
 void CalendarEditor::slotContextMenuDay( QModelIndex index, const QPoint& pos )
 {
+    if ( ! isReadWrite() ) {
+        return;
+    }
     kDebug()<<index.row()<<","<<index.column()<<":"<<pos;
 /*    QString name;
     if ( index.isValid() ) {
@@ -592,12 +592,12 @@ void CalendarEditor::setupGui()
     KActionCollection *coll = actionCollection();
     QString name = "calendareditor_calendar_list";
     
-    actionAddCalendar   = new KAction(KIcon( "document-new" ), i18n("Add Calendar"), this);
+    actionAddCalendar   = new KAction(KIcon( "resource-calendar-insert" ), i18n("Add Calendar"), this);
     coll->addAction("add_calendar", actionAddCalendar  );
     actionAddCalendar ->setShortcut( KShortcut( Qt::CTRL + Qt::Key_I ) );
     connect( actionAddCalendar , SIGNAL( triggered( bool ) ), SLOT( slotAddCalendar () ) );
     
-    actionAddSubCalendar   = new KAction(KIcon( "document-new" ), i18n("Add Subcalendar"), this);
+    actionAddSubCalendar   = new KAction(KIcon( "resource-calendar-child-insert" ), i18n("Add Subcalendar"), this);
     coll->addAction("add_subcalendar", actionAddSubCalendar  );
     actionAddSubCalendar ->setShortcut( KShortcut( Qt::SHIFT + Qt::CTRL + Qt::Key_I ) );
     connect( actionAddSubCalendar , SIGNAL( triggered( bool ) ), SLOT( slotAddSubCalendar () ) );
@@ -623,7 +623,8 @@ void CalendarEditor::setupGui()
 void CalendarEditor::updateReadWrite( bool readwrite )
 {
     m_calendarview->setReadWrite( readwrite );
-//    m_dayview->setReadWrite( readwrite );
+    m_dayview->setReadWrite( readwrite );
+    ViewBase::updateReadWrite( readwrite );
 }
 
 void CalendarEditor::slotAddCalendar ()
