@@ -26,13 +26,13 @@
 #include "kpttask.h"
 #include "kptresource.h"
 #include "kptdatetime.h"
-#include "kpttaskappointmentsview.h"
 #include "kptrelation.h"
 #include "kptschedule.h"
 #include "kptviewbase.h"
 #include "kptitemviewsettup.h"
 #include "kptduration.h"
 #include "kptdatetime.h"
+#include "kptresourceappointmentsmodel.h"
 
 #include <kdganttproxymodel.h>
 #include <kdganttconstraintmodel.h>
@@ -328,7 +328,7 @@ MyKDGanttView::MyKDGanttView( QWidget *parent )
         }
     }
 
-    setConstraintModel( new KDGantt::ConstraintModel() );
+    setConstraintModel( new KDGantt::ConstraintModel( this ) );
     KDGantt::ProxyModel *m = static_cast<KDGantt::ProxyModel*>( ganttProxyModel() );
 
     m->setRole( KDGantt::ItemTypeRole, KDGantt::ItemTypeRole ); // To provide correct format
@@ -341,7 +341,7 @@ MyKDGanttView::MyKDGanttView( QWidget *parent )
     m->setColumn( KDGantt::TaskCompletionRole, NodeModel::NodeCompleted );
     
     static_cast<KDGantt::DateTimeGrid*>( grid() )->setDayWidth( 30 );
-    static_cast<KDGantt::DateTimeGrid*>( grid() )->setRowSeparators( treeView()->alternatingRowColors() );
+    //static_cast<KDGantt::DateTimeGrid*>( grid() )->setRowSeparators( treeView()->alternatingRowColors() );
     
     connect( model(), SIGNAL( nodeInserted( Node* ) ), this, SLOT( slotNodeInserted( Node* ) ) );
 }
@@ -450,7 +450,6 @@ void MyKDGanttView::createDependencies()
 GanttView::GanttView( KoDocument *part, QWidget *parent, bool readWrite )
     : ViewBase( part, parent ),
     m_readWrite( readWrite ),
-    m_taskView( 0 ),
     m_project( 0 )
 {
     kDebug() <<" ---------------- KPlato: Creating GanttView ----------------";
@@ -465,9 +464,6 @@ GanttView::GanttView( KoDocument *part, QWidget *parent, bool readWrite )
 
     setupGui();
     
-    m_taskView = new TaskAppointmentsView( m_splitter );
-    m_taskView->hide();
-
     updateReadWrite( readWrite );
     //connect( m_gantt->constraintModel(), SIGNAL( constraintAdded( const Constraint& )), this, SLOT( update() ) );
     kDebug() <<m_gantt->constraintModel();
@@ -486,7 +482,6 @@ void GanttView::setZoom( double )
 {
     //kDebug() <<"setting gantt zoom:" << zoom;
     //m_gantt->setZoomFactor(zoom,true); NO!!! setZoomFactor() is something else
-    //m_taskView->setZoom( zoom );
 }
 
 void GanttView::setupGui()
@@ -507,7 +502,6 @@ void GanttView::slotOptions()
 void GanttView::clear()
 {
 //    m_gantt->clear();
-    m_taskView->clear();
 }
 
 void GanttView::setShowResources( bool on )
@@ -598,6 +592,7 @@ void GanttView::slotContextMenuRequested( QModelIndex idx, const QPoint &pos )
                 name = "taskview_milestone_popup";
                 break;
             case Node::Type_Summarytask:
+                name = "taskview_summary_popup";
                 break;
             default:
                 break;
@@ -666,7 +661,7 @@ MilestoneKDGanttView::MilestoneKDGanttView( QWidget *parent )
     m->setColumn( KDGantt::TaskCompletionRole, NodeModel::NodeCompleted );
 
     static_cast<KDGantt::DateTimeGrid*>( grid() )->setDayWidth( 30 );
-    static_cast<KDGantt::DateTimeGrid*>( grid() )->setRowSeparators( treeView()->alternatingRowColors() );
+    //static_cast<KDGantt::DateTimeGrid*>( grid() )->setRowSeparators( treeView()->alternatingRowColors() );
 }
 
 MilestoneItemModel *MilestoneKDGanttView::model() const
@@ -815,6 +810,7 @@ void MilestoneGanttView::slotContextMenuRequested( QModelIndex idx, const QPoint
                 name = "taskview_milestone_popup";
                 break;
             case Node::Type_Summarytask:
+                name = "taskview_summary_popup";
                 break;
             default:
                 break;
@@ -858,6 +854,128 @@ KoPrintJob *MilestoneGanttView::createPrintJob()
     return new GanttPrintingDialog( this, m_gantt );
 }
 
+//------------------------------------------
+
+ResourceAppointmentsGanttView::ResourceAppointmentsGanttView( KoDocument *part, QWidget *parent, bool readWrite )
+    : ViewBase( part, parent ),
+    m_project( 0 ),
+    m_model( new ResourceAppointmentsGanttModel( this ) )
+{
+    kDebug() <<" ---------------- KPlato: Creating ResourceAppointmentsGanttView ----------------";
+
+    m_gantt = new KDGantt::View( this );
+    m_gantt->graphicsView()->setItemDelegate( new ResourceGanttItemDelegate( m_gantt ) );
+
+    GanttTreeView *tv = new GanttTreeView( m_gantt );
+    tv->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+    tv->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+    tv->setVerticalScrollMode( QAbstractItemView::ScrollPerPixel ); // needed since qt 4.2
+    m_gantt->setLeftView( tv );
+    m_gantt->setRowController( new KDGantt::TreeViewRowController( tv, m_gantt->ganttProxyModel() ) );
+    tv->header()->setStretchLastSection( true );
+    
+
+    KDGantt::ProxyModel *m = static_cast<KDGantt::ProxyModel*>( m_gantt->ganttProxyModel() );
+    m->setRole( KDGantt::ItemTypeRole, KDGantt::ItemTypeRole );
+    m->setRole( KDGantt::StartTimeRole, KDGantt::StartTimeRole );
+    m->setRole( KDGantt::EndTimeRole, KDGantt::EndTimeRole );
+    m->setRole( KDGantt::TaskCompletionRole, KDGantt::TaskCompletionRole );
+
+    m_gantt->setModel( m_model );
+
+    QVBoxLayout *l = new QVBoxLayout( this );
+    l->setMargin( 0 );
+    l->addWidget( m_gantt );
+
+    setupGui();
+
+    updateReadWrite( readWrite );
+
+    connect( m_gantt->leftView(), SIGNAL( contextMenuRequested( QModelIndex, const QPoint& ) ), SLOT( slotContextMenuRequested( QModelIndex, const QPoint& ) ) );
+
+    connect( m_gantt->leftView(), SIGNAL( headerContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
+}
+
+void ResourceAppointmentsGanttView::setZoom( double )
+{
+    //kDebug() <<"setting gantt zoom:" << zoom;
+    //m_gantt->setZoomFactor(zoom,true); NO!!! setZoomFactor() is something else
+}
+
+Project *ResourceAppointmentsGanttView::project() const
+{
+    return m_model->project();
+}
+
+void ResourceAppointmentsGanttView::setProject( Project *project )
+{
+    static_cast<KDGantt::DateTimeGrid*>( m_gantt->grid() )->setStartDateTime( project->startTime().dateTime() );
+    m_model->setProject( project );
+}
+
+void ResourceAppointmentsGanttView::setScheduleManager( ScheduleManager *sm )
+{
+    //kDebug()<<id<<endl;
+    m_model->setScheduleManager( sm );
+}
+
+void ResourceAppointmentsGanttView::setupGui()
+{
+    createOptionAction();
+}
+
+Node *ResourceAppointmentsGanttView::currentNode() const
+{
+    QModelIndex idx = treeView()->selectionModel()->currentIndex();
+    return m_model->node( idx );
+}
+
+void ResourceAppointmentsGanttView::slotContextMenuRequested( QModelIndex idx, const QPoint &pos )
+{
+    kDebug();
+    QString name;
+    if ( idx.isValid() ) {
+        Node *n = m_model->node( idx );
+        if ( n ) {
+            name = "taskview_popup";
+        }
+    }
+    if ( name.isEmpty() ) {
+        slotHeaderContextMenuRequested( pos );
+        return;
+    }
+    emit requestPopupMenu( name, pos );
+}
+
+void ResourceAppointmentsGanttView::slotOptions()
+{
+    kDebug();
+    ItemViewSettupDialog *dlg = new ItemViewSettupDialog( treeView(), true, this );
+    dlg->exec();
+    delete dlg;
+}
+
+bool ResourceAppointmentsGanttView::loadContext( const KoXmlElement &settings )
+{
+    kDebug();
+    return treeView()->loadContext( m_model->columnMap(), settings );;
+}
+
+void ResourceAppointmentsGanttView::saveContext( QDomElement &settings ) const
+{
+    kDebug();
+    treeView()->saveContext( m_model->columnMap(), settings );
+}
+
+void ResourceAppointmentsGanttView::updateReadWrite( bool on )
+{
+    m_readWrite = on;
+}
+
+KoPrintJob *ResourceAppointmentsGanttView::createPrintJob()
+{
+    return 0;
+}
 
 }  //KPlato namespace
 
