@@ -29,6 +29,7 @@
 #include "kptdatetime.h"
 #include "kptpertresult.h"
 #include "kptitemviewsettup.h"
+#include "kptrecalculatedialog.h"
 
 #include <KoDocument.h>
 
@@ -37,6 +38,8 @@
 #include <QObject>
 #include <QVBoxLayout>
 #include <QHeaderView>
+#include <QApplication>
+#include <QClipboard>
 
 #include <kaction.h>
 #include <kicon.h>
@@ -44,6 +47,7 @@
 #include <klocale.h>
 #include <kxmlguifactory.h>
 #include <kactioncollection.h>
+#include <QKeyEvent>
 
 #include <kdebug.h>
 
@@ -64,15 +68,10 @@ ScheduleTreeView::ScheduleTreeView( QWidget *parent )
     createItemDelegates( m );
 }
 
-void ScheduleTreeView::slotActivated( const QModelIndex index )
-{
-    kDebug()<<index.column();
-}
-
 void ScheduleTreeView::selectionChanged( const QItemSelection &sel, const QItemSelection &desel )
 {
     //kDebug()<<sel.indexes().count();
-    foreach( QModelIndex i, selectionModel()->selectedIndexes() ) {
+    foreach( const QModelIndex &i, selectionModel()->selectedIndexes() ) {
         //kDebug()<<i.row()<<","<<i.column();
     }
     QTreeView::selectionChanged( sel, desel );
@@ -112,7 +111,9 @@ ScheduleEditor::ScheduleEditor( KoDocument *part, QWidget *parent )
         << ScheduleModel::ScheduleOverbooking
         << ScheduleModel::ScheduleDistribution
         << ScheduleModel::SchedulePlannedStart
-        << ScheduleModel::SchedulePlannedFinish;
+        << ScheduleModel::SchedulePlannedFinish
+        //<< ScheduleModel::ScheduleScheduler
+        ;
 
     QList<int> lst;
     for ( int c = 0; c < model()->columnCount(); ++c ) {
@@ -200,6 +201,7 @@ void ScheduleEditor::slotEnableActions( const ScheduleManager *sm )
     bool on = isReadWrite() && sm != 0;
     actionAddSubSchedule->setEnabled( on );
     
+    actionBaselineSchedule->setIcon( KIcon( ( sm && sm->isBaselined() ? "view-time-schedule-baselined-remove" : "view-time-schedule-baselined-add" ) ) );
     if ( on && ( sm->isBaselined() || sm->isChildBaselined() ) ) {
         actionBaselineSchedule->setEnabled( sm->isBaselined() );
         actionCalculateSchedule->setEnabled( false );
@@ -215,13 +217,13 @@ void ScheduleEditor::setupGui()
 {
     QString name = "scheduleeditor_edit_list";
     
-    actionAddSchedule  = new KAction(KIcon( "document-new" ), i18n("Add Schedule"), this);
+    actionAddSchedule  = new KAction(KIcon( "view-time-schedule-insert" ), i18n("Add Schedule"), this);
     actionAddSchedule->setShortcut( KShortcut( Qt::CTRL + Qt::Key_I ) );
     actionCollection()->addAction("add_schedule", actionAddSchedule );
     connect( actionAddSchedule, SIGNAL( triggered( bool ) ), SLOT( slotAddSchedule() ) );
     addAction( name, actionAddSchedule );
     
-    actionAddSubSchedule  = new KAction(KIcon( "document-new" ), i18n("Add Sub-schedule"), this);
+    actionAddSubSchedule  = new KAction(KIcon( "view-time-schedule-child-insert" ), i18n("Add Sub-schedule"), this);
     actionAddSubSchedule->setShortcut( KShortcut( Qt::CTRL + Qt::SHIFT + Qt::Key_I ) );
     actionCollection()->addAction("add_subschedule", actionAddSubSchedule );
     connect( actionAddSubSchedule, SIGNAL( triggered( bool ) ), SLOT( slotAddSubSchedule() ) );
@@ -233,13 +235,13 @@ void ScheduleEditor::setupGui()
     connect( actionDeleteSelection, SIGNAL( triggered( bool ) ), SLOT( slotDeleteSelection() ) );
     addAction( name, actionDeleteSelection );
 
-    actionCalculateSchedule  = new KAction(KIcon( "project_calculate" ), i18n("Calculate"), this);
+    actionCalculateSchedule  = new KAction(KIcon( "view-time-schedule-calculus" ), i18n("Calculate"), this);
 //    actionCalculateSchedule->setShortcut( KShortcut( Qt::CTRL + Qt::Key_C ) );
     actionCollection()->addAction("calculate_schedule", actionCalculateSchedule );
     connect( actionCalculateSchedule, SIGNAL( triggered( bool ) ), SLOT( slotCalculateSchedule() ) );
     addAction( name, actionCalculateSchedule );
     
-    actionBaselineSchedule  = new KAction(KIcon( "" ), i18n("Baseline"), this);
+    actionBaselineSchedule  = new KAction(KIcon( "view-time-schedule-baselined-add" ), i18n("Baseline"), this);
 //    actionBaselineSchedule->setShortcut( KShortcut( Qt::CTRL + Qt::Key_B ) );
     actionCollection()->addAction("schedule_baseline", actionBaselineSchedule );
     connect( actionBaselineSchedule, SIGNAL( triggered( bool ) ), SLOT( slotBaselineSchedule() ) );
@@ -274,7 +276,14 @@ void ScheduleEditor::slotCalculateSchedule()
     if ( sm == 0 ) {
         return;
     }
-    sm->setRecalculate( sm->parentManager() );
+    if ( sm->parentManager() ) {
+        RecalculateDialog dlg;
+        if ( dlg.exec() == QDialog::Rejected ) {
+            return;
+        }
+        sm->setRecalculate( true );
+        sm->setRecalculateFrom( DateTime( KDateTime( dlg.dateTime() ) ) );
+    }
     emit calculateSchedule( m_view->project(), sm );
 }
 
@@ -286,8 +295,23 @@ void ScheduleEditor::slotAddSchedule()
         sm = sm->parentManager();
         ScheduleManager *m = m_view->project()->createScheduleManager( sm->name() + QString(".%1").arg( sm->children().count() + 1 ) );
         part()->addCommand( new AddScheduleManagerCmd( sm, m, i18n( "Create sub-schedule" ) ) );
+        m_view->expand( model()->index( sm ) );
+        QModelIndex idx = model()->index( m );
+        if ( idx.isValid() ) {
+            m_view->selectionModel()->select( idx, QItemSelectionModel::Rows | QItemSelectionModel::ClearAndSelect );
+            m_view->selectionModel()->setCurrentIndex( idx, QItemSelectionModel::NoUpdate );
+        }
     } else {
-        emit addScheduleManager( m_view->project() );
+        Project *p = m_view->project();
+        ScheduleManager *m = p->createScheduleManager();
+        AddScheduleManagerCmd *cmd =  new AddScheduleManagerCmd( *p, m, i18n( "Add schedule %1", m->name() ) );
+        part() ->addCommand( cmd );
+        m_view->expand( model()->index( m ) );
+        QModelIndex idx = model()->index( m );
+        if ( idx.isValid() ) {
+            m_view->selectionModel()->select( idx, QItemSelectionModel::Rows | QItemSelectionModel::ClearAndSelect );
+            m_view->selectionModel()->setCurrentIndex( idx, QItemSelectionModel::NoUpdate );
+        }
     }
 }
 
@@ -297,10 +321,15 @@ void ScheduleEditor::slotAddSubSchedule()
     ScheduleManager *sm = m_view->currentManager();
     if ( sm ) {
         ScheduleManager *m = m_view->project()->createScheduleManager( sm->name() + QString(".%1").arg( sm->children().count() + 1 ) );
-        
         part()->addCommand( new AddScheduleManagerCmd( sm, m, i18n( "Create sub-schedule" ) ) );
+        m_view->expand( model()->index( sm ) );
+        QModelIndex idx = model()->index( m );
+        if ( idx.isValid() ) {
+            m_view->selectionModel()->select( idx, QItemSelectionModel::Rows | QItemSelectionModel::ClearAndSelect );
+            m_view->selectionModel()->setCurrentIndex( idx, QItemSelectionModel::NoUpdate );
+        }
     } else {
-        emit addScheduleManager( m_view->project() );
+        slotAddSchedule();
     }
 }
 
@@ -354,17 +383,20 @@ ScheduleLogTreeView::ScheduleLogTreeView( QWidget *parent )
     setModel( m_model );
     
     setRootIsDecorated( false );
-    setSelectionMode( QAbstractItemView::SingleSelection );
+    setSelectionMode( QAbstractItemView::ExtendedSelection );
     setSelectionBehavior( QAbstractItemView::SelectRows );
     
     connect( header(), SIGNAL( customContextMenuRequested ( const QPoint& ) ), this, SLOT( headerContextMenuRequested( const QPoint& ) ) );
-    connect( this, SIGNAL( activated ( const QModelIndex ) ), this, SLOT( slotActivated( const QModelIndex ) ) );
-
 }
 
-void ScheduleLogTreeView::slotActivated( const QModelIndex index )
+void ScheduleLogTreeView::setFilterWildcard( const QString &filter )
 {
-    kDebug()<<index.column();
+    m_model->setFilterWildcard( filter );
+}
+
+QRegExp ScheduleLogTreeView::filterRegExp() const
+{
+    return m_model->filterRegExp();
 }
 
 void ScheduleLogTreeView::headerContextMenuRequested( const QPoint &pos )
@@ -375,7 +407,7 @@ void ScheduleLogTreeView::headerContextMenuRequested( const QPoint &pos )
 void ScheduleLogTreeView::selectionChanged( const QItemSelection &sel, const QItemSelection &desel )
 {
     //kDebug()<<sel.indexes().count();
-    foreach( QModelIndex i, selectionModel()->selectedIndexes() ) {
+    foreach( const QModelIndex &i, selectionModel()->selectedIndexes() ) {
         //kDebug()<<i.row()<<","<<i.column();
     }
     QTreeView::selectionChanged( sel, desel );
@@ -390,6 +422,33 @@ void ScheduleLogTreeView::currentChanged( const QModelIndex & current, const QMo
 //    selectionModel()->select( current, QItemSelectionModel::Rows | QItemSelectionModel::ClearAndSelect );
 }
 
+void ScheduleLogTreeView::slotEditCopy()
+{
+    QStringList lst;
+    int row = 0;
+    QString s;
+    QHeaderView *h = header();
+    foreach( const QModelIndex &i, selectionModel()->selectedRows() ) {
+        QString s;
+        for ( int section = 0; section < h->count(); ++section ) {
+            QModelIndex idx = model()->index( i.row(), h->logicalIndex( section ) );
+            if ( ! idx.isValid() || isColumnHidden( idx.column() ) ) {
+                continue;
+            }
+            if ( ! s.isEmpty() ) {
+                s += " ";
+            }
+            s = QString( "%1%2" ).arg( s ).arg( idx.data().toString(), -10 );
+        }
+        if ( ! s.isEmpty() ) {
+            lst << s;
+        }
+    }
+    if ( ! lst.isEmpty() ) {
+        QApplication::clipboard()->setText( lst.join( "\n" ) );
+    }
+}
+
 //-----------------------------------
 ScheduleLogView::ScheduleLogView( KoDocument *part, QWidget *parent )
     : ViewBase( part, parent )
@@ -412,7 +471,6 @@ ScheduleLogView::ScheduleLogView( KoDocument *part, QWidget *parent )
 
     connect( m_view, SIGNAL( contextMenuRequested( QModelIndex, const QPoint& ) ), this, SLOT( slotContextMenuRequested( QModelIndex, const QPoint& ) ) );
     
-    connect( m_view, SIGNAL( headerContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
 }
 
 void ScheduleLogView::setProject( Project *project )
@@ -483,6 +541,10 @@ void ScheduleLogView::slotOptions()
     kDebug();
 }
 
+void ScheduleLogView::slotEditCopy()
+{
+    m_view->slotEditCopy();
+}
 
 bool ScheduleLogView::loadContext( const KoXmlElement &context )
 {
@@ -533,20 +595,8 @@ ViewBase *ScheduleHandlerView::hitView( const QPoint &glpos )
 
 void ScheduleHandlerView::setGuiActive( bool active ) // virtual slot
 {
-    //kDebug()<<active;
-    for ( int i = 0; i < m_splitter->count(); ++i ) {
-        ViewBase *v = dynamic_cast<ViewBase*>( m_splitter->widget( i ) );
-        if ( v  ) {
-            v->setGuiActive( active );
-        } else {
-            QTabWidget *tw = dynamic_cast<QTabWidget*>( m_splitter->widget( i ) );
-            if (tw ) {
-                v = dynamic_cast<ViewBase*>( tw->currentWidget() );
-                if ( v ) {
-                    v->setGuiActive( active );
-                }
-            }
-        }
+    foreach ( ViewBase *v, findChildren<ViewBase*>() ) {
+        v->setGuiActive( active );
     }
     m_activeview = active ? this : 0;
     emit guiActivated( this, active );
@@ -559,19 +609,9 @@ void ScheduleHandlerView::slotGuiActivated( ViewBase *, bool )
 QStringList ScheduleHandlerView::actionListNames() const
 {
     QStringList lst;
-    for ( int i = 0; i < m_splitter->count(); ++i ) {
-        ViewBase *v = dynamic_cast<ViewBase*>( m_splitter->widget( i ) );
-        if ( v  ) {
-            lst += v->actionListNames();
-        } else {
-            QTabWidget *tw = dynamic_cast<QTabWidget*>( m_splitter->widget( i ) );
-            if (tw ) {
-                v = dynamic_cast<ViewBase*>( tw->currentWidget() );
-                lst += v->actionListNames();
-            }
-        }
+    foreach ( ViewBase *v, findChildren<ViewBase*>() ) {
+        lst += v->actionListNames();
     }
-    //kDebug()<<lst;
     return lst;
 }
     
@@ -579,21 +619,9 @@ QList<QAction*> ScheduleHandlerView::actionList( const QString name ) const
 {
     //kDebug()<<name;
     QList<QAction*> lst;
-    for ( int i = 0; i < m_splitter->count(); ++i ) {
-        ViewBase *v = dynamic_cast<ViewBase*>( m_splitter->widget( i ) );
-        if ( v ) {
-            lst += v->actionList( name );
-        } else {
-            QTabWidget *tw = dynamic_cast<QTabWidget*>( m_splitter->widget( i ) );
-            if (tw ) {
-                v = dynamic_cast<ViewBase*>( tw->currentWidget() );
-                if ( v  ) {
-                    lst += v->actionList( name );
-                }
-            }
-        }
+    foreach ( ViewBase *v, findChildren<ViewBase*>() ) {
+        lst += v->actionList( name );
     }
-    //kDebug()<<lst;
     return lst;
 }
 
