@@ -91,7 +91,7 @@ QVariant UsedEffortItemModel::data ( const QModelIndex &index, int role ) const
                 // Total
                 //kDebug()<<index.row()<<","<<index.column()<<" total"<<endl;
                 double res = 0.0;
-                foreach ( QDate d, m_dates ) {
+                foreach ( const QDate &d, m_dates ) {
                     Completion::UsedEffort::ActualEffort *e = ue->effort( d );
                     if ( e ) {
                         res += e->normalEffort().toDouble( Duration::Unit_h );
@@ -150,7 +150,7 @@ QVariant UsedEffortItemModel::data ( const QModelIndex &index, int role ) const
 
 bool UsedEffortItemModel::setData ( const QModelIndex &idx, const QVariant &value, int role )
 {
-    kDebug()<<endl;
+    kDebug();
     switch ( role ) {
         case Qt::EditRole: {
             if ( idx.column() == 8 ) {
@@ -194,13 +194,13 @@ bool UsedEffortItemModel::setData ( const QModelIndex &idx, const QVariant &valu
 
 bool UsedEffortItemModel::submit()
 {
-    kDebug()<<endl;
+    kDebug();
     return QAbstractItemModel::submit();
 }
 
 void UsedEffortItemModel::revert()
 {
-    kDebug()<<endl;
+    kDebug();
     QList<const Resource*> lst = m_resourcelist;
     foreach ( const Resource *r, lst ) {
         if ( ! m_completion->usedEffortMap().contains( r ) ) {
@@ -749,14 +749,22 @@ QModelIndex CompletionEntryItemModel::addRow()
 
 void CompletionEntryItemModel::removeEntry( const QDate date )
 {
-    int i = m_datelist.indexOf( date );
-    if ( i != -1 ) {
-        beginRemoveRows( QModelIndex(), i, i );
-        m_datelist.removeAt( i );
-        endRemoveRows();
-        kDebug()<<date<<" removed"<<endl;
+    removeRow( m_datelist.indexOf( date ) );
+}
+
+void CompletionEntryItemModel::removeRow( int row )
+{
+    kDebug()<<row;
+    if ( row < 0 && row >= rowCount() ) {
+        return;
     }
+    QDate date = m_datelist.value( row );
+    beginRemoveRows( QModelIndex(), row, row );
+    m_datelist.removeAt( row );
+    endRemoveRows();
+    kDebug()<<date<<" removed row"<<row;
     m_completion->takeEntry( date );
+    emit changed();
 }
 
 void CompletionEntryItemModel::addEntry( const QDate date )
@@ -789,15 +797,16 @@ CompletionEntryEditor::CompletionEntryEditor( QWidget *parent )
     CompletionEntryItemModel *m = new CompletionEntryItemModel(this );
     setModel( m );
     
-//    setItemDelegateForColumn ( 1, new DoubleSpinBoxDelegate( this ) );
+    setItemDelegateForColumn ( 1, new ProgressBarDelegate( this ) );
     setItemDelegateForColumn ( 2, new DurationSpinBoxDelegate( this ) );
     setItemDelegateForColumn ( 3, new DurationSpinBoxDelegate( this ) );
     
     connect ( m, SIGNAL( rowInserted( const QDate ) ), SIGNAL( rowInserted( const QDate ) ) );
+    connect ( m, SIGNAL( rowRemoved( const QDate ) ), SIGNAL( rowRemoved( const QDate ) ) );
     connect ( model(), SIGNAL( dataChanged( const QModelIndex&, const QModelIndex& ) ), SIGNAL( changed() ) );
+    connect ( model(), SIGNAL( changed() ), SIGNAL( changed() ) );
 
     connect( selectionModel(), SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ), SIGNAL( selectionChanged( const QItemSelection&, const QItemSelection& ) ) );
-
 }
 
 void CompletionEntryEditor::setCompletion( Completion *completion )
