@@ -17,6 +17,7 @@
  */
 
 #include "kis_config_widget.h"
+#include "kis_debug.h"
 #include <QTimer>
 
 KisConfigWidget::KisConfigWidget(QWidget * parent, Qt::WFlags f)
@@ -24,7 +25,7 @@ KisConfigWidget::KisConfigWidget(QWidget * parent, Qt::WFlags f)
 {
     m_timer.setSingleShot(true);
     connect(&m_timer, SIGNAL(timeout()), SLOT(slotConfigChanged()));
-    connect(this, SIGNAL(sigConfigChanged()), SLOT(kickTimer()));
+    connect(this, SIGNAL(sigConfigurationItemChanged()), SLOT(kickTimer()));
 }
 
 KisConfigWidget::~KisConfigWidget()
@@ -33,12 +34,12 @@ KisConfigWidget::~KisConfigWidget()
 
 void KisConfigWidget::slotConfigChanged()
 {
-    emit sigPleaseUpdatePreview();
+    emit sigConfigurationUpdated();
 }
 
 void KisConfigWidget::kickTimer()
 {
-    m_timer.start(50);
+    m_timer.start(500);
 }
 
 #include "kis_config_widget.moc"
