@@ -44,6 +44,8 @@
 #include "recorder/kis_action_recorder.h"
 #include "kis_adjustment_layer.h"
 #include "kis_annotation.h"
+#include "kis_projection.h"
+#include "kis_background.h"
 #include "kis_change_profile_visitor.h"
 #include "kis_colorspace_convert_visitor.h"
 #include "kis_count_visitor.h"
@@ -52,7 +54,7 @@
 #include "commands/kis_image_commands.h"
 #include "kis_iterators_pixel.h"
 #include "kis_layer.h"
-
+#include "kis_meta_data_merge_strategy_registry.h"
 #include "kis_name_server.h"
 #include "kis_paint_device.h"
 #include "kis_paint_device_action.h"
@@ -65,13 +67,14 @@
 #include "kis_transform_visitor.h"
 #include "kis_types.h"
 #include "kis_crop_visitor.h"
-#include "kis_projection_update_strategy.h"
 #include "kis_meta_data_merge_strategy.h"
 
+#include "kis_refresh_visitor.h"
+
 class KisImage::KisImagePrivate
 {
 public:
-    KoColor backgroundColor;
+    KisBackgroundSP  backgroundPattern;
     quint32 lockCount;
     bool sizeChangedWhileLocked;
     KisPerspectiveGrid* perspectiveGrid;
@@ -98,13 +101,19 @@ public:
     KisSelectionSP globalSelection;
     KisSelectionSP deselectedGlobalSelection;
 
+    KisProjection* projection;
+
+    bool startProjection;
+
 };
-KisImage::KisImage(KisUndoAdapter *adapter, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name)
+KisImage::KisImage(KisUndoAdapter *adapter, qint32 width, qint32 height, const KoColorSpace * colorSpace, const QString& name, bool startProjection)
         : QObject(0)
         , KisShared()
         , m_d(new KisImagePrivate())
 {
     setObjectName(name);
+    dbgImage << "creating" << name;
+    m_d->startProjection = startProjection;
     init(adapter, width, height, colorSpace);
 }
 
@@ -115,8 +124,11 @@ KisImage::KisImage(const KisImage& rhs)
         , KisShared(rhs)
         , m_d(new KisImagePrivate())
 {
+
     if (this != &rhs) {
 
+        dbgImage << "copying" << objectName() << "from" << rhs.objectName();
+
         if (rhs.m_d->perspectiveGrid)
             m_d->perspectiveGrid = new KisPerspectiveGrid(*rhs.m_d->perspectiveGrid);
         else
@@ -133,13 +145,25 @@ KisImage::KisImage(const KisImage& rhs)
         m_d->deselectedGlobalSelection = 0;
         setRootLayer(static_cast<KisGroupLayer*>(rhs.m_d->rootLayer->clone().data()));
         m_d->annotations = rhs.m_d->annotations; // XXX the annotations would probably need to be deep-copied
-        m_d->nserver = new KisNameServer( *rhs.m_d->nserver );
+        m_d->nserver = new KisNameServer(*rhs.m_d->nserver);
+        m_d->startProjection = rhs.m_d->startProjection;
         Q_CHECK_PTR(m_d->nserver);
 
+        m_d->projection = 0;
+        if (m_d->startProjection) {
+            m_d->projection = new KisProjection(this);
+            m_d->projection->start();
+        }
     }
 }
+
 KisImage::~KisImage()
 {
+    dbgImage << "deleting kisimage" << objectName();
+    if (m_d->projection) {
+        m_d->projection->stop();
+        m_d->projection->deleteLater();
+    }
     delete m_d->perspectiveGrid;
     delete m_d->nserver;
     delete m_d;
@@ -153,22 +177,8 @@ void KisImage::aboutToAddANode(KisNode *parent, int index)
 
 void KisImage::nodeHasBeenAdded(KisNode *parent, int index)
 {
-    // XXX: Temporarily for compatibility
-
     KisLayer * layer = dynamic_cast<KisLayer*>(parent->at(index).data());
     if (layer) {
-
-        KisPaintLayerSP player = KisPaintLayerSP(dynamic_cast<KisPaintLayer*>(layer));
-        if (!player.isNull()) {
-            // XXX: This should also be done whenever a layer grows!
-            QList<KisPaintDeviceAction *> actions =
-                KoColorSpaceRegistry::instance()->paintDeviceActionsFor(player->paintDevice()->colorSpace());
-            for (int i = 0; i < actions.count(); i++) {
-                actions.at(i)->act(player.data()->paintDevice(), width(), height());
-            }
-
-        }
-
         // The addition of temporary layers is not interesting
         KoProperties props;
         props.setProperty("temporary", false);
@@ -202,14 +212,14 @@ void KisImage::nodeHasBeenRemoved(KisNode *parent, int index)
     }
 }
 
-void KisImage::aboutToMoveNode(KisNode * parent, int oldIndex, int newIndex)
+void KisImage::aboutToMoveNode(KisNode *parent, int oldIndex, int newIndex)
 {
     Q_UNUSED(parent);
     Q_UNUSED(oldIndex);
     Q_UNUSED(newIndex);
 }
 
-void KisImage::nodeHasBeenMoved(KisNode * parent, int oldIndex, int newIndex)
+void KisImage::nodeHasBeenMoved(KisNode *parent, int oldIndex, int newIndex)
 {
     Q_UNUSED(oldIndex);
 
@@ -257,18 +267,19 @@ void KisImage::setDeleselectedGlobalSelection(KisSelectionSP selection)
     m_d->deselectedGlobalSelection = selection;
 }
 
-
-KoColor KisImage::backgroundColor() const
+KisBackgroundSP KisImage::backgroundPattern() const
 {
-    return m_d->backgroundColor;
+    return m_d->backgroundPattern;
 }
 
-void KisImage::setBackgroundColor(const KoColor & color)
+void KisImage::setBackgroundPattern(KisBackgroundSP background)
 {
-    m_d->backgroundColor = color;
+    if (background != m_d->backgroundPattern) {
+        m_d->backgroundPattern = background;
+        emit sigImageUpdated(bounds());
+    }
 }
 
-
 QString KisImage::nextLayerName() const
 {
     if (m_d->nserver->currentSeed() == 0) {
@@ -284,13 +295,12 @@ void KisImage::rollBackLayerName()
     m_d->nserver->rollback();
 }
 
-void KisImage::init(KisUndoAdapter *adapter, qint32 width, qint32 height, const KoColorSpace * colorSpace)
+void KisImage::init(KisUndoAdapter *adapter, qint32 width, qint32 height, const KoColorSpace *colorSpace)
 {
     if (colorSpace == 0) {
         colorSpace = KoColorSpaceRegistry::instance()->rgb8();
     }
 
-    m_d->backgroundColor = KoColor(Qt::white, colorSpace);
     m_d->lockCount = 0;
     m_d->sizeChangedWhileLocked = false;
     m_d->perspectiveGrid = 0;
@@ -309,7 +319,13 @@ void KisImage::init(KisUndoAdapter *adapter, qint32 width, qint32 height, const
     m_d->width = width;
     m_d->height = height;
 
-    m_d->recorder = new KisActionRecorder(this);
+    m_d->recorder = new KisActionRecorder();
+
+    m_d->projection = 0;
+    if (m_d->startProjection) {
+        m_d->projection = new KisProjection(this);
+        m_d->projection->start();
+    }
 }
 
 bool KisImage::locked() const
@@ -320,10 +336,10 @@ bool KisImage::locked() const
 void KisImage::lock()
 {
     if (!locked()) {
-        if (m_d->rootLayer)
-            m_d->rootLayer->updateStrategy()->lock();
+        if (m_d->projection) {
+            m_d->projection->lock();
+        }
         m_d->sizeChangedWhileLocked = false;
-//         blockSignals(true);
     }
     m_d->lockCount++;
 }
@@ -336,15 +352,13 @@ void KisImage::unlock()
         m_d->lockCount--;
 
         if (m_d->lockCount == 0) {
-//             blockSignals(false);
-
             if (m_d->sizeChangedWhileLocked) {
                 emit sigSizeChanged(m_d->width, m_d->height);
             }
 
-            if (m_d->rootLayer)
-                m_d->rootLayer->updateStrategy()->unlock();
-
+            if (m_d->projection) {
+                m_d->projection->unlock();
+            }
         }
     }
 }
@@ -379,8 +393,8 @@ void KisImage::resize(qint32 w, qint32 h, qint32 x, qint32 y, bool cropLayers)
             else
                 m_d->adapter->beginMacro(i18n("Resize Image"));
 
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
-            m_d->adapter->addCommand(new KisImageResizeCommand(KisImageSP(this), w, h, width(), height()));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+            m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
         }
 
         m_d->width = w;
@@ -396,12 +410,37 @@ void KisImage::resize(qint32 w, qint32 h, qint32 x, qint32 y, bool cropLayers)
         unlock();
 
         if (undo()) {
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
             m_d->adapter->endMacro();
         }
     }
 }
 
+void KisImage::resizeWithOffset(qint32 w, qint32 h, qint32 xOffset, qint32 yOffset)
+{
+    if (w == width() && h == height() && xOffset == 0 && yOffset == 0)
+        return;
+
+    lock();
+    if (undo()) {
+        m_d->adapter->beginMacro(i18n("Size Canvas"));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+        m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
+    }
+
+    KisCropVisitor v(QRect(-xOffset, -yOffset, w, h), m_d->adapter);
+    m_d->rootLayer->accept(v);
+
+    emitSizeChanged();
+
+    unlock();
+
+    if (undo()) {
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
+        m_d->adapter->endMacro();
+    }
+
+}
 
 void KisImage::emitSizeChanged()
 {
@@ -434,16 +473,16 @@ void KisImage::scale(double sx, double sy, KoUpdater *progress, KisFilterStrateg
 
         if (undo()) {
             m_d->adapter->beginMacro(i18n("Scale Image"));
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
         }
 
         {
-            KisTransformVisitor visitor(KisImageSP(this), sx, sy, 0.0, 0.0, 0.0, 0, 0, progress, filterStrategy);
+            KisTransformVisitor visitor(KisImageWSP(this), sx, sy, 0.0, 0.0, 0.0, 0, 0, progress, filterStrategy);
             m_d->rootLayer->accept(visitor);
         }
 
         if (undo()) {
-            m_d->adapter->addCommand(new KisImageResizeCommand(KisImageSP(this), w, h, width(), height()));
+            m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
         }
 
         m_d->width = w;
@@ -454,7 +493,7 @@ void KisImage::scale(double sx, double sy, KoUpdater *progress, KisFilterStrateg
         unlock();
 
         if (undo()) {
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
             m_d->adapter->endMacro();
         }
     }
@@ -475,14 +514,14 @@ void KisImage::rotate(double radians, KoUpdater *progress)
 
     if (undo()) {
         m_d->adapter->beginMacro(i18n("Rotate Image"));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
     }
 
     KisFilterStrategy *filter = KisFilterStrategyRegistry::instance()->value("Triangle");
-    KisTransformVisitor visitor(KisImageSP(this), 1.0, 1.0, 0, 0, radians, -tx, -ty, progress, filter);
+    KisTransformVisitor visitor(KisImageWSP(this), 1.0, 1.0, 0, 0, radians, -tx, -ty, progress, filter);
     m_d->rootLayer->accept(visitor);
 
-    if (undo()) m_d->adapter->addCommand(new KisImageResizeCommand(KisImageSP(this), w, h, width(), height()));
+    if (undo()) m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
 
     m_d->width = w;
     m_d->height = h;
@@ -492,7 +531,7 @@ void KisImage::rotate(double radians, KoUpdater *progress)
     unlock();
 
     if (undo()) {
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
         m_d->adapter->endMacro();
     }
 }
@@ -510,7 +549,7 @@ void KisImage::shear(double angleX, double angleY, KoUpdater *progress)
         double deltaY = height() * qAbs(tan(angleX * pi / 180) * tan(angleY * pi / 180));
         w = (qint32)(width() + qAbs(height() * tan(angleX * pi / 180)));
         //ugly fix for the problem of having two extra pixels if only a shear along one
-        //axis is done. This has to be fixed in the cropping code in KisRotateVisitor!
+        //axis is done.
         if (angleX == 0 || angleY == 0)
             h = (qint32)(height() + qAbs(w * tan(angleY * pi / 180)));
         else if (angleX > 0 && angleY > 0)
@@ -527,14 +566,14 @@ void KisImage::shear(double angleX, double angleY, KoUpdater *progress)
 
         if (undo()) {
             m_d->adapter->beginMacro(i18n("Shear Image"));
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
         }
 
         KisShearVisitor v(angleX, angleY, progress);
         v.setUndoAdapter(m_d->adapter);
         rootLayer()->accept(v);
 
-        if (undo()) m_d->adapter->addCommand(new KisImageResizeCommand(KisImageSP(this), w, h, width(), height()));
+        if (undo()) m_d->adapter->addCommand(new KisImageResizeCommand(KisImageWSP(this), w, h, width(), height()));
 
         m_d->width = w;
         m_d->height = h;
@@ -544,13 +583,13 @@ void KisImage::shear(double angleX, double angleY, KoUpdater *progress)
         unlock();
 
         if (undo()) {
-            m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
+            m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
             m_d->adapter->endMacro();
         }
     }
 }
 
-void KisImage::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
+void KisImage::convertTo(const KoColorSpace *dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
 {
     if (*m_d->colorSpace == *dstColorSpace) {
         return;
@@ -562,22 +601,23 @@ void KisImage::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTr
 
     if (undo()) {
         m_d->adapter->beginMacro(i18n("Convert Image Type"));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
     }
 
-    setColorSpace(dstColorSpace);
-
-    KisColorSpaceConvertVisitor visitor(dstColorSpace, renderingIntent);
+    KisColorSpaceConvertVisitor visitor(this, dstColorSpace, renderingIntent);
     m_d->rootLayer->accept(visitor);
 
-    unlock();
-
     if (undo()) {
 
-        m_d->adapter->addCommand(new KisImageConvertTypeCommand(KisImageSP(this), oldCs, dstColorSpace));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
+        m_d->adapter->addCommand(new KisImageConvertTypeCommand(KisImageWSP(this), oldCs, dstColorSpace));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
         m_d->adapter->endMacro();
+    } else {
+        setColorSpace(dstColorSpace);
     }
+
+    unlock();
+    refreshGraph();
 }
 
 const KoColorProfile * KisImage::profile() const
@@ -585,19 +625,18 @@ const KoColorProfile * KisImage::profile() const
     return colorSpace()->profile();
 }
 
-void KisImage::setProfile(const KoColorProfile * profile)
+void KisImage::setProfile(const KoColorProfile *profile)
 {
     if (profile == 0) return;
 
-    const KoColorSpace * dstCs = KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->id(),
-                                 profile);
+    const KoColorSpace *dstCs = KoColorSpaceRegistry::instance()->colorSpace(colorSpace()->id(), profile);
     if (dstCs) {
 
         lock();
 
-        const KoColorSpace * oldCs = colorSpace();
+        const KoColorSpace *oldCs = colorSpace();
         setColorSpace(dstCs);
-        emit(sigProfileChanged(const_cast<KoColorProfile *>(profile)));
+        emit(sigProfileChanged(const_cast<KoColorProfile*>(profile)));
 
         KisChangeProfileVisitor visitor(oldCs, dstCs);
         m_d->rootLayer->accept(visitor);
@@ -654,6 +693,11 @@ QPointF KisImage::pixelToDocument(const QPoint &pixelCoord) const
     return QPointF((pixelCoord.x() + 0.5) / xRes(), (pixelCoord.y() + 0.5) / yRes());
 }
 
+QRectF KisImage::pixelToDocument(const QRectF &pixelCoord) const
+{
+    return QRectF(pixelToDocument(pixelCoord.topLeft()), pixelToDocument(pixelCoord.bottomRight()));
+}
+
 qint32 KisImage::width() const
 {
     return m_d->width;
@@ -672,9 +716,6 @@ KisGroupLayerSP KisImage::rootLayer() const
 
 KisPaintDeviceSP KisImage::projection()
 {
-    // XXX: Projection moved to the updateStrategy -- is synching really needed?
-    //m_d->projection->sync();
-
     Q_ASSERT(m_d->rootLayer);
     KisPaintDeviceSP projection = m_d->rootLayer->projection();
     Q_ASSERT(projection);
@@ -713,14 +754,15 @@ void KisImage::flatten()
     QRect rc = mergedImage()->extent();
 
     KisPainter gc(dst->paintDevice());
-    gc.bitBlt(rc.x(), rc.y(), COMPOSITE_COPY, mergedImage(), OPACITY_OPAQUE, rc.left(), rc.top(), rc.width(), rc.height());
+    gc.setCompositeOp(COMPOSITE_OVER);
+    gc.bitBlt(rc.x(), rc.y(), mergedImage(), rc.left(), rc.top(), rc.width(), rc.height());
 
     setRootLayer(new KisGroupLayer(this, "root", OPACITY_OPAQUE));
 
     if (undo()) {
         m_d->adapter->beginMacro(i18n("Flatten Image"));
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), true));
-        m_d->adapter->addCommand(new KisImageChangeLayersCommand(KisImageSP(this), oldRootLayer, m_d->rootLayer, ""));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), true));
+        m_d->adapter->addCommand(new KisImageChangeLayersCommand(KisImageWSP(this), oldRootLayer, m_d->rootLayer, ""));
     }
 
     lock();
@@ -732,7 +774,7 @@ void KisImage::flatten()
     notifyLayersChanged();
 
     if (undo()) {
-        m_d->adapter->addCommand(new KisImageLockCommand(KisImageSP(this), false));
+        m_d->adapter->addCommand(new KisImageLockCommand(KisImageWSP(this), false));
         m_d->adapter->endMacro();
     }
 }
@@ -749,7 +791,6 @@ KisLayerSP KisImage::mergeLayer(KisLayerSP layer, const KisMetaData::MergeStrate
     KisPaintLayerSP newLayer = new KisPaintLayer(this, layer->name(), OPACITY_OPAQUE, colorSpace());
     Q_CHECK_PTR(newLayer);
 
-
     QRect layerExtent = layer->extent();
     QRect layerPrevSiblingExtent = layer->prevSibling()->extent();
 
@@ -778,14 +819,50 @@ KisLayerSP KisImage::mergeLayer(KisLayerSP layer, const KisMetaData::MergeStrate
     strategy->merge(newLayer->metaData(), srcs, scores);
 
     KisNodeSP parent = layer->parent(); // parent is set to null when the layer is removed from the node
-    dbgImage << ppVar( parent );
-//     addNode(newLayer, parent, layer);
-//     removeNode(layer->prevSibling());
-//     removeNode(layer);
+    dbgImage << ppVar(parent);
 
-    undoAdapter()->addCommand(new KisImageLayerAddCommand(this, newLayer, parent, layer ));
-    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer->prevSibling() ));
-    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer ));
+    // XXX: merge the masks!
+
+    undoAdapter()->addCommand(new KisImageLayerAddCommand(this, newLayer, parent, layer));
+    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer->prevSibling()));
+    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer));
+
+    undoAdapter()->endMacro();
+
+    return newLayer;
+}
+
+KisLayerSP KisImage::flattenLayer(KisLayerSP layer)
+{
+    if (!layer->firstChild()) return layer;
+
+    undoAdapter()->beginMacro(i18n("Flatten Layer"));
+    KisPaintLayerSP newLayer = new KisPaintLayer(this, layer->name(), layer->opacity(), colorSpace());
+    newLayer->setCompositeOp(layer->compositeOp()->id());
+    newLayer->metaData();
+    QRect rc = layer->extent();
+
+    KisPainter gc(newLayer->paintDevice());
+    gc.setCompositeOp(newLayer->colorSpace()->compositeOp(COMPOSITE_COPY));
+    gc.bitBlt(rc.topLeft(), layer->projection(), rc);
+    gc.end();
+    undoAdapter()->addCommand(new KisImageLayerAddCommand(this, newLayer, layer->parent(), layer->nextSibling()));
+
+    KisNodeSP node = layer->firstChild();
+    while (node) {
+        undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, node));
+        node = node->nextSibling();
+    }
+    undoAdapter()->addCommand(new KisImageLayerRemoveCommand(this, layer));
+
+
+    QList<const KisMetaData::Store*> srcs;
+    srcs.append(layer->metaData());
+
+    const KisMetaData::MergeStrategy* strategy = KisMetaData::MergeStrategyRegistry::instance()->get("Smart");
+    QList<double> scores;
+    scores.append(1.0); //Just give some score, there only is one layer
+    strategy->merge(newLayer->metaData(), srcs, scores);
 
     undoAdapter()->endMacro();
 
@@ -821,8 +898,10 @@ QImage KisImage::convertToQImage(qint32 x,
     if (!dev) return QImage();
     QImage img = dev->convertToQImage(const_cast<KoColorProfile*>(profile), x, y, w, h);
 
+    if (m_d->backgroundPattern) {
+        m_d->backgroundPattern->paintBackground(img, QRect(x, y, w, h));
+    }
     if (!img.isNull()) {
-
 #ifdef WORDS_BIGENDIAN
         uchar * data = img.bits();
         for (int i = 0; i < w * h; ++i) {
@@ -846,82 +925,6 @@ QImage KisImage::convertToQImage(qint32 x,
 }
 
 
-QImage KisImage::convertToQImage(const QRect& r, const double xScale, const double yScale, const KoColorProfile *profile, KisSelectionSP mask)
-{
-    Q_UNUSED(mask);
-
-    qDebug() << "KisImage::convertToQimage " << r << ", x scale " << xScale << ", y scale " << yScale;
-
-#ifdef __GNUC__
-#warning "KisImage::convertToQImage: Implement direct rendering of current mask onto scaled image pixels"
-#endif
-
-    if (r.isEmpty()) {
-        return QImage();
-    }
-
-    quint32 pixelSize = colorSpace()->pixelSize();
-
-    QRect srcRect;
-
-    srcRect.setLeft(static_cast<int>(r.left() * xScale));
-    srcRect.setRight(static_cast<int>(ceil((r.right() + 1) * xScale)) - 1);
-    srcRect.setTop(static_cast<int>(r.top() * yScale));
-    srcRect.setBottom(static_cast<int>(ceil((r.bottom() + 1) * yScale)) - 1);
-
-    KisPaintDeviceSP mergedImage = m_d->rootLayer->projection();
-
-    quint8 *scaledImageData = new quint8[r.width() * r.height() * pixelSize];
-
-    quint8 *imageRow = new quint8[srcRect.width() * pixelSize];
-    const qint32 imageRowX = srcRect.x();
-
-    for (qint32 y = 0; y < r.height(); ++y) {
-
-        qint32 dstY = r.y() + y;
-        qint32 dstX = r.x();
-        qint32 srcY = int(dstY * yScale);
-
-        mergedImage->readBytes(imageRow, imageRowX, srcY, srcRect.width(), 1);
-
-        quint8 *dstPixel = scaledImageData + (y * r.width() * pixelSize);
-        quint32 columnsRemaining = r.width();
-
-        while (columnsRemaining > 0) {
-
-            qint32 srcX = int(dstX * xScale);
-
-            memcpy(dstPixel, imageRow + ((srcX - imageRowX) * pixelSize), pixelSize);
-
-            ++dstX;
-            dstPixel += pixelSize;
-            --columnsRemaining;
-        }
-    }
-
-    delete [] imageRow;
-
-    QImage image = colorSpace()->convertToQImage(scaledImageData, r.width(), r.height(), const_cast<KoColorProfile*>(profile), KoColorConversionTransformation::IntentPerceptual);
-    delete [] scaledImageData;
-
-#ifdef WORDS_BIGENDIAN
-    uchar * data = image.bits();
-    for (int i = 0; i < image.width() * image.height(); ++i) {
-        uchar r, g, b, a;
-        a = data[0];
-        b = data[1];
-        g = data[2];
-        r = data[3];
-        data[0] = r;
-        data[1] = g;
-        data[2] = b;
-        data[3] = a;
-        data += 4;
-    }
-#endif
-    return image;
-}
-
 
 QImage KisImage::convertToQImage(const QRect& r, const QSize& scaledImageSize, const KoColorProfile *profile)
 {
@@ -981,21 +984,25 @@ QImage KisImage::convertToQImage(const QRect& r, const QSize& scaledImageSize, c
 
     QImage image = colorSpace()->convertToQImage(scaledImageData, r.width(), r.height(), const_cast<KoColorProfile*>(profile), KoColorConversionTransformation::IntentPerceptual);
 
+    if (m_d->backgroundPattern) {
+        m_d->backgroundPattern->paintBackground(image, r, scaledImageSize, QSize(imageWidth, imageHeight));
+    }
+
     delete [] scaledImageData;
 
 #ifdef __BIG_ENDIAN__
     uchar * data = image.bits();
     for (int i = 0; i < image.width() * image.height(); ++i) {
-      uchar r, g, b, a;
-      a = data[0];
-      b = data[1];
-      g = data[2];
-      r = data[3];
-      data[0] = r;
-      data[1] = g;
-      data[2] = b;
-      data[3] = a;
-      data += 4;
+        uchar r, g, b, a;
+        a = data[0];
+        b = data[1];
+        g = data[2];
+        r = data[3];
+        data[0] = r;
+        data[1] = g;
+        data[2] = b;
+        data[3] = a;
+        data += 4;
     }
 #endif
 
@@ -1005,6 +1012,7 @@ QImage KisImage::convertToQImage(const QRect& r, const QSize& scaledImageSize, c
 
 KisPaintDeviceSP KisImage::mergedImage()
 {
+    refreshGraph();
     return m_d->rootLayer->projection();
 }
 
@@ -1054,7 +1062,7 @@ const KoColorSpace * KisImage::colorSpace() const
 void KisImage::setColorSpace(const KoColorSpace * colorSpace)
 {
     m_d->colorSpace = colorSpace;
-    m_d->rootLayer->resetProjection();
+    m_d->rootLayer->resetCache();
     emit sigColorSpaceChanged(colorSpace);
 }
 
@@ -1063,7 +1071,6 @@ void KisImage::setRootLayer(KisGroupLayerSP rootLayer)
     m_d->rootLayer = rootLayer;
     m_d->rootLayer->disconnect();
     m_d->rootLayer->setGraphListener(this);
-    m_d->rootLayer->updateStrategy()->setImage(this);
     setRoot(m_d->rootLayer.data());
 }
 
@@ -1115,7 +1122,7 @@ vKisAnnotationSP_it KisImage::beginAnnotations()
         // XXX productName(), or just "ICC Profile"?
         if (profile->valid()) {
             const KoIccColorProfile* iccprofile = dynamic_cast<const KoIccColorProfile*>(profile);
-            if (!iccprofile->rawData().isEmpty()) {
+            if (iccprofile && !iccprofile->rawData().isEmpty()) {
                 annotation = new  KisAnnotation("icc", iccprofile->name(), iccprofile->rawData());
             }
         }
@@ -1141,10 +1148,24 @@ KisPerspectiveGrid* KisImage::perspectiveGrid()
     return m_d->perspectiveGrid;
 }
 
+void KisImage::refreshGraph()
+{
+    KisRefreshVisitor refresher(this);
+    rootLayer()->accept(refresher);
+}
+
 void KisImage::slotProjectionUpdated(const QRect & rc)
 {
     emit sigImageUpdated(rc);
 }
 
+void KisImage::updateProjection(KisNodeSP node, const QRect& rc)
+{
+    if (!locked() && m_d->projection) {
+        dbgImage << "Updating node " << node << "rect" << rc;
+        m_d->projection->updateProjection(node, rc);
+    }
+}
+
 #include "kis_image.moc"
 
