@@ -44,8 +44,7 @@
 #include "kis_random_sub_accessor.h"
 #include "kis_selection.h"
 #include "kis_node.h"
-#include "kis_painterly_overlay.h"
-
+#include "commands/kis_paintdevice_convert_type_command.h"
 #include "kis_datamanager.h"
 
 #include "kis_selection_component.h"
@@ -60,11 +59,9 @@ public:
     KisNodeWSP parent;
     qint32 x;
     qint32 y;
-    KoColorSpace * colorSpace;
+    KoColorSpace* colorSpace;
     qint32 pixelSize;
     qint32 nChannels;
-    KisPainterlyOverlaySP painterlyOverlay;
-
 };
 
 KisPaintDevice::KisPaintDevice(const KoColorSpace * colorSpace, const QString& name)
@@ -172,6 +169,11 @@ void KisPaintDevice::setDirty()
         m_d->parent->setDirty();
 }
 
+void KisPaintDevice::setParentNode(KisNodeWSP parent)
+{
+    m_d->parent = parent;
+}
+
 void KisPaintDevice::move(qint32 x, qint32 y)
 {
     QRect dirtyRegion = extent();
@@ -209,7 +211,8 @@ QRect KisPaintDevice::extent() const
     return QRect(x, y, w, h);
 }
 
-void KisPaintDevice::exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const {
+void KisPaintDevice::exactBounds(qint32 &x, qint32 &y, qint32 &w, qint32 &h) const
+{
     QRect rc = exactBounds();
     x = rc.x();
     y = rc.y();
@@ -223,6 +226,7 @@ QRect KisPaintDevice::exactBounds() const
     // Solution n°2
     qint32  x, y, w, h, boundX2, boundY2, boundW2, boundH2;
     QRect rc = extent();
+
     x = boundX2 = rc.x();
     y = boundY2 = rc.y();
     w = boundW2 = rc.width();
@@ -248,7 +252,7 @@ QRect KisPaintDevice::exactBounds() const
 
     found = false;
 
-    for (qint32 y2 = y + h; y2 >= y ; --y2) {
+    for (qint32 y2 = y + h - 1; y2 >= y ; --y2) {
         KisHLineConstIterator it = const_cast<KisPaintDevice *>(this)->createHLineConstIterator(x, y2, w);
         while (!it.isDone() && found == false) {
             if (memcmp(it.rawData(), defaultPixel, m_d->pixelSize) != 0) {
@@ -283,7 +287,7 @@ QRect KisPaintDevice::exactBounds() const
     // Look for right edge )
     {
 
-        for (qint32 x2 = x + w; x2 >= x; --x2) {
+        for (qint32 x2 = x + w - 1; x2 >= x; --x2) {
 
             KisVLineConstIterator it = const_cast<KisPaintDevice *>(this)->createVLineConstIterator(x2, boundY2, boundH2);
             while (!it.isDone() && found == false) {
@@ -358,17 +362,17 @@ bool KisPaintDevice::read(KoStore *store)
     return retval;
 }
 
-void KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
+QUndoCommand* KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConversionTransformation::Intent renderingIntent)
 {
+    dbgImage << this << colorSpace()->id() << dstColorSpace->id() << renderingIntent;
     if (*colorSpace() == *dstColorSpace) {
-        return;
+        return 0;
     }
 
     KisPaintDevice dst(dstColorSpace);
     dst.setX(x());
     dst.setY(y());
 
-
     qint32 x, y, w, h;
     QRect rc = exactBounds();
     x = rc.x();
@@ -406,10 +410,18 @@ void KisPaintDevice::convertTo(const KoColorSpace * dstColorSpace, KoColorConver
     KisDataManagerSP oldData = m_datamanager;
     const KoColorSpace *oldColorSpace = m_d->colorSpace;
 
+    KisPaintDeviceConvertTypeCommand* cmd = new KisPaintDeviceConvertTypeCommand(this,
+            oldData,
+            oldColorSpace,
+            dst.m_datamanager,
+            dstColorSpace);
+
     setDataManager(dst.m_datamanager, dstColorSpace);
 
     emit colorSpaceChanged(dstColorSpace);
     delete oldColorSpace;
+
+    return cmd;
 }
 
 void KisPaintDevice::setProfile(const KoColorProfile * profile)
@@ -426,10 +438,10 @@ void KisPaintDevice::setProfile(const KoColorProfile * profile)
 void KisPaintDevice::setDataManager(KisDataManagerSP data, const KoColorSpace * colorSpace)
 {
     m_datamanager = data;
+    //delete m_d->colorSpace;
     m_d->colorSpace = colorSpace->clone();
     m_d->pixelSize = m_d->colorSpace->pixelSize();
     m_d->nChannels = m_d->colorSpace->channelCount();
-
 }
 
 void KisPaintDevice::convertFromQImage(const QImage& image, const QString &srcProfileName,
@@ -442,19 +454,19 @@ void KisPaintDevice::convertFromQImage(const QImage& image, const QString &srcPr
     }
     // Don't convert if not no profile is given and both paint dev and qimage are rgba.
     if (srcProfileName.isEmpty() && colorSpace()->id() == "RGBA") {
-        writeBytes(img.bits(), 0, 0, img.width(), img.height());
+        writeBytes(img.bits(), offsetX, offsetY, img.width(), img.height());
     } else {
         quint8 * dstData = new quint8[img.width() * img.height() * pixelSize()];
         KoColorSpaceRegistry::instance()
-            ->colorSpace("RGBA", srcProfileName)
-            ->convertPixelsTo(img.bits(), dstData, colorSpace(), img.width() * img.height());
+        ->colorSpace("RGBA", srcProfileName)
+        ->convertPixelsTo(img.bits(), dstData, colorSpace(), img.width() * img.height());
 
         writeBytes(dstData, offsetX, offsetY, img.width(), img.height());
         delete[] dstData;
     }
 }
 
-QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile)
+QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile) const
 {
     qint32 x1;
     qint32 y1;
@@ -473,7 +485,7 @@ QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile)
     return convertToQImage(dstProfile, x1, y1, w, h);
 }
 
-QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint32 x1, qint32 y1, qint32 w, qint32 h)
+QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint32 x1, qint32 y1, qint32 w, qint32 h) const
 {
 
     if (w < 0)
@@ -496,46 +508,47 @@ QImage KisPaintDevice::convertToQImage(const KoColorProfile *  dstProfile, qint3
     //      This copies potentially a lot of data.
     readBytes(data, x1, y1, w, h);
     QImage image = colorSpace()->convertToQImage(data, w, h, dstProfile,
-                                                 KoColorConversionTransformation::IntentPerceptual);
+                   KoColorConversionTransformation::IntentPerceptual);
     delete[] data;
 
     return image;
 }
 
-KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h) const
+KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h, const KisSelection * selection) const
 {
     KisPaintDeviceSP thumbnail = new KisPaintDevice(colorSpace());
     thumbnail->clear();
 
-    int srcw, srch;
+    int srcWidth, srcHeight;
+    int srcX0, srcY0;
     const QRect e = extent();
-    srcw = e.width();
-    srch = e.height();
+    e.getRect(&srcX0, &srcY0, &srcWidth, &srcHeight);
 
-    if (w > srcw) {
-        w = srcw;
-        h = qint32(double(srcw) / w * h);
+
+    if (w > srcWidth) {
+        w = srcWidth;
+        h = qint32(double(srcWidth) / w * h);
     }
-    if (h > srch) {
-        h = srch;
-        w = qint32(double(srch) / h * w);
+    if (h > srcHeight) {
+        h = srcHeight;
+        w = qint32(double(srcHeight) / h * w);
     }
 
-    if (srcw > srch)
-        h = qint32(double(srch) / srcw * w);
-    else if (srch > srcw)
-        w = qint32(double(srcw) / srch * h);
+    if (srcWidth > srcHeight)
+        h = qint32(double(srcHeight) / srcWidth * w);
+    else if (srcHeight > srcWidth)
+        w = qint32(double(srcWidth) / srcHeight * h);
 
-    KisRandomConstAccessorPixel iter = createRandomConstAccessor(0, 0);
-    KisRandomAccessorPixel dstIter = thumbnail->createRandomAccessor( 0, 0 );
+    KisRandomConstAccessorPixel iter = createRandomConstAccessor(0, 0, selection);
+    KisRandomAccessorPixel dstIter = thumbnail->createRandomAccessor(0, 0);
 
     for (qint32 y = 0; y < h; ++y) {
-        qint32 iY = (y * srch) / h;
+        qint32 iY = srcY0 + (y * srcHeight) / h;
         for (qint32 x = 0; x < w; ++x) {
-            qint32 iX = (x * srcw) / w;
+            qint32 iX = srcX0 + (x * srcWidth) / w;
             iter.moveTo(iX, iY);
-            dstIter.moveTo(x,  y );
-            memcpy( dstIter.rawData(), iter.rawData(), m_d->pixelSize );
+            dstIter.moveTo(x,  y);
+            memcpy(dstIter.rawData(), iter.rawData(), m_d->pixelSize);
         }
     }
 
@@ -543,29 +556,9 @@ KisPaintDeviceSP KisPaintDevice::createThumbnailDevice(qint32 w, qint32 h) const
 
 }
 
-KisPainterlyOverlaySP KisPaintDevice::painterlyOverlay()
+QImage KisPaintDevice::createThumbnail(qint32 w, qint32 h, const KisSelection *selection)
 {
-    return m_d->painterlyOverlay;
-}
-
-void KisPaintDevice::createPainterlyOverlay()
-{
-    if (!m_d->painterlyOverlay) {
-        m_d->painterlyOverlay = new KisPainterlyOverlay();
-        emit(painterlyOverlayCreated());
-    }
-
-}
-
-void KisPaintDevice::removePainterlyOverlay()
-{
-    m_d->painterlyOverlay = 0;
-}
-
-
-QImage KisPaintDevice::createThumbnail(qint32 w, qint32 h)
-{
-    KisPaintDeviceSP dev = createThumbnailDevice(w, h);
+    KisPaintDeviceSP dev = createThumbnailDevice(w, h, selection);
     return dev->convertToQImage(KoColorSpaceRegistry::instance()->rgb8()->profile());
 }
 
@@ -611,7 +604,7 @@ KisHLineConstIteratorPixel  KisPaintDevice::createHLineConstIterator(qint32 x, q
     return KisHLineConstIteratorPixel(dm, selectionDm, x, y, w, m_d->x, m_d->y);
 }
 
-KisRepeatHLineConstIteratorPixel KisPaintDevice::createRepeatHLineConstIterator(qint32 x, qint32 y, qint32 w, const QRect& _dataWidth, const KisSelection * selection ) const
+KisRepeatHLineConstIteratorPixel KisPaintDevice::createRepeatHLineConstIterator(qint32 x, qint32 y, qint32 w, const QRect& _dataWidth, const KisSelection * selection) const
 {
     KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
     KisDataManager* selectionDm = 0;
@@ -622,7 +615,7 @@ KisRepeatHLineConstIteratorPixel KisPaintDevice::createRepeatHLineConstIterator(
     return KisRepeatHLineConstIteratorPixel(dm, selectionDm, x, y, w, m_d->x, m_d->y, _dataWidth);
 }
 
-KisRepeatVLineConstIteratorPixel KisPaintDevice::createRepeatVLineConstIterator(qint32 x, qint32 y, qint32 h, const QRect& _dataWidth, const KisSelection * selection ) const
+KisRepeatVLineConstIteratorPixel KisPaintDevice::createRepeatVLineConstIterator(qint32 x, qint32 y, qint32 h, const QRect& _dataWidth, const KisSelection * selection) const
 {
     KisDataManager* dm = const_cast< KisDataManager*>(m_datamanager.data()); // TODO: don't do this
     KisDataManager* selectionDm = 0;
@@ -812,7 +805,7 @@ void KisPaintDevice::setY(qint32 y)
 }
 
 
-void KisPaintDevice::readBytes(quint8 * data, qint32 x, qint32 y, qint32 w, qint32 h)
+void KisPaintDevice::readBytes(quint8 * data, qint32 x, qint32 y, qint32 w, qint32 h) const
 {
     m_datamanager->readBytes(data, x - m_d->x, y - m_d->y, w, h);
 }
