@@ -50,6 +50,7 @@
 #include "filter/kis_filter.h"
 #include "kis_layer.h"
 #include "kis_paint_device.h"
+#include "kis_fixed_paint_device.h"
 #include "kis_transaction.h"
 #include "kis_types.h"
 #include "kis_vec.h"
@@ -69,44 +70,49 @@
 #define BEZIER_FLATNESS_THRESHOLD 0.5
 
 struct KisPainter::Private {
-    KisPaintDeviceSP device;
-    KisSelectionSP selection;
-    KisTransaction  *transaction;
-    KoUpdater * progressUpdater;
+    KisPaintDeviceSP            device;
+    KisSelectionSP              selection;
+    KisTransaction*             transaction;
+    KoUpdater*                  progressUpdater;
 
-    QRegion dirtyRegion;
-    QRect dirtyRect;
-    KisPaintOp * paintOp;
-    QRect bounds;
-    KoColor paintColor;
-    KoColor backgroundColor;
-    KoColor fillColor;
-    KisFilterConfiguration * generator;
-    KisPaintLayer *sourceLayer;
-    FillStyle fillStyle;
-    StrokeStyle strokeStyle;
-    bool antiAliasPolygonFill;
-    KisPattern *pattern;
-    QPointF duplicateOffset;
-    quint8 opacity;
-    qint32 pixelSize;
-    const KoColorSpace * colorSpace;
-    KoColorProfile *  profile;
-    const KoCompositeOp * compositeOp;
-    QBitArray channelFlags;
-    bool useBoundingDirtyRect;
-    KoAbstractGradient* gradient;
-    KisPaintOpPresetSP paintOpPreset;
+    QRegion                     dirtyRegion;
+    QRect                       dirtyRect;
+    KisPaintOp*                 paintOp;
+    QRect                       bounds;
+    KoColor                     paintColor;
+    KoColor                     backgroundColor;
+    KoColor                     fillColor;
+    KisFilterConfiguration*     generator;
+    KisPaintLayer*              sourceLayer;
+    FillStyle                   fillStyle;
+    StrokeStyle                 strokeStyle;
+    bool                        antiAliasPolygonFill;
+    KisPattern*                 pattern;
+    QPointF                     duplicateOffset;
+    quint8                      opacity;
+    quint32                     pixelSize;
+    const KoColorSpace*         colorSpace;
+    KoColorProfile*             profile;
+    const KoCompositeOp*        compositeOp;
+    QBitArray                   channelFlags;
+    bool                        useBoundingDirtyRect;
+    const KoAbstractGradient*   gradient;
+    KisPaintOpPresetSP          paintOpPreset;
+    QImage                      polygonMaskImage;
+    QPainter*                   maskPainter;
+    KisFillPainter*             fillPainter;
+    qint32                      maskImageWidth;
+    qint32                      maskImageHeight;
 };
 
 KisPainter::KisPainter()
-    : d(new Private)
+        : d(new Private)
 {
     init();
 }
 
 KisPainter::KisPainter(KisPaintDeviceSP device)
-    : d(new Private)
+        : d(new Private)
 {
     init();
     Q_ASSERT(device);
@@ -114,7 +120,7 @@ KisPainter::KisPainter(KisPaintDeviceSP device)
 }
 
 KisPainter::KisPainter(KisPaintDeviceSP device, KisSelectionSP selection)
-    : d(new Private)
+        : d(new Private)
 {
     init();
     Q_ASSERT(device);
@@ -136,6 +142,10 @@ void KisPainter::init()
     d->bounds = QRect();
     d->progressUpdater = 0;
     d->gradient = 0;
+    d->maskPainter = 0;
+    d->fillPainter = 0;
+    d->maskImageWidth = 255;
+    d->maskImageHeight = 255;
 
     KConfigGroup cfg = KGlobal::config()->group("");
     d->useBoundingDirtyRect = cfg.readEntry("aggregate_dirty_regions", true);
@@ -145,6 +155,8 @@ KisPainter::~KisPainter()
 {
     end();
     delete d->paintOp;
+    delete d->maskPainter;
+    delete d->fillPainter;
     delete d;
 }
 
@@ -159,9 +171,8 @@ void KisPainter::begin(KisPaintDeviceSP device, KisSelectionSP selection)
     d->selection = selection;
     Q_ASSERT(device->colorSpace());
 
-    if (d->transaction) {
-        delete d->transaction;
-    }
+    delete d->transaction;
+    d->transaction = 0;
 
     d->device = device;
     d->colorSpace = device->colorSpace();
@@ -175,21 +186,15 @@ QUndoCommand *KisPainter::end()
 
 void KisPainter::beginTransaction(const QString& customName)
 {
-    if (d->transaction) {
-        delete d->transaction;
-    }
+    delete d->transaction;
     d->transaction = new KisTransaction(customName, d->device);
     Q_CHECK_PTR(d->transaction);
-    d->device->connect(d->device.data(), SIGNAL(painterlyOverlayCreated()), d->transaction, SLOT(painterlyOverlayAdded()));
 }
 
 void KisPainter::beginTransaction(KisTransaction* command)
 {
-    if (d->transaction) {
-        delete d->transaction;
-    }
+    delete d->transaction;
     d->transaction = command;
-    d->device->connect(d->device.data(), SIGNAL(painterlyOverlayCreated()), d->transaction, SLOT(painterlyOverlayAdded()));
 }
 
 
@@ -220,9 +225,14 @@ QRegion KisPainter::dirtyRegion()
 }
 
 
-QRegion KisPainter::addDirtyRect(const QRect & r)
+QRegion KisPainter::addDirtyRect(const QRect & rc)
 {
-    Q_ASSERT(r.width() >= 0 || r.height() >= 0);
+    QRect r = rc.normalized();
+
+    if (!r.isValid() || r.width() <= 0 || r.height() <= 0) {
+        return d->dirtyRegion;
+    }
+
     if (d->useBoundingDirtyRect) {
         d->dirtyRect = d->dirtyRect.united(r);
         return QRegion(d->dirtyRect);
@@ -234,43 +244,20 @@ QRegion KisPainter::addDirtyRect(const QRect & r)
 
 
 void KisPainter::bitBlt(qint32 dx, qint32 dy,
-                        const KoCompositeOp* op,
-                        const KisPaintDeviceSP src,
-                        qint32 sx, qint32 sy,
-                        qint32 sw, qint32 sh)
-{
-    bitBlt(dx, dy, op, src, OPACITY_OPAQUE, sx, sy, sw, sh);
-}
-
-void KisPainter::bitBlt(qint32 dx, qint32 dy,
-                        const QString & op,
-                        const KisPaintDeviceSP src,
-                        quint8 opacity,
-                        qint32 sx, qint32 sy,
-                        qint32 sw, qint32 sh)
-{
-    bitBlt(dx, dy, d->colorSpace->compositeOp(op), src, opacity, sx, sy, sw, sh);
-}
-
-void KisPainter::bitBlt(const QPoint & pos, const KisPaintDeviceSP src, const QRect & srcRect)
-{
-    bitBlt(pos.x(), pos.y(), d->compositeOp, src, d->opacity, srcRect.x(), srcRect.y(), srcRect.width(), srcRect.height());
-}
-
-void KisPainter::bitBlt(qint32 dx, qint32 dy,
-                        const KoCompositeOp* op,
                         const KisPaintDeviceSP srcdev,
-                        quint8 opacity,
                         qint32 sx, qint32 sy,
                         qint32 sw, qint32 sh)
 {
-    if (srcdev.isNull() || sw == 0 || sh == 0) {
-        return;
-    }
+    if (sw == 0 || sh == 0) return;
+    if (srcdev.isNull()) return;
+    if (d->device.isNull()) return;
 
     QRect srcRect = QRect(sx, sy, sw, sh);
 
-    if (op->id() != COMPOSITE_COPY) { // In case of COMPOSITE_COPY restricting bitblt to extent can have unexpected behavior since it would reduce the area that is copied
+    // In case of COMPOSITE_COPY restricting bitblt to extent can
+    // have unexpected behavior since it would reduce the area that
+    // is copied.
+    if (d->compositeOp->id() != COMPOSITE_COPY) {
         srcRect &= srcdev->extent();
     }
 
@@ -286,8 +273,6 @@ void KisPainter::bitBlt(qint32 dx, qint32 dy,
     sw = srcRect.width();
     sh = srcRect.height();
 
-    addDirtyRect(QRect(dx, dy, sw, sh));
-
     const KoColorSpace * srcCs = srcdev->colorSpace();
 
     qint32 dstY = dy;
@@ -297,123 +282,144 @@ void KisPainter::bitBlt(qint32 dx, qint32 dy,
     KisRandomConstAccessorPixel srcIt = srcdev->createRandomConstAccessor(sx, sy);
     KisRandomAccessorPixel dstIt = d->device->createRandomAccessor(dx, dy);
 
-    while (rowsRemaining > 0) {
+    if (d->selection) {
 
-        qint32 dstX = dx;
-        qint32 srcX = sx;
-        qint32 columnsRemaining = sw;
-        qint32 numContiguousDstRows = d->device->numContiguousRows(dstY, dstX, dstX + sw - 1);
-        qint32 numContiguousSrcRows = srcdev->numContiguousRows(srcY, srcX, srcX + sw - 1);
+        KisRandomConstAccessorPixel maskIt = d->selection->createRandomConstAccessor(dx, dy);
 
-        qint32 rows = qMin(numContiguousDstRows, numContiguousSrcRows);
-        rows = qMin(rows, rowsRemaining);
+        while (rowsRemaining > 0) {
 
-        while (columnsRemaining > 0) {
+            qint32 dstX = dx;
+            qint32 srcX = sx;
+            qint32 columnsRemaining = sw;
+            qint32 numContiguousDstRows = d->device->numContiguousRows(dstY, dstX, dstX + sw - 1);
+            qint32 numContiguousSrcRows = srcdev->numContiguousRows(srcY, srcX, srcX + sw - 1);
+            qint32 numContiguousSelRows = d->selection->numContiguousRows(srcY, srcX, srcX + sw - 1);
 
-            qint32 numContiguousDstColumns = d->device->numContiguousColumns(dstX, dstY, dstY + rows - 1);
-            qint32 numContiguousSrcColumns = srcdev->numContiguousColumns(srcX, srcY, srcY + rows - 1);
+            qint32 rows = qMin(numContiguousDstRows, numContiguousSrcRows);
+            rows = qMin(rows, numContiguousSelRows);
+            rows = qMin(rows, rowsRemaining);
 
-            qint32 columns = qMin(numContiguousDstColumns, numContiguousSrcColumns);
-            columns = qMin(columns, columnsRemaining);
+            while (columnsRemaining > 0) {
 
-            qint32 srcRowStride = srcdev->rowStride(srcX, srcY);
-            srcIt.moveTo(srcX, srcY);
-            const quint8 *srcData = srcIt.rawData();
+                qint32 numContiguousDstColumns = d->device->numContiguousColumns(dstX, dstY, dstY + rows - 1);
+                qint32 numContiguousSrcColumns = srcdev->numContiguousColumns(srcX, srcY, srcY + rows - 1);
+                qint32 numContiguousSelColumns = d->selection->numContiguousColumns(srcX, srcY, srcY + rows - 1);
 
-            qint32 dstRowStride = d->device->rowStride(dstX, dstY);
-            dstIt.moveTo(dstX, dstY);
-            quint8 *dstData = dstIt.rawData();
+                qint32 columns = qMin(numContiguousDstColumns, numContiguousSrcColumns);
+                columns = qMin(columns, numContiguousSelColumns);
+                columns = qMin(columns, columnsRemaining);
 
-            d->colorSpace->bitBlt(dstData,
-                                  dstRowStride,
-                                  srcCs,
-                                  srcData,
-                                  srcRowStride,
-                                  0,
-                                  0,
-                                  opacity,
-                                  rows,
-                                  columns,
-                                  op,
-                                  d->channelFlags);
+                qint32 srcRowStride = srcdev->rowStride(srcX, srcY);
+                srcIt.moveTo(srcX, srcY);
 
-            srcX += columns;
-            dstX += columns;
-            columnsRemaining -= columns;
+                qint32 dstRowStride = d->device->rowStride(dstX, dstY);
+                dstIt.moveTo(dstX, dstY);
+
+                qint32 maskRowStride = d->selection->rowStride(dstX, dstY);
+                maskIt.moveTo(dstX, dstY);
+
+                d->colorSpace->bitBlt(dstIt.rawData(),
+                                      dstRowStride,
+                                      srcCs,
+                                      srcIt.rawData(),
+                                      srcRowStride,
+                                      maskIt.rawData(),
+                                      maskRowStride,
+                                      d->opacity,
+                                      rows,
+                                      columns,
+                                      d->compositeOp,
+                                      d->channelFlags);
+                srcX += columns;
+                dstX += columns;
+                columnsRemaining -= columns;
+            }
+
+            srcY += rows;
+            dstY += rows;
+            rowsRemaining -= rows;
         }
-
-        srcY += rows;
-        dstY += rows;
-        rowsRemaining -= rows;
-    }
-}
-
-void KisPainter::bltSelection(qint32 dx, qint32 dy,
-                              const QString & op,
-                              const KisPaintDeviceSP src,
-                              quint8 opacity,
-                              qint32 sx, qint32 sy,
-                              qint32 sw, qint32 sh)
-{
-    bltSelection(dx, dy, d->colorSpace->compositeOp(op), src, opacity, sx, sy, sw, sh);
-}
-
-void KisPainter::bltSelection(qint32 dx, qint32 dy,
-                              const KoCompositeOp *op,
-                              const KisPaintDeviceSP srcdev,
-                              quint8 opacity,
-                              qint32 sx, qint32 sy,
-                              qint32 sw, qint32 sh)
-{
-    if (d->device.isNull()) return;
-    if (!d->selection) {
-        bitBlt(dx, dy, op, srcdev, opacity, sx, sy, sw, sh);
     } else {
-        bltSelection(dx, dy, op, srcdev, d->selection, opacity, sx, sy, sw, sh);
+
+        while (rowsRemaining > 0) {
+
+            qint32 dstX = dx;
+            qint32 srcX = sx;
+            qint32 columnsRemaining = sw;
+            qint32 numContiguousDstRows = d->device->numContiguousRows(dstY, dstX, dstX + sw - 1);
+            qint32 numContiguousSrcRows = srcdev->numContiguousRows(srcY, srcX, srcX + sw - 1);
+
+            qint32 rows = qMin(numContiguousDstRows, numContiguousSrcRows);
+            rows = qMin(rows, rowsRemaining);
+
+            while (columnsRemaining > 0) {
+
+                qint32 numContiguousDstColumns = d->device->numContiguousColumns(dstX, dstY, dstY + rows - 1);
+                qint32 numContiguousSrcColumns = srcdev->numContiguousColumns(srcX, srcY, srcY + rows - 1);
+
+                qint32 columns = qMin(numContiguousDstColumns, numContiguousSrcColumns);
+                columns = qMin(columns, columnsRemaining);
+
+                qint32 srcRowStride = srcdev->rowStride(srcX, srcY);
+                srcIt.moveTo(srcX, srcY);
+
+                qint32 dstRowStride = d->device->rowStride(dstX, dstY);
+                dstIt.moveTo(dstX, dstY);
+
+                d->colorSpace->bitBlt(dstIt.rawData(),
+                                      dstRowStride,
+                                      srcCs,
+                                      srcIt.rawData(),
+                                      srcRowStride,
+                                      0,
+                                      0,
+                                      d->opacity,
+                                      rows,
+                                      columns,
+                                      d->compositeOp,
+                                      d->channelFlags);
+
+                srcX += columns;
+                dstX += columns;
+                columnsRemaining -= columns;
+            }
+
+            srcY += rows;
+            dstY += rows;
+            rowsRemaining -= rows;
+        }
     }
+
+    addDirtyRect(QRect(dx, dy, sw, sh));
 }
 
-void KisPainter::bltSelection(const QPoint & pos, const KisPaintDeviceSP src, const KisSelectionSP selDev, const QRect & srcRect)
+void KisPainter::bitBlt(const QPoint & pos, const KisPaintDeviceSP src, const QRect & srcRect)
 {
-    bltSelection(pos.x(), pos.y(), d->compositeOp, src, selDev, d->opacity, srcRect.x(), srcRect.y(), srcRect.width(), srcRect.height());
+    bitBlt(pos.x(), pos.y(), src, srcRect.x(), srcRect.y(), srcRect.width(), srcRect.height());
 }
 
-void KisPainter::bltSelection(qint32 dx, qint32 dy,
-                              const QString & op,
-                              const KisPaintDeviceSP src,
-                              const KisSelectionSP selMask,
-                              quint8 opacity,
-                              qint32 sx, qint32 sy,
-                              qint32 sw, qint32 sh)
-{
-    bltSelection(dx, dy, d->colorSpace->compositeOp(op), src, selMask, opacity, sx, sy, sw, sh);
-}
 
-void KisPainter::bltSelection(qint32 dx, qint32 dy,
-                              const KoCompositeOp  *op,
-                              const KisPaintDeviceSP srcdev,
-                              const KisSelectionSP selMask,
-                              quint8 opacity,
-                              qint32 sx, qint32 sy,
-                              qint32 sw, qint32 sh)
+
+void KisPainter::bltFixed(qint32 dx, qint32 dy,
+                          const KisFixedPaintDeviceSP srcDev,
+                          qint32 sx, qint32 sy,
+                          qint32 sw, qint32 sh)
 {
     if (sw == 0 || sh == 0) return;
-    if (srcdev.isNull()) return;
-
-    if (selMask.isNull()) return;
-
+    if (srcDev.isNull()) return;
     if (d->device.isNull()) return;
+    Q_ASSERT(srcDev->pixelSize() == d->pixelSize);
 
-// This is an optimization that break the fill painter
-//     if (selMask->isProbablyTotallyUnselected(QRect(dx, dy, sw, sh))) {
-//         return;
-//     }
     QRect srcRect = QRect(sx, sy, sw, sh);
-    srcRect &= srcdev->exactBounds();
+
     if (srcRect.isEmpty()) {
         return;
     }
 
+    if (!srcRect.contains(srcDev->bounds())) {
+        srcRect = srcDev->bounds();
+    }
+
     dx += srcRect.x() - sx;
     dy += srcRect.y() - sy;
 
@@ -422,76 +428,56 @@ void KisPainter::bltSelection(qint32 dx, qint32 dy,
     sw = srcRect.width();
     sh = srcRect.height();
 
-    const KoColorSpace * srcCs = srcdev->colorSpace();
+    const KoColorSpace * srcCs = d->colorSpace;
+    quint8* dstBytes = new quint8[sw * sh * d->device->pixelSize()];
+    d->device->readBytes(dstBytes, dx, dy, sw, sh);
 
-    qint32 dstY = dy;
-    qint32 srcY = sy;
-    qint32 rowsRemaining = sh;
+    if (d->selection) {
 
-    KisRandomConstAccessorPixel srcIt = srcdev->createRandomConstAccessor(sx, sy);
-    KisRandomAccessorPixel dstIt = d->device->createRandomAccessor(dx, dy);
-    KisRandomConstAccessorPixel maskIt = selMask->createRandomConstAccessor(dx, dy);
+        quint8* selBytes = new quint8[sw * sh * d->selection->pixelSize()];
+        d->selection->readBytes(selBytes, dx, dy, sw, sh);
 
-    while (rowsRemaining > 0) {
+        d->colorSpace->bitBlt(dstBytes,
+                              sw * d->device->pixelSize(),
+                              srcCs,
+                              srcDev->data() + sx,
+                              srcDev->bounds().width() * srcDev->pixelSize(),
+                              selBytes,
+                              sw  * d->selection->pixelSize(),
+                              d->opacity,
+                              sh,
+                              sw,
+                              d->compositeOp,
+                              d->channelFlags);
 
-        qint32 dstX = dx;
-        qint32 srcX = sx;
-        qint32 columnsRemaining = sw;
-        qint32 numContiguousDstRows = d->device->numContiguousRows(dstY, dstX, dstX + sw - 1);
-        qint32 numContiguousSrcRows = srcdev->numContiguousRows(srcY, srcX, srcX + sw - 1);
-        qint32 numContiguousSelRows = selMask->numContiguousRows(srcY, srcX, srcX + sw - 1);
+        delete[] selBytes;
+    } else {
+        d->colorSpace->bitBlt(dstBytes,
+                              sw * d->device->pixelSize(),
+                              srcCs,
+                              srcDev->data() + sx,
+                              srcDev->bounds().width() * srcDev->pixelSize(),
+                              0,
+                              0,
+                              d->opacity,
+                              sh,
+                              sw,
+                              d->compositeOp,
+                              d->channelFlags);
 
-        qint32 rows = qMin(numContiguousDstRows, numContiguousSrcRows);
-        rows = qMin(rows, numContiguousSelRows);
-        rows = qMin(rows, rowsRemaining);
-
-        while (columnsRemaining > 0) {
-
-            qint32 numContiguousDstColumns = d->device->numContiguousColumns(dstX, dstY, dstY + rows - 1);
-            qint32 numContiguousSrcColumns = srcdev->numContiguousColumns(srcX, srcY, srcY + rows - 1);
-            qint32 numContiguousSelColumns = selMask->numContiguousColumns(srcX, srcY, srcY + rows - 1);
-
-            qint32 columns = qMin(numContiguousDstColumns, numContiguousSrcColumns);
-            columns = qMin(columns, numContiguousSelColumns);
-            columns = qMin(columns, columnsRemaining);
-
-            qint32 srcRowStride = srcdev->rowStride(srcX, srcY);
-            srcIt.moveTo(srcX, srcY);
-            const quint8 *srcData = srcIt.rawData();
-
-            qint32 dstRowStride = d->device->rowStride(dstX, dstY);
-            dstIt.moveTo(dstX, dstY);
-            quint8 *dstData = dstIt.rawData();
-
-            qint32 maskRowStride = selMask->rowStride(dstX, dstY);
-            maskIt.moveTo(dstX, dstY);
-            const quint8 *maskData = maskIt.rawData();
-
-            d->colorSpace->bitBlt(dstData,
-                                  dstRowStride,
-                                  srcCs,
-                                  srcData,
-                                  srcRowStride,
-                                  maskData,
-                                  maskRowStride,
-                                  opacity,
-                                  rows,
-                                  columns,
-                                  op,
-                                  d->channelFlags);
-
-            srcX += columns;
-            dstX += columns;
-            columnsRemaining -= columns;
-        }
-
-        srcY += rows;
-        dstY += rows;
-        rowsRemaining -= rows;
     }
+
+    d->device->writeBytes(dstBytes, dx, dy, sw, sh);
+
+    delete[] dstBytes;
+
     addDirtyRect(QRect(dx, dy, sw, sh));
 }
 
+void KisPainter::bltFixed(const QPoint & pos, const KisFixedPaintDeviceSP src, const QRect & srcRect)
+{
+    bltFixed(pos.x(), pos.y(), src, srcRect.x(), srcRect.y(), srcRect.width(), srcRect.height());
+}
 
 double KisPainter::paintLine(const KisPaintInformation &pi1,
                              const KisPaintInformation &pi2,
@@ -563,7 +549,10 @@ double KisPainter::paintBezierCurve(const KisPaintInformation &pi1,
                                     const KisPaintInformation &pi2,
                                     const double savedDist)
 {
-    return d->paintOp->paintBezierCurve(pi1, control1, control2, pi2, savedDist);
+    if (d->paintOp) {
+        return d->paintOp->paintBezierCurve(pi1, control1, control2, pi2, savedDist);
+    }
+    return 0.5;
 }
 
 void KisPainter::paintRect(const QRectF &rect)
@@ -728,7 +717,12 @@ void KisPainter::fillPainterPath(const QPainterPath& path)
     KisPaintDeviceSP polygon = new KisPaintDevice(d->device->colorSpace());
     Q_CHECK_PTR(polygon);
 
-    KisFillPainter fillPainter(polygon);
+    if (!d->fillPainter) {
+        d->fillPainter = new KisFillPainter(polygon);
+    } else {
+        d->fillPainter->begin(polygon);
+    }
+
 
     QRectF boundingRect = path.boundingRect();
     QRect fillRect;
@@ -755,60 +749,63 @@ void KisPainter::fillPainterPath(const QPainterPath& path)
         // Currently unsupported, fall through
         warnImage << "Unknown or unsupported fill style in fillPolygon\n";
     case FillStyleForegroundColor:
-        fillPainter.fillRect(fillRect, paintColor(), OPACITY_OPAQUE);
+        d->fillPainter->fillRect(fillRect, paintColor(), OPACITY_OPAQUE);
         break;
     case FillStyleBackgroundColor:
-        fillPainter.fillRect(fillRect, backgroundColor(), OPACITY_OPAQUE);
+        d->fillPainter->fillRect(fillRect, backgroundColor(), OPACITY_OPAQUE);
         break;
     case FillStylePattern:
         Q_ASSERT(d->pattern != 0);
-        fillPainter.fillRect(fillRect, d->pattern);
+        d->fillPainter->fillRect(fillRect, d->pattern);
         break;
     case FillStyleGenerator:
         Q_ASSERT(d->generator != 0);
-        fillPainter.fillRect(fillRect.x(), fillRect.y(), fillRect.width(), fillRect.height(), generator());
+        d->fillPainter->fillRect(fillRect.x(), fillRect.y(), fillRect.width(), fillRect.height(), generator());
         break;
     }
 
-    KisSelectionSP polygonMask = new KisSelection(polygon);
-
-    const qint32 MASK_IMAGE_WIDTH = 256;
-    const qint32 MASK_IMAGE_HEIGHT = 256;
-
-    QImage polygonMaskImage(MASK_IMAGE_WIDTH, MASK_IMAGE_HEIGHT, QImage::Format_ARGB32);
-    QPainter maskPainter(&polygonMaskImage);
-    maskPainter.setRenderHint(QPainter::Antialiasing, antiAliasPolygonFill());
-
-    // Break the mask up into chunks so we don't have to allocate a potentially very large QImage.
-
-    for (qint32 x = fillRect.x(); x < fillRect.x() + fillRect.width(); x += MASK_IMAGE_WIDTH) {
-        for (qint32 y = fillRect.y(); y < fillRect.y() + fillRect.height(); y += MASK_IMAGE_HEIGHT) {
-
-            maskPainter.fillRect(polygonMaskImage.rect(), QColor(OPACITY_TRANSPARENT, OPACITY_TRANSPARENT, OPACITY_TRANSPARENT, 255));
-            maskPainter.translate(-x, -y);
-            maskPainter.fillPath(path, QColor(OPACITY_OPAQUE, OPACITY_OPAQUE, OPACITY_OPAQUE, 255));
-            maskPainter.translate(x, y);
-
-            qint32 rectWidth = qMin(fillRect.x() + fillRect.width() - x, MASK_IMAGE_WIDTH);
-            qint32 rectHeight = qMin(fillRect.y() + fillRect.height() - y, MASK_IMAGE_HEIGHT);
-
-            KisRectIterator rectIt = polygonMask->createRectIterator(x, y, rectWidth, rectHeight);
-
-            while (!rectIt.isDone()) {
-                (*rectIt.rawData()) = qRed(polygonMaskImage.pixel(rectIt.x() - x, rectIt.y() - y));
-                ++rectIt;
-            }
-        }
+    if (d->polygonMaskImage.isNull() || (d->maskPainter == 0)) {
+        d->polygonMaskImage = QImage(d->maskImageWidth, d->maskImageHeight, QImage::Format_ARGB32);
+        d->maskPainter = new QPainter(&d->polygonMaskImage);
+        d->maskPainter->setRenderHint(QPainter::Antialiasing, antiAliasPolygonFill());
     }
 
-    polygon->applySelectionMask(polygonMask);
+    // Break the mask up into chunks so we don't have to allocate a potentially very large QImage.
+    const QColor opaqueColor(OPACITY_OPAQUE, OPACITY_OPAQUE, OPACITY_OPAQUE, 255);
+    const QBrush brush(opaqueColor);
+    for (qint32 x = fillRect.x(); x < fillRect.x() + fillRect.width(); x += d->maskImageWidth) {
+        for (qint32 y = fillRect.y(); y < fillRect.y() + fillRect.height(); y += d->maskImageHeight) {
+
+            d->polygonMaskImage.fill(qRgba(OPACITY_TRANSPARENT, OPACITY_TRANSPARENT, OPACITY_TRANSPARENT, 255));
+            d->maskPainter->translate(-x, -y);
+            d->maskPainter->fillPath(path, brush);
+            d->maskPainter->translate(x, y);
+
+            qint32 rectWidth = qMin(fillRect.x() + fillRect.width() - x, d->maskImageWidth);
+            qint32 rectHeight = qMin(fillRect.y() + fillRect.height() - y, d->maskImageHeight);
+
+            KisHLineIterator lineIt = polygon->createHLineIterator(x, y, rectWidth);
+
+            quint8 tmp;
+            for (int row = y; row < y + rectHeight; row++) {
+                QRgb* line = reinterpret_cast<QRgb*>(d->polygonMaskImage.scanLine(row - y));
+                while (!lineIt.isDone()) {
+                    tmp = qRed(line[lineIt.x() - x]);
+                    polygon->colorSpace()->applyAlphaU8Mask(lineIt.rawData(),
+                                                            &tmp, 1);
+                    ++lineIt;
+                }
+                lineIt.nextRow();
+            }
+
+        }
+    }
 
     QRect r = polygon->extent();
 
     // The strokes for the outline may have already added updated the dirtyrect, but it can't hurt,
     // and if we're painting without outlines, then there will be no dirty rect. Let's do it ourselves...
-
-    bltSelection(r.x(), r.y(), d->compositeOp, polygon, opacity(), r.x(), r.y(), r.width(), r.height());
+    bitBlt(r.x(), r.y(), polygon, r.x(), r.y(), r.width(), r.height());
 }
 
 void KisPainter::drawLine(const QPointF & start, const QPointF & end)
@@ -1015,8 +1012,8 @@ void KisPainter::drawWuLine(const QPointF & start, const QPointF & end)
         // line have to be paint from left to right
         if (x1 > x2) {
             float tmp;
-            tmp = x1;x1 = x2;x2 = tmp;
-            tmp = y1;y1 = y2;y2 = tmp;
+            tmp = x1; x1 = x2; x2 = tmp;
+            tmp = y1; y1 = y2; y2 = tmp;
             xd = (x2 - x1);
             yd = (y2 - y1);
         }
@@ -1104,8 +1101,8 @@ void KisPainter::drawWuLine(const QPointF & start, const QPointF & end)
         // line have to be painted from left to right
         if (y1 > y2) {
             float tmp;
-            tmp = x1;x1 = x2;x2 = tmp;
-            tmp = y1;y1 = y2;y2 = tmp;
+            tmp = x1; x1 = x2; x2 = tmp;
+            tmp = y1; y1 = y2; y2 = tmp;
             xd = (x2 - x1);
             yd = (y2 - y1);
         }
@@ -1207,7 +1204,7 @@ void KisPainter::drawThickLine(const QPointF & start, const QPointF & end, int s
     KoColor col2(c1);
 
     float grada, gradb, dxa, dxb, dya, dyb, adya, adyb, fraca, fracb,
-        xfa, yfa, xfb, yfb, b1a, b2a, b1b, b2b, dx, dy;
+    xfa, yfa, xfb, yfb, b1a, b2a, b1b, b2b, dx, dy;
     int x, y, ix1, ix2, iy1, iy2;
 
     KoColor pix;
@@ -1580,13 +1577,12 @@ KisPattern * KisPainter::pattern() const
 void KisPainter::setPaintColor(const KoColor& color)
 {
     d->paintColor = color;
-    if ( d->device )
-    {
-        d->paintColor.convertTo( d->device->colorSpace() );
+    if (d->device) {
+        d->paintColor.convertTo(d->device->colorSpace());
     }
 }
 
-KoColor KisPainter::paintColor() const
+const KoColor &KisPainter::paintColor() const
 {
     return d->paintColor;
 }
@@ -1594,13 +1590,12 @@ KoColor KisPainter::paintColor() const
 void KisPainter::setBackgroundColor(const KoColor& color)
 {
     d->backgroundColor = color;
-    if ( d->device )
-    {
-        d->backgroundColor.convertTo( d->device->colorSpace() );
+    if (d->device) {
+        d->backgroundColor.convertTo(d->device->colorSpace());
     }
 }
 
-KoColor KisPainter::backgroundColor() const
+const KoColor &KisPainter::backgroundColor() const
 {
     return d->backgroundColor;
 }
@@ -1610,7 +1605,7 @@ void KisPainter::setFillColor(const KoColor& color)
     d->fillColor = color;
 }
 
-KoColor KisPainter::fillColor() const
+const KoColor &KisPainter::fillColor() const
 {
     return d->fillColor;
 }
@@ -1684,6 +1679,11 @@ const KoCompositeOp * KisPainter::compositeOp()
     return d->compositeOp;
 }
 
+void KisPainter::setCompositeOp(const QString& op)
+{
+    d->compositeOp = d->colorSpace->compositeOp(op);
+}
+
 void KisPainter::setSelection(KisSelectionSP selection)
 {
     d->selection = selection;
@@ -1699,21 +1699,21 @@ KoUpdater * KisPainter::progressUpdater()
     return d->progressUpdater;
 }
 
-void KisPainter::setGradient(KoAbstractGradient* gradient)
+void KisPainter::setGradient(const KoAbstractGradient* gradient)
 {
     d->gradient = gradient;
 }
-KoAbstractGradient* KisPainter::gradient()
+
+const KoAbstractGradient* KisPainter::gradient()
 {
     return d->gradient;
 }
 
-void KisPainter::setPaintOpPreset(KisPaintOpPresetSP preset, KisImageSP image)
+void KisPainter::setPaintOpPreset(KisPaintOpPresetSP preset, KisImageWSP image)
 {
     d->paintOpPreset = preset;
     delete d->paintOp;
     d->paintOp = KisPaintOpRegistry::instance()->paintOp(preset, this, image);
-    Q_ASSERT( d->paintOp );
 }
 
 KisPaintOpPresetSP KisPainter::preset() const
@@ -1725,3 +1725,13 @@ KisPaintOp* KisPainter::paintOp() const
 {
     return d->paintOp;
 }
+
+
+void KisPainter::setMaskImageSize(qint32 width, qint32 height)
+{
+
+    d->maskImageWidth = qBound(1, width, 256);
+    d->maskImageHeight = qBound(1, height, 256);
+    d->fillPainter = 0;
+    d->polygonMaskImage = QImage();
+}
