@@ -39,7 +39,7 @@ KisTransformWorker::KisTransformWorker(KisPaintDeviceSP dev,
                                        double xshear, double yshear,
                                        double rotation,
                                        qint32 xtranslate, qint32 ytranslate,
-                                       KoUpdater *progress,
+                                       KoUpdaterPtr progress,
                                        KisFilterStrategy *filter,
                                        bool fixBorderAlpha)
 {
@@ -49,7 +49,7 @@ KisTransformWorker::KisTransformWorker(KisPaintDeviceSP dev,
     m_xshear = xshear;
     m_yshear = yshear;
     m_rotation = rotation,
-    m_xtranslate = xtranslate;
+                 m_xtranslate = xtranslate;
     m_ytranslate = ytranslate;
     m_progressUpdater = progress;
     m_filter = filter;
@@ -58,7 +58,9 @@ KisTransformWorker::KisTransformWorker(KisPaintDeviceSP dev,
 
 KisTransformWorker::~KisTransformWorker()
 {
-}void KisTransformWorker::rotateNone(KisPaintDeviceSP src, KisPaintDeviceSP dst)
+}
+
+void KisTransformWorker::rotateNone(KisPaintDeviceSP src, KisPaintDeviceSP dst)
 {
     qint32 pixelSize = src->pixelSize();
     QRect r;
@@ -81,11 +83,11 @@ KisTransformWorker::~KisTransformWorker()
 
         //progress info
         m_progressStep += r.width();
-        if(m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
+        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
             m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            m_progressUpdater->setProgress(m_lastProgressReport);
+            if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(m_lastProgressReport);
         }
-        if (m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
             break;
         }
     }
@@ -112,11 +114,11 @@ void KisTransformWorker::rotateRight90(KisPaintDeviceSP src, KisPaintDeviceSP ds
 
         //progress info
         m_progressStep += r.width();
-        if(m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
+        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
             m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            m_progressUpdater->setProgress(m_lastProgressReport);
+            if (m_progressUpdater) m_progressUpdater->setProgress(m_lastProgressReport);
         }
-        if (m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
             break;
         }
     }
@@ -147,11 +149,11 @@ void KisTransformWorker::rotateLeft90(KisPaintDeviceSP src, KisPaintDeviceSP dst
 
         //progress info
         m_progressStep += r.width();
-        if(m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
+        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
             m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            m_progressUpdater->setProgress(m_lastProgressReport);
+            if (m_progressUpdater) m_progressUpdater->setProgress(m_lastProgressReport);
         }
-        if (m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
             break;
         }
     }
@@ -180,11 +182,11 @@ void KisTransformWorker::rotate180(KisPaintDeviceSP src, KisPaintDeviceSP dst)
 
         //progress info
         m_progressStep += r.width();
-        if(m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
+        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
             m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            m_progressUpdater->setProgress(m_lastProgressReport);
+            if (m_progressUpdater) m_progressUpdater->setProgress(m_lastProgressReport);
         }
-        if (m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
             break;
         }
     }
@@ -346,7 +348,7 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
         // Build a temporary line
         T srcIt = createIterator <T>(src, srcStart, lineNum, srcLen);
         quint8 *data;
-        qint32 i=0;
+        qint32 i = 0;
         data = srcIt.rawData(); // take the first pixel as source - in effect duplicating the pixel
         while (i < extraLen) {
             memcpy(&tmpLine[i*pixelSize], data, pixelSize);
@@ -357,7 +359,7 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
             data = srcIt.rawData();
             memcpy(&tmpLine[i*pixelSize], data, pixelSize);
             cs->setAlpha(data, 0, 1);
-            if(i < srcLen + extraLen-1) // duplicate pixels along edge
+            if (i < srcLen + extraLen - 1) // duplicate pixels along edge
                 ++srcIt;
             i++;
         }
@@ -400,22 +402,22 @@ void KisTransformWorker::transformPass(KisPaintDevice *src, KisPaintDevice *dst,
             data = dstIt.rawData();
             mixOp->mixColors(colors, filterWeights[center&255].weight, filterWeights[center&255].numWeights, data);
 
-/*
-            //possibly fix the alpha of the border if user wants it
-            if (fixBorderAlpha && (i == 0 || i == dstLen - 1))
-                cs->setAlpha(data, cs->alpha(&tmpLine[(center>>8)*pixelSize]), 1);
-*/
+            /*
+                        //possibly fix the alpha of the border if user wants it
+                        if (fixBorderAlpha && (i == 0 || i == dstLen - 1))
+                            cs->setAlpha(data, cs->alpha(&tmpLine[(center>>8)*pixelSize]), 1);
+            */
             ++dstIt;
             i++;
         }
 
         //progress info
         m_progressStep += dstLen;
-        if(m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
+        if (m_lastProgressReport != (m_progressStep * 100) / m_progressTotalSteps) {
             m_lastProgressReport = (m_progressStep * 100) / m_progressTotalSteps;
-            m_progressUpdater->setProgress(m_lastProgressReport);
+            if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(m_lastProgressReport);
         }
-        if (m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
             break;
         }
     }
@@ -519,20 +521,19 @@ bool KisTransformWorker::run()
             rotateNone(srcdev, m_dev); //copy it back
         }
         //progress info
-        m_progressUpdater->setProgress(100);
-
+        if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(100);
         return false;
-        return m_progressUpdater->interrupted();
+
     }
 
-        if ( m_progressUpdater->interrupted()) {
-            m_progressUpdater->setProgress(100);
-            return false;
-        }
+    if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
+        if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(100);
+        return false;
+    }
 
     transformPass <KisHLineIteratorPixel>(srcdev.data(), srcdev.data(), xscale, yscale*xshear, 0, m_filter, m_fixBorderAlpha);
 
-    if ( m_progressUpdater->interrupted()) {
+    if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
         m_progressUpdater->setProgress(100);
         return false;
     }
@@ -540,7 +541,7 @@ bool KisTransformWorker::run()
     // Now do the second pass
     transformPass <KisVLineIteratorPixel>(srcdev.data(), srcdev.data(), yscale, yshear, ytranslate, m_filter, m_fixBorderAlpha);
 
-    if ( m_progressUpdater->interrupted()) {
+    if (!m_progressUpdater.isNull() && m_progressUpdater->interrupted()) {
         m_progressUpdater->setProgress(100);
         return false;
     }
@@ -557,9 +558,10 @@ bool KisTransformWorker::run()
     //CBRm_dev->setDirty();
 
     //progress info
-    m_progressUpdater->setProgress(100);
+    if (!m_progressUpdater.isNull()) m_progressUpdater->setProgress(100);
 
-    return m_progressUpdater->interrupted();
+    if (!m_progressUpdater.isNull()) return m_progressUpdater->interrupted();
+    return true;
 }
 
 QRect KisTransformWorker::mirrorX(KisPaintDeviceSP dev, const KisSelection* selection)
@@ -569,7 +571,7 @@ QRect KisTransformWorker::mirrorX(KisPaintDeviceSP dev, const KisSelection* sele
     int pixelSize = dev->pixelSize();
 
     if (!hasCurrentMemento) {
-            dst = new KisPaintDevice(dev->colorSpace());
+        dst = new KisPaintDevice(dev->colorSpace());
     }
 
     QRect r;
@@ -602,7 +604,7 @@ QRect KisTransformWorker::mirrorX(KisPaintDeviceSP dev, const KisSelection* sele
         }
     }
     if (!hasCurrentMemento) {
-        dev->setDataManager( dst->dataManager(), dst->colorSpace() );
+        dev->setDataManager(dst->dataManager(), dst->colorSpace());
     }
 
     return r;
@@ -646,7 +648,7 @@ QRect KisTransformWorker::mirrorY(KisPaintDeviceSP dev, const KisSelection* sele
         }
     }
     if (!hasCurrentMemento) {
-        dev->setDataManager( dst->dataManager(), dst->colorSpace() );
+        dev->setDataManager(dst->dataManager(), dst->colorSpace());
     }
 
     return r;
