@@ -23,13 +23,17 @@
 
 class KisRecordedAction;
 
+/**
+ * This class record actions and allow other \ref KisMacro to connect to it
+ * to get the action that are currently created.
+ */
 class KRITAIMAGE_EXPORT KisActionRecorder : public KisMacro
 {
 
     Q_OBJECT
 
 public:
-    KisActionRecorder(KisImageSP image);
+    KisActionRecorder();
     ~KisActionRecorder();
 
 public slots:
@@ -37,7 +41,9 @@ public slots:
     void addAction(const KisRecordedAction& action);
 
 signals:
-
+    /**
+     * This signal is emitted each time an action is added to this recorder.
+     */
     void addedAction(const KisRecordedAction& action);
 };
 
