@@ -22,15 +22,25 @@
 #include <KoColorTransformation.h>
 #include <KoColorSpace.h>
 #include <KoColorSpaceRegistry.h>
-#include "kis_colorspace_convert_visitor.h"
 
+#include <testutil.h>
+
+#include "kis_colorspace_convert_visitor.h"
+#include "kis_paint_layer.h"
 #include "kis_image.h"
 
 void KisColorSpaceConvertVisitorTest::testCreation()
 {
-    const KoColorSpace * colorSpace = KoColorSpaceRegistry::instance()->colorSpace("CMYK", 0);
-    QVERIFY(colorSpace);
-    KisColorSpaceConvertVisitor test(colorSpace, KoColorConversionTransformation::IntentPerceptual);
+    const KoColorSpace * rgb = KoColorSpaceRegistry::instance()->rgb16();
+    QVERIFY(rgb);
+    const KoColorSpace * lab = KoColorSpaceRegistry::instance()->lab16();
+    QVERIFY(lab);
+    TestUtil::KisUndoAdapterDummy* undoAdapterDummy = new TestUtil::KisUndoAdapterDummy();
+    KisImageWSP img = new KisImage(undoAdapterDummy, 100, 100, lab, "test");
+    KisPaintLayerSP layer = new KisPaintLayer(img, "test", OPACITY_OPAQUE, lab);
+    KisColorSpaceConvertVisitor test(img, rgb, KoColorConversionTransformation::IntentPerceptual);
+    layer->accept(test);
+    QVERIFY(layer->colorSpace()->colorModelId() == rgb->colorModelId());
 }
 
 
