@@ -25,6 +25,7 @@ class KisPaintDeviceTest : public QObject
 {
     Q_OBJECT
 
+
     void testCreation();
     void testStore();
     void testGeometry();
@@ -32,15 +33,16 @@ class KisPaintDeviceTest : public QObject
     void testCrop();
     void testThumbnail();
     void testPixel();
-    void testDirty();
     void testRoundtripReadWrite();
     void testPlanarReadWrite();
-    void testColorSpaceConversion();
+    void testRoundtripConversion();
+    void testBltPerformance();
+
 
 private slots:
 
-    void testRoundtripConversion();
 
+    void testColorSpaceConversion();
 };
 
 #endif
