@@ -18,7 +18,7 @@
 
 #include "kis_tileddatamanager.h"
 
-#include <q3valuevector.h>
+#include <QVector>
 
 #include <KoStore.h>
 
@@ -29,6 +29,28 @@
 #include "kis_memento.h"
 #include "kis_tilemanager.h"
 
+
+class KisTileDataWrapper
+{
+    KisTile* m_tile;
+    qint32 m_offset;
+public:
+    KisTileDataWrapper(KisTile* tile, qint32 offset)
+            : m_tile(tile), m_offset(offset) {
+        m_tile->addReader();
+    }
+
+    virtual ~KisTileDataWrapper() {
+        m_tile->removeReader();
+    }
+
+    quint8* data() const {
+        return m_tile->data() + m_offset;
+    }
+};
+
+
+
 /* The data area is divided into tiles each say 64x64 pixels (defined at compiletime)
  * The tiles are laid out in a matrix that can have negative indexes.
  * The matrix grows automatically if needed (a call for writeacces to a tile outside the current extent)
@@ -231,9 +253,7 @@ QRect KisTiledDataManager::extent() const
 void KisTiledDataManager::setExtent(qint32 x, qint32 y, qint32 w, qint32 h)
 {
     QRect newRect = QRect(x, y, w, h).normalized();
-    //printRect("newRect", newRect);
     QRect oldRect = QRect(m_extentMinX, m_extentMinY, m_extentMaxX - m_extentMinX + 1, m_extentMaxY - m_extentMinY + 1).normalized();
-    //printRect("oldRect", oldRect);
 
     // Do nothing if the desired size is bigger than we currently are: that is handled by the autoextending automatically
     if (newRect.contains(oldRect)) return;
@@ -246,7 +266,6 @@ void KisTiledDataManager::setExtent(qint32 x, qint32 y, qint32 w, qint32 h)
 
         while (tile) {
             QRect tileRect = QRect(tile->getCol() * KisTile::WIDTH, tile->getRow() * KisTile::HEIGHT, KisTile::WIDTH, KisTile::HEIGHT);
-            //printRect("tileRect", tileRect);
 
             if (newRect.contains(tileRect)) {
                 // Completely inside, do nothing
@@ -259,7 +278,6 @@ void KisTiledDataManager::setExtent(qint32 x, qint32 y, qint32 w, qint32 h)
 
                     // Create the intersection of the tile and new rect
                     QRect intersection = newRect.intersect(tileRect);
-                    //printRect("intersection", intersection);
                     intersection.setRect(intersection.x() - tileRect.x(), intersection.y() - tileRect.y(), intersection.width(), intersection.height());
 
                     // This can be done a lot more efficiently, no doubt, by clearing runs of pixels to the left and the right of
@@ -794,15 +812,7 @@ KisTile *KisTiledDataManager::getOldTile(qint32 col, qint32 row, KisTile *def)
     return tile;
 }
 
-quint8* KisTiledDataManager::pixelPtr(qint32 x, qint32 y, bool writable)
-{
-    // Ahem, this is a bit not as good. The point is, this function needs the tile data,
-    // but it might be swapped out. This code swaps it in, but at function exit it might
-    // be swapped out again! THIS MAKES THE RETURNED POINTER QUITE VOLATILE
-    return pixelPtrSafe(x, y, writable) -> data();
-}
-
-KisTileDataWrapperSP KisTiledDataManager::pixelPtrSafe(qint32 x, qint32 y, bool writable)
+KisTileDataWrapper* KisTiledDataManager::pixelPtrSafe(qint32 x, qint32 y) const
 {
     qint32 row = yToRow(y);
     qint32 col = xToCol(x);
@@ -812,21 +822,41 @@ KisTileDataWrapperSP KisTiledDataManager::pixelPtrSafe(qint32 x, qint32 y, bool
     qint32 xInTile = x - col * KisTile::WIDTH;
     qint32 offset = m_pixelSize * (yInTile * KisTile::WIDTH + xInTile);
 
-    KisTile *tile = getTile(col, row, writable);
+    KisTile *tile = const_cast<KisTiledDataManager*>(this)->getTile(col, row, false);
+    KisTileDataWrapper* ktdw = new KisTileDataWrapper(tile, offset);
 
-    return new KisTileDataWrapper(tile, offset);
+    return ktdw;
 }
 
+KisTileDataWrapper* KisTiledDataManager::pixelPtrSafe(qint32 x, qint32 y)
+{
+    qint32 row = yToRow(y);
+    qint32 col = xToCol(x);
+
+    // calc limits within the tile
+    qint32 yInTile = y - row * KisTile::HEIGHT;
+    qint32 xInTile = x - col * KisTile::WIDTH;
+    qint32 offset = m_pixelSize * (yInTile * KisTile::WIDTH + xInTile);
+
+    KisTile *tile = getTile(col, row, true);
+
+    KisTileDataWrapper* ktdw = new KisTileDataWrapper(tile, offset);
+
+    return ktdw;
+}
+
+
 void KisTiledDataManager::setPixel(qint32 x, qint32 y, const quint8 * data)
 {
-    quint8 *pixel = pixelPtr(x, y, true);
-    memcpy(pixel, data, m_pixelSize);
+    KisTileDataWrapper* tileData = pixelPtrSafe(x, y);
+    memcpy(tileData->data(), data, m_pixelSize);
+    delete tileData;
 }
 
 
 void KisTiledDataManager::readBytes(quint8 * data,
                                     qint32 x, qint32 y,
-                                    qint32 w, qint32 h)
+                                    qint32 w, qint32 h) const
 {
     if (data == 0) return;
     //Q_ASSERT(data != 0);
@@ -855,8 +885,8 @@ void KisTiledDataManager::readBytes(quint8 * data,
 
             qint32 columns = qMin(numContiguousSrcColumns, columnsRemaining);
 
-            KisTileDataWrapperSP tileData = pixelPtrSafe(srcX, srcY, false);
-            const quint8 *srcData = tileData -> data();
+            KisTileDataWrapper* tileData = pixelPtrSafe(srcX, srcY);
+            const quint8 *srcData = tileData->data();
             qint32 srcRowStride = rowStride(srcX, srcY);
 
             quint8 *dstData = data + ((dstX + (dstY * w)) * m_pixelSize);
@@ -871,6 +901,8 @@ void KisTiledDataManager::readBytes(quint8 * data,
             srcX += columns;
             dstX += columns;
             columnsRemaining -= columns;
+
+            delete tileData;
         }
 
         srcY += rows;
@@ -914,7 +946,7 @@ void KisTiledDataManager::writeBytes(const quint8 * bytes,
 
             qint32 columns = qMin(numContiguousdstColumns, columnsRemaining);
 
-            KisTileDataWrapperSP tileData = pixelPtrSafe(dstX, dstY, true);
+            KisTileDataWrapper* tileData = pixelPtrSafe(dstX, dstY);
             quint8 *dstData = tileData->data();
             qint32 dstRowStride = rowStride(dstX, dstY);
 
@@ -927,6 +959,7 @@ void KisTiledDataManager::writeBytes(const quint8 * bytes,
                 dstData += dstRowStride;
             }
 
+            delete tileData;
             dstX += columns;
             srcX += columns;
             columnsRemaining -= columns;
@@ -982,8 +1015,8 @@ QVector<quint8*> KisTiledDataManager::readPlanarBytes(QVector<qint32> channelsiz
 
             qint32 columns = qMin(numContiguousSrcColumns, columnsRemaining);
 
-            KisTileDataWrapperSP tileData = pixelPtrSafe(srcX, srcY, false);
-            const quint8 *srcData = tileData -> data();
+            KisTileDataWrapper* tileData = pixelPtrSafe(srcX, srcY);
+            const quint8 *srcData = tileData->data();
 
             for (qint32 row = 0; row < rows; row++) {
                 for (qint32 col = 0; col < columns; ++col) {
@@ -1000,7 +1033,7 @@ QVector<quint8*> KisTiledDataManager::readPlanarBytes(QVector<qint32> channelsiz
 
                 }
             }
-
+            delete tileData;
             srcX += columns;
             columnsRemaining -= columns;
         }
@@ -1046,7 +1079,7 @@ void KisTiledDataManager::writePlanarBytes(QVector<quint8*> planes, QVector<qint
 
             qint32 columns = qMin(numContiguousdstColumns, columnsRemaining);
 
-            KisTileDataWrapperSP tileData = pixelPtrSafe(dstX, dstY, true);
+            KisTileDataWrapper* tileData = pixelPtrSafe(dstX, dstY);
             quint8 *dstData = tileData->data();
 
             for (qint32 row = 0; row < rows; ++row) {
@@ -1059,7 +1092,7 @@ void KisTiledDataManager::writePlanarBytes(QVector<quint8*> planes, QVector<qint
                     }
                 }
             }
-
+            delete tileData;
             dstX += columns;
             columnsRemaining -= columns;
         }
@@ -1116,13 +1149,3 @@ qint32 KisTiledDataManager::numTiles(void) const
     return m_numTiles;
 }
 
-KisTileDataWrapper::KisTileDataWrapper(KisTile* tile, qint32 offset)
-        : m_tile(tile), m_offset(offset)
-{
-    m_tile->addReader();
-}
-
-KisTileDataWrapper::~KisTileDataWrapper()
-{
-    m_tile->removeReader();
-}
