@@ -21,9 +21,6 @@
 
 #include <kis_paintop_options_widget.h>
 
-#include "ui_wdgdynaoptions.h"
-#include "widgets/kis_popup_button.h"
-
 class KisPaintActionTypeOption;
 class KisDynaOpOption;
 
@@ -35,9 +32,7 @@ public:
     KisDynaPaintOpSettingsWidget(QWidget* parent = 0);
     virtual ~KisDynaPaintOpSettingsWidget();
 
-    void setConfiguration( const KisPropertiesConfiguration * config);
     KisPropertiesConfiguration* configuration() const;
-    void writeConfiguration( KisPropertiesConfiguration *config ) const;
 
 public:
     KisPaintActionTypeOption* m_paintActionTypeOption;
