@@ -21,8 +21,6 @@
 
 #include <kparts/plugin.h>
 
-class KisView;
-
 /**
  * A module that provides a crop tool.
  */
@@ -33,10 +31,6 @@ public:
     ToolCrop(QObject *parent, const QStringList &);
     virtual ~ToolCrop();
 
-private:
-
-    KisView * m_view;
-
 };
 
 #endif // TOOL_CROP_H_
