@@ -20,8 +20,6 @@
 
 #include <kparts/plugin.h>
 
-class KisView;
-
 /**
  * A module that provides a tool for doinge perspective transformation.
  */
@@ -32,10 +30,6 @@ public:
     ToolPerspectiveTransform(QObject *parent, const QStringList &);
     virtual ~ToolPerspectiveTransform();
 
-private:
-
-    KisView * m_view;
-
 };
 
 #endif // TOOL_PERSPECTIVE_TRANSFORM_H_
