@@ -21,8 +21,6 @@
 
 #include <kparts/plugin.h>
 
-class KisView;
-
 /**
  * A module that provides a star tool.
  */
@@ -33,10 +31,6 @@ public:
     ToolStar(QObject *parent, const QStringList &);
     virtual ~ToolStar();
 
-private:
-
-    KisView * m_view;
-
 };
 
 #endif // TOOL_STAR_H_
