@@ -44,6 +44,7 @@
 #include <KoCompositeOp.h>
 #include <KoShapeManager.h>
 #include <KoProgressUpdater.h>
+#include <KoUpdater.h>
 
 #include <kis_global.h>
 #include <canvas/kis_canvas2.h>
@@ -61,12 +62,14 @@
 #include <kis_pixel_selection.h>
 #include <kis_shape_selection.h>
 #include <kis_selection_manager.h>
+
 #include <KoShapeTransformCommand.h>
 
 #include "flake/kis_node_shape.h"
 #include "flake/kis_layer_container_shape.h"
 #include "flake/kis_shape_layer.h"
 #include "kis_canvas_resource_provider.h"
+#include "widgets/kis_progress_widget.h"
 
 namespace
 {
@@ -80,9 +83,9 @@ public:
 public:
     virtual void redo();
     virtual void undo();
-    void transformArgs(double &sx, double &sy, QPointF &translate, double &a);
-    KisSelectionSP origSelection(QPoint &startPos, QPoint &endPos);
-
+    void transformArgs(double &sx, double &sy, QPointF &translate, double &a) const;
+    KisSelectionSP origSelection(QPoint &startPos, QPoint &endPos) const;
+    void setNewPosition(int x, int y);
 private:
     double m_scaleX;
     double m_scaleY;
@@ -92,6 +95,7 @@ private:
     KisSelectionSP m_origSelection;
     QPoint m_originalTopLeft;
     QPoint m_originalBottomRight;
+    QPoint m_newPosition;
 };
 
 TransformCmd::TransformCmd(KisToolTransform *tool, KisNodeSP node, double scaleX, double scaleY, QPointF translate, double a, KisSelectionSP origSel, QPoint originalTopLeft, QPoint originalBottomRight)
@@ -111,7 +115,7 @@ TransformCmd::~TransformCmd()
 {
 }
 
-void TransformCmd::transformArgs(double &sx, double &sy, QPointF &translate, double &a)
+void TransformCmd::transformArgs(double &sx, double &sy, QPointF &translate, double &a) const
 {
     sx = m_scaleX;
     sy = m_scaleY;
@@ -119,7 +123,7 @@ void TransformCmd::transformArgs(double &sx, double &sy, QPointF &translate, dou
     a = m_a;
 }
 
-KisSelectionSP TransformCmd::origSelection(QPoint &originalTopLeft, QPoint &originalBottomRight)
+KisSelectionSP TransformCmd::origSelection(QPoint &originalTopLeft, QPoint &originalBottomRight) const
 {
     originalTopLeft = m_originalTopLeft;
     originalBottomRight = m_originalBottomRight;
@@ -129,12 +133,21 @@ KisSelectionSP TransformCmd::origSelection(QPoint &originalTopLeft, QPoint &orig
 void TransformCmd::redo()
 {
     KisSelectedTransaction::redo();
+    layer()->paintDevice()->move(m_newPosition);
 }
 
 void TransformCmd::undo()
 {
     KisSelectedTransaction::undo();
+    layer()->paintDevice()->move(m_originalTopLeft);
 }
+
+void TransformCmd::setNewPosition(int x, int y)
+{
+    m_newPosition.setX(x);
+    m_newPosition.setY(y);
+}
+
 }
 
 KisToolTransform::KisToolTransform(KoCanvasBase * canvas)
@@ -178,13 +191,12 @@ void KisToolTransform::activate(bool temporary)
     Q_UNUSED(temporary);
 
     if (currentNode() && currentNode()->paintDevice()) {
-        //connect(m_subject, commandExecuted(K3Command *c), this, notifyCommandAdded( KCommand * c));
-        //m_subject->undoAdapter()->setCommandHistoryListener( this );
+        image()->undoAdapter()->setCommandHistoryListener(this);
 
-        TransformCmd * cmd = 0;
+        const TransformCmd * cmd = 0;
 
         if (image()->undoAdapter()->presentCommand())
-            cmd = dynamic_cast<TransformCmd*>(image()->undoAdapter()->presentCommand());
+            cmd = dynamic_cast<const TransformCmd*>(image()->undoAdapter()->presentCommand());
 
         if (cmd == 0) {
             initHandles();
@@ -235,7 +247,7 @@ void KisToolTransform::initHandles()
 
 void KisToolTransform::mousePressEvent(KoPointerEvent *e)
 {
-    KisImageSP img = image();
+    KisImageWSP img = image();
 
     if (!currentNode())
         return;
@@ -343,7 +355,7 @@ void KisToolTransform::mouseMoveEvent(KoPointerEvent *e)
     QPointF bottomleft = m_bottomleft;
     QPointF bottomright = m_bottomright;
 
-    KisImageSP img = image();
+    KisImageWSP img = image();
     QPointF mousePos = QPointF(e->point.x() * img->xRes(), e->point.y() * img->yRes());
 
     if (m_selecting) {
@@ -602,7 +614,7 @@ void KisToolTransform::paint(QPainter& gc, const KoViewConverter &converter)
     pen.setWidth(0);
 
     recalcOutline();
-    KisImageSP img = image();
+    KisImageWSP img = image();
     QPointF topleft = converter.documentToView(QPointF(m_topleft.x() / img->xRes(), m_topleft.y() / img->yRes()));
     QPointF topright = converter.documentToView(QPointF(m_topright.x() / img->xRes(), m_topright.y() / img->yRes()));
     QPointF bottomleft = converter.documentToView(QPointF(m_bottomleft.x() / img->xRes(), m_bottomleft.y() / img->yRes()));
@@ -635,13 +647,13 @@ void KisToolTransform::transform()
         return;
 
     KisCanvas2 *canvas = dynamic_cast<KisCanvas2 *>(m_canvas);
-    if(!canvas)
+    if (!canvas)
         return;
 
     QPointF t = m_translate - rot(m_originalCenter.x() * m_scaleX, m_originalCenter.y() * m_scaleY);
-    KoProgressUpdater updater(canvas->view()->statusBar()->progress());
-    updater.start( 100, i18n("Transform") );
-    KoUpdater progress = updater.startSubtask();
+    KoProgressUpdater* updater = canvas->view()->createProgressUpdater();
+    updater->start(100, i18n("Transform"));
+    KoUpdaterPtr progress = updater->startSubtask();
 
     // This mementoes the current state of the active device.
     TransformCmd * transaction = new TransformCmd(this, currentNode(), m_scaleX,
@@ -653,7 +665,8 @@ void KisToolTransform::transform()
     rc = rc.normalized();
     currentNode()->paintDevice()->clear();
     KisPainter gc(currentNode()->paintDevice());
-    gc.bitBlt(rc.x(), rc.y(), COMPOSITE_COPY, m_origDevice, rc.x(), rc.y(), rc.width(), rc.height());
+    gc.setCompositeOp(COMPOSITE_COPY);
+    gc.bitBlt(rc.topLeft(), m_origDevice, rc);
     gc.end();
 
     // Also restore the original selection.
@@ -663,12 +676,12 @@ void KisToolTransform::transform()
         if (currentSelection()) {
             currentSelection()->getOrCreatePixelSelection()->clear();
             KisPainter sgc(KisPaintDeviceSP(currentSelection()->getOrCreatePixelSelection()));
-            sgc.bitBlt(rc.x(), rc.y(), COMPOSITE_COPY, m_origSelection, rc.x(), rc.y(), rc.width(), rc.height());
+            sgc.setCompositeOp(COMPOSITE_COPY);
+            sgc.bitBlt(rc.topLeft(), m_origSelection, rc);
             sgc.end();
         }
-    } else
-        if (currentSelection())
-            currentSelection()->clear();
+    } else if (currentSelection())
+        currentSelection()->clear();
 
     // Perform the transform. Since we copied the original state back, this doesn't degrade
     // after many tweaks. Since we started the transaction before the copy back, the memento
@@ -677,27 +690,26 @@ void KisToolTransform::transform()
         KisPaintDeviceSP tmpDevice = new KisPaintDevice(m_origDevice->colorSpace());
         QRect selectRect = currentSelection()->selectedRect();
         KisPainter gc(tmpDevice, currentSelection());
-        gc.bltSelection(selectRect.x(), selectRect.y(), COMPOSITE_OVER, m_origDevice, OPACITY_OPAQUE,
-                        selectRect.x(), selectRect.y(), selectRect.width(), selectRect.height());
+        gc.bitBlt(selectRect.topLeft(), m_origDevice, selectRect);
         gc.end();
 
-        KisTransformWorker worker(tmpDevice, m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), &progress, m_filter);
+        KisTransformWorker worker(tmpDevice, m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), progress, m_filter);
         worker.run();
 
         currentNode()->paintDevice()->clearSelection(currentSelection());
 
-        KisTransformWorker selectionWorker(currentSelection()->getOrCreatePixelSelection(), m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), &progress, m_filter);
+        KisTransformWorker selectionWorker(currentSelection()->getOrCreatePixelSelection(), m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), progress, m_filter);
         selectionWorker.run();
 
 
-        if(currentSelection()->hasShapeSelection()) {
+        if (currentSelection()->hasShapeSelection()) {
             QMatrix resolutionMatrix;
-            resolutionMatrix.scale(1/image()->xRes(), 1/image()->yRes());
+            resolutionMatrix.scale(1 / image()->xRes(), 1 / image()->yRes());
             QPointF center = resolutionMatrix.map(m_originalCenter);
 
             QMatrix matrix;
             matrix.translate(center.x(), center.y());
-            matrix.rotate(m_a/M_PI*180);
+            matrix.rotate(m_a / M_PI*180);
             matrix.translate(-center.x(), -center.y());
 
             KisShapeSelection* shapeSelection = static_cast<KisShapeSelection*>(currentSelection()->shapeSelection());
@@ -708,17 +720,16 @@ void KisToolTransform::transform()
                 m_oldMatrixList << shape->transformation();
                 m_newMatrixList << shape->transformation()*matrix;
             }
-            KoShapeTransformCommand* cmd = new KoShapeTransformCommand( shapes, m_oldMatrixList, m_newMatrixList, transaction );
+            KoShapeTransformCommand* cmd = new KoShapeTransformCommand(shapes, m_oldMatrixList, m_newMatrixList, transaction);
             cmd->redo();
         }
 
         QRect tmpRc = tmpDevice->extent();
         KisPainter painter(currentNode()->paintDevice());
-        painter.bitBlt(tmpRc.x(), tmpRc.y(), COMPOSITE_OVER, tmpDevice,
-                       tmpRc.x(), tmpRc.y(), tmpRc.width(), tmpRc.height());
+        painter.bitBlt(tmpRc.topLeft(), tmpDevice, tmpRc);
         painter.end();
     } else {
-        KisTransformWorker worker(currentNode()->paintDevice(), m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), &progress, m_filter);
+        KisTransformWorker worker(currentNode()->paintDevice(), m_scaleX, m_scaleY, 0, 0, m_a, int(t.x()), int(t.y()), progress, m_filter);
         worker.run();
     }
 
@@ -734,23 +745,26 @@ void KisToolTransform::transform()
     currentNode()->paintDevice()->setDirty(rc); // XXX: This is not enough - should union with new extent
 
     canvas->view()->selectionManager()->selectionChanged();
-    if(currentSelection() && currentSelection()->hasShapeSelection())
+    if (currentSelection() && currentSelection()->hasShapeSelection())
         canvas->view()->selectionManager()->shapeSelectionChanged();
 
     // Else add the command -- this will have the memento from the previous state,
     // and the transformed state from the original device we cached in our activated()
     // method.
     if (transaction) {
+        transaction->setNewPosition(currentNode()->paintDevice()->x(), currentNode()->paintDevice()->y());
         if (image()->undo())
             image()->undoAdapter()->addCommand(transaction);
         else
             delete transaction;
     }
+    updater->deleteLater();
+
 }
 
-void KisToolTransform::notifyCommandAdded(QUndoCommand * command)
+void KisToolTransform::notifyCommandAdded(const QUndoCommand * command)
 {
-    TransformCmd * cmd = dynamic_cast<TransformCmd*>(command);
+    const TransformCmd * cmd = dynamic_cast<const TransformCmd*>(command);
     if (cmd == 0) {
         // The last added command wasn't one of ours;
         // we should reset to the new state of the canvas.
@@ -759,13 +773,13 @@ void KisToolTransform::notifyCommandAdded(QUndoCommand * command)
     }
 }
 
-void KisToolTransform::notifyCommandExecuted(QUndoCommand * command)
+void KisToolTransform::notifyCommandExecuted(const QUndoCommand * command)
 {
     Q_UNUSED(command);
-    TransformCmd * cmd = 0;
+    const TransformCmd * cmd = 0;
 
     if (image()->undoAdapter()->presentCommand())
-        cmd = dynamic_cast<TransformCmd*>(image()->undoAdapter()->presentCommand());
+        cmd = dynamic_cast<const TransformCmd*>(image()->undoAdapter()->presentCommand());
 
     if (cmd == 0) {
         // The command now on the top of the stack isn't one of ours
@@ -790,7 +804,7 @@ QWidget* KisToolTransform::createOptionWidget()
     m_optWidget = new WdgToolTransform(0);
     Q_CHECK_PTR(m_optWidget);
     m_optWidget->setObjectName(toolId() + " option widget");
-    
+
     m_optWidget->cmbFilter->clear();
     m_optWidget->cmbFilter->setIDList(KisFilterStrategyRegistry::instance()->listKeys());
 
@@ -815,6 +829,7 @@ QWidget* KisToolTransform::createOptionWidget()
     m_optWidget->textLabel2->hide();
     m_optWidget->textLabel3->hide();
     m_optWidget->textLabel4->hide();
+    m_optWidget->setFixedHeight(m_optWidget->sizeHint().height());
     return m_optWidget;
 }
 
