@@ -5,12 +5,14 @@
 
 class KisView;
 
-class %{APPNAME}Plugin : public KParts::Plugin
+class %
 {
+    APPNAME
+} Plugin : public KParts::Plugin {
     Q_OBJECT
 public:
-    %{APPNAME}Plugin(QObject *parent, const QStringList &);
-    virtual ~%{APPNAME}Plugin();
+    % {APPNAME} Plugin(QObject *parent, const QStringList &);
+    virtual ~ % {APPNAME} Plugin();
 
 private:
 
