@@ -8,12 +8,14 @@ class KisView2;
 /**
  * Template of view plugin
  */
-class %{APPNAME}Plugin : public KParts::Plugin
+class %
 {
+    APPNAME
+} Plugin : public KParts::Plugin {
     Q_OBJECT
 public:
-    %{APPNAME}Plugin(QObject *parent, const QStringList &);
-    virtual ~%{APPNAME}Plugin();
+    % {APPNAME} Plugin(QObject *parent, const QStringList &);
+    virtual ~ % {APPNAME} Plugin();
 
 private slots:
 
