@@ -40,6 +40,7 @@
 #include <KoColorProfile.h>
 #include <KoSelection.h>
 #include <KoShapeManager.h>
+#include <KoProgressUpdater.h>
 
 #include <filter/kis_filter_configuration.h>
 #include <filter/kis_filter.h>
@@ -58,6 +59,7 @@
 #include <kis_shear_visitor.h>
 #include <kis_transform_worker.h>
 #include <kis_undo_adapter.h>
+#include <kis_painter.h>
 
 #include "kis_config.h"
 #include "kis_cursor.h"
@@ -78,32 +80,21 @@
 #include "canvas/kis_canvas2.h"
 #include "widgets/kis_meta_data_merge_strategy_chooser_widget.h"
 #include "widgets/kis_wdg_generator.h"
-#include "kis_layer_box.h"
+#include "kis_progress_widget.h"
 #include "kis_node_commands_adapter.h"
+#include "kis_node_manager.h"
 
 KisLayerManager::KisLayerManager(KisView2 * view, KisDoc2 * doc)
         : m_view(view)
         , m_doc(doc)
         , m_imgFlatten(0)
         , m_imgMergeLayer(0)
-        , m_actionAdjustmentLayer(0)
-        , m_actionGeneratorLayer(0)
-        , m_layerAdd(0)
-        , m_layerAddCloneLayer(0)
-        , m_layerAddShapeLayer(0)
-        , m_layerBottom(0)
-        , m_layerDup(0)
-        , m_layerHide(0)
-        , m_layerLower(0)
-        , m_layerProperties(0)
-        , m_layerRaise(0)
-        , m_layerRm(0)
         , m_layerSaveAs(0)
-        , m_layerTop(0)
         , m_actLayerVis(false)
         , m_imgResizeToLayer(0)
+        , m_flattenLayer(0)
         , m_activeLayer(0)
-        , m_commandsAdapter( new KisNodeCommandsAdapter( m_view ) )
+        , m_commandsAdapter(new KisNodeCommandsAdapter(m_view))
 {
 }
 
@@ -130,8 +121,7 @@ void KisLayerManager::activateLayer(KisLayerSP layer)
     m_activeLayer = layer;
     emit sigLayerActivated(layer);
     layersUpdated();
-    if(layer)
-    {
+    if (layer) {
         m_view->resourceProvider()->slotNodeActivated(layer.data());
     }
 }
@@ -149,79 +139,15 @@ void KisLayerManager::setup(KActionCollection * actionCollection)
     m_imgMergeLayer->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_E));
     connect(m_imgMergeLayer, SIGNAL(triggered()), this, SLOT(mergeLayer()));
 
-    m_layerAdd  = new KAction(i18n("&Add..."), this);
-    actionCollection->addAction("insert_layer", m_layerAdd);
-    m_layerAdd->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_N));
-    connect(m_layerAdd, SIGNAL(triggered()), this, SLOT(layerAdd()));
-
-    m_actionAdjustmentLayer  = new KAction(i18n("&Filter Layer"), this);
-    actionCollection->addAction("insert_adjustment_layer", m_actionAdjustmentLayer);
-    connect(m_actionAdjustmentLayer, SIGNAL(triggered()), this, SLOT(addAdjustmentLayer()));
-
-    m_actionGeneratorLayer  = new KAction(i18n("&Generator Layer"), this);
-    actionCollection->addAction("insert_Generator_layer", m_actionGeneratorLayer);
-    connect(m_actionGeneratorLayer, SIGNAL(triggered()), this, SLOT(addGeneratorLayer()));
-
-    m_layerAddCloneLayer  = new KAction(i18n("&Clone Layer"), this);
-    actionCollection->addAction("insert_clone_layer", m_layerAddCloneLayer);
-    connect(m_layerAddCloneLayer, SIGNAL(triggered()), this, SLOT(addCloneLayer()));
-
-    m_layerAddShapeLayer  = new KAction(i18n("&Shape Layer"), this);
-    actionCollection->addAction("insert_shape_layer", m_layerAddShapeLayer);
-    connect(m_layerAddShapeLayer, SIGNAL(triggered()), this, SLOT(addShapeLayer()));
-
-    m_layerRm  = new KAction(i18n("&Remove"), this);
-    actionCollection->addAction("remove_layer", m_layerRm);
-    connect(m_layerRm, SIGNAL(triggered()), this, SLOT(layerRemove()));
-
-    m_layerDup  = new KAction(i18n("Duplicate"), this);
-    actionCollection->addAction("duplicate_layer", m_layerDup);
-    connect(m_layerDup, SIGNAL(triggered()), this, SLOT(layerDuplicate()));
-
-    m_layerHide  = new KToggleAction(i18n("&Hide"), this);
-    actionCollection->addAction("hide_layer", m_layerHide);
-    connect(m_layerHide, SIGNAL(triggered()), this,  SLOT(layerToggleVisible()));
-
-    m_layerHide->setCheckedState(KGuiItem(i18n("&Show")));
-    m_layerHide->setChecked(false);
-
-    m_layerRaise  = new KAction(KIcon("go-up"), i18n("Raise"), this);
-    actionCollection->addAction("raiselayer", m_layerRaise);
-    m_layerRaise->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_BracketRight));
-    connect(m_layerRaise, SIGNAL(triggered()), this, SLOT(layerRaise()));
-
-    m_layerLower  = new KAction(KIcon("go-down"), i18n("Lower"), this);
-    actionCollection->addAction("lowerlayer", m_layerLower);
-    m_layerLower->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_BracketLeft));
-    connect(m_layerLower, SIGNAL(triggered()), this, SLOT(layerLower()));
-
-    m_layerTop  = new KAction(KIcon("go-top"), i18n("To Top"), this);
-    actionCollection->addAction("toplayer", m_layerTop);
-    m_layerTop->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_BracketRight));
-    connect(m_layerTop, SIGNAL(triggered()), this, SLOT(layerFront()));
-
-    m_layerBottom  = new KAction(KIcon("go-down"), i18n("To Bottom"), this);
-    actionCollection->addAction("bottomlayer", m_layerBottom);
-    m_layerBottom->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_BracketLeft));
-    connect(m_layerBottom, SIGNAL(triggered()), this, SLOT(layerBack()));
-
-    m_layerProperties  = new KAction(i18n("Properties..."), this);
-    actionCollection->addAction("layer_properties", m_layerProperties);
-    connect(m_layerProperties, SIGNAL(triggered()), this, SLOT(layerProperties()));
+    m_flattenLayer  = new KAction(i18n("&Flatten Layer"), this);
+    actionCollection->addAction("flatten_layer", m_flattenLayer);
+    connect(m_flattenLayer, SIGNAL(triggered()), this, SLOT(flattenLayer()));
 
     m_layerSaveAs  = new KAction(KIcon("document-save"), i18n("Save Layer as Image..."), this);
     actionCollection->addAction("save_layer_as_image", m_layerSaveAs);
     connect(m_layerSaveAs, SIGNAL(triggered()), this, SLOT(saveLayerAsImage()));
 
-    KAction * action  = new KAction(KIcon("view-split-left-right"), i18n("Flip on &X Axis"), this);
-    actionCollection->addAction("mirrorLayerX", action);
-    connect(action, SIGNAL(triggered()), this, SLOT(mirrorLayerX()));
-
-    action  = new KAction(KIcon("view-split-top-bottom"), i18n("Flip on &Y Axis"), this);
-    actionCollection->addAction("mirrorLayerY", action);
-    connect(action, SIGNAL(triggered()), this, SLOT(mirrorLayerY()));
-
-    m_imgResizeToLayer  = new KAction(i18n("Resize Image to Size of Current Layer"), this);
+    m_imgResizeToLayer  = new KAction(i18n("Size Canvas to Size of Current Layer"), this);
     actionCollection->addAction("resizeimgtolayer", m_imgResizeToLayer);
     connect(m_imgResizeToLayer, SIGNAL(triggered()), this, SLOT(imgResizeToActiveLayer()));
 }
@@ -233,8 +159,7 @@ void KisLayerManager::addAction(QAction * action)
 
 void KisLayerManager::updateGUI()
 {
-
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
 
     KisLayerSP layer;
     qint32 nlayers = 0;
@@ -249,25 +174,12 @@ void KisLayerManager::updateGUI()
 
     bool enable = img && layer && layer->visible() && !layer->userLocked() && !layer->systemLocked();
 
-    m_layerDup->setEnabled(enable);
-    m_layerRm->setEnabled(enable);
-    m_layerHide->setEnabled(img && layer);
-    m_layerProperties->setEnabled(enable);
     m_layerSaveAs->setEnabled(enable);
-    m_layerRaise->setEnabled(enable && layer->prevSibling());
-    m_layerLower->setEnabled(enable && layer->nextSibling());
-    m_layerTop->setEnabled(enable && nlayers > 1 && layer.data() != img->rootLayer()->firstChild().data());
-    m_layerBottom->setEnabled(enable && nlayers > 1 && layer.data() != img->rootLayer()->lastChild().data());
 
     // XXX these should be named layer instead of img
     m_imgFlatten->setEnabled(nlayers > 1);
     m_imgMergeLayer->setEnabled(nlayers > 1 && layer && layer->prevSibling());
-
-
-    // XXX: Check whether the colorspace was really changed
-    if (activeDevice()) {
-        emit currentColorSpaceChanged(activeDevice()->colorSpace());
-    }
+    m_flattenLayer->setEnabled(nlayers > 1 && layer && layer->firstChild());
 
     m_imgResizeToLayer->setEnabled(activeLayer());
 
@@ -280,7 +192,7 @@ void KisLayerManager::imgResizeToActiveLayer()
     KisLayerSP layer;
     KisUndoAdapter * undoAdapter = m_view->undoAdapter();
 
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
 
     if (img && (layer = activeLayer())) {
 
@@ -329,24 +241,6 @@ void KisLayerManager::layerOpacity(double opacity, bool final)
     }
 }
 
-void KisLayerManager::layerToggleVisible()
-{
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    layer->setVisible(!layer->visible());
-
-    layer->setDirty();
-}
-
-void KisLayerManager::layerToggleLocked()
-{
-    KisLayerSP layer = activeLayer();
-    if (!layer) return;
-
-    layer->setUserLocked(!layer->userLocked());
-}
-
 void KisLayerManager::actLayerVisChanged(int show)
 {
     m_actLayerVis = (show != 0);
@@ -373,10 +267,10 @@ void KisLayerManager::layerProperties()
         if (prev) dev = prev->projection();
 
         KisDlgAdjLayerProps dlg(dev, alayer->image(), alayer->filter(), alayer->name(), i18n("Filter Layer Properties"), m_view, "dlgadjlayerprops");
-        dlg.resize( dlg.minimumSizeHint() );
+        dlg.resize(dlg.minimumSizeHint());
         KisFilterConfiguration* config = dlg.filterConfiguration();
         QString before;
-        if ( config ) {
+        if (config) {
             before = config->toLegacyXML();
         }
         if (dlg.exec() == QDialog::Accepted) {
@@ -385,14 +279,14 @@ void KisLayerManager::layerProperties()
             if (dlg.filterConfiguration()) {
                 after = dlg.filterConfiguration()->toLegacyXML();
             }
-            if ( after != before ) {
+            if (after != before) {
                 alayer->setName(dlg.layerName());
 
                 KisChangeFilterCmd<KisAdjustmentLayerSP> * cmd
-                    = new KisChangeFilterCmd<KisAdjustmentLayerSP>(alayer,
-                                                                   dlg.filterConfiguration(),
-                                                                   before,
-                                                                   after);
+                = new KisChangeFilterCmd<KisAdjustmentLayerSP>(alayer,
+                        dlg.filterConfiguration(),
+                        before,
+                        after);
                 cmd->redo();
                 m_view->undoAdapter()->addCommand(cmd);
                 m_doc->setModified(true);
@@ -405,18 +299,18 @@ void KisLayerManager::layerProperties()
 
         QString before = alayer->generator()->toLegacyXML();
         dlg.setConfiguration(alayer->generator());
-        dlg.resize( dlg.minimumSizeHint() );
+        dlg.resize(dlg.minimumSizeHint());
 
         if (dlg.exec() == QDialog::Accepted) {
 
             QString after = dlg.configuration()->toLegacyXML();
-            if ( after != before ) {
+            if (after != before) {
                 KisChangeGeneratorCmd<KisGeneratorLayerSP> * cmd
-                                    = new KisChangeGeneratorCmd<KisGeneratorLayerSP>(alayer,
-                                                                                     dlg.configuration(),
-                                                                                     before,
-                                                                                     after
-                                        );
+                = new KisChangeGeneratorCmd<KisGeneratorLayerSP>(alayer,
+                        dlg.configuration(),
+                        before,
+                        after
+                                                                );
 
                 cmd->redo();
                 m_view->undoAdapter()->addCommand(cmd);
@@ -430,7 +324,7 @@ void KisLayerManager::layerProperties()
                                   layer->compositeOp(),
                                   cs,
                                   layer->channelFlags());
-        dlg.resize( dlg.minimumSizeHint() );
+        dlg.resize(dlg.minimumSizeHint());
 
         if (dlg.exec() == QDialog::Accepted) {
 
@@ -447,13 +341,13 @@ void KisLayerManager::layerProperties()
 
             if (layer->name() != dlg.getName() ||
                     layer->opacity() != dlg.getOpacity() ||
-                    layer->compositeOp() != dlg.getCompositeOp() ||
+                    layer->compositeOp()->id() != dlg.getCompositeOp() ||
                     oldChannelFlags != newChannelFlags
                ) {
                 QApplication::setOverrideCursor(KisCursor::waitCursor());
                 m_view->undoAdapter()->addCommand(new KisLayerPropsCommand(layer,
                                                   layer->opacity(), dlg.getOpacity(),
-                                                  layer->compositeOpId(), dlg.getCompositeOp()->id(),
+                                                  layer->compositeOpId(), dlg.getCompositeOp(),
                                                   layer->name(), dlg.getName(),
                                                   oldChannelFlags, newChannelFlags));
                 QApplication::restoreOverrideCursor();
@@ -466,7 +360,7 @@ void KisLayerManager::layerProperties()
 
 void KisLayerManager::layerAdd()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (img && activeLayer()) {
         addLayer(activeLayer()->parent(), activeLayer());
     } else if (img)
@@ -475,7 +369,7 @@ void KisLayerManager::layerAdd()
 
 void KisLayerManager::addLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (img) {
         KisConfig cfg;
         QString profilename;
@@ -484,7 +378,7 @@ void KisLayerManager::addLayer(KisNodeSP parent, KisNodeSP above)
             layer->setCompositeOp(COMPOSITE_OVER);
             m_commandsAdapter->addNode(layer.data(), parent.data(), above.data());
             m_view->canvas()->update();
-            m_view->layerBox()->setCurrentNode( layer );
+            m_view->nodeManager()->activateNode(layer);
         } else {
             KMessageBox::error(m_view, i18n("Could not add layer to image."), i18n("Layer Error"));
         }
@@ -493,7 +387,7 @@ void KisLayerManager::addLayer(KisNodeSP parent, KisNodeSP above)
 
 void KisLayerManager::addGroupLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (img) {
         KisLayerSP layer = KisLayerSP(new KisGroupLayer(img.data(), img->nextLayerName(), OPACITY_OPAQUE));
         if (layer) {
@@ -509,7 +403,7 @@ void KisLayerManager::addGroupLayer(KisNodeSP parent, KisNodeSP above)
 
 void KisLayerManager::addCloneLayer()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (img && activeLayer()) {
         addCloneLayer(activeLayer()->parent(), activeLayer());
     } else if (img)
@@ -518,7 +412,7 @@ void KisLayerManager::addCloneLayer()
 
 void KisLayerManager::addCloneLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (img) {
         // Check whether we are not cloning a parent layer
         if (KisGroupLayer * from = dynamic_cast<KisGroupLayer*>(m_activeLayer.data())) {
@@ -551,7 +445,7 @@ void KisLayerManager::addCloneLayer(KisNodeSP parent, KisNodeSP above)
 
 void KisLayerManager::addShapeLayer()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (img && activeLayer()) {
         addShapeLayer(activeLayer()->parent(), activeLayer());
     } else if (img)
@@ -561,7 +455,7 @@ void KisLayerManager::addShapeLayer()
 
 void KisLayerManager::addShapeLayer(KisNodeSP parent, KisNodeSP above)
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (img) {
         // XXX: Make work with nodes!
         KisLayer * parentLayer = dynamic_cast<KisLayer*>(parent.data());
@@ -590,7 +484,7 @@ void KisLayerManager::addAdjustmentLayer(KisNodeSP parent, KisNodeSP above)
 {
     Q_ASSERT(parent);
 
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisLayerSP l = activeLayer();
@@ -606,7 +500,7 @@ void KisLayerManager::addAdjustmentLayer(KisNodeSP parent, KisNodeSP above)
     KisAdjustmentLayerSP adjl = addAdjustmentLayer(parent, above, QString(), 0, selection);
 
     KisDlgAdjustmentLayer dlg(adjl, adjl.data(), dev, adjl->image(), img->nextLayerName(), i18n("New Filter Layer"), m_view, "dlgadjustmentlayer");
-    dlg.resize( dlg.minimumSizeHint() );
+    dlg.resize(dlg.minimumSizeHint());
 
     if (dlg.exec() != QDialog::Accepted) {
         m_view->image()->removeNode(adjl);
@@ -620,7 +514,7 @@ KisAdjustmentLayerSP KisLayerManager::addAdjustmentLayer(KisNodeSP parent, KisNo
 {
     Q_ASSERT(parent);
 
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return 0;
 
     KisAdjustmentLayerSP l = new KisAdjustmentLayer(img, name, filter, selection);
@@ -638,11 +532,11 @@ void KisLayerManager::addGeneratorLayer(KisNodeSP parent, KisNodeSP above)
 {
     Q_ASSERT(parent);
 
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisDlgGeneratorLayer dlg(img->nextLayerName(), m_view);
-    dlg.resize( dlg.minimumSizeHint() );
+    dlg.resize(dlg.minimumSizeHint());
 
     if (dlg.exec() == QDialog::Accepted) {
         KisSelectionSP selection = m_view->selection();
@@ -658,7 +552,7 @@ void KisLayerManager::addGeneratorLayer(KisNodeSP parent, KisNodeSP above, const
     Q_ASSERT(parent);
     Q_ASSERT(generator);
 
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisGeneratorLayerSP l = new KisGeneratorLayer(img, name, generator, selection);
@@ -673,7 +567,7 @@ void KisLayerManager::addGeneratorLayer(KisNodeSP parent, KisNodeSP above, const
 
 void KisLayerManager::layerRemove()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
 
     if (img) {
         KisLayerSP layer = activeLayer();
@@ -694,7 +588,7 @@ void KisLayerManager::layerRemove()
 
 void KisLayerManager::layerDuplicate()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
 
     if (!img)
         return;
@@ -718,7 +612,7 @@ void KisLayerManager::layerDuplicate()
 
 void KisLayerManager::layerRaise()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     KisLayerSP layer;
 
     if (!img)
@@ -732,7 +626,7 @@ void KisLayerManager::layerRaise()
 
 void KisLayerManager::layerLower()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     KisLayerSP layer;
 
     if (!img)
@@ -740,13 +634,13 @@ void KisLayerManager::layerLower()
 
     layer = activeLayer();
 
-    m_commandsAdapter->lower( layer );
+    m_commandsAdapter->lower(layer);
     layer->parent()->setDirty();
 }
 
 void KisLayerManager::layerFront()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     KisLayerSP layer;
 
     if (!img)
@@ -759,7 +653,7 @@ void KisLayerManager::layerFront()
 
 void KisLayerManager::layerBack()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisLayerSP layer;
@@ -795,14 +689,13 @@ void KisLayerManager::mirrorLayerX()
         Q_CHECK_PTR(t);
     }
 
-    QRect dirty = KisTransformWorker::mirrorX(dev, m_view->selection() );
+    QRect dirty = KisTransformWorker::mirrorX(dev, m_view->selection());
     m_activeLayer->setDirty(dirty);
 
     if (t) m_view->undoAdapter()->addCommand(t);
 
     m_doc->setModified(true);
     layersUpdated();
-
     m_view->canvas()->update();
 }
 
@@ -817,7 +710,7 @@ void KisLayerManager::mirrorLayerY()
         Q_CHECK_PTR(t);
     }
 
-    QRect dirty = KisTransformWorker::mirrorY(dev, m_view->selection() );
+    QRect dirty = KisTransformWorker::mirrorY(dev, m_view->selection());
     m_activeLayer->setDirty(dirty);
 
     if (t) m_view->undoAdapter()->addCommand(t);
@@ -835,20 +728,23 @@ void KisLayerManager::scaleLayer(double sx, double sy, KisFilterStrategy *filter
     if (!layer) return;
 
     KisSelectedTransaction * t = 0;
-    if (m_view->undoAdapter() && m_view->undoAdapter()->undo() && layer->selection()) {
+    if (m_view->undoAdapter() && m_view->undoAdapter()->undo()) {
         t = new KisSelectedTransaction(i18n("Scale Layer"), layer);
         Q_CHECK_PTR(t);
     }
 
-    KoProgressUpdater pu(m_view->statusBar()->progress());
-    KoUpdater u = pu.startSubtask();
-    KisTransformWorker worker(layer->paintDevice(), sx, sy, 0, 0, 0.0, 0, 0, &u, filterStrategy);
+    KoProgressUpdater* updater = m_view->createProgressUpdater();
+    KoUpdaterPtr u = updater->startSubtask();
+
+    KisTransformWorker worker(layer->paintDevice(), sx, sy, 0, 0, 0.0, 0, 0, u, filterStrategy);
     worker.run();
 
     if (t) m_view->undoAdapter()->addCommand(t);
     m_doc->setModified(true);
     layersUpdated();
     m_view->canvas()->update();
+
+    updater->deleteLater();
 }
 
 void KisLayerManager::rotateLayer(double radians)
@@ -897,11 +793,11 @@ void KisLayerManager::shearLayer(double angleX, double angleY)
         undo->beginMacro(i18n("Shear layer"));
     }
 
-    KoProgressUpdater updater(m_view->statusBar()->progress());
-    updater.start( 100, i18n("Shear layer") );
-    KoUpdater up = updater.startSubtask();
+    KoProgressUpdater* updater = m_view->statusBar()->progress()->createUpdater();
+    updater->start(100, i18n("Shear layer"));
+    KoUpdaterPtr up = updater->startSubtask();
 
-    KisShearVisitor v(angleX, angleY, &up);
+    KisShearVisitor v(angleX, angleY, up);
     v.setUndoAdapter(undo);
     layer->accept(v);
 
@@ -910,11 +806,14 @@ void KisLayerManager::shearLayer(double angleX, double angleY)
     m_doc->setModified(true);
     layersUpdated();
     m_view->canvas()->update();
+
+    m_view->statusBar()->progress()->detachUpdater(updater);
+    updater->deleteLater();
 }
 
 void KisLayerManager::flattenImage()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
 
     if (img) {
         bool doIt = true;
@@ -939,7 +838,7 @@ void KisLayerManager::flattenImage()
 
 void KisLayerManager::mergeLayer()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisLayerSP layer = activeLayer();
@@ -949,7 +848,7 @@ void KisLayerManager::mergeLayer()
     if (!strategy) return;
 
     KisLayerSP  newLayer = img->mergeLayer(layer, strategy);
-    if(newLayer) {
+    if (newLayer) {
         newLayer->setDirty();
     }
 
@@ -957,6 +856,21 @@ void KisLayerManager::mergeLayer()
 
 }
 
+void KisLayerManager::flattenLayer()
+{
+    KisImageWSP img = m_view->image();
+    if (!img) return;
+
+    KisLayerSP layer = activeLayer();
+    if (!layer) return;
+
+    KisLayerSP newLayer = img->flattenLayer(layer);
+    if (newLayer) {
+        newLayer->setDirty();
+    }
+    m_view->updateGUI();
+}
+
 void KisLayerManager::layersUpdated()
 {
     KisLayerSP layer = activeLayer();
@@ -981,24 +895,35 @@ void KisLayerManager::saveLayerAsImage()
     KUrl url = fd.selectedUrl();
     QString mimefilter = fd.currentMimeFilter();
 
+    if (mimefilter.isNull()) {
+        KMimeType::Ptr mime = KMimeType::findByUrl(url);
+        mimefilter = mime->name();
+    }
+
     if (url.isEmpty())
         return;
 
-
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisLayerSP l = activeLayer();
     if (!l) return;
 
-    QRect r = l->exactBounds();
+    QRect r = img->bounds();
 
     KisDoc2 d;
     d.prepareForImport();
 
-    KisImageSP dst = KisImageSP(new KisImage(d.undoAdapter(), r.width(), r.height(), img->colorSpace(), l->name()));
+    KisImageWSP dst = new KisImage(d.undoAdapter(), r.width(), r.height(), img->colorSpace(), l->name());
+    dst->setResolution(img->xRes(), img->yRes());
     d.setCurrentImage(dst);
-    dst->addNode(l->clone(), dst->rootLayer(), KisLayerSP(0));
+    KisPaintLayer* paintLayer = new KisPaintLayer(dst, "projection", l->opacity());
+    KisPainter gc(paintLayer->paintDevice());
+    gc.bitBlt(QPoint(0, 0), l->projection(), l->projection()->exactBounds());
+    dst->addNode(paintLayer, dst->rootLayer(), KisLayerSP(0));
+
+    dst->resize(paintLayer->exactBounds());
+    dst->refreshGraph();
 
     d.setOutputMimeType(mimefilter.toLatin1());
     d.exportDocument(url);
