@@ -34,11 +34,12 @@
 #include <kstandardaction.h>
 #include <kactioncollection.h>
 
+#include "KoCanvasController.h"
 #include "KoChannelInfo.h"
 #include "KoIntegerMaths.h"
 #include <KoDocument.h>
 #include <KoMainWindow.h>
-#include <KoQueryTrader.h>
+#include <KoDocumentEntry.h>
 #include <KoViewConverter.h>
 #include <KoSelection.h>
 #include <KoShapeManager.h>
@@ -114,10 +115,10 @@ KisSelectionManager::KisSelectionManager(KisView2 * view, KisDoc2 * doc)
     Q_ASSERT(selection);
     connect(selection, SIGNAL(selectionChanged()), this, SLOT(shapeSelectionChanged()));
 
-    KisSelectionDecoration* decoration = new KisSelectionDecoration( m_view );
+    KisSelectionDecoration* decoration = new KisSelectionDecoration(m_view);
     connect(this, SIGNAL(currentSelectionChanged()), decoration, SLOT(selectionChanged()));
     decoration->setVisible(true);
-    m_view->canvasBase()->addDecoration( decoration );
+    m_view->canvasBase()->addDecoration(decoration);
 }
 
 KisSelectionManager::~KisSelectionManager()
@@ -138,11 +139,14 @@ void KisSelectionManager::setup(KActionCollection * collection)
     collection->addAction("paste_new", m_pasteNew);
     connect(m_pasteNew, SIGNAL(triggered()), this, SLOT(pasteNew()));
 
+    m_pasteAt = new KAction(i18n("Paste at cursor"), this);
+    collection->addAction("paste_at", m_pasteAt);
+    connect(m_pasteAt, SIGNAL(triggered()), this, SLOT(pasteAt()));
+
     m_selectAll = collection->addAction(KStandardAction::SelectAll,  "select_all", this, SLOT(selectAll()));
 
     m_deselect = collection->addAction(KStandardAction::Deselect,  "deselect", this, SLOT(deselect()));
 
-
     m_clear = collection->addAction(KStandardAction::Clear,  "clear", this, SLOT(clear()));
 
     m_delete = new KAction(KIcon("edit-delete"), i18n("D&elete"), this);
@@ -247,7 +251,7 @@ void KisSelectionManager::updateGUI()
         return;
     }
 
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     KisLayerSP l;
     KisPaintDeviceSP dev;
 
@@ -369,7 +373,7 @@ void KisSelectionManager::selectionChanged()
 
 void KisSelectionManager::cut()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisLayerSP layer = m_view->activeLayer();
@@ -383,10 +387,11 @@ void KisSelectionManager::cut()
     Q_CHECK_PTR(t);
 
     layer->paintDevice()->clearSelection(m_view->selection());
-#if 0 // XXX_SELECTION
-    dev->deselect();
-#endif
+    QRect rect = m_view->selection()->selectedRect();
+    deselect();
+
     m_view->document()->addCommand(t);
+    layer->setDirty(rect);
 }
 
 void KisSelectionManager::copy()
@@ -399,7 +404,7 @@ void KisSelectionManager::copy()
         m_view->canvasBase()->toolProxy()->copy();
     } else {
 
-        KisImageSP img = m_view->image();
+        KisImageWSP img = m_view->image();
         if (!img) return;
 
         KisPaintDeviceSP dev = m_view->activeDevice();
@@ -419,7 +424,8 @@ void KisSelectionManager::copy()
         // Copy image data
         KisPainter gc;
         gc.begin(clip);
-        gc.bitBlt(0, 0, COMPOSITE_COPY, dev, r.x(), r.y(), r.width(), r.height());
+        gc.setCompositeOp(COMPOSITE_COPY);
+        gc.bitBlt(0, 0, dev, r.x(), r.y(), r.width(), r.height());
         gc.end();
 
         if (selection) {
@@ -452,7 +458,7 @@ void KisSelectionManager::copy()
 
 void KisSelectionManager::paste()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisPaintDeviceSP clip = m_clipboard->clip();
@@ -464,7 +470,8 @@ void KisSelectionManager::paste()
         QRect r = clip->exactBounds();
         KisPainter gc;
         gc.begin(layer->paintDevice());
-        gc.bitBlt(0, 0, COMPOSITE_COPY, clip, r.x(), r.y(), r.width(), r.height());
+        gc.setCompositeOp(COMPOSITE_COPY);
+        gc.bitBlt(0, 0, clip, r.x(), r.y(), r.width(), r.height());
         gc.end();
 
         //figure out where to position the clip
@@ -477,8 +484,14 @@ void KisSelectionManager::paste()
         if (bottomright.y() > img->height())
             center.setY(img->height() / 2);
         center -= QPoint(r.width() / 2, r.height() / 2);
-        layer->setX(center.x());
-        layer->setY(center.y());
+
+        const KoCanvasBase* canvasBase = m_view->canvasBase();
+        const KoViewConverter* viewConverter = m_view->canvasBase()->viewConverter();
+
+        layer->setX(viewConverter->viewToDocumentX(canvasBase->canvasController()->canvasOffsetX() + center.x()));
+        layer->setY(viewConverter->viewToDocumentY(canvasBase->canvasController()->canvasOffsetY() + center.y()));
+
+        qDebug() << "layer x,y:" << layer->x() << ", " << layer->y();
 
         /*XXX CBR have an idea of asking the user if he is about to paste a clip in another cs than that of
           the image if that is what he want rather than silently converting
@@ -486,17 +499,21 @@ void KisSelectionManager::paste()
           if (dlg->exec() == QDialog::Accepted)
           layer->convertTo(img->colorSpace());
         */
-        if( m_view->activeLayer() )
-        {
+        if (m_view->activeLayer()) {
             m_adapter->addNode(layer , m_view->activeLayer()->parent(), m_view->activeLayer().data());
         } else {
-            m_adapter->addNode(layer , img->rootLayer(), 0 );
+            m_adapter->addNode(layer , img->rootLayer(), 0);
         }
         layer->setDirty();
     } else
         m_view->canvasBase()->toolProxy()->paste();
 }
 
+void KisSelectionManager::pasteAt()
+{
+    //XXX
+}
+
 void KisSelectionManager::pasteNew()
 {
     KisPaintDeviceSP clip = m_clipboard->clip();
@@ -511,18 +528,19 @@ void KisSelectionManager::pasteNew()
     const QByteArray mimetype = KoDocument::readNativeFormatMimeType();
     KoDocumentEntry entry = KoDocumentEntry::queryByMimeType(mimetype);
 
-    KisDoc2 * doc = dynamic_cast<KisDoc2*>(entry.createDoc());
+    KisDoc2* doc = dynamic_cast<KisDoc2*>(entry.createDoc());
     if (!doc) return;
 
     Q_ASSERT(doc->undoAdapter() != 0);
     doc->undoAdapter()->setUndo(false);
 
-    KisImageSP img = new KisImage(doc->undoAdapter(), r.width(), r.height(),
-                                  KoColorSpaceRegistry::instance()->colorSpace( clip->colorSpace()->id(), clip->colorSpace()->profile() ), "Pasted" ); // TODO should be translated ?
+    KisImageWSP img = new KisImage(doc->undoAdapter(), r.width(), r.height(),
+                                   KoColorSpaceRegistry::instance()->colorSpace(clip->colorSpace()->id(), clip->colorSpace()->profile()), "Pasted");    // TODO should be translated ?
     KisPaintLayerSP layer = new KisPaintLayer(img.data(), clip->objectName(), OPACITY_OPAQUE, clip->colorSpace());
 
     KisPainter p(layer->paintDevice());
-    p.bitBlt(0, 0, COMPOSITE_COPY, clip, OPACITY_OPAQUE, r.x(), r.y(), r.width(), r.height());
+    p.setCompositeOp(COMPOSITE_COPY);
+    p.bitBlt(0, 0, clip, r.x(), r.y(), r.width(), r.height());
     p.end();
 
     img->addNode(layer.data(), img->rootLayer());
@@ -537,7 +555,7 @@ void KisSelectionManager::pasteNew()
 
 void KisSelectionManager::selectAll()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     QUndoCommand* selectionCmd = new QUndoCommand(i18n("Select All"));
@@ -556,7 +574,7 @@ void KisSelectionManager::selectAll()
 
 void KisSelectionManager::deselect()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisSelectionSP sel = m_view->selection();
@@ -579,7 +597,7 @@ void KisSelectionManager::deselect()
 
 void KisSelectionManager::clear()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisPaintDeviceSP dev = m_view->activeDevice();
@@ -609,7 +627,7 @@ void KisSelectionManager::deleteSelection()
 
 void KisSelectionManager::fill(const KoColor& color, bool fillWithPattern, const QString& transactionText)
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisPaintDeviceSP dev = m_view->activeDevice();
@@ -633,8 +651,8 @@ void KisSelectionManager::fill(const KoColor& color, bool fillWithPattern, const
     KisPainter painter2(dev, selection);
 
     if (img->undo()) painter2.beginTransaction(transactionText);
-    painter2.bltSelection(0, 0, COMPOSITE_OVER, filled, OPACITY_OPAQUE,
-                          0, 0, img->width(), img->height());
+
+    painter2.bitBlt(0, 0, filled, 0, 0, img->width(), img->height());
 
     dev->setDirty();
 
@@ -660,7 +678,7 @@ void KisSelectionManager::fillPattern()
 
 void KisSelectionManager::reselect()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisLayerSP layer = m_view->activeLayer();
@@ -680,7 +698,7 @@ void KisSelectionManager::reselect()
 
 void KisSelectionManager::invert()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisSelectionSP selection = m_view->selection();
@@ -700,7 +718,7 @@ void KisSelectionManager::invert()
 
 void KisSelectionManager::copySelectionToNewLayer()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisLayerSP layer = m_view->activeLayer();
@@ -712,7 +730,7 @@ void KisSelectionManager::copySelectionToNewLayer()
 
 void KisSelectionManager::cutToNewLayer()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisPaintDeviceSP dev = m_view->activeDevice();
@@ -725,7 +743,7 @@ void KisSelectionManager::cutToNewLayer()
 
 void KisSelectionManager::feather()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
     if (!m_view->selection())
         return;
@@ -769,7 +787,7 @@ void KisSelectionManager::feather()
 void KisSelectionManager::toggleDisplaySelection()
 {
     KisCanvasDecoration* decoration = m_view->canvasBase()->decoration("selection");
-    if(decoration)
+    if (decoration)
         decoration->toggleVisibility();
 }
 
@@ -783,7 +801,7 @@ bool KisSelectionManager::displaySelection()
 
 void KisSelectionManager::grow(qint32 xradius, qint32 yradius)
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     if (!m_view->selection()) return;
@@ -809,7 +827,7 @@ void KisSelectionManager::grow(qint32 xradius, qint32 yradius)
     for (qint32 i = 0; i < yradius + 1; i++) {
         buf[i] = new quint8[layerSize.width()];
     }
-    quint8* buffer = new quint8[(layerSize.width() + 2 * xradius) * (yradius + 1)];
+    quint8* buffer = new quint8[(layerSize.width() + 2 * xradius) *(yradius + 1)];
     for (qint32 i = 0; i < layerSize.width() + 2 * xradius; i++) {
         if (i < xradius)
             max[i] = buffer;
@@ -908,7 +926,7 @@ void KisSelectionManager::grow(qint32 xradius, qint32 yradius)
 void KisSelectionManager::shrink(qint32 xradius, qint32 yradius, bool edge_lock)
 {
 
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     if (!m_view->selection()) return;
@@ -956,11 +974,10 @@ void KisSelectionManager::shrink(qint32 xradius, qint32 yradius, bool edge_lock)
                 max[i] = &buffer[(yradius + 1) * (layerSize.width() + xradius)];
         else if (i < layerSize.width() + xradius)
             max[i] = &buffer[(yradius + 1) * (i - xradius)];
+        else if (edge_lock)
+            max[i] = &buffer[(yradius + 1) * (layerSize.width() + xradius - 1)];
         else
-            if (edge_lock)
-                max[i] = &buffer[(yradius + 1) * (layerSize.width() + xradius - 1)];
-            else
-                max[i] = &buffer[(yradius + 1) * (layerSize.width() + xradius)];
+            max[i] = &buffer[(yradius + 1) * (layerSize.width() + xradius)];
     }
     if (!edge_lock)
         for (qint32 j = 0 ; j < xradius + 1; j++) max[0][j] = 0;
@@ -1060,7 +1077,7 @@ void KisSelectionManager::shrink(qint32 xradius, qint32 yradius, bool edge_lock)
 
 void KisSelectionManager::smooth()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     if (!m_view->selection()) return;
@@ -1122,7 +1139,7 @@ void KisSelectionManager::smooth()
 
 void KisSelectionManager::erode()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisSelectionSP selection = m_view->selection();
@@ -1190,7 +1207,7 @@ void KisSelectionManager::erode()
 
 void KisSelectionManager::dilate()
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     KisSelectionSP selection = m_view->selection();
@@ -1255,7 +1272,7 @@ void KisSelectionManager::dilate()
 
 void KisSelectionManager::border(qint32 xradius, qint32 yradius)
 {
-    KisImageSP img = m_view->image();
+    KisImageWSP img = m_view->image();
     if (!img) return;
 
     if (!m_view->selection()) return;
@@ -1399,13 +1416,12 @@ void KisSelectionManager::border(qint32 xradius, qint32 yradius)
                         max[x] = yradius;
                     else
                         max[x]--;
-                } else
-                    if (transition[-max[x]][x])
-                        max[x] = -max[x];
-                    else if (transition[-max[x] + 1][x])
-                        max[x] = -max[x] + 1;
-                    else
-                        max[x]--;
+                } else if (transition[-max[x]][x])
+                    max[x] = -max[x];
+                else if (transition[-max[x] + 1][x])
+                    max[x] = -max[x] + 1;
+                else
+                    max[x]--;
             } else
                 max[x]--;
             if (max[x] < -yradius - 1)
