@@ -19,7 +19,7 @@
 #define KIS_PATTERN_CHOOSER_H_
 
 #include <QFrame>
-#include "kis_item_chooser.h"
+#include <KoResourceItemChooser.h>
 
 class QLabel;
 
@@ -32,18 +32,22 @@ public:
     KisPatternChooser(QWidget *parent = 0, const char *name = 0);
     virtual ~KisPatternChooser();
 
-    KisItemChooser* itemChooser() {
-        return m_itemChooser;
-    }
+    /// Gets the currently selected resource
+    /// @returns the selected resource, 0 is no resource is selected
+    KoResource * currentResource();
 
+    void setCurrentItem(int row, int column);
+
+signals:
+    /// Emitted when a resource was selected
+    void resourceSelected(KoResource * resource);
 
 private slots:
-    void slotImportPattern();
-    void update(QTableWidgetItem *item);
+    void update(KoResource * resource);
 
 private:
     QLabel *m_lbName;
-    KisItemChooser * m_itemChooser;
+    KoResourceItemChooser * m_itemChooser;
 };
 
 #endif // KIS_PATTERN_CHOOSER_H_
