@@ -1068,7 +1068,7 @@ void Cell::saveOdfAnnotation( KoXmlWriter &xmlwriter )
     {
         //<office:annotation draw:style-name="gr1" draw:text-style-name="P1" svg:width="2.899cm" svg:height="2.691cm" svg:x="2.858cm" svg:y="0.001cm" draw:caption-point-x="-2.858cm" draw:caption-point-y="-0.001cm">
         xmlwriter.startElement( "office:annotation" );
-        const QStringList text = comment.split( "\n", QString::SkipEmptyParts );
+        const QStringList text = comment.split( '\n', QString::SkipEmptyParts );
         for ( QStringList::ConstIterator it = text.begin(); it != text.end(); ++it ) {
             xmlwriter.startElement( "text:p" );
             xmlwriter.addTextNode( *it );
@@ -1129,7 +1129,8 @@ bool Cell::saveOdf( KoXmlWriter& xmlwriter, KoGenStyles &mainStyles,
         saveOdfCellStyle( currentCellStyle, mainStyles );
         // skip 'table:style-name' attribute for the default style
         if ( !currentCellStyle.isDefaultStyle() )
-            xmlwriter.addAttribute( "table:style-name", mainStyles.styles().find(currentCellStyle).value() );
+            if ( mainStyles.styles().contains(currentCellStyle) )
+                xmlwriter.addAttribute( "table:style-name", mainStyles.styles().find(currentCellStyle).value() );
     }
 
     // group empty cells with the same style
@@ -1352,7 +1353,7 @@ bool Cell::loadOdf(const KoXmlElement& element, OdfLoadingContext& tableContext)
         // each spreadsheet application likes to safe formulas with a different namespace
         // prefix, so remove all of them
         QStringList prefixes = QStringList() << "oooc:" << "kspr:" << "of:" << "msoxl:";
-        foreach (QString prefix, prefixes) {
+        foreach (const QString &prefix, prefixes) {
             if (oasisFormula.startsWith( prefix )) {
                 oasisFormula = oasisFormula.mid( prefix.length() );
                 break;
