@@ -42,7 +42,7 @@
 #include <QFont>
 #include <QTimer>
 //Added by qt3to4:
-#include <Q3ValueList>
+#include <QList>
 #include <QPainter>
 
 #include <kcompletion.h>
@@ -397,7 +397,7 @@ bool Doc::saveOdfHelper( SavingContext & documentContext, SaveFlag saveFlag,
     settingsWriter->startElement("config:config-item-set");
     settingsWriter->addAttribute("config:name", "view-settings");
 
-    KoUnit::saveOdf(settingsWriter, unit());
+    saveUnitOdf(settingsWriter);
 
     saveOdfSettings( *settingsWriter );
 
@@ -888,14 +888,6 @@ void Doc::paintCellRegions( QPainter& painter, const QRect &viewRect,
 //   }
 //   else {
 //     matrix = painter.matrix();
-//   }
-//
-//   QPtrListIterator<KoDocumentChild> it( children() );
-//   for( ; it.current(); ++it ) {
-//     // if ( ((Child*)it.current())->sheet() == sheet &&
-//     //    !m_pView->hasDocumentInWindow( it.current()->document() ) )
-//     if ( ((Child*)it.current())->sheet() == sheet)
-//       rgn -= it.current()->region( matrix );
 //   }
     painter.setClipRegion( rgn );
 
