@@ -28,6 +28,7 @@
 
 // Local
 #include "View.h"
+#include "TabBar.h"
 
 // standard C/C++ includes
 #include <assert.h>
@@ -53,7 +54,7 @@
 #include <QSqlDatabase>
 #include <QSizePolicy>
 //Added by qt3to4:
-#include <Q3ValueList>
+#include <QList>
 
 // KDE includes
 #include <kactioncollection.h>
@@ -61,7 +62,7 @@
 #include <kcomponentdata.h>
 #include <kdatatool.h>
 #include <kdebug.h>
-#include <kdeversion.h>
+
 #include <KFontChooser>
 #include <kinputdialog.h>
 #include <kmessagebox.h>
@@ -85,8 +86,8 @@
 
 // KOffice includes
 #include <KoGlobal.h>
+#include <KoDpi.h>
 #include <KoCanvasController.h>
-#include <KoColor.h>
 #include <KoMainWindow.h>
 #include <KoOdfLoadingContext.h>
 #include <KoOdfReadStore.h>
@@ -95,9 +96,7 @@
 #include <KoShapeManager.h>
 #include <KoSelection.h>
 #include <KoStore.h>
-#include <KoTabBar.h>
 #include <KoToolBoxFactory.h>
-#include <KoToolDocker.h>
 #include <KoDockerManager.h>
 #include <KoToolManager.h>
 #include <KoToolRegistry.h>
@@ -106,6 +105,7 @@
 #include <KoZoomAction.h>
 #include <KoZoomController.h>
 #include <KoZoomHandler.h>
+#include <KoToolProxy.h>
 
 // KSpread includes
 #include "ApplicationSettings.h"
@@ -189,7 +189,7 @@ public:
     SelectAllButton* selectAllButton;
     QScrollBar *horzScrollBar;
     QScrollBar *vertScrollBar;
-    KoTabBar *tabBar;
+    TabBar *tabBar;
     QLabel* calcLabel;
     QGridLayout* viewLayout;
     QGridLayout* tabScrollBarLayout;
@@ -237,6 +237,9 @@ public:
     QAction * hideSheet;
     QAction * showSheet;
 
+    //Shape manipulation
+    KAction * deleteShape;
+
     // page layout
     QAction * paperLayout;
     QAction * resetPrintRange;
@@ -508,6 +511,13 @@ void View::Private::initActions()
     actions->calcCountA->setToolTip(i18n("Calculate using the countA"));
     actions->calcCountA->setActionGroup( groupCalc );
 
+    //Shape actions
+    actions->deleteShape = new KAction( KIcon("edit-delete"), i18n("Delete"), view );
+    actions->deleteShape->setShortcut(QKeySequence("Del"));
+    connect(actions->deleteShape, SIGNAL(triggered()), view, SLOT(editDeleteSelection()));
+    connect(canvas->toolProxy(), SIGNAL(selectionChanged(bool)), actions->deleteShape, SLOT(setEnabled(bool)));
+    ac->addAction("edit_delete", actions->deleteShape );
+
     // -- special action, only for developers --
     //
 
@@ -634,15 +644,16 @@ View::View( QWidget *_parent, Doc *_doc )
         setZoom( 100, true );
     }
 
+    connect (&d->statusBarOpTimer, SIGNAL(timeout()), this, SLOT(calcStatusBarOp()));
+
     // Delay the setting of the initial position, because
     // we have to wait for the widget to be shown. Otherwise,
     // we get a wrong widget size.
     // This is the last operation for the "View loading" process.
     // The loading flag will be unset at its end.
-    if ( !doc()->map()->sheetList().isEmpty() )
-      QTimer::singleShot(50, this, SLOT(initialPosition()));
-
-    connect (&d->statusBarOpTimer, SIGNAL(timeout()), this, SLOT(calcStatusBarOp()));
+    // Don't try to delay this init cause following operations will assume
+    // the sheets are already fully setup.
+    initialPosition();
 
     new ViewAdaptor(this);
     d->canvas->setFocus();
@@ -664,7 +675,7 @@ View::~View()
     qDeleteAll( d->sheetViews );
 
     d->selection->emitCloseEditor(false);
-    d->selection->endReferenceSelection();
+    d->selection->endReferenceSelection(false);
     d->activeSheet = 0; // set the active sheet to 0 so that when during destruction
     // of embedded child documents possible repaints in Sheet are not
     // performed. The repains can happen if you delete an embedded document,
@@ -676,7 +687,6 @@ View::~View()
     delete d->calcLabel;
 
     delete d->actions;
-    delete d->zoomController;
     delete d->zoomHandler;
     // NOTE Stefan: Delete the Canvas explicitly, even if it has this view as
     //              parent. Otherwise, it leads to crashes, because it tries to
@@ -750,12 +760,12 @@ void View::initView()
         dockerMng = new KoDockerManager(this);
         setDockerManager(dockerMng);
     }
-    connect( d->canvasController, SIGNAL( toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, KoView *) ),
-             dockerMng, SLOT( newOptionWidgets(const  QMap<QString, QWidget *> &, KoView *) ) );
+    connect( d->canvasController, SIGNAL( toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*) ),
+             dockerMng, SLOT( newOptionWidgets(const  QMap<QString, QWidget *> &, QWidget*) ) );
 
     // Setup the zoom controller.
     d->zoomHandler = new KoZoomHandler();
-    d->zoomController = new KoZoomController( d->canvasController, d->zoomHandler, actionCollection() );
+    d->zoomController = new KoZoomController(d->canvasController, d->zoomHandler, actionCollection(), 0, this);
     d->zoomController->zoomAction()->setZoomModes( KoZoomMode::ZOOM_CONSTANT );
     addStatusBarItem( d->zoomController->zoomAction()->createWidget( statusBar() ), 0, true );
     connect( d->zoomController, SIGNAL(zoomChanged(KoZoomMode::Mode, qreal)),
@@ -787,7 +797,7 @@ void View::initView()
     d->tabScrollBarLayout = new QGridLayout( bottomPart );
     d->tabScrollBarLayout->setMargin(0);
     d->tabScrollBarLayout->setSpacing(0);
-    d->tabBar = new KoTabBar( 0 );
+    d->tabBar = new TabBar( 0 );
     d->tabScrollBarLayout->addWidget( d->tabBar, 0, 0 );
     d->horzScrollBar = new QScrollBar( 0 );
     d->canvasController->setHorizontalScrollBar( d->horzScrollBar );
@@ -846,6 +856,11 @@ Canvas* View::canvasWidget() const
     return d->canvas;
 }
 
+KoZoomController *View::zoomController() const
+{
+    return d->zoomController;
+}
+
 KoCanvasController* View::canvasController() const
 {
     return d->canvasController;
@@ -871,7 +886,7 @@ QScrollBar* View::vertScrollBar()const
     return d->vertScrollBar;
 }
 
-KoTabBar* View::tabBar() const
+TabBar* View::tabBar() const
 {
     return d->tabBar;
 }
@@ -1082,11 +1097,17 @@ void View::shapeSelectionChanged()
         {
             // If the anchoring differs between shapes, deselect the anchoring action and stop here.
             d->actions->shapeAnchor->setCurrentAction(0);
-            break; 
+            break;
         }
     }
 }
 
+
+void View::editDeleteSelection()
+{
+    d->canvas->toolProxy()->deleteSelection();
+}
+
 void View::initialPosition()
 {
     // Loading completed, pick initial worksheet
@@ -1149,7 +1170,7 @@ void View::initialPosition()
     refreshView();
 
     // Activate the cell tool.
-    if (shell()) 
+    if (shell())
         KoToolManager::instance()->switchToolRequested("KSpreadCellToolId");
 }
 
@@ -1780,7 +1801,7 @@ void View::setZoom( int zoom, bool /*updateViews*/ )
   kDebug(36005) <<"---------SetZoom:" << zoom;
 
   // Set the zoom in KoView (for embedded views)
-  zoomHandler()->setZoomAndResolution( zoom, KoGlobal::dpiX(), KoGlobal::dpiY());
+  zoomHandler()->setZoomAndResolution( zoom, KoDpi::dpiX(), KoDpi::dpiY());
   //KoView::setZoom( zoomHandler()->zoomedResolutionY() /* KoView only supports one zoom */ );
 
   doc()->refreshInterface();
