@@ -40,7 +40,7 @@
 #include <kstatusbar.h>
 #include <knuminput.h>
 #include <kmessagebox.h>
-#include <kdeversion.h>
+
 #include <kcolorbutton.h>
 #include <KPluginInfo>
 #include <KPluginSelector>
@@ -49,8 +49,6 @@
 #include <KStandardDirs>
 #include <sonnet/configwidget.h>
 
-#include <KoTabBar.h>
-
 #include "ApplicationSettings.h"
 #include "CalculationSettings.h"
 #include "part/Doc.h"
@@ -328,6 +326,7 @@ PreferenceDialog::PreferenceDialog(View* view)
     d->interfaceOptions.m_cursorMovement->addItem(i18n("Right"), Right);
     d->interfaceOptions.m_cursorMovement->addItem(i18n("Left"), Left);
     d->interfaceOptions.m_cursorMovement->addItem(i18n("Down, First Column"), BottomFirst);
+    d->interfaceOptions.m_cursorMovement->addItem(i18n("None"), NoMovement);
 
     d->interfaceOptions.m_statusBarFunction->addItem(i18n("Sum"), SumOfNumber);
     d->interfaceOptions.m_statusBarFunction->addItem(i18n("Min"), Min);
