@@ -32,7 +32,7 @@
 #include "Style.h"
 
 // KOffice
-#include <KoGlobal.h>
+#include <KoDpi.h>
 #include <KoViewConverter.h>
 
 // KDE
@@ -845,7 +845,7 @@ void CellEditor::copy()
 
 void CellEditor::setEditorFont(QFont const & font, bool updateSize, const KoViewConverter *viewConverter)
 {
-    const double scaleY = POINT_TO_INCH(double(KoGlobal::dpiY()));
+    const qreal scaleY = POINT_TO_INCH(static_cast<qreal>((KoDpi::dpiY())));
     d->textEdit->setFont(QFont(font.family(), viewConverter->documentToViewY(font.pointSizeF()) / scaleY));
 
     if (updateSize) {
@@ -1145,7 +1145,7 @@ void CellEditor::handleKeyPressEvent( QKeyEvent * _ev )
   QApplication::sendEvent( d->textEdit, _ev );
 }
 
-void CellEditor::handleIMEvent( QInputMethodEvent  * _ev )
+void CellEditor::handleInputMethodEvent( QInputMethodEvent  * _ev )
 {
     // send the IM event to the KLineEdit
     QApplication::sendEvent( d->textEdit, _ev );
