@@ -41,6 +41,7 @@
 #include <KoColorBackground.h>
 #include <KoImageCollection.h>
 #include <KoImageData.h>
+#include <KoBorder.h>
 
 // KDE + Qt includes
 #include <QTextBlock>
@@ -197,9 +198,9 @@ bool KWDLoader::load(KoXmlElement &root)
             if (pgLayout.bottom == 0.0)
                 pgLayout.bottom = paperborders.attribute("bottom").toDouble();
         } else
-            kWarning() << "No <PAPERBORDERS> tag!";
+            kWarning(32001) << "No <PAPERBORDERS> tag!";
     } else
-        kWarning() << "No <PAPER> tag! This is a mandatory tag! Expect weird page sizes...";
+        kWarning(32001) << "No <PAPER> tag! This is a mandatory tag! Expect weird page sizes...";
 
     m_pageManager->pageStyle("Standard").setPageLayout(pgLayout);
 
@@ -478,7 +479,7 @@ void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem, bool loadFrames,
         ImageKey imageKey;
         fill(&imageKey, key);
         if (imageKey.filename.isEmpty()) {
-            kWarning() << "could not find image in the store\n";
+            kWarning(32001) << "could not find image in the store";
             return;
         }
 
@@ -491,13 +492,13 @@ void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem, bool loadFrames,
         KoShape *shape = factory->createDefaultShapeAndInit(m_document->dataCenterMap());
         shape->setKeepAspectRatio(image.attribute("keepAspectRatio", "true") == "true");
 
-        KoImageCollection * collection = dynamic_cast< KoImageCollection *>( m_document->dataCenterMap()["ImageCollection"] );
-        Q_ASSERT( collection );
-        KoImageData * data = collection->getImage( imageKey.filename, m_store );
-        shape->setUserData( data );
+        KoImageCollection *collection = dynamic_cast<KoImageCollection*>(m_document->dataCenterMap()["ImageCollection"]);
+        Q_ASSERT(collection);
+        KoImageData *data = collection->createImageData(imageKey.filename, m_store);
+        shape->setUserData(data);
 
-        KWFrame * f = new KWFrame( shape, fs );
-        fill( f, frame );
+        KWFrame *f = new KWFrame(shape, fs);
+        fill(f, frame);
         m_document->addFrameSet(fs);
         return;
     }
@@ -649,11 +650,11 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
                             formatCursor.setPosition(block.position() + pos);
                             formatCursor.setPosition(block.position() + pos + length, QTextCursor::KeepAnchor);
                         } else {
-                            kWarning("Format has missing or invalid 'len' value, ignoring\n");
+                            kWarning(32001) << "Format has missing or invalid 'len' value, ignoring";
                             continue;
                         }
                     } else {
-                        kWarning("Format has missing or invalid 'pos' value, ignoring\n");
+                        kWarning(32001) << "Format has missing or invalid 'pos' value, ignoring";
                         continue;
                     }
                     if (id == "1") {
@@ -662,23 +663,23 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
                         fill(&s2, format);
                         s2.applyStyle(&formatCursor);
                     } else if (id == "2") {
-                        kWarning("File to old, image can not be recovered\n");
+                        kWarning(32001) << "File to old, image can not be recovered";
                     } else if (id == "4") {
                         KoXmlElement variable = format.namedItem("VARIABLE").toElement();
                         if (variable.isNull()) {
-                            kWarning() << "Missing VARIABLE tag\n";
+                            kWarning(32001) << "Missing VARIABLE tag";
                             continue;
                         }
                         KoXmlElement type = variable.namedItem("TYPE").toElement();
                         int typeId = type.attribute("type", "1").toInt();
-                        switch(typeId) {
+                        switch (typeId) {
                         case 11: { // footnote
                             KoXmlElement footnote = variable.namedItem("FOOTNOTE").toElement();
                             KoInlineNote *note = new KoInlineNote(KoInlineNote::Footnote);
                             note->setLabel(footnote.attribute("value"));
-                            note->setAutoNumbering( footnote.attribute("numberingtype", "auto") == "auto" );
+                            note->setAutoNumbering(footnote.attribute("numberingtype", "auto") == "auto");
                             note->setText(i18n("Unable to locate footnote text"));
-                            KoTextDocumentLayout *layout = dynamic_cast<KoTextDocumentLayout*>(
+                            KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(
                                     fs->document()->documentLayout());
                             Q_ASSERT(layout);
                             Q_ASSERT(layout->inlineTextObjectManager());
@@ -696,9 +697,9 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
                                         ? KoInlineNote::Footnote : KoInlineNote::Endnote;
                             KoInlineNote *note = new KoInlineNote(type);
                             note->setLabel(footEndNote.attribute("value"));
-                            note->setAutoNumbering( footEndNote.attribute("numberingtype", "auto") == "auto" );
+                            note->setAutoNumbering(footEndNote.attribute("numberingtype", "auto") == "auto");
                             note->setText(i18n("Unable to locate note-text"));
-                            KoTextDocumentLayout *layout = dynamic_cast<KoTextDocumentLayout*>(
+                            KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(
                                     fs->document()->documentLayout());
                             Q_ASSERT(layout);
                             Q_ASSERT(layout->inlineTextObjectManager());
@@ -721,17 +722,17 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
                         }
                         }
                     } else if (id == "5") {
-                        kWarning("File to old, footnote can not be recovered\n");
+                        kWarning(32001) << "File to old, footnote can not be recovered";
                     } else if (id == "6") { // anchor for floating frame.
                         KoXmlElement anchor = format.namedItem("ANCHOR").toElement();
                         if (anchor.isNull()) {
-                            kWarning() << "Missing ANCHOR tag\n";
+                            kWarning(32001) << "Missing ANCHOR tag";
                             continue;
                         }
                         QString type = anchor.attribute("type", "frameset");
                         if (type == "frameset") {
                             if (! anchor.hasAttribute("instance")) {
-                                kWarning() << "ANCHOR is missing its instance attribute\n";
+                                kWarning(32001) << "ANCHOR is missing its instance attribute";
                                 continue;
                             }
                             AnchorData data;
@@ -839,7 +840,7 @@ void KWDLoader::fill(KoParagraphStyle *style, const KoXmlElement &layout)
         case 10: llp.setStyle(KoListStyle::DiscItem); break;
         case 11: llp.setStyle(KoListStyle::BoxItem); break;
         case 7: llp.setStyle(KoListStyle::CustomCharItem);
-            kWarning() << "According to spec COUNTER with type 7 is not supported, ignoring\n";
+            kWarning(32001) << "According to spec COUNTER with type 7 is not supported, ignoring";
             // fall through
         default: {
             lstyle = 0;
@@ -871,20 +872,20 @@ void KWDLoader::fill(KoParagraphStyle *style, const KoXmlElement &layout)
                     innerWidth = 0.0;
             spacing = 0.0;
             switch (element.attribute("style").toInt()) {
-            case 0: borderStyle = KoParagraphStyle::BorderSolid; break;
-            case 1: borderStyle = KoParagraphStyle::BorderDashed; break;
-            case 2: borderStyle = KoParagraphStyle::BorderDotted; break;
-            case 3: borderStyle = KoParagraphStyle::BorderDashDotPattern; break;
-            case 4: borderStyle = KoParagraphStyle::BorderDashDotDotPattern; break;
+            case 0: borderStyle = KoBorder::BorderSolid; break;
+            case 1: borderStyle = KoBorder::BorderDashed; break;
+            case 2: borderStyle = KoBorder::BorderDotted; break;
+            case 3: borderStyle = KoBorder::BorderDashDotPattern; break;
+            case 4: borderStyle = KoBorder::BorderDashDotDotPattern; break;
             case 5:
-                borderStyle = KoParagraphStyle::BorderDouble;
+                borderStyle = KoBorder::BorderDouble;
                 spacing = width;
                 innerWidth = width;
                 break;
             }
         }
         qreal width, innerWidth, spacing;
-        KoParagraphStyle::BorderStyle borderStyle;
+        KoBorder::BorderStyle borderStyle;
     };
     element = layout.namedItem("LEFTBORDER").toElement();
     if (!element.isNull()) {
@@ -1070,7 +1071,7 @@ void KWDLoader::fill(KWFrame *frame, const KoXmlElement &frameElem)
     if (tf) {
         if (zIndex <= 0 && static_cast<KWTextFrameSet*>(tf->frameSet())->textFrameSetType() == KWord::OtherTextFrameSet)
             zIndex = 1; // OtherTextFrameSet types always live on top of the main frames.
-        KoTextShapeData *textShapeData = dynamic_cast<KoTextShapeData*>(frame->shape()->userData());
+        KoTextShapeData *textShapeData = qobject_cast<KoTextShapeData*>(frame->shape()->userData());
         Q_ASSERT(textShapeData);
         KoInsets margins;
         margins.left = frameElem.attribute("bleftpt", "0.0").toDouble();
@@ -1160,7 +1161,7 @@ void KWDLoader::insertAnchors()
     foreach (const AnchorData &anchor, m_anchors) {
         KWFrameSet *fs = m_document->frameSetByName(anchor.frameSetName);
         if (fs == 0) {
-            kWarning() << "Anchored frameset not found: '" << anchor.frameSetName;
+            kWarning(32001) << "Anchored frameset not found: '" << anchor.frameSetName;
             continue;
         }
         if (fs->frames().count() == 0)  continue;
@@ -1174,7 +1175,7 @@ void KWDLoader::insertAnchors()
         QTextCursor cursor(anchor.document);
         cursor.setPosition(anchor.cursorPosition);
         cursor.setPosition(anchor.cursorPosition + 1, QTextCursor::KeepAnchor);
-        KoTextDocumentLayout *layout = dynamic_cast<KoTextDocumentLayout*>(cursor.block().document()->documentLayout());
+        KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(cursor.block().document()->documentLayout());
         Q_ASSERT(layout);
         Q_ASSERT(layout->inlineTextObjectManager());
         layout->inlineTextObjectManager()->insertInlineObject(cursor, textAnchor);
@@ -1187,10 +1188,10 @@ void KWDLoader::insertNotes()
     foreach (const NotesData &note, m_notes) {
         KWFrameSet *fs = m_document->frameSetByName(note.frameSetName);
         if (fs == 0) {
-            kWarning() << "Frameset data for note not found: '" << note.frameSetName;
+            kWarning(32001) << "Frameset data for note not found: '" << note.frameSetName;
             continue;
         }
-        KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*> (fs);
+        KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(fs);
         if (tfs && tfs->document()) {
             note.note->setText(tfs->document()->toPlainText());
 //kDebug(32001) << "setting the text to" << note.note->text();
