@@ -23,6 +23,7 @@
 
 // kword includes
 #include "KWDocument.h"
+#include "KWDocument_p.h"
 #include "KWFactory.h"
 #include "KWView.h"
 #include "KWCanvas.h"
@@ -61,6 +62,7 @@
 #include <KoListLevelProperties.h>
 #include <KoDataCenter.h>
 #include <KoTextShapeData.h>
+#include <KoUndoStack.h>
 
 // KDE + Qt includes
 #include <klocale.h>
@@ -132,8 +134,6 @@ KWDocument::KWDocument(QWidget *parentWidget, QObject* parent, bool singleViewMo
 {
     m_frameLayout.setDocument(this);
 
-    connect(documentInfo(), SIGNAL(infoUpdated(const QString &, const QString &)),
-            inlineTextObjectManager(), SLOT(documentInformationUpdated(const QString &, const QString &)));
     setComponentData(KWFactory::componentData(), false);
     setTemplateType("kword_template");
 
@@ -149,6 +149,12 @@ KWDocument::KWDocument(QWidget *parentWidget, QObject* parent, bool singleViewMo
         shapeFactory->populateDataCenterMap(m_dataCenterMap);
     }
 
+    //Populate the document undoStack in the dataCenterMap. This can be used later by shapes for their undo/redo mechanism.
+    m_dataCenterMap["UndoStack"] = undoStack();
+
+    connect(documentInfo(), SIGNAL(infoUpdated(const QString &, const QString &)),
+            inlineTextObjectManager(), SLOT(documentInformationUpdated(const QString &, const QString &)));
+
     m_config.load(this);
     clear();
 }
@@ -208,14 +214,14 @@ KoView* KWDocument::createViewInstance(QWidget* parent)
 {
     KWView *view = new KWView(m_viewMode, this, parent);
     if (m_magicCurtain)
-        view->kwcanvas()->shapeManager()->add(m_magicCurtain, false);
+        view->kwcanvas()->shapeManager()->add(m_magicCurtain, KoShapeManager::AddWithoutRepaint);
 
     bool switchToolCalled = false;
     foreach (KWFrameSet *fs, m_frameSets) {
         if (fs->frameCount() == 0)
             continue;
         foreach (KWFrame *frame, fs->frames())
-            view->kwcanvas()->shapeManager()->add(frame->shape(), false);
+            view->kwcanvas()->shapeManager()->add(frame->shape(), KoShapeManager::AddWithoutRepaint);
         if (switchToolCalled)
             continue;
         KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(fs);
@@ -258,7 +264,7 @@ void KWDocument::removePage(int pageNumber)
 
     KWPage page = m_pageManager.page(pageNumber);
     if (! page.isValid()) {
-        kWarning() << "remove page requested for a non exiting page!\n";
+        kWarning(32001) << "remove page requested for a non exiting page!";
         return;
     }
     addCommand(new KWPageRemoveCommand(this, page));
@@ -369,7 +375,7 @@ void KWDocument::removeFrame(KWFrame *frame)
     removeFrameFromViews(frame);
     KWPage page = pageManager()->page(frame->shape());
     if (!page.isValid()) return;
-    if (page != pageManager()->last())
+    if (page != pageManager()->last() || page == pageManager()->begin())
         return; // can only delete last page.
     foreach (KWFrameSet *fs, m_frameSets) {
         foreach (KWFrame *f, fs->frames()) {
@@ -602,7 +608,7 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
     for (int i = frameSetCount() - 1; i > -1; i--) {
         KWFrameSet *fs = frameSet(i);
         if (!fs) {
-            kWarning() << "frameset " << i << " is NULL!!" << endl;
+            kWarning() << "frameset " << i << " is NULL!!";
             m_lstFrameSet.remove(i);
             continue;
         }
@@ -612,24 +618,24 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
             for (int f = fs->frameCount() - 1; f >= 0; f--) {
                 KWFrame *frame = fs->frame(f);
                 if (frame->left() < 0) {
-                    kWarning() << fs->name() << " frame " << f << " pos.x is < 0, moving frame" << endl;
+                    kWarning() << fs->name() << " frame " << f << " pos.x is < 0, moving frame";
                     frame->moveBy(0 - frame->left(), 0);
                 }
                 if (frame->right() > m_pageLayout.ptWidth) {
                     kWarning() << fs->name() << " frame " << f << " rightborder outside page ("
-                    << frame->right() << ">" << m_pageLayout.ptWidth << "), shrinking" << endl;
+                    << frame->right() << ">" << m_pageLayout.ptWidth << "), shrinking";
                     frame->setRight(m_pageLayout.ptWidth);
                 }
                 if (fs->isProtectSize())
                     continue; // don't make frames bigger of a protected frameset.
                 if (frame->height() < s_minFrameHeight) {
                     kWarning() << fs->name() << " frame " << f << " height is so small no text will fit, adjusting (was: "
-                    << frame->height() << " is: " << s_minFrameHeight << ")" << endl;
+                    << frame->height() << " is: " << s_minFrameHeight << ")";
                     frame->setHeight(s_minFrameHeight);
                 }
                 if (frame->width() < s_minFrameWidth) {
                     kWarning() << fs->name() << " frame " << f << " width is so small no text will fit, adjusting (was: "
-                    << frame->width() << " is: " << s_minFrameWidth  << ")" << endl;
+                    << frame->width() << " is: " << s_minFrameWidth  << ")";
                     frame->setWidth(s_minFrameWidth);
                 }
             }
@@ -642,7 +648,7 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
                 fs->addFrame(frame);
             }
         } else if (fs->frameCount() == 0) {
-            kWarning() << "frameset " << i << " " << fs->name() << " has no frames" << endl;
+            kWarning() << "frameset " << i << " " << fs->name() << " has no frames";
             removeFrameSet(fs);
             if (fs->type() == FT_PART)
                 delete static_cast<KWPartFrameSet *>(fs)->getChild();
@@ -652,7 +658,7 @@ void KWDocument::endOfLoading() // called by both oasis and oldxml
         if (fs->frameCount() > 0) {
             KWFrame *frame = fs->frame(0);
             if (frame->isCopy()) {
-                kWarning() << "First frame in a frameset[" << fs->name() << "] was set to be a copy; resetting\n";
+                kWarning() << "First frame in a frameset[" << fs->name() << "] was set to be a copy; resetting";
                 frame->setCopy(false);
             }
         }
@@ -739,7 +745,7 @@ void KWDocument::requestMoreSpace(KWTextFrameSet *fs)
     if (fs == mainFrameSet()) {
         KoShape *shape = lastFrame->shape();
         if (shape) {
-            KoTextShapeData *data = dynamic_cast<KoTextShapeData*>(shape->userData());
+            KoTextShapeData *data = qobject_cast<KoTextShapeData*>(shape->userData());
             if (data) {
                 QTextBlock block = fs->document()->findBlock(data->endPosition() + 1);
                 if (block.isValid()) {
@@ -815,18 +821,18 @@ void KWDocument::printDebug()
     kDebug(32001) << "                 Debug info";
     kDebug(32001) << "Document:" << this;
     /*kDebug(32001) <<"Type of document:" << (m_pageStyle.hasMainTextFrame()?"WP":"DTP"); */
-    kDebug(32001) <<"Units:" << KoUnit::unitName( unit() );
+    kDebug(32001) << "Units:" << KoUnit::unitName(unit());
     kDebug(32001) << "# Framesets:" << frameSetCount();
     int i = 0;
     foreach (KWFrameSet *fs, m_frameSets) {
-        kDebug(32001) << "Frameset" << i++ << ": '" <<
-        fs->name() << "' (" << fs << ")" << endl;
+        kDebug(32001) << "Frameset" << i++ << ":" <<
+        fs->name() << '(' << fs << ')';
         fs->printDebug();
     }
 
     kDebug(32001) << "PageManager holds" << pageCount() << " pages";
     KWPage page = m_pageManager.begin();
-    while(page.isValid()) {
+    while (page.isValid()) {
         int pgnum = page.pageNumber();
         QString side = "[Left] ";
         QString num = QString::number(pgnum);
@@ -854,7 +860,7 @@ void KWDocument::printDebug()
                 << layout.bottom << layout.right << layout.pageEdge << layout.bindingSide;
     }
 
-    kDebug(32001) << "  The height of the doc (in pt) is:" << pageManager()->bottomOfPage(pageManager()->pageCount() - 1) << endl;
+    kDebug(32001) << "  The height of the doc (in pt) is:" << pageManager()->bottomOfPage(pageManager()->pageCount() - 1);
 }
 #endif
 
@@ -890,6 +896,7 @@ void KWDocument::saveConfig()
 
 // ************* PageProcessingQueue ************
 PageProcessingQueue::PageProcessingQueue(KWDocument *parent)
+    : QObject(parent)
 {
     m_document = parent;
     m_triggered = false;
@@ -905,12 +912,12 @@ void PageProcessingQueue::addPage(KWPage page)
 
 void PageProcessingQueue::process()
 {
+    m_triggered = false;
     const bool docIsEmpty = m_document->isEmpty();
     const bool docIsModified = m_document->isModified();
     foreach (const KWPage &page, m_pages) {
         if (! page.isValid())
             continue;
-        emit m_document->pageSetupChanged();
         m_document->m_frameLayout.createNewFramesForPage(page.pageNumber());
     }
     if (docIsEmpty)
@@ -919,4 +926,5 @@ void PageProcessingQueue::process()
         m_document->setModified(false);
     m_pages.clear();
     deleteLater();
+    emit m_document->pageSetupChanged();
 }
