@@ -25,24 +25,30 @@
 #include "KWDocument.h"
 #include "KWPage.h"
 
+#include <ktemporaryfile.h>
+
 #include "frames/KWTextFrameSet.h"
 #include "frames/KWTextFrame.h"
 #include <KoXmlWriter.h>
 #include <KoOdfWriteStore.h>
 #include <KoShapeSavingContext.h>
+
 #include <KoTextShapeData.h>
 #include <KoStyleManager.h>
 #include <KoParagraphStyle.h>
 #include <KoShapeGroup.h>
 #include <KoShapeLayer.h>
 
+#include <KoGenChanges.h>
+#include <KoTextSharedSavingData.h>
+
 #include <KoStoreDevice.h>
 
 #include <QBuffer>
 #include <QTextCursor>
 #include <KDebug>
 
-QByteArray KWOdfWriter::serializeHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGenStyles& mainStyles, KWTextFrameSet* fs)
+QByteArray KWOdfWriter::serializeHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGenStyles& mainStyles, KoGenChanges & changes, KWTextFrameSet* fs)
 {
     QByteArray tag;
     switch (fs->textFrameSetType()) {
@@ -57,10 +63,15 @@ QByteArray KWOdfWriter::serializeHeaderFooter(KoEmbeddedDocumentSaver& embeddedS
     QBuffer buffer(&content);
     buffer.open(QIODevice::WriteOnly);
     KoXmlWriter writer(&buffer);
+
     KoShapeSavingContext context(writer, mainStyles, embeddedSaver);
 
+    KoTextSharedSavingData *sharedData = new KoTextSharedSavingData;
+    sharedData->setGenChanges(changes);
+    context.addSharedData(KOTEXT_SHARED_SAVING_ID, sharedData);
+
     Q_ASSERT(!fs->frames().isEmpty());
-    KoTextShapeData *shapedata = dynamic_cast<KoTextShapeData *>(fs->frames().first()->shape()->userData());
+    KoTextShapeData *shapedata = qobject_cast<KoTextShapeData *>(fs->frames().first()->shape()->userData());
     Q_ASSERT(shapedata);
 
     writer.startElement(tag);
@@ -71,7 +82,7 @@ QByteArray KWOdfWriter::serializeHeaderFooter(KoEmbeddedDocumentSaver& embeddedS
 }
 
 // rename to save pages ?
-void KWOdfWriter::saveHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGenStyles& mainStyles)
+void KWOdfWriter::saveHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGenStyles& mainStyles, KoGenChanges& changes)
 {
     //kDebug(32001 )<< "START saveHeaderFooter ############################################";
     // first get all the framesets in a nice quick-to-access data structure
@@ -100,7 +111,7 @@ void KWOdfWriter::saveHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGen
         KoGenStyle layoutStyle = pageStyle.saveOdf();
         masterStyle.addProperty("style:page-layout-name", mainStyles.lookup(layoutStyle, "pm"));
         QString name = mainStyles.lookup(masterStyle, pageStyle.name(), KoGenStyles::DontForceNumbering);
-        masterPages.insert(pageStyle, name);
+        m_masterPages.insert(pageStyle, name);
     }
 
     // We need to flush them out ordered as defined in the specs.
@@ -126,7 +137,8 @@ void KWOdfWriter::saveHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGen
             if (fs->frameCount() == 0) // don't save empty framesets
                 continue;
 
-            QByteArray content = serializeHeaderFooter(embeddedSaver, mainStyles, fs);
+            QByteArray content = serializeHeaderFooter(embeddedSaver, mainStyles, changes, fs);
+
             if (content.isNull())
                 continue;
 
@@ -135,7 +147,7 @@ void KWOdfWriter::saveHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGen
         // append the headerfooter-style to the main-style
         if (! masterStyle.isEmpty()) {
             QString name = mainStyles.lookup(masterStyle, pageStyle.name(), KoGenStyles::DontForceNumbering);
-            masterPages.insert(pageStyle, name);
+            m_masterPages.insert(pageStyle, name);
         }
     }
 
@@ -148,7 +160,7 @@ void KWOdfWriter::saveHeaderFooter(KoEmbeddedDocumentSaver& embeddedSaver, KoGen
 KWOdfWriter::KWOdfWriter(KWDocument *document)
         : QObject(),
         m_document(document),
-        m_shapeTree(4,2)
+        m_shapeTree(4, 2)
 {
 }
 
@@ -163,25 +175,39 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
 
     KoStore * store = odfStore.store();
 
-    if ( ! store->open( "settings.xml" ) ) {
+    if (! store->open("settings.xml")) {
         return false;
     }
 
-    saveOdfSettings( store );
+    saveOdfSettings(store);
 
-    if(!store->close())
+    if (!store->close())
         return false;
 
     KoXmlWriter * manifestWriter = odfStore.manifestWriter();
 
-    manifestWriter->addManifestEntry( "settings.xml", "text/xml" );
+    manifestWriter->addManifestEntry("settings.xml", "text/xml");
 
     KoXmlWriter* contentWriter = odfStore.contentWriter();
     if (!contentWriter)
         return false;
 
+    KTemporaryFile tmpChangeFile;
+    tmpChangeFile.open();
+    KoXmlWriter *changeWriter = new KoXmlWriter(&tmpChangeFile, 1);
+    if (!changeWriter)
+        return false;
+
+    KTemporaryFile tmpTextBodyFile;
+    tmpTextBodyFile.open();
+    KoXmlWriter *tmpBodyWriter = new KoXmlWriter(&tmpTextBodyFile, 1);
+    if (!tmpBodyWriter)
+        return false;
+
     KoGenStyles mainStyles;
 
+    KoGenChanges changes;
+
     // Save the named styles
     KoStyleManager *styleManager = dynamic_cast<KoStyleManager *>(m_document->dataCenterMap()["StyleManager"]);
     styleManager->saveOdf(mainStyles);
@@ -190,14 +216,19 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
 
     // Header and footers save their content into master-styles/master-page, and their
     // styles into the page-layout automatic-style.
-    saveHeaderFooter(embeddedSaver, mainStyles);
+
+    saveHeaderFooter(embeddedSaver, mainStyles, changes);
 
     KoXmlWriter *bodyWriter = odfStore.bodyWriter();
-    KoShapeSavingContext context(*bodyWriter, mainStyles, embeddedSaver);
-
     bodyWriter->startElement("office:body");
     bodyWriter->startElement("office:text");
 
+    KoShapeSavingContext context(*tmpBodyWriter, mainStyles, embeddedSaver);
+
+    KoTextSharedSavingData *sharedData = new KoTextSharedSavingData;
+    sharedData->setGenChanges(changes);
+    context.addSharedData(KOTEXT_SHARED_SAVING_ID, sharedData);
+
     calculateZindexOffsets();
 
     KWTextFrameSet *mainTextFrame = 0;
@@ -276,7 +307,7 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
                 value.clear();
                 KWTextFrame *tf = dynamic_cast<KWTextFrame*>(frame);
                 if (tf && tf->minimumFrameHeight() > 1)
-                    shape->setAdditionalAttribute("fo:min-height", QString::number(tf->minimumFrameHeight())+ "pt");
+                    shape->setAdditionalAttribute("fo:min-height", QString::number(tf->minimumFrameHeight()) + "pt");
                 break;
             }
             if (!value.isEmpty())
@@ -310,7 +341,7 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
             shape->setAdditionalAttribute("draw:z-index", QString::number(effectiveZIndex));
             shape->setAdditionalAttribute("text:anchor-type", "page");
             shape->setAdditionalAttribute("text:anchor-page-number", QString::number(page.pageNumber()));
-            context.addShapeOffset(shape, QMatrix(1, 0, 0 , 1, 0, -pagePos ));
+            context.addShapeOffset(shape, QMatrix(1, 0, 0 , 1, 0, -pagePos));
             shape->saveOdf(context);
             context.removeShapeOffset(shape);
             shape->removeAdditionalAttribute("draw:copy-of");
@@ -325,14 +356,14 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
 
     if (mainTextFrame) {
         if (! mainTextFrame->frames().isEmpty() && mainTextFrame->frames().first()) {
-            KoTextShapeData * shapeData = dynamic_cast<KoTextShapeData *>(mainTextFrame->frames().first()->shape()->userData());
+            KoTextShapeData * shapeData = qobject_cast<KoTextShapeData *>(mainTextFrame->frames().first()->shape()->userData());
             if (shapeData) {
                 KWPageManager *pm = m_document->pageManager();
                 if (pm->pageCount()) { // make the first page refer to our page master
                     QTextCursor cursor(shapeData->document());
                     QTextBlockFormat tbf;
                     KWPageStyle style = pm->pages().first().pageStyle();
-                    tbf.setProperty(KoParagraphStyle::MasterPageName, masterPages.value(style));
+                    tbf.setProperty(KoParagraphStyle::MasterPageName, m_masterPages.value(style));
                     cursor.mergeBlockFormat(tbf);
                 }
                 shapeData->saveOdf(context);
@@ -340,20 +371,35 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
         }
     }
 
+    //we save the changes before starting the page sequence element because odf validator insist on having <tracked-changes> right after the <office:text> tag
+    mainStyles.saveOdfAutomaticStyles(contentWriter, false);
+
+    changes.saveOdfChanges(changeWriter);
+
+    delete changeWriter;
+    changeWriter = 0;
+
+    tmpChangeFile.close();
+    bodyWriter->addCompleteElement(&tmpChangeFile);
+
     bodyWriter->startElement("text:page-sequence");
     foreach (KWPage page, m_document->pageManager()->pages()) {
-        Q_ASSERT(masterPages.contains(page.pageStyle()));
+        Q_ASSERT(m_masterPages.contains(page.pageStyle()));
         bodyWriter->startElement("text:page");
-        bodyWriter->addAttribute("text:master-page-name", masterPages.value(page.pageStyle()));
+        bodyWriter->addAttribute("text:master-page-name", m_masterPages.value(page.pageStyle()));
         bodyWriter->endElement(); // text:page
     }
     bodyWriter->endElement(); // text:page-sequence
 
+    delete tmpBodyWriter;
+    tmpBodyWriter = 0;
+
+    tmpTextBodyFile.close();
+    bodyWriter->addCompleteElement(&tmpTextBodyFile);
+
     bodyWriter->endElement(); // office:text
     bodyWriter->endElement(); // office:body
 
-    mainStyles.saveOdfAutomaticStyles(contentWriter, false);
-
     odfStore.closeContentWriter();
 
     // add manifest line for content.xml
@@ -372,28 +418,28 @@ bool KWOdfWriter::save(KoOdfWriteStore & odfStore, KoEmbeddedDocumentSaver & emb
 
 bool KWOdfWriter::saveOdfSettings(KoStore *store)
 {
-    KoStoreDevice settingsDev( store );
-    KoXmlWriter * settingsWriter = KoOdfWriteStore::createOasisXmlWriter( &settingsDev, "office:document-settings" );
+    KoStoreDevice settingsDev(store);
+    KoXmlWriter * settingsWriter = KoOdfWriteStore::createOasisXmlWriter(&settingsDev, "office:document-settings");
 
     // add this so that OOo reads guides lines and grid data from ooo:view-settings
-    settingsWriter->addAttribute( "xmlns:ooo", "http://openoffice.org/2004/office" );
+    settingsWriter->addAttribute("xmlns:ooo", "http://openoffice.org/2004/office");
 
     settingsWriter->startElement("office:settings");
     settingsWriter->startElement("config:config-item-set");
     settingsWriter->addAttribute("config:name", "view-settings");
 
-    KoUnit::saveOdf( settingsWriter, m_document->unit() );
+    m_document->saveUnitOdf(settingsWriter);
 
     settingsWriter->endElement(); // config:config-item-set
 
     settingsWriter->startElement("config:config-item-set");
     settingsWriter->addAttribute("config:name", "ooo:view-settings");
-    settingsWriter->startElement("config:config-item-map-indexed" );
-    settingsWriter->addAttribute("config:name", "Views" );
-    settingsWriter->startElement("config:config-item-map-entry" );
+    settingsWriter->startElement("config:config-item-map-indexed");
+    settingsWriter->addAttribute("config:name", "Views");
+    settingsWriter->startElement("config:config-item-map-entry");
 
-    m_document->guidesData().saveOdfSettings( *settingsWriter );
-    m_document->gridData().saveOdfSettings( *settingsWriter );
+    m_document->guidesData().saveOdfSettings(*settingsWriter);
+    m_document->gridData().saveOdfSettings(*settingsWriter);
 
     settingsWriter->endElement(); // config:config-item-map-entry
     settingsWriter->endElement(); // config:config-item-map-indexed
@@ -421,6 +467,7 @@ void KWOdfWriter::calculateZindexOffsets()
     }
 
     foreach (const KWPage &page, m_document->pageManager()->pages()) {
+        // TODO handle pageSpread here.
         int minZIndex = 0;
         foreach (KoShape *shape, m_shapeTree.intersects(page.rect()))
             minZIndex = qMin(shape->zIndex(), minZIndex);
@@ -436,7 +483,7 @@ void KWOdfWriter::addShapeToTree(KoShape *shape)
     // add the children of a KoShapeContainer
     KoShapeContainer* container = dynamic_cast<KoShapeContainer*>(shape);
     if (container) {
-        foreach(KoShape* containerShape, container->iterator()) {
+        foreach(KoShape* containerShape, container->childShapes()) {
             addShapeToTree(containerShape);
         }
     }
