@@ -43,6 +43,10 @@
 
 // koffice libs includes
 #include <KoCopyController.h>
+#include <KoTextDocument.h>
+#include <KoShapeCreateCommand.h>
+#include <KoImageSelectionWidget.h>
+#include <KoCanvasResourceProvider.h>
 #include <KoCutController.h>
 #include <KoPasteController.h>
 #include <KoShape.h>
@@ -55,7 +59,7 @@
 #include <KoZoomAction.h>
 #include <KoToolManager.h>
 #include <KoMainWindow.h>
-#include <KoTextSelectionHandler.h>
+#include <KoTextEditor.h>
 #include <KoToolProxy.h>
 #include <KoCanvasResourceProvider.h>
 #include <KoTextAnchor.h>
@@ -78,12 +82,13 @@
 #include <kxmlguifactory.h>
 #include <kstatusbar.h>
 #include <kfiledialog.h>
+#include <kmessagebox.h>
 
 static KWFrame *frameForShape(KoShape *shape)
 {
     KoShape *parent = shape;
     while (parent->parent())
-        parent = shape->parent();
+        parent = parent->parent();
     return dynamic_cast<KWFrame*>(parent->applicationData());
 }
 
@@ -111,16 +116,16 @@ KWView::KWView(const QString& viewMode, KWDocument* document, QWidget *parent)
 
     new KoFind(this, m_canvas->resourceProvider(), actionCollection());
 
-    m_zoomController = new KoZoomController(m_gui->canvasController(), &m_zoomHandler, actionCollection());
+    m_zoomController = new KoZoomController(m_gui->canvasController(), &m_zoomHandler, actionCollection(), 0, this);
 
     KWStatisticsDockerFactory statisticsFactory(this);
     KWStatisticsDocker *docker = dynamic_cast<KWStatisticsDocker *>(createDockWidget(&statisticsFactory));
     if (docker && docker->view() != this) docker->setView(this);
 
     if (statusBar())
-        new KWStatusBar(statusBar(), this);
+        KWStatusBar::addViewControls(statusBar(), this);
 
-    // the zoom controller needs to be initialized after the status bar gets initialized as 
+    // the zoom controller needs to be initialized after the status bar gets initialized as
     // that resulted in bug 180759
     m_zoomController->setPageSize(m_currentPage.rect().size());
     KoZoomMode::Modes modes = KoZoomMode::ZOOM_WIDTH;
@@ -135,7 +140,6 @@ KWView::KWView(const QString& viewMode, KWDocument* document, QWidget *parent)
 
 KWView::~KWView()
 {
-    delete m_zoomController;
 }
 
 KWCanvas *KWView::kwcanvas() const
@@ -143,6 +147,11 @@ KWCanvas *KWView::kwcanvas() const
     return m_canvas;
 }
 
+const KWPage KWView::currentPage() const
+{
+    return m_currentPage;
+}
+
 QWidget *KWView::canvas() const
 {
     return m_canvas;
@@ -236,6 +245,11 @@ void KWView::setupActions()
     actionCollection()->addAction("select_bookmark", action);
     connect(action, SIGNAL(triggered()), this, SLOT(selectBookmark()));
 
+    action = new KAction(i18n("Insert Picture..."), this);
+    action->setToolTip(i18n("Insert a picture into document"));
+    actionCollection()->addAction("insert_picture", action);
+    connect(action, SIGNAL(triggered()), this, SLOT(insertImage()));
+
     action = new KAction(i18n("Frame Borders"), this);
     action->setToolTip(i18n("Turns the border display on and off"));
     action->setCheckable(true);
@@ -305,11 +319,9 @@ if (false) { // TODO move this to the text tool as soon as  a) the string freeze
     action->setShortcut(QKeySequence("Del"));
     connect(action, SIGNAL(triggered()), this, SLOT(editDeleteSelection()));
     connect(kwcanvas()->toolProxy(), SIGNAL(selectionChanged(bool)), action, SLOT(setEnabled(bool)));
-    actionCollection()->addAction("edit_delete", action );
+    actionCollection()->addAction("edit_delete", action);
 
     action = m_document->gridData().gridToggleAction(m_canvas);
-    // XXX remove the translated string when the string freeze is lifted, the KoGridData should have those
-    action->setText(i18n("Show Grid"));
     actionCollection()->addAction("view_grid", action);
 
     // -------------- Frame menu
@@ -320,6 +332,7 @@ if (false) { // TODO move this to the text tool as soon as  a) the string freeze
     connect(action, SIGNAL(triggered()), this, SLOT(createLinkedFrame()));
 
     action = new KAction(i18n("Create custom outline"), this);
+    actionCollection()->addAction("create_custom_outline", action);
     action->setToolTip(i18n("Create a custom vector outline that text will run around"));
     action->setWhatsThis(i18n("Text normally runs around the content of a shape, when you want a custom outline that is independent of the content you can create one and alter it with the vector tools"));
     connect(action, SIGNAL(triggered()), this, SLOT(createCustomOutline()));
@@ -817,6 +830,7 @@ void KWView::editFrameProperties()
 KoPrintJob * KWView::createPrintJob()
 {
     KWPrintingDialog *dia = new KWPrintingDialog(this);
+    dia->printer().setResolution(600);
     dia->printer().setCreator("KWord 2.0");
     dia->printer().setFullPage(true); // ignore printer margins
     return dia;
@@ -825,7 +839,7 @@ KoPrintJob * KWView::createPrintJob()
 void KWView::insertFrameBreak()
 {
     // TODO should this be removed??
-    KoTextSelectionHandler *handler = qobject_cast<KoTextSelectionHandler*> (kwcanvas()->toolProxy()->selection());
+    KoTextEditor *handler = qobject_cast<KoTextEditor*> (kwcanvas()->toolProxy()->selection());
     if (handler)
         handler->insertFrameBreak();
 }
@@ -846,7 +860,7 @@ void KWView::addBookmark()
 
     QString tool = KoToolManager::instance()->preferredToolForSelection(selection->selectedShapes());
     KoToolManager::instance()->switchToolRequested(tool);
-    KoTextSelectionHandler *handler = qobject_cast<KoTextSelectionHandler*> (kwcanvas()->toolProxy()->selection());
+    KoTextEditor *handler = qobject_cast<KoTextEditor*> (kwcanvas()->toolProxy()->selection());
     Q_ASSERT(handler);
 
     KoBookmarkManager *manager = m_document->inlineTextObjectManager()->bookmarkManager();
@@ -919,7 +933,7 @@ void KWView::deleteBookmark(const QString &name)
 
     QString tool = KoToolManager::instance()->preferredToolForSelection(selection->selectedShapes());
     KoToolManager::instance()->switchToolRequested(tool);
-    KoTextSelectionHandler *handler = qobject_cast<KoTextSelectionHandler*> (kwcanvas()->toolProxy()->selection());
+    KoTextEditor *handler = qobject_cast<KoTextEditor*> (kwcanvas()->toolProxy()->selection());
     Q_ASSERT(handler);
 
     KoCanvasResourceProvider *provider = m_canvas->resourceProvider();
@@ -997,7 +1011,12 @@ void KWView::formatPage()
     if (! m_currentPage.isValid())
         return;
     KWPageSettingsDialog *dia = new KWPageSettingsDialog(this, m_document, m_currentPage);
-    dia->showTextDirection(kwcanvas()->resourceProvider()->boolResource(KoText::BidiDocument));
+    if (m_document->mainFrameSet()) {
+        KoTextDocument doc(m_document->mainFrameSet()->document());
+        KoTextEditor *editor = doc.textEditor();
+        if (editor)
+            dia->showTextDirection(editor->isBidiDocument());
+    }
     dia->show();
 }
 
@@ -1008,7 +1027,7 @@ void KWView::inlineFrame()
 
     KoShape *targetShape = 0;
     foreach (KoShape *shape, selection->selectedShapes(KoFlake::TopLevelSelection)) {
-        if (shape->isLocked())
+        if (shape->isGeometryProtected())
             continue;
         if (shape->parent())
             continue;
@@ -1016,8 +1035,7 @@ void KWView::inlineFrame()
         break; // TODO group before...
     }
     if (targetShape == 0) {
-        // message:  "Please select at least one non locked shape and try again"
-        kDebug() << "Please select at least one non locked shape and try again";
+        KMessageBox::error(this, i18n("Please select at least one non-locked shape and try again"));
         return;
     }
 
@@ -1049,7 +1067,7 @@ void KWView::inlineFrame()
 
     QString tool = KoToolManager::instance()->preferredToolForSelection(selection->selectedShapes());
     KoToolManager::instance()->switchToolRequested(tool);
-    KoTextSelectionHandler *handler = qobject_cast<KoTextSelectionHandler*> (kwcanvas()->toolProxy()->selection());
+    KoTextEditor *handler = qobject_cast<KoTextEditor*> (kwcanvas()->toolProxy()->selection());
     Q_ASSERT(handler);
     KoTextAnchor *anchor = new KoTextAnchor(targetShape);
     anchor->setOffset(QPointF(0, -targetShape->size().height()));
@@ -1082,7 +1100,7 @@ void KWView::createLinkedFrame()
     foreach (KoShape *shape, oldSelection) {
         KWFrame *frame = dynamic_cast<KWFrame*>(shape->applicationData());
         Q_ASSERT(frame);
-        KWCopyShape *copy = new KWCopyShape(frame->shape());
+        KWCopyShape *copy = new KWCopyShape(frame->shape(), m_document->pageManager());
         copy->setPosition(frame->shape()->position());
         QPointF offset(40, 40);
         kwcanvas()->clipToDocument(copy, offset);
@@ -1166,6 +1184,39 @@ void KWView::createCustomOutline()
     KoToolManager::instance()->switchToolRequested("PathToolFactoryId");
 }
 
+void KWView::insertImage()
+{
+    KoShape *shape = KoImageSelectionWidget::selectImageShape(m_document->dataCenterMap(), this);
+    if (shape) {
+        if (m_currentPage.isValid()) {
+            QRectF page = m_currentPage.rect();
+            // make the shape be on the current page, and fit inside the current page.
+            if (page.width() < shape->size().width() || page.height() < shape->size().height()) {
+                QSizeF newSize(page.width() * 0.9, page.height() * 0.9);
+                const qreal xRatio = newSize.width() / shape->size().width();
+                const qreal yRatio = newSize.height() / shape->size().height();
+                if (xRatio > yRatio) // then lets make the vertical set the size.
+                    newSize.setWidth(shape->size().width() * yRatio);
+                else
+                    newSize.setHeight(shape->size().height() * xRatio);
+                shape->setSize(newSize);
+            }
+            shape->setPosition(page.topLeft());
+
+            int zIndex = 0;
+            foreach (KoShape *s, m_canvas->shapeManager()->shapesAt(page))
+                zIndex = qMax(s->zIndex(), zIndex);
+            shape->setZIndex(zIndex+1);
+        }
+
+        KoShapeCreateCommand *cmd = new KoShapeCreateCommand(m_document, shape);
+        KoSelection *selection = m_canvas->shapeManager()->selection();
+        selection->deselectAll();
+        selection->select(shape);
+        m_document->addCommand(cmd);
+    }
+}
+
 
 // end of actions
 
@@ -1192,14 +1243,7 @@ void KWView::selectionChanged()
     m_actionFormatFrameSet->setEnabled(shape != 0);
     m_actionAddBookmark->setEnabled(shape != 0);
     if (shape) {
-        KWPage currentPage = m_document->pageManager()->page(shape);
-        if (currentPage != m_currentPage) {
-            m_currentPage = currentPage;
-            m_canvas->resourceProvider()->setResource(KWord::CurrentPage, m_currentPage.pageNumber());
-            m_zoomController->setPageSize(m_currentPage.rect().size());
-            m_actionViewHeader->setChecked(m_currentPage.pageStyle().headerPolicy() != KWord::HFTypeNone);
-            m_actionViewFooter->setChecked(m_currentPage.pageStyle().footerPolicy() != KWord::HFTypeNone);
-        }
+        setCurrentPage(m_document->pageManager()->page(shape));
         KWFrame *frame = frameForShape(shape);
         KWTextFrameSet *fs = frame == 0 ? 0 : dynamic_cast<KWTextFrameSet*>(frame->frameSet());
         if (fs)
@@ -1222,6 +1266,18 @@ void KWView::selectionChanged()
     }
 }
 
+void KWView::setCurrentPage(const KWPage &currentPage)
+{
+    Q_ASSERT(currentPage.isValid());
+    if (currentPage != m_currentPage) {
+        m_currentPage = currentPage;
+        m_canvas->resourceProvider()->setResource(KoCanvasResource::CurrentPage, m_currentPage.pageNumber());
+        m_zoomController->setPageSize(m_currentPage.rect().size());
+        m_actionViewHeader->setChecked(m_currentPage.pageStyle().headerPolicy() != KWord::HFTypeNone);
+        m_actionViewFooter->setChecked(m_currentPage.pageStyle().footerPolicy() != KWord::HFTypeNone);
+    }
+}
+
 void KWView::showEvent(QShowEvent *e)
 {
     KoView::showEvent(e);
@@ -1234,3 +1290,35 @@ void KWView::updateStatusBarAction()
     if (action && statusBar())
         action->setChecked(! statusBar()->isHidden());
 }
+
+void KWView::offsetInDocumentMoved(int yOffset)
+{
+    const qreal offset = -m_zoomHandler.viewToDocumentY(yOffset);
+    const qreal height = m_zoomHandler.viewToDocumentY(m_gui->viewportSize().height());
+    if (m_currentPage.isValid()) { // most of the time the current will not change.
+        const qreal pageTop = m_currentPage.offsetInDocument();
+        const qreal pageBottom = pageTop + m_currentPage.height();
+        const qreal visibleArea = qMin(offset + height, pageBottom) - qMax(pageTop, offset);
+        if (visibleArea / height >= 0.45) // current page is just fine if > 45% is shown
+            return;
+
+        // using 'next'/'prev' is much cheaper than using a documentOffset, so try that first.
+        if (pageTop > offset && pageTop < offset + height) { // check if the page above is a candidate.
+            KWPage page = m_currentPage.previous();
+            if (page.isValid() && pageTop - offset > visibleArea) {
+                setCurrentPage(page);
+                return;
+            }
+        }
+        if (pageBottom > offset && pageBottom < offset + height) { // check if the page above is a candidate.
+            KWPage page = m_currentPage.next();
+            if (page.isValid() && m_currentPage.height() - height > visibleArea) {
+                setCurrentPage(page);
+                return;
+            }
+        }
+    }
+    KWPage page = m_document->pageManager()->page(qreal(offset + height / 2.0));
+    if (page.isValid())
+        setCurrentPage(page);
+}
