@@ -31,10 +31,9 @@
 #include <KoUnit.h>
 #include "KoConnectionShapeLoadingUpdater.h"
 
-#include <kdebug.h>
-
 #include <QPainter>
 
+#include <KDebug>
 // XXX: Add editable text in path shapes so we can get a label here
 
 struct KoConnectionShape::Private {
@@ -66,6 +65,8 @@ KoConnectionShape::KoConnectionShape()
     int connectionPointCount = connectionPoints().size();
     for (int i = 0; i < connectionPointCount; ++i)
         removeConnectionPoint(0);
+
+    m_hasMoved = true;
 }
 
 KoConnectionShape::~KoConnectionShape()
@@ -86,7 +87,7 @@ void KoConnectionShape::saveOdf(KoShapeSavingContext & context) const
 {
     context.xmlWriter().startElement("draw:connector");
     saveOdfAttributes( context, OdfMandatories | OdfAdditionalAttributes );
-    
+
     switch( d->connectionType )
     {
         case Lines:
@@ -102,7 +103,7 @@ void KoConnectionShape::saveOdf(KoShapeSavingContext & context) const
             context.xmlWriter().addAttribute( "draw:type", "standard" );
             break;
     }
-    
+
     if (d->shape1) {
         context.xmlWriter().addAttribute( "draw:start-shape", context.drawId(d->shape1) );
         context.xmlWriter().addAttribute( "draw:start-glue-point", d->connectionPointIndex1 );
@@ -117,13 +118,13 @@ void KoConnectionShape::saveOdf(KoShapeSavingContext & context) const
         context.xmlWriter().addAttribute( "draw:end-glue-point", d->connectionPointIndex2 );
     }
     else {
-        QPointF p((m_handles[1] + position()) * context.shapeOffset(this));
+        QPointF p((m_handles[m_handles.count()-1] + position()) * context.shapeOffset(this));
         context.xmlWriter().addAttributePt( "svg:x2", p.x() );
         context.xmlWriter().addAttributePt( "svg:y2", p.y() );
     }
 
     saveOdfCommonChildElements(context);
-    
+
     context.xmlWriter().endElement();
 }
 
@@ -167,8 +168,8 @@ bool KoConnectionShape::loadOdf(const KoXmlElement & element, KoShapeLoadingCont
             context.updateShape(shapeId2, new KoConnectionShapeLoadingUpdater(this, KoConnectionShapeLoadingUpdater::Second));
         }
     } else {
-        m_handles[1].setX(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "x2", QString())));
-        m_handles[1].setY(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "y2", QString())));
+        m_handles[m_handles.count() - 1].setX(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "x2", QString())));
+        m_handles[m_handles.count() - 1].setY(KoUnit::parseValue(element.attributeNS(KoXmlNS::svg, "y2", QString())));
     }
 
     QString skew = element.attributeNS(KoXmlNS::draw, "line-skew", "");
@@ -182,14 +183,14 @@ bool KoConnectionShape::loadOdf(const KoXmlElement & element, KoShapeLoadingCont
 
 bool KoConnectionShape::hitTest(const QPointF &position) const
 {
-    return KoShape::hitTest(position);
+    return KoParameterShape::hitTest(position);
 }
 
 void KoConnectionShape::moveHandleAction(int handleId, const QPointF & point, Qt::KeyboardModifiers modifiers)
 {
     Q_UNUSED(modifiers);
 
-    if (handleId > m_handles.size())
+    if (handleId >= m_handles.size())
         return;
 
     m_handles[handleId] = point;
@@ -199,19 +200,80 @@ void KoConnectionShape::updatePath(const QSizeF &size)
 {
     Q_UNUSED(size);
 
-    const qreal MinimumEscapeLength = 20.0;
-
+    QPointF dst = 0.3 * ( m_handles[0] - m_handles[m_handles.count() - 1]);
+    const qreal MinimumEscapeLength = (qreal)20.;
     clear();
     switch (d->connectionType) {
     case Standard: {
+        normalPath(MinimumEscapeLength);
+        if( m_path.count() != 0 ){
+            moveTo( m_path[0] );
+            for(int index = 1; index < m_path.count(); ++index )
+                lineTo( m_path[index] );
+        }
+
+        break;
+    }
+    case Lines: {
+        QPointF direction1 = escapeDirection(0);
+        QPointF direction2 = escapeDirection(m_handles.count() - 1);
+        moveTo(m_handles[0]);
+        if (! direction1.isNull())
+            lineTo(m_handles[0] + MinimumEscapeLength * direction1);
+        if (! direction2.isNull())
+            lineTo(m_handles[m_handles.count() - 1] + MinimumEscapeLength * direction2);
+        lineTo(m_handles[m_handles.count() - 1]);
+        break;
+    }
+    case Straight:
+        moveTo(m_handles[0]);
+        lineTo(m_handles[m_handles.count() - 1]);
+        break;
+    case Curve:
+        // TODO
+        QPointF direction1 = escapeDirection(0);
+        QPointF direction2 = escapeDirection(m_handles.count() - 1);
+        moveTo(m_handles[0]);
+        if (! direction1.isNull() && ! direction2.isNull()) {
+            QPointF curvePoint1 = m_handles[0] + 5.0 * MinimumEscapeLength * direction1;
+            QPointF curvePoint2 = m_handles[m_handles.count() - 1] + 5.0 * MinimumEscapeLength * direction2;
+            curveTo(curvePoint1, curvePoint2, m_handles[m_handles.count() - 1]);
+        } else {
+            lineTo(m_handles[m_handles.count() - 1]);
+        }
+        break;
+    }
+    normalize();
+}
+
+void KoConnectionShape::normalPath( const qreal MinimumEscapeLength )
+{
+    if(m_hasMoved) {
+
+        m_hasMoved = false;
+        QPointF firstHandle;
+        QPointF lastHandle;
+
+        // Clear handles keeping the first and last one.
+        firstHandle = m_handles[0];
+        lastHandle = m_handles[m_handles.count() - 1];
+
+        m_handles.clear();
+        m_handles.append(firstHandle);
+        m_handles.append(lastHandle);
+
+        // Clear the path to build it again.
+        m_path.clear();
+        m_path.append( m_handles[0] );
+
         QList<QPointF> edges1;
         QList<QPointF> edges2;
 
         QPointF direction1 = escapeDirection(0);
-        QPointF direction2 = escapeDirection(1);
-
+        QPointF direction2 = escapeDirection(m_handles.count() - 1);
+        
         QPointF edgePoint1 = m_handles[0] + MinimumEscapeLength * direction1;
-        QPointF edgePoint2 = m_handles[1] + MinimumEscapeLength * direction2;
+        QPointF edgePoint2 = m_handles[m_handles.count() - 1] + MinimumEscapeLength * direction2;
 
         edges1.append(edgePoint1);
         edges2.prepend(edgePoint2);
@@ -245,47 +307,12 @@ void KoConnectionShape::updatePath(const QSizeF &size)
                 }
             } while (! connected);
         }
-        moveTo(m_handles[0]);
-        uint edgeCount1 = edges1.count();
-        for (uint i = 0; i < edgeCount1; ++i)
-            lineTo(edges1[i]);
-        uint edgeCount2 = edges2.count();
-        for (uint i = 0; i < edgeCount2; ++i)
-            lineTo(edges2[i]);
-        lineTo(m_handles[1]);
 
-        break;
+        m_path.append(edges1);
+        m_path.append(edges2);
+
+        m_path.append( m_handles[m_handles.count() - 1] );
     }
-    case Lines: {
-        QPointF direction1 = escapeDirection(0);
-        QPointF direction2 = escapeDirection(1);
-        moveTo(m_handles[0]);
-        if (! direction1.isNull())
-            lineTo(m_handles[0] + MinimumEscapeLength * direction1);
-        if (! direction2.isNull())
-            lineTo(m_handles[1] + MinimumEscapeLength * direction2);
-        lineTo(m_handles[1]);
-        break;
-    }
-    case Straight:
-        moveTo(m_handles[0]);
-        lineTo(m_handles[1]);
-        break;
-    case Curve:
-        // TODO
-        QPointF direction1 = escapeDirection(0);
-        QPointF direction2 = escapeDirection(1);
-        moveTo(m_handles[0]);
-        if (! direction1.isNull() && ! direction2.isNull()) {
-            QPointF curvePoint1 = m_handles[0] + 5.0 * MinimumEscapeLength * direction1;
-            QPointF curvePoint2 = m_handles[1] + 5.0 * MinimumEscapeLength * direction2;
-            curveTo(curvePoint1, curvePoint2, m_handles[1]);
-        } else {
-            lineTo(m_handles[1]);
-        }
-        break;
-    }
-    normalize();
 }
 
 bool KoConnectionShape::setConnection1(KoShape * shape1, int connectionPointIndex1)
@@ -293,14 +320,19 @@ bool KoConnectionShape::setConnection1(KoShape * shape1, int connectionPointInde
     // refuse to connect to a shape that depends on us (e.g. a artistic text shape)
     if (hasDependee(shape1))
         return false;
-    
+
+    // check if the connection point does exist
+    if (shape1 && connectionPointIndex1 >= shape1->connectionPoints().count())
+        return false;
+
     if (d->shape1)
         d->shape1->removeDependee(this);
     d->shape1 = shape1;
     if (d->shape1)
         d->shape1->addDependee(this);
+
     d->connectionPointIndex1 = connectionPointIndex1;
-    
+
     return true;
 }
 
@@ -309,14 +341,19 @@ bool KoConnectionShape::setConnection2(KoShape * shape2, int connectionPointInde
     // refuse to connect to a shape that depends on us (e.g. a artistic text shape)
     if (hasDependee(shape2))
         return false;
-    
+
+    // check if the connection point does exist
+    if (shape2 && connectionPointIndex2 >= shape2->connectionPoints().count())
+        return false;
+
     if (d->shape2)
         d->shape2->removeDependee(this);
     d->shape2 = shape2;
     if (d->shape2)
         d->shape2->addDependee(this);
+
     d->connectionPointIndex2 = connectionPointIndex2;
-    
+
     return true;
 }
 
@@ -350,8 +387,8 @@ void KoConnectionShape::updateConnections()
         if (d->connectionPointIndex2 < connectionPoints.count()) {
             // map connection point into our shape coordinates
             QPointF p = documentToShape(d->shape2->absoluteTransformation(0).map(connectionPoints[d->connectionPointIndex2]));
-            if (m_handles[1] != p) {
-                m_handles[1] = p;
+            if (m_handles[m_handles.count() - 1] != p) {
+                m_handles[m_handles.count() - 1] = p;
                 updateHandles = true;
             }
         }
@@ -469,32 +506,43 @@ QPointF KoConnectionShape::perpendicularDirection(const QPointF &p1, const QPoin
     return perpendicular;
 }
 
-void KoConnectionShape::shapeChanged(ChangeType type)
+void KoConnectionShape::shapeChanged(ChangeType type, KoShape * shape)
 {
-    if (! isParametricShape())
-        return;
+    // check if we are during a forced update
+    const bool updateIsActive = d->forceUpdate;
 
+    m_hasMoved = true;
     switch (type) {
-    case RotationChanged:
-    case ShearChanged:
-    case ScaleChanged:
-    case GenericMatrixChange:
-        d->forceUpdate = true;
-        break;
-    default:
-        return;
-    }
-}
-
-void KoConnectionShape::notifyShapeChanged(KoShape * shape, ChangeType type)
-{
-    if (type == KoShape::Deleted) {
-        if (shape == d->shape1)
-            setConnection1(0, -1);
-        else if (shape == d->shape2)
-            setConnection2(0, -1);
+        case PositionChanged:
+        case RotationChanged:
+        case ShearChanged:
+        case ScaleChanged:
+        case GenericMatrixChange:
+        case ParameterChanged:
+            if (isParametricShape() && shape == 0)
+                d->forceUpdate = true;
+            break;
+        case Deleted:
+            if (shape != d->shape1 && shape != d->shape2)
+                return;
+            if (shape == d->shape1)
+                setConnection1(0, -1);
+            if (shape == d->shape2)
+                setConnection2(0, -1);
+            break;
+        default:
+            return;
     }
 
-    if (isParametricShape())
+    // the connection was moved while it is connected to some other shapes
+    const bool connectionChanged = !shape && (d->shape1 || d->shape2);
+    // one of the connected shape has moved
+    const bool connectedShapeChanged = shape && (shape == d->shape1 || shape == d->shape2);
+    
+    if (!updateIsActive && (connectionChanged || connectedShapeChanged) && isParametricShape())
         updateConnections();
+    
+    // reset the forced update flag
+    d->forceUpdate = false;
 }
+
