@@ -19,6 +19,8 @@
 
 #include "KoShapeContainerModel.h"
 
+#include "KoShapeContainer.h"
+
 KoShapeContainerModel::KoShapeContainerModel()
 {
 }
@@ -27,3 +29,20 @@ KoShapeContainerModel::~KoShapeContainerModel()
 {
 }
 
+void KoShapeContainerModel::proposeMove(KoShape *child, QPointF &move)
+{
+    Q_UNUSED(child);
+    Q_UNUSED(move);
+}
+
+void KoShapeContainerModel::childChanged(KoShape *child, KoShape::ChangeType type)
+{
+    Q_UNUSED( type );
+    KoShapeContainer * parent = child->parent();
+    Q_ASSERT(parent);
+    // propagate the change up the hierarchy
+    KoShapeContainer * grandparent = parent->parent();
+    if (grandparent) {
+        grandparent->model()->childChanged(parent, KoShape::ChildChanged);
+    }
+}
