@@ -21,6 +21,7 @@
 #include "KoPADocument.h"
 
 #include <KoStore.h>
+#include <KoUndoStack.h>
 #include <KoXmlWriter.h>
 #include <KoXmlReader.h>
 #include <KoOdfStylesReader.h>
@@ -49,6 +50,7 @@
 #include "KoPASavingContext.h"
 #include "KoPALoadingContext.h"
 #include "KoPAViewMode.h"
+#include "KoPAPageProvider.h"
 #include "commands/KoPAPageDeleteCommand.h"
 
 #include <kdebug.h>
@@ -80,6 +82,9 @@ KoPADocument::KoPADocument( QWidget* parentWidget, QObject* parent, bool singleV
         shapeFactory->populateDataCenterMap(d->dataCenterMap);
     }
 
+    //Populate the document undoStack in the dataCenterMap. This can be used later by shapes for their undo/redo mechanism.
+    d->dataCenterMap["UndoStack"] = undoStack();
+    d->dataCenterMap[KoPageProvider::ID] = new KoPAPageProvider();
     loadConfig();
 }
 
@@ -96,7 +101,7 @@ void KoPADocument::paintContent( QPainter &painter, const QRect &rect)
 {
     KoPAPageBase * page = pageByIndex( 0, false );
     Q_ASSERT( page );
-    QPixmap thumbnail = page->thumbnail( rect.size() );
+    QPixmap thumbnail( pageThumbnail( page, rect.size() ) );
     painter.drawPixmap( rect, thumbnail );
 }
 
@@ -119,14 +124,14 @@ bool KoPADocument::loadOdf( KoOdfReadStore & odfStore )
     KoXmlElement realBody ( KoXml::namedItemNS( content, KoXmlNS::office, "body" ) );
 
     if ( realBody.isNull() ) {
-        kError() << "No body tag found!" << endl;
+        kError(30010) << "No body tag found!" << endl;
         return false;
     }
 
     KoXmlElement body = KoXml::namedItemNS(realBody, KoXmlNS::office, odfTagName( false ));
 
     if ( body.isNull() ) {
-        kError() << "No office:" << odfTagName( false ) << " tag found!" << endl;
+        kError(30010) << "No office:" << odfTagName( false ) << " tag found!" << endl;
         return false;
     }
 
@@ -139,6 +144,14 @@ bool KoPADocument::loadOdf( KoOdfReadStore & odfStore )
     d->masterPages = loadOdfMasterPages( odfStore.styles().masterPages(), paContext );
     d->pages = loadOdfPages( body, paContext );
 
+    // create pages if there are none
+    if (d->masterPages.empty()) {
+        d->masterPages.append(newMasterPage());
+    }
+    if (d->pages.empty()) {
+        d->pages.append(newPage(static_cast<KoPAMasterPage*>(d->masterPages.first())));
+    }
+
     if ( !loadOdfEpilogue( body, paContext ) ) {
         return false;
     }
@@ -149,6 +162,8 @@ bool KoPADocument::loadOdf( KoOdfReadStore & odfStore )
         setActionEnabled( KoPAView::ActionDeletePage, false );
     }
 
+    updatePageCount();
+
     emit sigProgress( -1 );
     return true;
 }
@@ -245,12 +260,16 @@ QList<KoPAPageBase *> KoPADocument::loadOdfMasterPages( const QHash<QString, KoX
 
 QList<KoPAPageBase *> KoPADocument::loadOdfPages( const KoXmlElement & body, KoPALoadingContext & context )
 {
+    if (d->masterPages.isEmpty()) { // we require at least one master page. Auto create one if the doc was faulty.
+        d->masterPages << newMasterPage();
+    }
+
     QList<KoPAPageBase *> pages;
     KoXmlElement element;
     forEachElement( element, body )
     {
         if ( element.tagName() == "page" && element.namespaceURI() == KoXmlNS::draw ) {
-            KoPAPage* page = newPage();
+            KoPAPage *page = newPage(static_cast<KoPAMasterPage*>(d->masterPages.first()));
             page->loadOdf( element, context );
             pages.append( page );
         }
@@ -307,7 +326,7 @@ bool KoPADocument::saveOdfSettings( KoStore * store )
     settingsWriter->startElement("config:config-item-set");
     settingsWriter->addAttribute("config:name", "view-settings");
 
-    KoUnit::saveOdf( settingsWriter, unit() );
+    saveUnitOdf(settingsWriter);
 
     settingsWriter->endElement(); // config:config-item-set
 
@@ -511,8 +530,16 @@ void KoPADocument::updateViews(KoPAPageBase *page)
 
     foreach (KoView *view, views()) {
         KoPAView *paView = static_cast<KoPAView *>(view);
-        if (paView->activePage() == page)
-            paView->updateActivePage(page);
+        if ( paView->activePage() == page ) {
+            paView->updateActivePage( page );
+        }
+        else if ( dynamic_cast<KoPAMasterPage *>( page ) ) {
+            // if the page changed is a master page, we need to check whether it is the current page's master page
+            KoPAPage *activePage = dynamic_cast<KoPAPage *>( paView->activePage() );
+            if ( activePage && activePage->masterPage() == page ) {
+                paView->updateActivePage( activePage );
+            }
+        }
     }
 }
 
@@ -521,6 +548,13 @@ KoPageApp::PageType KoPADocument::pageType() const
     return KoPageApp::Page;
 }
 
+QPixmap KoPADocument::pageThumbnail(KoPAPageBase* page, const QSize& size)
+{
+    int pageNumber = pageIndex(page) + 1;
+    static_cast<KoPAPageProvider*>(d->dataCenterMap[KoPAPageProvider::ID])->setMasterPageNumber(pageNumber);
+    return page->thumbnail(size);
+}
+
 void KoPADocument::initEmpty()
 {
     d->masterPages.clear();
@@ -554,6 +588,7 @@ void KoPADocument::insertPage( KoPAPageBase* page, int index )
     }
 
     pages.insert( index, page );
+    updatePageCount();
 
     setActionEnabled( KoPAView::ActionDeletePage, pages.size() > 1 );
 
@@ -579,6 +614,7 @@ void KoPADocument::insertPage( KoPAPageBase* page, KoPAPageBase* after )
     }
 
     pages.insert( index, page );
+    updatePageCount();
 
     setActionEnabled( KoPAView::ActionDeletePage, pages.size() > 1 );
 
@@ -609,6 +645,7 @@ int KoPADocument::takePage( KoPAPageBase *page )
                 kopaView->updateActivePage( newActivePage );
             }
         }
+        updatePageCount();
     }
 
     if ( pages.size() == 1 ) {
@@ -722,12 +759,20 @@ bool KoPADocument::rulersVisible() const
     return d->rulersVisible;
 }
 
-
 int KoPADocument::pageCount() const
 {
     return d->pages.count();
 }
 
+void KoPADocument::updatePageCount()
+{
+    KoInlineTextObjectManager * om = dynamic_cast<KoInlineTextObjectManager*>( dataCenterMap()["InlineTextObjectManager"] );
+
+    if ( om ) {
+        om->setProperty( KoInlineObject::PageCount, pageCount() );
+    }
+}
+
 void KoPADocument::insertIntoDataCenterMap(QString key, KoDataCenter *dc)
 {
     d->dataCenterMap[key] = dc;
