@@ -20,8 +20,9 @@
 
 #include "KoPADocumentModel.h"
 
-#include <KoPADocument.h>
-#include <KoPAPageBase.h>
+#include "KoPADocument.h"
+#include "KoPAPageBase.h"
+#include "KoPAPageProvider.h"
 #include <KoShapePainter.h>
 #include <KoShapeManager.h>
 #include <KoShapeBorderModel.h>
@@ -213,7 +214,7 @@ QVariant KoPADocumentModel::data( const QModelIndex &index, int role ) const
             if( container )
             {
                 bbox = QRectF();
-                foreach( KoShape* shape, container->iterator() )
+                foreach( KoShape* shape, container->childShapes() )
                     bbox = bbox.united( shape->outline().boundingRect() );
             }
             return qreal(bbox.width()) / bbox.height();
@@ -288,19 +289,19 @@ KoDocumentSectionModel::PropertyList KoPADocumentModel::properties( KoShape* sha
 {
     PropertyList l;
     l << Property(i18n("Visible"), SmallIcon("14_layer_visible"), SmallIcon("14_layer_novisible"), shape->isVisible());
-    l << Property(i18n("Locked"), SmallIcon("object-locked"), SmallIcon("object-unlocked"), shape->isLocked());
+    l << Property(i18n("Locked"), SmallIcon("object-locked"), SmallIcon("object-unlocked"), shape->isGeometryProtected());
     return l;
 }
 
 void KoPADocumentModel::setProperties( KoShape* shape, const PropertyList &properties )
 {
     bool oldVisibleState = shape->isVisible();
-    bool oldLockedState = shape->isLocked();
+    bool oldLockedState = shape->isGeometryProtected();
 
     shape->setVisible( properties.at( 0 ).state.toBool() );
-    shape->setLocked( properties.at( 1 ).state.toBool() );
+    shape->setGeometryProtected( properties.at( 1 ).state.toBool() );
 
-    if( ( oldVisibleState != shape->isVisible() ) || ( oldLockedState != shape->isLocked() ) )
+    if( ( oldVisibleState != shape->isVisible() ) || ( oldLockedState != shape->isGeometryProtected() ) )
         shape->update();
 }
 
@@ -322,13 +323,13 @@ QImage KoPADocumentModel::createThumbnail( KoShape* shape, const QSize &thumbSiz
         else {
             size.setWidth( size.height() * ratio );
         }
-        QPixmap pixmap = page->thumbnail( size );
+        QPixmap pixmap = m_document->pageThumbnail( page, size );
         return pixmap.toImage();
     }
 
     KoShapeContainer * container = dynamic_cast<KoShapeContainer*>( shape );
     if( container )
-        shapes = container->iterator();
+        shapes = container->childShapes();
     else
         shapes.append( shape );
 
@@ -344,12 +345,12 @@ QImage KoPADocumentModel::createThumbnail( KoShape* shape, const QSize &thumbSiz
 
 KoShape * KoPADocumentModel::childFromIndex( KoShapeContainer *parent, int row ) const
 {
-    return parent->iterator().at(row);
+    return parent->childShapes().at(row);
 
     if( parent != m_lastContainer )
     {
         m_lastContainer = parent;
-        m_childs = parent->iterator();
+        m_childs = parent->childShapes();
         qSort( m_childs.begin(), m_childs.end(), KoShape::compareShapeZIndex );
     }
     return m_childs.at( row );
@@ -360,12 +361,12 @@ int KoPADocumentModel::indexFromChild( KoShapeContainer *parent, KoShape *child
     if ( !m_document )
         return 0;
 
-    return parent->iterator().indexOf( child );
+    return parent->childShapes().indexOf( child );
 
     if( parent != m_lastContainer )
     {
         m_lastContainer = parent;
-        m_childs = parent->iterator();
+        m_childs = parent->childShapes();
         qSort( m_childs.begin(), m_childs.end(), KoShape::compareShapeZIndex );
     }
     return m_childs.indexOf( child );
@@ -476,20 +477,8 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
             if ( row < 0 ) {
                 return false;
             }
-            QUndoCommand *command;
             KoPAPageBase *after = ( row != 0 ) ? m_document->pageByIndex( row - 1, false ) : 0;
-            // TODO: After string freeze is lifted, the command name can be changed to
-            //  "Move slides" and "Move pages" to reflect the actual behavior
-            if ( m_document->pageType() == KoPageApp::Slide ) {
-                command = new QUndoCommand( i18n( "Move slide") );
-            }
-            else {
-                command = new QUndoCommand( i18n( "Move page" ) );
-            }
-            foreach ( KoPAPageBase *page, pages ) {
-                KoPAPageMoveCommand *moveCommand = new KoPAPageMoveCommand( m_document, page, after, command );
-                after = page;
-            }
+            KoPAPageMoveCommand *command = new KoPAPageMoveCommand( m_document, pages, after );
             m_document->addCommand( command );
             kDebug(30010) << "KoPADocumentModel::dropMimeData parent = root, dropping page(s) as root, moving page(s)";
             return true;
@@ -517,7 +506,7 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
             cmd->setText( i18n("Reparent shapes") );
 
             foreach( KoShape * shape, toplevelShapes )
-                new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, cmd );
+                new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, QList<KoShape*>(), cmd );
 
             new KoShapeGroupCommand( group, toplevelShapes, cmd );
             KoCanvasController * canvasController = KoToolManager::instance()->activeCanvasController();
@@ -547,7 +536,7 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
                     }
 
                     clipped.append( shape->parent()->childClipped( shape ) );
-                    new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, cmd );
+                    new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, QList<KoShape*>(), cmd );
                 }
                 // shapes are dropped on a container, so add them to the container
                 new KoShapeGroupCommand( container, toplevelShapes, clipped, cmd );
