@@ -22,6 +22,7 @@
 #include "KoTextWriter.h"
 
 #include <QTextDocument>
+#include <QTextTable>
 
 #include "KoInlineObject.h"
 #include "KoInlineTextObjectManager.h"
@@ -33,31 +34,78 @@
 #include "KoTextDocumentLayout.h"
 #include "KoTextBlockData.h"
 #include "KoTextDocument.h"
+
 #include <KoShapeSavingContext.h>
 #include <KoXmlWriter.h>
 #include <KoGenStyle.h>
 #include <KoGenStyles.h>
 
-KoTextWriter::KoTextWriter(KoShapeSavingContext &m_context)
-    : m_context(m_context),
-      m_layout(0),
-      m_styleManager(0)
+#include <opendocument/KoTextSharedSavingData.h>
+#include <changetracker/KoChangeTracker.h>
+#include <changetracker/KoChangeTrackerElement.h>
+#include <changetracker/KoDeleteChangeMarker.h>
+#include <KoGenChange.h>
+#include <KoGenChanges.h>
+
+class KoTextWriter::Private
 {
-    m_writer = &m_context.xmlWriter();
+public:
+    explicit Private(KoShapeSavingContext &context)
+    : context(context),
+    sharedData(0),
+    writer(0),
+    layout(0),
+    styleManager(0),
+    changeTracker(0)
+    {
+        writer = &context.xmlWriter();
+    }
+
+    ~Private() {}
+
+    KoShapeSavingContext &context;
+    KoTextSharedSavingData *sharedData;
+    KoXmlWriter *writer;
+
+    KoTextDocumentLayout *layout;
+    KoStyleManager *styleManager;
+    KoChangeTracker *changeTracker;
+};
+
+KoTextWriter::KoTextWriter(KoShapeSavingContext &context)
+    : d(new Private(context))
+{
+    KoSharedSavingData *sharedData = context.sharedData(KOTEXT_SHARED_SAVING_ID);
+    if (sharedData) {
+        d->sharedData = dynamic_cast<KoTextSharedSavingData *>(sharedData);
+    }
+
+    if (!d->sharedData) {
+        d->sharedData = new KoTextSharedSavingData();
+        KoGenChanges *changes = new KoGenChanges();
+        d->sharedData->setGenChanges(*changes);
+        if (!sharedData) {
+            context.addSharedData(KOTEXT_SHARED_SAVING_ID, d->sharedData);
+        } else {
+            kWarning(32500) << "A different type of sharedData was found under the" << KOTEXT_SHARED_SAVING_ID;
+            Q_ASSERT(false);
+        }
+    }
 }
 
 KoTextWriter::~KoTextWriter()
 {
+    delete d;
 }
 
 QString KoTextWriter::saveParagraphStyle(const QTextBlock &block)
 {
-    KoParagraphStyle *defaultParagraphStyle = m_styleManager->defaultParagraphStyle();
+    KoParagraphStyle *defaultParagraphStyle = d->styleManager->defaultParagraphStyle();
 
     QTextBlockFormat blockFormat = block.blockFormat();
     QTextCharFormat charFormat = QTextCursor(block).blockCharFormat();
 
-    KoParagraphStyle *originalParagraphStyle = m_styleManager->paragraphStyle(blockFormat.intProperty(KoParagraphStyle::StyleId));
+    KoParagraphStyle *originalParagraphStyle = d->styleManager->paragraphStyle(blockFormat.intProperty(KoParagraphStyle::StyleId));
     if (!originalParagraphStyle)
         originalParagraphStyle = defaultParagraphStyle;
 
@@ -72,26 +120,26 @@ QString KoTextWriter::saveParagraphStyle(const QTextBlock &block)
         // therefore we would need to store that information in the saving context
         if (originalParagraphStyle != defaultParagraphStyle) {
             KoGenStyle style(KoGenStyle::StyleUser, "paragraph");
-            originalParagraphStyle->saveOdf(style, m_context.mainStyles());
-            generatedName = m_context.mainStyles().lookup(style, internalName, KoGenStyles::DontForceNumbering);
+            originalParagraphStyle->saveOdf(style, d->context.mainStyles());
+            generatedName = d->context.mainStyles().lookup(style, internalName, KoGenStyles::DontForceNumbering);
         }
     } else { // There are manual changes... We'll have to store them then
         KoGenStyle style(KoGenStyle::StyleAuto, "paragraph", internalName);
-        if (m_context.isSet(KoShapeSavingContext::AutoStyleInStyleXml))
+        if (d->context.isSet(KoShapeSavingContext::AutoStyleInStyleXml))
             style.setAutoStyleInStylesDotXml(true);
         if (originalParagraphStyle)
             paragStyle.removeDuplicates(*originalParagraphStyle);
-        paragStyle.saveOdf(style, m_context.mainStyles());
-        generatedName = m_context.mainStyles().lookup(style, "P");
+        paragStyle.saveOdf(style, d->context.mainStyles());
+        generatedName = d->context.mainStyles().lookup(style, "P");
     }
     return generatedName;
 }
 
 QString KoTextWriter::saveCharacterStyle(const QTextCharFormat &charFormat, const QTextCharFormat &blockCharFormat)
 {
-    KoCharacterStyle *defaultCharStyle = m_styleManager->defaultParagraphStyle()->characterStyle();
+    KoCharacterStyle *defaultCharStyle = d->styleManager->defaultParagraphStyle()->characterStyle();
 
-    KoCharacterStyle *originalCharStyle = m_styleManager->characterStyle(charFormat.intProperty(KoCharacterStyle::StyleId));
+    KoCharacterStyle *originalCharStyle = d->styleManager->characterStyle(charFormat.intProperty(KoCharacterStyle::StyleId));
     if (!originalCharStyle)
         originalCharStyle = defaultCharStyle;
 
@@ -108,17 +156,17 @@ QString KoTextWriter::saveCharacterStyle(const QTextCharFormat &charFormat, cons
             if (!charStyle.isEmpty()) {
                 KoGenStyle style(KoGenStyle::StyleUser, "text");
                 originalCharStyle->saveOdf(style);
-                generatedName = m_context.mainStyles().lookup(style, internalName, KoGenStyles::DontForceNumbering);
+                generatedName = d->context.mainStyles().lookup(style, internalName, KoGenStyles::DontForceNumbering);
             }
         }
     } else { // There are manual changes... We'll have to store them then
         KoGenStyle style(KoGenStyle::StyleAuto, "text", originalCharStyle != defaultCharStyle ? internalName : "" /*parent*/);
-        if (m_context.isSet(KoShapeSavingContext::AutoStyleInStyleXml))
+        if (d->context.isSet(KoShapeSavingContext::AutoStyleInStyleXml))
             style.setAutoStyleInStylesDotXml(true);
         charStyle.removeDuplicates(*originalCharStyle);
         if (!charStyle.isEmpty()) {
             charStyle.saveOdf(style);
-            generatedName = m_context.mainStyles().lookup(style, "T");
+            generatedName = d->context.mainStyles().lookup(style, "T");
         }
     }
 
@@ -145,7 +193,7 @@ QHash<QTextList *, QString> KoTextWriter::saveListStyles(QTextBlock block, int t
             bool automatic = listStyle->styleId() == 0;
             KoGenStyle style(automatic ? KoGenStyle::StyleListAuto : KoGenStyle::StyleList);
             listStyle->saveOdf(style);
-            QString generatedName = m_context.mainStyles().lookup(style, listStyle->name(), KoGenStyles::AllowDuplicates);
+            QString generatedName = d->context.mainStyles().lookup(style, listStyle->name(), KoGenStyles::AllowDuplicates);
             listStyles[textList] = generatedName;
             generatedLists.insert(list, generatedName);
         } else {
@@ -156,7 +204,7 @@ QHash<QTextList *, QString> KoTextWriter::saveListStyles(QTextBlock block, int t
             KoListStyle listStyle;
             listStyle.setLevelProperties(llp);
             listStyle.saveOdf(style);
-            QString generatedName = m_context.mainStyles().lookup(style, listStyle.name());
+            QString generatedName = d->context.mainStyles().lookup(style, listStyle.name());
             listStyles[textList] = generatedName;
         }
     }
@@ -165,21 +213,25 @@ QHash<QTextList *, QString> KoTextWriter::saveListStyles(QTextBlock block, int t
 
 void KoTextWriter::saveParagraph(const QTextBlock &block, int from, int to)
 {
+    QString changeName;
+
     QTextBlockFormat blockFormat = block.blockFormat();
-    int outlineLevel = blockFormat.intProperty(KoParagraphStyle::OutlineLevel);
+    const int outlineLevel = blockFormat.intProperty(KoParagraphStyle::OutlineLevel);
     if (outlineLevel > 0) {
-        m_writer->startElement("text:h", false);
-        m_writer->addAttribute("text:outline-level", outlineLevel);
-    } else 
-        m_writer->startElement("text:p", false);
+        d->writer->startElement("text:h", false);
+        d->writer->addAttribute("text:outline-level", outlineLevel);
+    } else {
+        d->writer->startElement("text:p", false);
+    }
 
     QString styleName = saveParagraphStyle(block);
     if (!styleName.isEmpty())
-        m_writer->addAttribute("text:style-name", styleName);
+        d->writer->addAttribute("text:style-name", styleName);
 
     // Write the fragments and their formats
     QTextCursor cursor(block);
     QTextCharFormat blockCharFormat = cursor.blockCharFormat();
+    QTextCharFormat previousCharFormat;
     QTextBlock::iterator it;
     for (it = block.begin(); !(it.atEnd()); ++it) {
         QTextFragment currentFragment = it.fragment();
@@ -189,54 +241,113 @@ void KoTextWriter::saveParagraph(const QTextBlock &block, int from, int to)
             break;
         if (currentFragment.isValid()) {
             QTextCharFormat charFormat = currentFragment.charFormat();
-            KoInlineObject *inlineObject = m_layout->inlineTextObjectManager()->inlineTextObject(charFormat);
+            QTextCharFormat compFormat = charFormat;
+            bool identical;
+            previousCharFormat.clearProperty(KoCharacterStyle::ChangeTrackerId);
+            compFormat.clearProperty(KoCharacterStyle::ChangeTrackerId);
+            if (previousCharFormat == compFormat)
+                identical = true;
+            else
+                identical = false;
+
+            if (d->changeTracker /*&& d->changeTracker->isEnabled()*/ && d->changeTracker->containsInlineChanges(charFormat)
+                    && d->changeTracker->elementById(charFormat.property(KoCharacterStyle::ChangeTrackerId).toInt())->isEnabled()) { //TODO uncomment changeTracker->isEnabled or implement another "security" measure to prevent saving changes at all
+                    if (d->changeTracker->elementById(charFormat.property(KoCharacterStyle::ChangeTrackerId).toInt())->getChangeType() == KoGenChange::deleteChange
+                        && dynamic_cast<KoDeleteChangeMarker*>(d->layout->inlineTextObjectManager()->inlineTextObject(charFormat))) {
+                        continue;
+                    }
+                KoGenChange change;
+                d->changeTracker->saveInlineChange(charFormat.property(KoCharacterStyle::ChangeTrackerId).toInt(), change);
+                changeName = d->sharedData->genChanges().insert(change);
+                d->writer->startElement("text:change-start", false);
+                d->writer->addAttribute("text:change-id", changeName);
+                d->writer->endElement();
+            }
+
+            KoInlineObject *inlineObject = d->layout->inlineTextObjectManager()->inlineTextObject(charFormat);
             if (currentFragment.length() == 1 && inlineObject
                     && currentFragment.text()[0].unicode() == QChar::ObjectReplacementCharacter) {
-                inlineObject->saveOdf(m_context);
+                inlineObject->saveOdf(d->context);
             } else {
                 QString styleName = saveCharacterStyle(charFormat, blockCharFormat);
                 if (charFormat.isAnchor()) {
-                    m_writer->startElement("text:a", false);
-                    m_writer->addAttribute("xlink:type", "simple");
-                    m_writer->addAttribute("xlink:href", charFormat.anchorHref());
-                } else if (!styleName.isEmpty()) {
-                    m_writer->startElement("text:span", false);
-                    m_writer->addAttribute("text:style-name", styleName);
+                    d->writer->startElement("text:a", false);
+                    d->writer->addAttribute("xlink:type", "simple");
+                    d->writer->addAttribute("xlink:href", charFormat.anchorHref());
+                } else if (!styleName.isEmpty() /*&& !identical*/) {
+                    d->writer->startElement("text:span", false);
+                    d->writer->addAttribute("text:style-name", styleName);
                 }
 
                 QString text = currentFragment.text();
                 int spanFrom = fragmentStart >= from ? 0 : from;
                 int spanTo = to == -1 ? fragmentEnd : (fragmentEnd > to ? to : fragmentEnd);
                 if (spanFrom != fragmentStart || spanTo != fragmentEnd) { // avoid mid, if possible
-                    m_writer->addTextSpan(text.mid(spanFrom - fragmentStart, spanTo - spanFrom));
+                    d->writer->addTextSpan(text.mid(spanFrom - fragmentStart, spanTo - spanFrom));
                 } else {
-                    m_writer->addTextSpan(text);
+                    d->writer->addTextSpan(text);
                 }
 
-                if (!styleName.isEmpty() || charFormat.isAnchor())
-                    m_writer->endElement();
+                if ((!styleName.isEmpty() /*&& !identical*/) || charFormat.isAnchor())
+                    d->writer->endElement();
             } // if (inlineObject)
-        } // if (fragment.valid())
-    } // foeach(fragment)
 
-    m_writer->endElement();
+            if (!changeName.isEmpty()) {
+                d->writer->startElement("text:change-end", false);
+                d->writer->addAttribute("text:change-id",changeName);
+                d->writer->endElement();
+                changeName=QString();
+            }
+            previousCharFormat = charFormat;
+        } // if (fragment.valid())
+    } // foreach(fragment)
+
+    d->writer->endElement();
 }
 
-void KoTextWriter::write(QTextDocument *document, int from, int to)
+void KoTextWriter::saveTable (QTextTable *table, QHash<QTextList *, QString> &listStyles)
 {
-    m_styleManager = KoTextDocument(document).styleManager();
-    m_layout = dynamic_cast<KoTextDocumentLayout*>(document->documentLayout());
-    Q_ASSERT(m_layout);
-    Q_ASSERT(m_layout->inlineTextObjectManager());
+    d->writer->startElement("table:table");
+    for (int c = 0 ; c < table->columns() ; c++) {
+        d->writer->startElement("table:table-column");
+        d->writer->endElement(); // table:table-column
+    }
+    for (int r = 0 ; r < table->rows() ; r++) {
+        d->writer->startElement("table:table-row");
+        for (int c = 0 ; c < table->columns() ; c++) {
+            QTextTableCell cell = table->cellAt(r, c);
+            if ((cell.row() == r) && (cell.column() == c)) {
+                d->writer->startElement("table:table-cell");
+                d->writer->addAttribute("rowSpan", cell.rowSpan());
+                d->writer->addAttribute("columnSpan", cell.columnSpan());
+                writeBlocks(table->document(), cell.firstPosition(), cell.lastPosition(), listStyles, table);
+            } else {
+                d->writer->startElement("table:covered-table-cell");
+            }
+            d->writer->endElement(); // table:table-cell
+        }
+        d->writer->endElement(); // table:table-row
+    }
+    d->writer->endElement(); // table:table
+}
 
-    QTextBlock block = document->findBlock(from);
+void KoTextWriter::writeBlocks(QTextDocument *document, int from, int to, QHash<QTextList *, QString> &listStyles, QTextTable *currentTable)
+{
     KoTextDocument textDocument(document);
-
-    QHash<QTextList *, QString> listStyles = saveListStyles(block, to);
+    QTextBlock block = document->findBlock(from);
     QList<QTextList*> textLists; // Store the current lists being stored.
     KoList *currentList = 0;
 
     while (block.isValid() && ((to == -1) || (block.position() <= to))) {
+        QTextCursor cursor(block);
+        if (cursor.currentTable() != currentTable) {
+            // Call the code to save the table....
+            saveTable(cursor.currentTable(), listStyles);
+            // We skip to the end of the table.
+            block = cursor.currentTable()->lastCursorPosition().block();
+            block = block.next();
+            continue;
+        }
         QTextBlockFormat blockFormat = block.blockFormat();
         QTextList *textList = block.textList();
         int headingLevel = 0, numberedParagraphLevel = 0;
@@ -250,51 +361,51 @@ void KoTextWriter::write(QTextDocument *document, int from, int to)
                 if (currentList != list) {
                     while (!textLists.isEmpty()) {
                         textLists.removeLast();
-                        m_writer->endElement(); // </text:list>
+                        d->writer->endElement(); // </text:list>
                         if (!textLists.isEmpty()) {
-                            m_writer->endElement(); // </text:list-element>
+                            d->writer->endElement(); // </text:list-element>
                         }
                     }
                     currentList = list;
                 } else if (!textLists.isEmpty()) // sublists should be written within a list-item
-                    m_writer->startElement("text:list-item", false);
-                m_writer->startElement("text:list", false);
-                m_writer->addAttribute("text:style-name", listStyles[textList]);
+                    d->writer->startElement("text:list-item", false);
+                d->writer->startElement("text:list", false);
+                d->writer->addAttribute("text:style-name", listStyles[textList]);
                 if (textList->format().hasProperty(KoListStyle::ContinueNumbering))
-                    m_writer->addAttribute("text:continue-numbering",
+                    d->writer->addAttribute("text:continue-numbering",
                                          textList->format().boolProperty(KoListStyle::ContinueNumbering) ? "true" : "false");
                 textLists.append(textList);
             } else if (textList != textLists.last()) {
                 while ((!textLists.isEmpty()) && (textList != textLists.last())) {
                     textLists.removeLast();
-                    m_writer->endElement(); // </text:list>
-                    m_writer->endElement(); // </text:list-element>
+                    d->writer->endElement(); // </text:list>
+                    d->writer->endElement(); // </text:list-element>
                 }
             }
             const bool listHeader = blockFormat.boolProperty(KoParagraphStyle::IsListHeader)
                                     || blockFormat.boolProperty(KoParagraphStyle::UnnumberedListItem);
-            m_writer->startElement(listHeader ? "text:list-header" : "text:list-item", false);
+                d->writer->startElement(listHeader ? "text:list-header" : "text:list-item", false);
             if (KoListStyle::isNumberingStyle(textList->format().style())) {
                 if (KoTextBlockData *blockData = dynamic_cast<KoTextBlockData *>(block.userData())) {
-                    m_writer->startElement("text:number", false);
-                    m_writer->addTextSpan(blockData->counterText());
-                    m_writer->endElement();
+                    d->writer->startElement("text:number", false);
+                    d->writer->addTextSpan(blockData->counterText());
+                    d->writer->endElement();
                 }
             }
         } else {
             // Close any remaining list...
             while (!textLists.isEmpty()) {
                 textLists.removeLast();
-                m_writer->endElement(); // </text:list>
+                d->writer->endElement(); // </text:list>
                 if (!textLists.isEmpty()) {
-                    m_writer->endElement(); // </text:list-element>
+                    d->writer->endElement(); // </text:list-element>
                 }
             }
 
             if (textList && numberedParagraphLevel) {
-                m_writer->startElement("text:numbered-paragraph", false);
-                m_writer->addAttribute("text:level", numberedParagraphLevel);
-                m_writer->addAttribute("text:style-name", listStyles.value(textList));
+                d->writer->startElement("text:numbered-paragraph", false);
+                d->writer->addAttribute("text:level", numberedParagraphLevel);
+                d->writer->addAttribute("text:style-name", listStyles.value(textList));
             }
         }
 
@@ -315,7 +426,7 @@ void KoTextWriter::write(QTextDocument *document, int from, int to)
 
         // We must check if we need to close a previously-opened text:list node.
         if ((block.textList() && !headingLevel) || numberedParagraphLevel)
-            m_writer->endElement();
+            d->writer->endElement();
 
         block = block.next();
     } // while
@@ -323,10 +434,25 @@ void KoTextWriter::write(QTextDocument *document, int from, int to)
     // Close any remaining lists
     while (!textLists.isEmpty()) {
         textLists.removeLast();
-        m_writer->endElement(); // </text:list>
+        d->writer->endElement(); // </text:list>
         if (!textLists.isEmpty()) {
-            m_writer->endElement(); // </text:list-element>
+            d->writer->endElement(); // </text:list-element>
         }
     }
 }
 
+void KoTextWriter::write(QTextDocument *document, int from, int to)
+{
+    d->styleManager = KoTextDocument(document).styleManager();
+    d->layout = qobject_cast<KoTextDocumentLayout*>(document->documentLayout());
+
+    d->changeTracker = KoTextDocument(document).changeTracker();
+
+    Q_ASSERT(d->layout);
+    Q_ASSERT(d->layout->inlineTextObjectManager());
+
+    QTextBlock block = document->findBlock(from);
+
+    QHash<QTextList *, QString> listStyles = saveListStyles(block, to);
+    writeBlocks(document, from, to, listStyles);
+}
