@@ -26,9 +26,7 @@
 #include "KoDockRegistry.h"
 #include "KoDocument.h"
 #include "KoMainWindow.h"
-#include "KoFrame.h"
 #include "KoViewAdaptor.h"
-#include "KoDocumentChild.h"
 #include "KoDockFactory.h"
 
 #include <kactioncollection.h>
@@ -38,13 +36,12 @@
 #include <kparts/event.h>
 #include <kstatusbar.h>
 #include <kdebug.h>
-#include <kconfiggroup.h>
 #include <QTimer>
 #include <QtGui/QDockWidget>
 #include <QToolBar>
 #include <QApplication>
-#include <Q3ValueList>
-#include <kdeversion.h>
+#include <QList>
+
 
 //static
 QString KoView::newObjectName()
@@ -58,30 +55,23 @@ class KoViewPrivate
 {
 public:
     KoViewPrivate() {
-        m_inOperation = false;
-        m_zoom = 1.0;
-        m_children.setAutoDelete(true);
-        m_manager = 0L;
-        m_tempActiveWidget = 0L;
-//     m_dcopObject = 0;
-        m_registered = false;
-        m_documentDeleted = false;
-        m_viewBar = 0L;
+        manager = 0;
+        tempActiveWidget = 0;
+        registered = false;
+        documentDeleted = false;
+        viewBar = 0;
     }
     ~KoViewPrivate() {
     }
 
-    QPointer<KoDocument> m_doc; // our KoDocument
-    QPointer<KParts::PartManager> m_manager;
-    qreal m_zoom;
-    Q3PtrList<KoViewChild> m_children;
-    QWidget *m_tempActiveWidget;
-//   KoViewIface *m_dcopObject;
-    bool m_registered;  // are we registered at the part manager?
-    bool m_documentDeleted; // true when m_doc gets deleted [can't use m_doc==0
+    QPointer<KoDocument> document; // our KoDocument
+    QPointer<KParts::PartManager> manager;
+    QWidget *tempActiveWidget;
+    bool registered;  // are we registered at the part manager?
+    bool documentDeleted; // true when document gets deleted [can't use document==0
     // since this only happens in ~QObject, and views
     // get deleted by ~KoDocument].
-    QTimer *m_scrollTimer;
+    QTimer *scrollTimer;
 
 
     // Hmm sorry for polluting the private class with such a big inner class.
@@ -94,6 +84,14 @@ public:
         StatusBarItem(QWidget * widget, int stretch, bool permanent)
                 : m_widget(widget), m_stretch(stretch), m_permanent(permanent), m_visible(false) {}
 
+        bool operator==(const StatusBarItem& rhs) {
+            return m_widget == rhs.m_widget;
+        }
+
+        bool operator!=(const StatusBarItem& rhs) {
+            return m_widget != rhs.m_widget;
+        }
+
         QWidget * widget() const {
             return m_widget;
         }
@@ -123,9 +121,10 @@ public:
         bool m_permanent;
         bool m_visible;  // true when the item has been added to the statusbar
     };
-    Q3ValueList<StatusBarItem> m_statusBarItems; // Our statusbar items
-    bool m_inOperation; //in the middle of an operation (no screen refreshing)?
-    QToolBar* m_viewBar;
+
+    QList<StatusBarItem> statusBarItems; // Our statusbar items
+    bool inOperation; //in the middle of an operation (no screen refreshing)?
+    QToolBar* viewBar;
 };
 
 KoView::KoView(KoDocument *document, QWidget *parent)
@@ -140,23 +139,11 @@ KoView::KoView(KoDocument *document, QWidget *parent)
     QDBusConnection::sessionBus().registerObject('/' + objectName(), this);
 
     //kDebug(30003) <<"KoView::KoView" << this;
-    d->m_doc = document;
+    d->document = document;
     KParts::PartBase::setPartObject(this);
 
     setFocusPolicy(Qt::StrongFocus);
 
-    setMouseTracking(true);
-
-    connect(d->m_doc, SIGNAL(childChanged(KoDocumentChild *)),
-            this, SLOT(slotChildChanged(KoDocumentChild *)));
-
-    connect(d->m_doc, SIGNAL(beginOperation()),
-            this, SLOT(beginOperation()));
-
-    connect(d->m_doc, SIGNAL(endOperation()),
-            this, SLOT(endOperation()));
-
-
     setupGlobalActions();
 
     KStatusBar * sb = statusBar();
@@ -172,15 +159,15 @@ KoView::KoView(KoDocument *document, QWidget *parent)
                 this, SLOT(slotClearStatusText()));
 #endif
 
-        connect(d->m_doc, SIGNAL(statusBarMessage(const QString&)),
+        connect(d->document, SIGNAL(statusBarMessage(const QString&)),
                 this, SLOT(slotActionStatusText(const QString&)));
-        connect(d->m_doc, SIGNAL(clearStatusBarMessage()),
+        connect(d->document, SIGNAL(clearStatusBarMessage()),
                 this, SLOT(slotClearStatusText()));
     }
-    d->m_doc->setCurrent();
+    d->document->setCurrent();
 
-    d->m_scrollTimer = new QTimer(this);
-    connect(d->m_scrollTimer, SIGNAL(timeout()), this, SLOT(slotAutoScroll()));
+    d->scrollTimer = new QTimer(this);
+    connect(d->scrollTimer, SIGNAL(timeout()), this, SLOT(slotAutoScroll()));
 
     // add all plugins.
     foreach(const QString & docker, KoDockRegistry::instance()->keys()) {
@@ -196,14 +183,14 @@ KoView::KoView(KoDocument *document, QWidget *parent)
 KoView::~KoView()
 {
     kDebug(30003) << "KoView::~KoView" << this;
-    delete d->m_scrollTimer;
+    delete d->scrollTimer;
 //   delete d->m_dcopObject;
-    if (!d->m_documentDeleted) {
+    if (!d->documentDeleted) {
         if (koDocument() && !koDocument()->isSingleViewMode()) {
-            if (d->m_manager && d->m_registered)   // if we aren't registered we mustn't unregister :)
-                d->m_manager->removePart(koDocument());
-            d->m_doc->removeView(this);
-            d->m_doc->setCurrent(false);
+            if (d->manager && d->registered)   // if we aren't registered we mustn't unregister :)
+                d->manager->removePart(koDocument());
+            d->document->removeView(this);
+            d->document->setCurrent(false);
         }
     }
     delete d;
@@ -211,38 +198,33 @@ KoView::~KoView()
 
 KoDocument *KoView::koDocument() const
 {
-    return d->m_doc;
+    return d->document;
 }
 
 void KoView::setDocumentDeleted()
 {
-    d->m_documentDeleted = true;
+    d->documentDeleted = true;
 }
 
 bool KoView::documentDeleted() const
 {
-    return d->m_documentDeleted;
-}
-
-bool KoView::hasDocumentInWindow(KoDocument *doc)
-{
-    return child(doc) != 0L;
+    return d->documentDeleted;
 }
 
 void KoView::setPartManager(KParts::PartManager *manager)
 {
-    d->m_manager = manager;
+    d->manager = manager;
     if (!koDocument()->isSingleViewMode() &&
             !manager->parts().contains(koDocument())) {  // is there another view registered?
-        d->m_registered = true; // no, so we have to register now and ungregister again in the DTOR
+        d->registered = true; // no, so we have to register now and ungregister again in the DTOR
         manager->addPart(koDocument(), false);
     } else
-        d->m_registered = false;  // There is already another view registered for that part...
+        d->registered = false;  // There is already another view registered for that part...
 }
 
 KParts::PartManager *KoView::partManager() const
 {
-    return d->m_manager;
+    return d->manager;
 }
 
 QAction *KoView::action(const QDomElement &element) const
@@ -253,7 +235,21 @@ QAction *KoView::action(const QDomElement &element) const
     QAction *act = KXMLGUIClient::action(name.toUtf8());
 
     if (!act)
-        act = d->m_doc->KXMLGUIClient::action(name.toUtf8());
+        act = d->document->KXMLGUIClient::action(name.toUtf8());
+
+    // last resort, try to get action from the main window if there is one
+    if (!act && shell())
+        act = shell()->actionCollection()->action(name);
+
+    return act;
+}
+
+QAction *KoView::action(const char* name) const
+{
+    QAction *act = KXMLGUIClient::action(name);
+
+    if (!act)
+        act = d->document->KXMLGUIClient::action(name);
 
     // last resort, try to get action from the main window if there is one
     if (!act && shell())
@@ -264,31 +260,8 @@ QAction *KoView::action(const QDomElement &element) const
 
 KoDocument *KoView::hitTest(const QPoint &viewPos)
 {
-    KoViewChild *viewChild;
-
-    QPoint pos = reverseViewTransformations(viewPos);
-
-    KoDocumentChild *docChild = selectedChild();
-    if (docChild) {
-        if ((viewChild = child(docChild->document()))) {
-            if (viewChild->frameRegion().contains(pos))
-                return 0;
-        } else
-            if (docChild->frameRegion().contains(pos))
-                return 0;
-    }
-
-    docChild = activeChild();
-    if (docChild) {
-        if ((viewChild = child(docChild->document()))) {
-            if (viewChild->frameRegion().contains(pos))
-                return 0;
-        } else
-            if (docChild->frameRegion().contains(pos))
-                return 0;
-    }
-
-    return koDocument()->hitTest(pos, this);
+    Q_UNUSED(viewPos);
+    return koDocument(); // we no longer have child documents
 }
 
 int KoView::leftBorder() const
@@ -311,17 +284,6 @@ int KoView::bottomBorder() const
     return 0;
 }
 
-void KoView::setZoom(qreal zoom)
-{
-    d->m_zoom = zoom;
-    update();
-}
-
-qreal KoView::zoom() const
-{
-    return d->m_zoom;
-}
-
 QWidget *KoView::canvas() const
 {
     //dfaure: since the view plays two roles in this method (the const means "you can modify the canvas
@@ -339,10 +301,6 @@ int KoView::canvasYOffset() const
     return 0;
 }
 
-void KoView::canvasAddChild(KoViewChild *)
-{
-}
-
 void KoView::customEvent(QEvent *ev)
 {
     if (KParts::PartActivateEvent::test(ev))
@@ -355,56 +313,12 @@ void KoView::customEvent(QEvent *ev)
 
 void KoView::partActivateEvent(KParts::PartActivateEvent *event)
 {
-    if (event->part() != (KParts::Part *)koDocument()) {
-        Q_ASSERT(event->part()->inherits("KoDocument"));
-
-        KoDocumentChild *child = koDocument()->child((KoDocument *)event->part());
-        if (child && event->activated()) {
-            if (child->isRectangle()) {
-                KoViewChild *viewChild = new KoViewChild(child, this);
-                d->m_children.append(viewChild);
-
-                QApplication::setOverrideCursor(Qt::WaitCursor);
-                // This is the long operation (due to toolbar layout stuff)
-                d->m_manager->setActivePart(child->document(), viewChild->frame()->view());
-                QApplication::restoreOverrideCursor();
-
-                // Now we can move the frame to the right place
-                viewChild->setInitialFrameGeometry();
-            } else {
-                emit regionInvalidated(child->frameRegion(matrix()), true);
-            }
-            emit childActivated(child);
-        } else if (child) {
-            emit regionInvalidated(child->frameRegion(matrix()), true);
-            emit childDeactivated(child);
-        } else
-            emit invalidated();
-    } else
-        emit activated(event->activated());
+    emit activated(event->activated());
 }
 
 void KoView::partSelectEvent(KParts::PartSelectEvent *event)
 {
-    if (event->part() != (KParts::Part *)koDocument()) {
-        Q_ASSERT(event->part()->inherits("KoDocument"));
-
-        KoDocumentChild *child = koDocument()->child((KoDocument *)event->part());
-
-        if (child && event->selected()) {
-            QRegion r = child->frameRegion(matrix());
-            r.translate(- canvasXOffset(), - canvasYOffset());
-            emit regionInvalidated(r, true);
-            emit childSelected(child);
-        } else if (child) {
-            QRegion r = child->frameRegion(matrix());
-            r.translate(- canvasXOffset(), - canvasYOffset());
-            emit regionInvalidated(r, true);
-            emit childUnselected(child);
-        } else
-            emit invalidated();
-    } else
-        emit selected(event->selected());
+    emit selected(event->selected());
 }
 
 void KoView::guiActivateEvent(KParts::GUIActivateEvent * ev)
@@ -417,77 +331,52 @@ void KoView::showAllStatusBarItems(bool show)
     KStatusBar * sb = statusBar();
     if (!sb)
         return;
-    Q3ValueListIterator<KoViewPrivate::StatusBarItem> it = d->m_statusBarItems.begin();
-    for (; it != d->m_statusBarItems.end() ; ++it)
-        if (show)
-            (*it).ensureItemShown(sb);
-        else
-            (*it).ensureItemHidden(sb);
+    int itemCount = d->statusBarItems.count();
+    for (int i = 0; i < itemCount; ++i) {
+        KoViewPrivate::StatusBarItem &sbItem = d->statusBarItems[i];
+        if (show) {
+            sbItem.ensureItemShown(sb);
+        } else {
+            sbItem.ensureItemHidden(sb);
+        }
+    }
 }
 
-
 void KoView::addStatusBarItem(QWidget * widget, int stretch, bool permanent)
 {
     KoViewPrivate::StatusBarItem item(widget, stretch, permanent);
-    d->m_statusBarItems.append(item);
-    Q3ValueListIterator<KoViewPrivate::StatusBarItem> it = d->m_statusBarItems.fromLast();
     KStatusBar * sb = statusBar();
-    if (sb)
-        (*it).ensureItemShown(sb);
+    if (sb) {
+        item.ensureItemShown(sb);
+    }
+    d->statusBarItems.append(item);
 }
 
-void KoView::removeStatusBarItem(QWidget * widget)
+void KoView::removeStatusBarItem(QWidget *widget)
 {
-    KStatusBar * sb = statusBar();
-    Q3ValueListIterator<KoViewPrivate::StatusBarItem> it = d->m_statusBarItems.begin();
-    for (; it != d->m_statusBarItems.end() ; ++it)
-        if ((*it).widget() == widget) {
-            if (sb)
-                (*it).ensureItemHidden(sb);
-            d->m_statusBarItems.remove(it);
+    KStatusBar *sb = statusBar();
+
+    int itemCount = d->statusBarItems.count();
+    for (int i = itemCount-1; i >= 0; --i) {
+        KoViewPrivate::StatusBarItem &sbItem = d->statusBarItems[i];
+        if (sbItem.widget() == widget) {
+            if (sb) {
+                sbItem.ensureItemHidden(sb);
+            }
+            d->statusBarItems.removeOne(sbItem);
             break;
         }
-    if (it == d->m_statusBarItems.end())
-        kWarning() << "KoView::removeStatusBarItem. Widget not found : " << widget;
-}
-
-
-
-
-KoDocumentChild *KoView::selectedChild()
-{
-    if (!d->m_manager)
-        return 0L;
-
-    KParts::Part *selectedPart = d->m_manager->selectedPart();
-
-    if (!selectedPart || !selectedPart->inherits("KoDocument"))
-        return 0L;
-
-    return koDocument()->child((KoDocument *)selectedPart);
-}
-
-KoDocumentChild *KoView::activeChild()
-{
-    if (!d->m_manager)
-        return 0L;
-
-    KParts::Part *activePart = d->m_manager->activePart();
-
-    if (!activePart || !activePart->inherits("KoDocument"))
-        return 0L;
-
-    return koDocument()->child((KoDocument *)activePart);
+    }
 }
 
 void KoView::enableAutoScroll()
 {
-    d->m_scrollTimer->start(50);
+    d->scrollTimer->start(50);
 }
 
 void KoView::disableAutoScroll()
 {
-    d->m_scrollTimer->stop();
+    d->scrollTimer->stop();
 }
 
 void KoView::paintEverything(QPainter &painter, const QRect &rect)
@@ -495,95 +384,6 @@ void KoView::paintEverything(QPainter &painter, const QRect &rect)
     koDocument()->paintEverything(painter, rect, this);
 }
 
-KoViewChild *KoView::child(KoView *view)
-{
-    Q3PtrListIterator<KoViewChild> it(d->m_children);
-    for (; it.current(); ++it)
-        if (it.current()->frame()->view() == view)
-            return it.current();
-
-    return 0L;
-}
-
-KoViewChild *KoView::child(KoDocument *doc)
-{
-    Q3PtrListIterator<KoViewChild> it(d->m_children);
-    for (; it.current(); ++it)
-        if (it.current()->documentChild()->document() == doc)
-            return it.current();
-
-    return 0L;
-}
-
-QMatrix KoView::matrix() const
-{
-    QMatrix m;
-    m.scale(zoom(), zoom());
-    //m.translate(  canvasXOffset() ,  canvasYOffset() );
-    return m;
-}
-
-void KoView::slotChildActivated(bool a)
-{
-    // Only interested in deactivate events
-    if (a)
-        return;
-
-    KoViewChild* ch = child((KoView*)sender());
-    if (!ch)
-        return;
-
-    KoView* view = ch->frame()->view();
-
-    QWidget *activeWidget = view->d->m_tempActiveWidget;
-
-    if (d->m_manager->activeWidget())
-        activeWidget = d->m_manager->activeWidget();
-
-    if (!activeWidget || !activeWidget->inherits("KoView"))
-        return;
-
-    // Is the new active view a child of this one ?
-    // In this case we may not delete!
-    //  QObject *n = d->m_manager->activeWidget();
-    QObject *n = activeWidget;
-    while (n)
-        if (n == (QObject *)view)
-            return;
-        else
-            n = n->parent();
-
-
-    d->m_tempActiveWidget = activeWidget;
-    QApplication::setOverrideCursor(Qt::WaitCursor);
-    d->m_manager->setActivePart(0L);
-
-    QPointer<KoDocumentChild> docChild = ch->documentChild();
-    QPointer<KoFrame> chFrame = ch->frame();
-    if (docChild && chFrame && chFrame->view()) {
-        docChild->setContentsPos(chFrame->view()->canvasXOffset(),
-                                 chFrame->view()->canvasYOffset());
-        docChild->document()->setViewBuildDocument(chFrame->view(), chFrame->view()->xmlguiBuildDocument());
-    }
-
-    d->m_children.remove(ch);
-
-    d->m_manager->addPart(docChild->document(), false);   // the destruction of the view removed the part from the partmanager. re-add it :)
-
-    QApplication::restoreOverrideCursor();
-
-    // #### HACK
-    // We want to delete as many views as possible and this
-    // trick is used to go upwards in the view-tree.
-    emit activated(false);
-}
-
-void KoView::slotChildChanged(KoDocumentChild *child)
-{
-    QRegion region(child->oldPointArray(matrix()));
-    emit regionInvalidated(child->frameRegion(matrix(), true).unite(region), true);
-}
-
 int KoView::autoScrollAcceleration(int offset) const
 {
     if (offset < 40)
@@ -631,43 +431,23 @@ KoPrintJob * KoView::createPrintJob()
     return 0;
 }
 
-
 void KoView::setupGlobalActions()
 {
-    actionNewView  = new KAction(KIcon("window-new"), i18n("&New View"), this);
+    KAction *actionNewView  = new KAction(KIcon("window-new"), i18n("&New View"), this);
     actionCollection()->addAction("view_newview", actionNewView);
     connect(actionNewView, SIGNAL(triggered(bool)), this, SLOT(newView()));
 }
 
 void KoView::newView()
 {
-    Q_ASSERT((d != 0L && d->m_doc));
+    Q_ASSERT((d != 0 && d->document));
 
-    KoDocument *thisDocument = d->m_doc;
+    KoDocument *thisDocument = d->document;
     KoMainWindow *shell = new KoMainWindow(thisDocument->componentData());
     shell->setRootDocument(thisDocument);
     shell->show();
 }
 
-bool KoView::isInOperation() const
-{
-    return d->m_inOperation;
-}
-
-void KoView::beginOperation()
-{
-    d->m_inOperation = true;
-    canvas()->setUpdatesEnabled(false);
-}
-
-void KoView::endOperation()
-{
-    canvas()->setUpdatesEnabled(true);
-    d->m_inOperation = false;
-
-//   canvas()->update();
-}
-
 KoDockerManager * KoView::dockerManager() const
 {
     KoMainWindow *mw = shell();
@@ -695,7 +475,7 @@ KXmlGuiWindow * KoView::mainWindow() const
 KStatusBar * KoView::statusBar() const
 {
     KoMainWindow *mw = shell();
-    return mw ? mw->statusBar() : 0L;
+    return mw ? mw->statusBar() : 0;
 }
 
 void KoView::slotActionStatusText(const QString &text)
@@ -736,49 +516,32 @@ void KoView::restoreDockWidget(QDockWidget *dock)
         shell()->restoreDockWidget(dock);
 }
 
-QPoint KoView::applyViewTransformations(const QPoint& p) const
-{
-    return QPoint(qRound(p.x() * zoom()), qRound(p.y() * zoom()));
-}
-
-QPoint KoView::reverseViewTransformations(const QPoint& v) const
-{
-    return QPoint(qRound(v.x() / zoom()), qRound(v.y() / zoom()));
-}
-
-QRect KoView::applyViewTransformations(const QRect& r) const
-{
-    return QRect(applyViewTransformations(r.topLeft()),
-                 applyViewTransformations(r.bottomRight()));
-}
-
-QRect KoView::reverseViewTransformations(const QRect& r) const
-{
-    return QRect(reverseViewTransformations(r.topLeft()),
-                 reverseViewTransformations(r.bottomRight()));
-}
-
 QToolBar* KoView::viewBar()
 {
-    if (!d->m_viewBar) {
-        d->m_viewBar = new QToolBar(statusBar());
-        addStatusBarItem(d->m_viewBar, 0 , true);
+    if (!d->viewBar) {
+        d->viewBar = new QToolBar(statusBar());
+        addStatusBarItem(d->viewBar, 0 , true);
     }
 
-    return d->m_viewBar;
+    return d->viewBar;
 }
 
 QList<QAction*> KoView::createChangeUnitActions()
 {
+    QActionGroup *unitGroup = new QActionGroup(this);
     QList<QAction*> answer;
-    answer.append(new UnitChangeAction(KoUnit::Millimeter, this, d->m_doc));
-    answer.append(new UnitChangeAction(KoUnit::Centimeter, this, d->m_doc));
-    answer.append(new UnitChangeAction(KoUnit::Decimeter, this, d->m_doc));
-    answer.append(new UnitChangeAction(KoUnit::Inch, this, d->m_doc));
-    answer.append(new UnitChangeAction(KoUnit::Pica, this, d->m_doc));
-    answer.append(new UnitChangeAction(KoUnit::Cicero, this, d->m_doc));
-    answer.append(new UnitChangeAction(KoUnit::Point, this, d->m_doc));
+    answer.append(new UnitChangeAction(KoUnit::Millimeter, unitGroup, d->document));
+    answer.append(new UnitChangeAction(KoUnit::Centimeter, unitGroup, d->document));
+    answer.append(new UnitChangeAction(KoUnit::Decimeter, unitGroup, d->document));
+    answer.append(new UnitChangeAction(KoUnit::Inch, unitGroup, d->document));
+    answer.append(new UnitChangeAction(KoUnit::Pica, unitGroup, d->document));
+    answer.append(new UnitChangeAction(KoUnit::Cicero, unitGroup, d->document));
+    answer.append(new UnitChangeAction(KoUnit::Point, unitGroup, d->document));
 
+    const int currentUnit = d->document.data()->unit().indexInList();
+    Q_ASSERT(currentUnit < answer.count());
+    if (currentUnit >= 0)
+        answer.value(currentUnit)->setChecked(true);
     return answer;
 }
 
