@@ -3,6 +3,7 @@
    Copyright (C) 2007-2008 Thorsten Zachmann <zachmann@kde.org>
    Copyright (C) 2009 Thomas Zander <zander@kde.org>
    Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
+   Copyright (C) 2009 Inge Wallin <inge@lysator.liu.se>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -36,6 +37,7 @@ static const struct {
 } styleData[] = {
     { KoGenStyle::StyleUser, "style:style", "style:paragraph-properties", false  },
     { KoGenStyle::StyleText, "style:style", "style:text-properties", false  },
+    { KoGenStyle::StyleChart, "style:style", "style:chart-properties", false  },
     { KoGenStyle::StyleTableColumn, "style:style", "style:table-column-properties", false  },
     { KoGenStyle::StyleTableRow, "style:style", "style:table-row-properties", false  },
     { KoGenStyle::StyleTableCell, "style:style", "style:table-cell-properties", false  },
@@ -62,6 +64,7 @@ static const struct {
     { KoGenStyle::StyleAuto, "style:style", "style:paragraph-properties", false  },
     { KoGenStyle::StyleTextAuto, "style:style", "style:text-properties", false  },
     { KoGenStyle::StyleGraphicAuto, "style:style", "style:graphic-properties", false  },
+    { KoGenStyle::StyleChartAuto, "style:style", "style:chart-properties", false  },
     { KoGenStyle::StylePresentationAuto, "style:style", "style:graphic-properties", false  },
     { KoGenStyle::StyleDrawingPage, "style:style", "style:drawing-page-properties", false  },
     { KoGenStyle::StyleAutoTable, "style:style", "style:table-properties", false  },
@@ -84,6 +87,16 @@ static const struct {
 
 static const unsigned int numAutoStyleData = sizeof(autoStyleData) / sizeof(*autoStyleData);
 
+static void addRawOdfStyles(const QByteArray& xml, QByteArray& styles)
+{
+    if (xml.isEmpty())
+        return;
+    if (!styles.isEmpty() && !styles.endsWith('\n') && !xml.startsWith('\n')) {
+        styles.append('\n');
+    }
+    styles.append(xml);
+}
+
 class KoGenStyles::Private
 {
 public:
@@ -114,6 +127,15 @@ public:
     };
     QHash<QString, RelationTarget> relations; // key is the name of the source style
 
+    QByteArray& rawOdfAutomaticStyles(bool stylesDotXml) {
+        return stylesDotXml ? rawOdfAutomaticStyles_stylesDotXml : rawOdfAutomaticStyles_contentDotXml;
+    }
+
+    QByteArray rawOdfDocumentStyles;
+    QByteArray rawOdfAutomaticStyles_stylesDotXml;
+    QByteArray rawOdfAutomaticStyles_contentDotXml;
+    QByteArray rawOdfMasterStyles;
+
     KoGenStyles *q;
 };
 
@@ -331,6 +353,11 @@ void KoGenStyles::saveOdfAutomaticStyles(KoXmlWriter* xmlWriter, bool stylesDotX
         }
     }
 
+    const QByteArray &rawOdfAutomaticStyles = d->rawOdfAutomaticStyles(stylesDotXml);
+    if (!rawOdfAutomaticStyles.isEmpty()) {
+        xmlWriter->addCompleteElement(rawOdfAutomaticStyles.constData());
+    }
+
     xmlWriter->endElement(); // office:automatic-styles
 }
 
@@ -364,6 +391,10 @@ void KoGenStyles::saveOdfDocumentStyles(KoXmlWriter* xmlWriter) const
         }
     }
 
+    if (!d->rawOdfDocumentStyles.isEmpty()) {
+        xmlWriter->addCompleteElement(d->rawOdfDocumentStyles.constData());
+    }
+
     xmlWriter->endElement(); // office:styles
 }
 
@@ -377,7 +408,26 @@ void KoGenStyles::saveOdfMasterStyles(KoXmlWriter* xmlWriter) const
         (*it).style->writeStyle(xmlWriter, *this, "style:master-page", (*it).name, 0);
     }
 
-    xmlWriter->endElement(); // office:styles
+    if (!d->rawOdfMasterStyles.isEmpty()) {
+        xmlWriter->addCompleteElement(d->rawOdfMasterStyles.constData());
+    }
+
+    xmlWriter->endElement(); // office:master-styles
+}
+
+void KoGenStyles::addRawOdfDocumentStyles(const QByteArray& xml)
+{
+    addRawOdfStyles(xml, d->rawOdfDocumentStyles);
+}
+
+void KoGenStyles::addRawOdfAutomaticStyles(const QByteArray& xml, bool stylesDotXml)
+{
+    addRawOdfStyles(xml, d->rawOdfAutomaticStyles(stylesDotXml));
+}
+
+void KoGenStyles::addRawOdfMasterStyles(const QByteArray& xml)
+{
+    addRawOdfStyles(xml, d->rawOdfMasterStyles);
 }
 
 void KoGenStyles::saveOdfFontFaceDecls(KoXmlWriter* xmlWriter) const
