@@ -17,8 +17,8 @@
  */
 
 
-#ifndef KIS_STRATEGY_COLORSPACE_RGB_TESTER_H
-#define KIS_STRATEGY_COLORSPACE_RGB_TESTER_H
+#ifndef KO_STRATEGY_COLORSPACE_RGB_TESTER_H
+#define KO_STRATEGY_COLORSPACE_RGB_TESTER_H
 
 #include <QtTest/QtTest>
 
