@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
 
    Copyright (C) 2006-2008 Thorsten Zachmann <zachmann@kde.org>
-   Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+   Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
    Copyright (C) 2008-2009 Jan Hambrecht <jaham@gmx.net>
    Copyright (C) 2008 Casper Boemann <cbr@boemann.dk>
 
@@ -188,63 +188,63 @@ bool DefaultTool::wantsAutoScroll()
 
 void DefaultTool::setupActions()
 {
-    KAction* actionBringToFront = new KAction(KIcon("object-order-front"),
+    KAction* actionBringToFront = new KAction(KIcon("object-order-front-koffice"),
                                                i18n("Bring to &Front"), this);
     addAction("object_order_front", actionBringToFront);
     actionBringToFront->setShortcut(QKeySequence("Ctrl+Shift+]"));
     connect(actionBringToFront, SIGNAL(triggered()), this, SLOT(selectionBringToFront()));
 
-    KAction* actionRaise = new KAction(KIcon("object-order-raise"), i18n("&Raise"), this);
+    KAction* actionRaise = new KAction(KIcon("object-order-raise-koffice"), i18n("&Raise"), this);
     addAction("object_order_raise", actionRaise);
     actionRaise->setShortcut(QKeySequence("Ctrl+]"));
     connect(actionRaise, SIGNAL(triggered()), this, SLOT(selectionMoveUp()));
 
-    KAction* actionLower = new KAction(KIcon("object-order-lower"), i18n("&Lower"), this);
+    KAction* actionLower = new KAction(KIcon("object-order-lower-koffice"), i18n("&Lower"), this);
     addAction("object_order_lower", actionLower);
     actionLower->setShortcut(QKeySequence("Ctrl+["));
     connect(actionLower, SIGNAL(triggered()), this, SLOT(selectionMoveDown()));
 
-    KAction* actionSendToBack = new KAction(KIcon("object-order-back"),
+    KAction* actionSendToBack = new KAction(KIcon("object-order-back-koffice"),
                                              i18n("Send to &Back"), this);
     addAction("object_order_back", actionSendToBack);
     actionSendToBack->setShortcut(QKeySequence("Ctrl+Shift+["));
     connect(actionSendToBack, SIGNAL(triggered()), this, SLOT(selectionSendToBack()));
 
-    KAction* actionAlignLeft = new KAction(KIcon("object-align-horizontal-left"),
+    KAction* actionAlignLeft = new KAction(KIcon("object-align-horizontal-left-koffice"),
                                             i18n("Align Left"), this);
     addAction("object_align_horizontal_left", actionAlignLeft);
     connect(actionAlignLeft, SIGNAL(triggered()), this, SLOT(selectionAlignHorizontalLeft()));
 
-    KAction* actionAlignCenter = new KAction(KIcon("object-align-horizontal-center"),
+    KAction* actionAlignCenter = new KAction(KIcon("object-align-horizontal-center-koffice"),
                                               i18n("Horizontally Center"), this);
     addAction("object_align_horizontal_center", actionAlignCenter);
     connect(actionAlignCenter, SIGNAL(triggered()), this, SLOT(selectionAlignHorizontalCenter()));
 
-    KAction* actionAlignRight = new KAction(KIcon("object-align-horizontal-right"),
+    KAction* actionAlignRight = new KAction(KIcon("object-align-horizontal-right-koffice"),
                                              i18n("Align Right"), this);
     addAction("object_align_horizontal_right", actionAlignRight);
     connect(actionAlignRight, SIGNAL(triggered()), this, SLOT(selectionAlignHorizontalRight()));
 
-    KAction* actionAlignTop = new KAction(KIcon("object-align-vertical-top"), i18n("Align Top"), this);
+    KAction* actionAlignTop = new KAction(KIcon("object-align-vertical-top-koffice"), i18n("Align Top"), this);
     addAction("object_align_vertical_top", actionAlignTop);
     connect(actionAlignTop, SIGNAL(triggered()), this, SLOT(selectionAlignVerticalTop()));
 
-    KAction* actionAlignMiddle = new KAction(KIcon("object-align-vertical-center"),
+    KAction* actionAlignMiddle = new KAction(KIcon("object-align-vertical-center-koffice"),
                                               i18n("Vertically Center"), this);
     addAction("object_align_vertical_center", actionAlignMiddle);
     connect(actionAlignMiddle, SIGNAL(triggered()), this, SLOT(selectionAlignVerticalCenter()));
 
-    KAction* actionAlignBottom = new KAction(KIcon("object-align-vertical-bottom"),
+    KAction* actionAlignBottom = new KAction(KIcon("object-align-vertical-bottom-koffice"),
                                               i18n("Align Bottom"), this);
     addAction("object_align_vertical_bottom", actionAlignBottom);
     connect(actionAlignBottom, SIGNAL(triggered()), this, SLOT(selectionAlignVerticalBottom()));
 
-    KAction* actionGroupBottom = new KAction(KIcon("object-group"),
+    KAction* actionGroupBottom = new KAction(KIcon("object-group-koffice"),
                                               i18n("Group"), this);
     addAction("object_group", actionGroupBottom);
     connect(actionGroupBottom, SIGNAL(triggered()), this, SLOT(selectionGroup()));
 
-    KAction* actionUngroupBottom = new KAction(KIcon("object-ungroup"),
+    KAction* actionUngroupBottom = new KAction(KIcon("object-ungroup-koffice"),
                                                 i18n("Ungroup"), this);
     addAction("object_ungroup", actionUngroupBottom);
     connect(actionUngroupBottom, SIGNAL(triggered()), this, SLOT(selectionUngroup()));
@@ -555,8 +555,8 @@ void DefaultTool::selectGuideAtPosition(const QPointF &position)
     // check if we are on a guide line
     KoGuidesData * guidesData = m_canvas->guidesData();
     if (guidesData && guidesData->showGuideLines()) {
-        qreal handleRadius = m_canvas->resourceProvider()->handleRadius();
-        qreal minDistance = m_canvas->viewConverter()->viewToDocumentX(handleRadius);
+        qreal grabSensitivity = m_canvas->resourceProvider()->grabSensitivity();
+        qreal minDistance = m_canvas->viewConverter()->viewToDocumentX(grabSensitivity);
         uint i = 0;
         foreach (qreal guidePos, guidesData->horizontalGuideLines()) {
             qreal distance = qAbs(guidePos - position.y());
@@ -598,15 +598,6 @@ QRectF DefaultTool::handlesSize()
     return bound;
 }
 
-QRectF DefaultTool::handleRectAt( const QPointF &point )
-{
-    qreal handleRadius = m_canvas->resourceProvider()->handleRadius();
-    QRectF handleRect = QRectF(QPointF(), QSizeF(2*handleRadius,2*handleRadius));
-    handleRect = m_canvas->viewConverter()->viewToDocument(handleRect);
-    handleRect.moveCenter(point);
-    return handleRect;
-}
-
 void DefaultTool::mouseReleaseEvent(KoPointerEvent *event)
 {
     KoInteractionTool::mouseReleaseEvent(event);
@@ -643,25 +634,28 @@ bool DefaultTool::moveSelection(int direction, Qt::KeyboardModifiers modifiers)
 {
     qreal x=0.0, y=0.0;
     if (direction == Qt::Key_Left)
-        x=-0.5;
+        x = -5;
     else if (direction == Qt::Key_Right)
-        x=0.5;
+        x = 5;
     else if (direction == Qt::Key_Up)
-        y=-0.5;
+        y = -5;
     else if (direction == Qt::Key_Down)
-        y=0.5;
+        y = 5;
 
     if (x != 0.0 || y != 0.0) { // actually move
-        if ((modifiers & Qt::ShiftModifier) == 0) { // no shift used
+        if ((modifiers & Qt::ShiftModifier) != 0) {
             x *= 10;
             y *= 10;
+        } else if ((modifiers & Qt::AltModifier) != 0) { // more precise
+            x /= 5;
+            y /= 5;
         }
 
         QList<QPointF> prevPos;
         QList<QPointF> newPos;
         QList<KoShape*> shapes;
-        foreach(KoShape* shape, koSelection()->selectedShapes(KoFlake::StrippedSelection)) {
-            if (shape->isLocked())
+        foreach(KoShape* shape, koSelection()->selectedShapes(KoFlake::TopLevelSelection)) {
+            if (shape->isGeometryProtected())
                 continue;
             shapes.append(shape);
             QPointF p = shape->position();
@@ -784,7 +778,7 @@ void DefaultTool::deleteSelection()
 {
     QList<KoShape *> shapes;
     foreach (KoShape *s, m_canvas->shapeManager()->selection()->selectedShapes(KoFlake::TopLevelSelection)) {
-        if (s->isLocked())
+        if (s->isGeometryProtected())
             continue;
         shapes << s;
     }
@@ -800,13 +794,7 @@ bool DefaultTool::paste()
     bool success = false;
     if (data->hasFormat(KoOdf::mimeType(KoOdf::Text))) {
         KoShapeManager * shapeManager = m_canvas->shapeManager();
-        int zIndex = 0;
-        foreach (KoShape *shape, shapeManager->shapes())
-        {
-            zIndex = qMax(zIndex, shape->zIndex());
-        }
-
-        KoShapePaste paste(m_canvas, zIndex + 1, shapeManager->selection()->activeLayer());
+        KoShapePaste paste(m_canvas, shapeManager->selection()->activeLayer());
         success = paste.paste(KoOdf::Text, data);
     }
     return success;
@@ -852,7 +840,7 @@ KoFlake::SelectionHandle DefaultTool::handleAt(const QPointF &point, bool *inner
     {
         QPainterPath path;
         path.addPolygon(m_selectionOutline);
-        *innerHandleMeaning = path.contains(point) || path.intersects(handleRectAt(point));
+        *innerHandleMeaning = path.contains(point) || path.intersects(handlePaintRect(point));
     }
     for (int i = 0; i < KoFlake::NoHandle; ++i) {
         KoFlake::SelectionHandle handle = handleOrder[i];
@@ -971,14 +959,10 @@ void DefaultTool::selectionGroup()
         }
     }
     KoShapeGroup *group = new KoShapeGroup();
-    // is this needed?
-    if (selection->activeLayer()) {
-        selection->activeLayer()->addChild(group);
-    }
     // TODO what if only one shape is left?
     QUndoCommand *cmd = new QUndoCommand(i18n("Group shapes"));
     m_canvas->shapeController()->addShapeDirect(group, cmd);
-    new KoShapeGroupCommand(group, groupedShapes, cmd);
+    KoShapeGroupCommand::createCommand(group, groupedShapes, cmd);
     m_canvas->addCommand(cmd);
 }
 
@@ -1005,7 +989,9 @@ void DefaultTool::selectionUngroup()
         KoShapeGroup *group = dynamic_cast<KoShapeGroup*>(shape);
         if (group) {
             cmd = cmd ? cmd : new QUndoCommand(i18n("Ungroup shapes"));
-            new KoShapeUngroupCommand(group, group->iterator(), cmd);
+            new KoShapeUngroupCommand(group, group->childShapes(),
+                                      group->parent()? QList<KoShape*>(): m_canvas->shapeManager()->topLevelShapes(),
+                                      cmd);
             m_canvas->shapeController()->removeShape(group, cmd);
         }
     }
@@ -1083,7 +1069,9 @@ void DefaultTool::selectionReorder(KoShapeReorderCommand::MoveShapeType order)
         return;
 
     QUndoCommand * cmd = KoShapeReorderCommand::createCommand(editableShapes, m_canvas->shapeManager(), order);
-    m_canvas->addCommand(cmd);
+    if (cmd) {
+        m_canvas->addCommand(cmd);
+    }
 }
 
 QMap<QString, QWidget *> DefaultTool::createOptionWidgets()
@@ -1105,7 +1093,7 @@ void DefaultTool::resourceChanged(int key, const QVariant & res)
 
 KoInteractionStrategy *DefaultTool::createStrategy(KoPointerEvent *event)
 {
-    // reset the move by keys when a new strategy is created otherwise we might change the 
+    // reset the move by keys when a new strategy is created otherwise we might change the
     // command after a new command was added. This happend when you where faster than the timer.
     m_moveCommand = 0;
 
@@ -1168,7 +1156,7 @@ KoInteractionStrategy *DefaultTool::createStrategy(KoPointerEvent *event)
         }
         if (! (selectMultiple || selectNextInStack) && event->buttons() == Qt::LeftButton) {
             const QPainterPath outlinePath = select->transformation().map(select->outline());
-            if (outlinePath.contains(event->point) || outlinePath.intersects(handleRectAt(event->point)))
+            if (outlinePath.contains(event->point) || outlinePath.intersects(handlePaintRect(event->point)))
                 return new ShapeMoveStrategy(this, m_canvas, event->point);
         }
     }
@@ -1176,9 +1164,9 @@ KoInteractionStrategy *DefaultTool::createStrategy(KoPointerEvent *event)
     if ((event->buttons() & Qt::LeftButton) == 0)
         return 0;  // Nothing to do for middle/right mouse button
 
-    KoShape * object(shapeManager->shapeAt(event->point, selectNextInStack ? KoFlake::NextUnselected : KoFlake::ShapeOnTop));
+    KoShape *shape(shapeManager->shapeAt(event->point, selectNextInStack ? KoFlake::NextUnselected : KoFlake::ShapeOnTop));
 
-    if (!object && handle == KoFlake::NoHandle) {
+    if (!shape && handle == KoFlake::NoHandle) {
         // check if we have hit a guide
         if (m_guideLine->isValid()) {
             m_guideLine->select();
@@ -1191,17 +1179,17 @@ KoInteractionStrategy *DefaultTool::createStrategy(KoPointerEvent *event)
         return new KoShapeRubberSelectStrategy(this, m_canvas, event->point);
     }
 
-    if (select->isSelected(object)) {
+    if (select->isSelected(shape)) {
         if (selectMultiple) {
             repaintDecorations();
-            select->deselect(object);
+            select->deselect(shape);
         }
     }
-    else if (handle == KoFlake::NoHandle) { // clicked on object which is not selected
+    else if (handle == KoFlake::NoHandle) { // clicked on shape which is not selected
         repaintDecorations();
         if (! selectMultiple)
             shapeManager->selection()->deselectAll();
-        select->select(object, selectNextInStack ? false : true);
+        select->select(shape, selectNextInStack ? false : true);
         repaintDecorations();
         return new ShapeMoveStrategy(this, m_canvas, event->point);
     }
