@@ -25,8 +25,10 @@
 
 class Canvas;
 class KoShape;
+class KoPointerEvent;
 
-class SelectStrategy : public QObject , public InteractionStrategy {
+class SelectStrategy : public QObject, public InteractionStrategy
+{
     Q_OBJECT
 public:
     SelectStrategy(Canvas *canvas, KoShape *clickedShape, KoPointerEvent &event);
