@@ -38,6 +38,7 @@
 
 #include "MusicShape.h"
 #include "Renderer.h"
+#include "MusicCursor.h"
 
 #include "SimpleEntryTool.h"
 #include "SimpleEntryTool.moc"
@@ -56,6 +57,8 @@
 #include "actions/SelectionAction.h"
 
 #include "commands/AddBarsCommand.h"
+#include "commands/CreateChordCommand.h"
+#include "commands/AddNoteCommand.h"
 
 #include "core/Sheet.h"
 #include "core/Part.h"
@@ -67,6 +70,8 @@
 #include "core/StaffSystem.h"
 #include "core/MusicXmlReader.h"
 #include "core/MusicXmlWriter.h"
+#include "core/KeySignature.h"
+#include "core/Note.h"
 
 #include <algorithm>
 
@@ -76,7 +81,8 @@ SimpleEntryTool::SimpleEntryTool( KoCanvasBase* canvas )
     : KoTool( canvas ),
     m_musicshape(0),
     m_voice(0),
-    m_selectionStart(-1)
+    m_selectionStart(-1),
+    m_cursor(0)
 {
     QActionGroup* actionGroup = new QActionGroup(this);
     connect(actionGroup, SIGNAL(triggered(QAction*)), this, SLOT(activeActionChanged(QAction*)));
@@ -94,38 +100,47 @@ SimpleEntryTool::SimpleEntryTool( KoCanvasBase* canvas )
     connect(addBars, SIGNAL(triggered()), this, SLOT(addBars()));
 
     AbstractMusicAction* actionBreveNote = new NoteEntryAction(BreveNote, false, this);
+    actionBreveNote->setShortcut(Qt::Key_9);
     addAction("note_breve", actionBreveNote);
     actionGroup->addAction(actionBreveNote);
 
     AbstractMusicAction* actionWholeNote = new NoteEntryAction(WholeNote, false, this);
+    actionWholeNote->setShortcut(Qt::Key_8);
     addAction("note_whole", actionWholeNote);
     actionGroup->addAction(actionWholeNote);
 
     AbstractMusicAction* actionHalfNote = new NoteEntryAction(HalfNote, false, this);
+    actionHalfNote->setShortcut(Qt::Key_7);
     addAction("note_half", actionHalfNote);
     actionGroup->addAction(actionHalfNote);
 
     AbstractMusicAction* actionQuarterNote = new NoteEntryAction(QuarterNote, false, this);
+    actionQuarterNote->setShortcut(Qt::Key_6);
     addAction("note_quarter", actionQuarterNote);
     actionGroup->addAction(actionQuarterNote);
 
     AbstractMusicAction* actionNote8 = new NoteEntryAction(EighthNote, false, this);
+    actionNote8->setShortcut(Qt::Key_5);
     addAction("note_eighth", actionNote8);
     actionGroup->addAction(actionNote8);
 
     AbstractMusicAction* actionNote16 = new NoteEntryAction(SixteenthNote, false, this);
+    actionNote16->setShortcut(Qt::Key_4);
     addAction("note_16th", actionNote16);
     actionGroup->addAction(actionNote16);
 
     AbstractMusicAction* actionNote32 = new NoteEntryAction(ThirtySecondNote, false, this);
+    actionNote32->setShortcut(Qt::Key_3);
     addAction("note_32nd", actionNote32);
     actionGroup->addAction(actionNote32);
 
     AbstractMusicAction* actionNote64 = new NoteEntryAction(SixtyFourthNote, false, this);
+    actionNote64->setShortcut(Qt::Key_2);
     addAction("note_64th", actionNote64);
     actionGroup->addAction(actionNote64);
 
     AbstractMusicAction* actionNote128 = new NoteEntryAction(HundredTwentyEighthNote, false, this);
+    actionNote128->setShortcut(Qt::Key_1);
     addAction("note_128th", actionNote128);
     actionGroup->addAction(actionNote128);
 
@@ -216,7 +231,8 @@ SimpleEntryTool::SimpleEntryTool( KoCanvasBase* canvas )
     connect(action, SIGNAL(triggered()), this, SLOT(actionTriggered()));
     clefMenu->addAction(action = new SetClefAction(Clef::Soprano, 1, 0, this));
     connect(action, SIGNAL(triggered()), this, SLOT(actionTriggered()));
-
+    m_menus.append(clefMenu);
+    
     QList<QAction*> contextMenu;
 
     KAction* clefAction = new KAction(i18n("Clef"), this);
@@ -246,7 +262,8 @@ SimpleEntryTool::SimpleEntryTool( KoCanvasBase* canvas )
     connect(action, SIGNAL(triggered()), this, SLOT(actionTriggered()));
     tsMenu->addAction(action = new TimeSignatureAction(this, 12, 8));
     connect(action, SIGNAL(triggered()), this, SLOT(actionTriggered()));
-
+    m_menus.append(tsMenu);
+    
     KAction* timeSigAction = new KAction(i18n("Time signature"), this);
     timeSigAction->setMenu(tsMenu);
     contextMenu.append(timeSigAction);
@@ -275,7 +292,8 @@ SimpleEntryTool::SimpleEntryTool( KoCanvasBase* canvas )
     ksMenu->addSeparator();
     ksMenu->addAction(action = new KeySignatureAction(this));
     connect(action, SIGNAL(triggered()), this, SLOT(actionTriggered()));
-
+    m_menus.append(ksMenu);
+    
     KAction* keySigAction = new KAction(i18n("Key signature"), this);
     keySigAction->setMenu(ksMenu);
     contextMenu.append(keySigAction);
@@ -284,12 +302,12 @@ SimpleEntryTool::SimpleEntryTool( KoCanvasBase* canvas )
     connect(removeBarAction, SIGNAL(triggered()), this, SLOT(actionTriggered()));
     contextMenu.append(removeBarAction);
 
-
     setPopupActionList(contextMenu);
 }
 
 SimpleEntryTool::~SimpleEntryTool()
 {
+    qDeleteAll(m_menus);
 }
 
 void SimpleEntryTool::activate (bool temporary)
@@ -300,7 +318,11 @@ void SimpleEntryTool::activate (bool temporary)
     {
         m_musicshape = dynamic_cast<MusicShape*>( shape );
         if ( m_musicshape )
+        {
+            //TODO get the cursor that was used the last time for that sheet from some map
+            m_cursor = new MusicCursor(m_musicshape->sheet(), m_musicshape->sheet());
             break;
+        }
     }
     if ( !m_musicshape )
     {
@@ -314,6 +336,7 @@ void SimpleEntryTool::deactivate()
 {
     //kDebug()<<"SimpleEntryTool::deactivate";
     m_musicshape = 0;
+    m_cursor = 0;
 }
 
 void SimpleEntryTool::paint( QPainter& painter, const KoViewConverter& viewConverter )
@@ -332,14 +355,14 @@ void SimpleEntryTool::paint( QPainter& painter, const KoViewConverter& viewConve
         // find first shape
         MusicShape* shape = m_musicshape;
         while (shape->predecessor()) shape = shape->predecessor();
-        
+
         // now loop over all shapes
         while (shape) {
             painter.save();
             painter.setMatrix( shape->absoluteTransformation(&viewConverter) * painter.matrix() );
             KoShape::applyConversion( painter, viewConverter );
             painter.setClipRect(QRectF(QPointF(0, 0), shape->size()));            
-            
+
             for (int b = qMax(shape->firstBar(), m_selectionStart); b <= m_selectionEnd && b < sheet->barCount() && b <= shape->lastBar(); b++) {
                 Bar* bar = sheet->bar(b);
                 bool selectedStaff = false;
@@ -385,6 +408,11 @@ void SimpleEntryTool::paint( QPainter& painter, const KoViewConverter& viewConve
         }
     }
     
+    // draw cursor
+    if (m_cursor) {
+        m_activeAction->renderKeyboardPreview(painter, *m_cursor);
+    }
+
     m_activeAction->renderPreview(painter, m_point);
 }
 
@@ -445,7 +473,7 @@ void SimpleEntryTool::mousePressEvent( KoPointerEvent* event )
     }
 
 //    int line = closestStaff->line(yrel - closestStaff->top());
-//    kDebug() << "line: " << line << endl;
+//    kDebug() << "line: " << line;
 
     Part* part = closestStaff->part();
     for (int i = part->voiceCount(); i <= m_voice; i++) {
@@ -553,7 +581,7 @@ void SimpleEntryTool::mouseMoveEvent( KoPointerEvent* event )
         }
         
         //    int line = closestStaff->line(yrel - closestStaff->top());
-        //    kDebug() << "line: " << line << endl;
+        //    kDebug() << "line: " << line;
         
         Part* part = closestStaff->part();
         for (int i = part->voiceCount(); i <= m_voice; i++) {
@@ -596,6 +624,35 @@ void SimpleEntryTool::mouseReleaseEvent( KoPointerEvent* )
 {
 }
 
+void SimpleEntryTool::keyPressEvent( QKeyEvent *event )
+{
+    event->ignore();
+    m_activeAction->keyPress(event, *m_cursor);
+    if (!event->isAccepted()) {
+        event->accept();
+        switch (event->key()) {
+            case Qt::Key_Left:
+                m_cursor->moveLeft();
+                m_musicshape->update();
+                break;
+            case Qt::Key_Right:
+                m_cursor->moveRight();
+                m_musicshape->update();
+                break;
+            case Qt::Key_Up:
+                m_cursor->moveUp();
+                m_musicshape->update();
+                break;
+            case Qt::Key_Down:
+                m_cursor->moveDown();
+                m_musicshape->update();
+                break;
+            default:
+                event->ignore();
+        }
+    }
+}
+
 void SimpleEntryTool::addCommand(QUndoCommand* command)
 {
     m_canvas->addCommand(command);
@@ -615,14 +672,15 @@ void SimpleEntryTool::activeActionChanged(QAction* action)
 {
     bool oldVoiceAware = m_activeAction->isVoiceAware();
     m_activeAction = qobject_cast<AbstractMusicAction*>(action);
-    if (m_activeAction->isVoiceAware() != oldVoiceAware) {
+    //if (m_activeAction->isVoiceAware() != oldVoiceAware) {
         m_musicshape->update();
     //    reinterpret_cast<SimpleEntryWidget*>(optionWidget())->setVoiceListEnabled(m_activeAction->isVoiceAware());
-    }
+    //}
 }
 
 void SimpleEntryTool::voiceChanged(int voice)
 {
+    m_cursor->setVoice(voice);
     m_voice = voice;
     m_musicshape->update();
 }
